#!/usr/bin/env python

"""
./7_BANN_predictionw.py ./4_top70/alldiseasef ./4_top70/alldiseasep ./parameterDbest.txt 0.5844
./7_BANN_predictionw.py ./4_top70/allneutralf ./4_top70/allneutralp ./parameterNbest.txt 0.4156
./7_BANN_predictionw.py ./4_top70/alldiseasef ./4_top70/alldiseasedp ./parameterDNbest.txt 0.1688
"""
import sys,math,random
import numpy as np

class NN:

	def __init__(self, NI, NH1, NH2, NO, Eta, Mu, R, C):
		"""
		number of nodes in layers. 
		NI: input layer; 
		NH1: the firet hidden layer; 
		NH2: the second hidden layer;
		NO: ouput layer,here NO = 1.
		P: a 1*NI array which is related with feature's probabilities
		"""
		self.ni = NI # +1 for bias
		self.nh1 = NH1
		self.nh2 = self.ni # the number of nodes = the number of nodes in input layer in my method
		self.no = NO

		#initialize node-activations
		self.ai = np.ones((self.ni,1))
		self.ah1 = np.ones((self.nh1,1))
		self.ah2 = np.ones((self.nh2,1))
		self.ao = np.ones((self.no,1))
		
		#create node weigh matirces, initialize node weights to random values
		self.w1 = np.random.uniform(-0.2,0.2,(self.nh1,self.ni))
		self.w2 = np.random.uniform(-0.2,0.2,(self.nh2,self.nh1))
		self.p = np.ones((1,self.ni))
		self.h = Eta
		self.u = Mu
		self.r = np.array([R]*self.nh1,dtype='float64').reshape(self.nh1,1)
		self.c = np.array([C]*self.nh1,dtype='float64').reshape(self.nh1,1) 


	def update_param(self,parameterFile):
		inf = open(parameterFile,'r')
		lines = inf.readlines()
		for i in range(self.nh1): 
			cells = lines[i].strip().split(" ")
			for j in range(self.ni):
				self.w1[i,j] = float(cells[j])
		print self.nh1,self.nh2+self.nh1

		for i in range(self.nh2): 
			cells = lines[i+self.nh1].strip().split(" ")
			for j in range(self.nh1):
				self.w2[i,j] = float(cells[j])
		
		cells = lines[self.nh1+self.nh2].strip().split(" ")
		for i in range(self.r.shape[0]):
			self.r[i,0] = float(cells[i])

		cells = lines[1+self.nh1+self.nh2].strip().split(" ")
		for i in range(self.c.shape[0]):
			self.c[i,0] = float(cells[i])


	def predict(self, inputs, P, W1, W2, R, C):
		"""
		inputs is a NI*N array:
		NI is the number of inputs for each data
		N is the number of datas

		P is a N*NI
		
		"""
		# Hidden Layer 1
		#print "*************inputs\n"
		#print inputs	
		#print "*************inputs end\n"
		net1 = np.dot(W1,inputs)
		#print "************* net\n"
		#print net1	
		#print "*************net end\n"
		ah1 = sigmoid(net1,R,C)

		# Hidden Layer 2
		net2 = np.dot(W2,ah1)
		ah2 = net2

		# Out layer
		ao =np.zeros((P.shape[0],1))
		for i in range(0,P.shape[0]):
			ao[i,0] = np.dot(P[i],ah2[:,i].reshape(len(P[i]),1))
		
		return ao

		
	
	
	#def trainV2(self, inputs, possibility, targets,parameterFile):
	def trainV2(self, inputs, possibility,parameterFile):

		Ndata = inputs.shape[0]
		Ninput = inputs.shape[1]
		testNdata = inputs.shape[0]
		test = self.predict(inputs.T,possibility,self.w1,self.w2,self.r,self.c)
		#error = math.sqrt(sum((targets-test)**2)/Ndata)
		#mcc = MCC(targets,test)
		#print mcc
		#print test
		return test

		
def sigmoid(x,r,c):
	return 1/(1+np.exp(-r*(x-c)))

def dsigmoid(x):
	return x*(1-x)

def readfile(fin):
	inlist = []
	f = open(fin,"r")
	line = f.readline()
	while line:
		cells = line.strip().split()
		for i in range(len(cells)):
			if cells[i] == "#":
				cells[i]=0.0
		inlist.append(cells)
		line = f.readline()
	f.close()
	return np.array(inlist).astype(np.float)

def MCC(targets,predictions):
	tp, fn, tn, fp = 0.0, 0.0, 0.0, 0.0
	for i in range(targets.shape[0]):
		if targets[i,0] == 1.0:
			if predictions[i,0] >= 0.5:
				tp += 1
			else:
				fn += 1
		else:
			if predictions[i,0] < 0.5:
				tn += 1
			else:
				fp += 1
	#print tp,fn,tn,fp
	if (tp+fp) == 0.0 or (tp+fn) == 0.0 or (tn+fp) == 0.0 or (tn+fn) ==0.0:
		return 0.0
	return (tp*tn-fp*fn)/math.sqrt((tp+fp)*(tp+fn)*(tn+fp)*(tn+fn))

#def runANN(inputf,possibilityf,targetf,parameterFile,classlogP):
def runANN(inputf,possibilityf,parameterFile,classlogP):
	
	inputs = readfile(inputf)
	possibility = readfile(possibilityf)
	#targets = readfile(targetf)
	#print targets.shape
	
	biasIn = np.ones((inputs.shape[0],1))
	biasP = np.array([classlogP]*possibility.shape[0]).reshape(possibility.shape[0],1)

	inputs = np.hstack((inputs,biasIn))
	possibility = np.hstack((possibility,biasP))
	possibility = possibility*10

	NI = inputs.shape[1]
	NH1 = 20
	NH2 = NI
	NO = 1
	Eta = 0.001
	Mu = 0.001
	R = 1
	C = 0.5
	myNN = NN(NI, NH1, NH2, NO, Eta, Mu, R, C)
	myNN.update_param(parameterFile)
	#test = myNN.trainV2(inputs,possibility,targets,parameterFile)
	test = myNN.trainV2(inputs,possibility,parameterFile)
	fout = open(possibilityf+"_results","w")
	print test.shape
	for i in range(test.shape[0]):
		fout.write(str(test[i,0])+"\n")
	fout.close()
	#print test
	#print "max MCC = "+str(MCC(targets,test))


if __name__ == "__main__":
	inputf = sys.argv[1]
	possibilityf = sys.argv[2]
	#targetf = sys.argv[3]
	parameterFile = sys.argv[3]
	classlogP = float(sys.argv[4])
	#runANN(inputf,possibilityf,targetf,parameterFile,classlogP)
	runANN(inputf,possibilityf,parameterFile,classlogP)

