#!/usr/bin/env python
import sys,os,math,re
sys.path.insert(1,'/nfs/amino-home/zhanglabs/PreDAM/bin/python')
import preparation

to1letter = {
	'ALA':'A',
	'ARG':'R',
	'ASN':'N',
	'ASP':'D',
	'CYS':'C',
	'GLU':'E',
	'GLN':'Q',
	'GLY':'G',
	'HIS':'H',
	'ILE':'I',
	'LEU':'L',
	'LYS':'K',
	'MET':'M',
	'PHE':'F',
	'PRO':'P',
	'SER':'S',
	'THR':'T',
	'TRP':'W',
	'TYR':'Y',
	'VAL':'V'}

volumeR={
    'A':2.30,
    'C':2.50,
    'D':2.80,
    'E':3.05,
    'F':3.40,
    'G':1.90,
    'H':3.10,
    'I':3.09,
    'K':3.15,
    'L':3.15,
    'M':3.10,
    'N':2.85,
    'P':2.80,
    'Q':3.05,
    'R':3.16,
    'S':2.40,
    'T':2.80,
    'V':2.91,
    'W':3.60,
    'Y':3.45}

weightR={
    'A':89.09,
    'C':121.16,
    'D':133.1,
    'E':147.13,
    'F':165.19,
    'G':75.07,
    'H':155.16,
    'I':131.18,
    'K':146.19,
    'L':131.18,
    'M':149.21,
    'N':132.12,
    'P':115.13,
    'Q':146.15,
    'R':174.2,
    'S':105.09,
    'T':119.12,
    'V':117.15,
    'W':204.23,
    'Y':181.19
     }
"""
hydrophobicityR={
    'A':-0.2,
    'C':0.4,
    'D':11.5,
    'E':9.5,
    'F':-1.5,
    'G':0,
    'H':6.8,
    'I':-2.6,
    'K':7.4,
    'L':-2.6,
    'M':1.3,
    'N':7.7,
    'P':2.8,
    'Q':5.4,
    'R':19.8,
    'S':0.8,
    'T':1.1,
    'V':-1.2,
    'W':1.3,
    'Y':4.3
     }

isoelectricR={
    'A':6.01,
    'C':5.07,
    'D':2.77,
    'E':3.22,
    'F':5.48,
    'G':5.97,
    'H':7.59,
    'I':6.02,
    'K':9.74,
    'L':5.98,
    'M':5.74,
    'N':5.41,
    'P':6.48,
    'Q':5.65,
    'R':10.76,
    'S':5.68,
    'T':5.87,
    'V':5.97,
    'W':5.89,
    'Y':5.66
     }
"""

psicAA = {
	'A':1, 
	'R':2, 
	'N':3, 
	'D':4, 
	'C':5, 
	'Q':6, 
	'E':7, 
	'G':8, 
	'H':9, 
	'I':10, 
	'L':11, 
	'K':12, 
	'M':13, 
	'F':14, 
	'P':15, 
	'S':16, 
	'T':17, 
	'W':18, 
	'Y':19, 
	'V':20} 


def AAinf(aa1,aa2):
	wVol = volumeR[aa1]
	mVol = volumeR[aa2] 
	dV = mVol - wVol

	wWeight = weightR[aa1] 
	mWeight = weightR[aa2] 
	dW = mWeight - wWeight

	"""
	wHydro = hydrophobicityR[aa1]
	mHydro = hydrophobicityR[aa2]
	dH = mWeight - wWeight
	
	wIso = isoelectricR[aa1] 
	mIso = isoelectricR[aa2] 
	dI = mIso - wIso

	return wVol, mVol, dV, wWeight, mWeight, dW, wHydro, mHydro, dH, wIso, mIso, dI
	"""
	return wVol, mVol, dV, wWeight, mWeight, dW

def psicScore(infile,aa1,aa2,pos):
	pos = int(pos)
	if not os.path.isfile(infile):
		print 'Error: '+infile+' does not exist!'
		return None, None

	f = open(infile,'r')
	content = f.read()
	lines = content.strip().split('\n')
	f.close()

	if len(lines) < 2:
		print 'Error: '+infile+' is defective.'
		return None, None
	for i in range(1,len(lines)):
		rows = lines[i].strip().split()	
		if int(rows[0]) == pos:
			wPsic = rows[psicAA[aa1]]
			mPsic = rows[psicAA[aa2]]
			#print 'psic pass!'
			return float(wPsic),float(mPsic)
	print "Warning: mutation"+aa1+str(pos)+aa2+' can not be found in '+ infile+'.'
	return None, None

def tpsicScore(infile,aa1,aa2,pos):
	pos = int(pos)
	if not os.path.isfile(infile):
		print 'Error: '+infile+' does not exist!'
		return None, None

	f = open(infile,'r')
	content = f.read()
	lines = content.strip().split('\n')
	f.close()

	if len(lines) < 3:
		print 'Error: '+infile+' is defective.'
		return None, None
	for i in range(2,len(lines)):
		rows = lines[i].strip().split()	
		if int(rows[0]) == pos:
			wPsic = rows[psicAA[aa1]]
			mPsic = rows[psicAA[aa2]]
			#print 'tpsic pass!'
			return float(wPsic), float(mPsic)
	print "Warning: mutation"+aa1+str(pos)+aa2+' can not be found in '+ infile+'.'
	return None, None

def pharmacophore(infile):
	if not os.path.isfile(infile):
		print 'Error: '+infile+' does not exist!'
		return None, None
	f = open(infile, "r")
	line = f.readline()
	wildVector = map(int,line.strip().split())
	#line = f.readline()
	neighborVector = [0,0,0,0,0,0,0,0,0,0,0,0]
	while line:
		cells = line.strip().split()
		for i in range(0,len(cells)):
			neighborVector[i] += int(cells[i])
		line = f.readline()
	f.close()
	return wildVector, neighborVector

def pfamScore(infile):
	#print infile
	if not os.path.isfile(infile):
		print 'Error: '+infile+' does not exist!'
		return None, None, None
	f = open(infile,"r")
	line = f.readline().strip().split()
	f.close()
	if line[0] == '#':
		#print "*********************-1  -1 -1"
		#return -1.0,-1.0,-1.0
		return '#','#','#'
	elif float(line[0]) == -1.0 and float(line[1]) == -1.0:
		#print '---------------------1 -1 -1'
		#return -1.0,-1.0,-1.0
		return '#','#','#'
	else:
		return float(line[0]),float(line[1]),float(line[2])

def concavity(infile,pos):
	if not os.path.isfile(infile):
		print 'Error: '+infile+' does not exist!'
		return None
	f = open(infile,'r')
	lines = f.read().strip().split('\n')
	concavity = 0.0
	for line in lines:
		if re.match('^ATOM',line):
			atomName = line[12:16].strip()
			resIndex = int(line[22:26])
			if atomName == 'CA' and resIndex == int(pos):
				concavity = float(line[61:66])
				return concavity
				break
	print "Warning: mutation"+str(pos)+' can not be found in '+ infile+'.'
	return None


def JSD_aa_score(infile,aa1,aa2,pos):
	posTemp = int(pos)-1
	#residue JSD_score
	if not os.path.isfile(infile):
		print 'Error: '+infile+' does not exist!'
		return None, None

	f = open(infile,'r')
	content = f.read()
	lines = content.strip().split('\n')
	f.close()

	if len(lines) < 2:
		print 'Error: '+infile+' is defective.'
		return None, None
	for i in range(1,len(lines)):
		rows = lines[i].strip().split()	
		if int(rows[0]) == posTemp:
			wJSDaaScore = rows[psicAA[aa1]]
			mJSDaaScore = rows[psicAA[aa2]]
			#print 'psic pass!'
			return float(wJSDaaScore),float(mJSDaaScore)
	print "Warning: mutation"+aa1+str(pos)+aa2+' can not be found in '+ infile+'.'
	return None, None

def JSD_score(infile,pos):
	posTemp = int(pos)-1
	#residue JSD_score
	if not os.path.isfile(infile):
		print 'Error: '+infile+' does not exist!'
		return None

	f = open(infile,'r')
	content = f.read()
	lines = content.strip().split('\n')
	f.close()

	if len(lines) < 1:
		print 'Error: '+infile+' is defective.'
		return None
	for i in range(len(lines)):
		rows = lines[i].strip().split()	
		if int(rows[0]) == posTemp:
			jsdScore = rows[2]
			return float(jsdScore)
	print "Warning: mutation "+str(pos)+' can not be found in '+ infile+'.'
	return None

def depth(infile, pos):
	#depth
	if not os.path.isfile(infile):
		print 'Error: '+infile+' is defective.'
		return None
	else:
		f = open(infile, "r")
		line = f.readline()
		line = f.readline()
		while line:
			cells = line.strip().split()
			residue = int(cells[0].strip().split(":")[1])
			if residue == int(pos):
				all_atom = cells[2]
				return all_atom
				break
			line = f.readline()
		f.close()
	print "Warning: mutation "+pos+' can not be found in '+ infile+'.'
	return None

def ddGSef(infile,aa2):
	#ddG
	if not os.path.isfile(infile):
		print 'Error: '+infile+' is defective.'
		return None, None
	else:
		f = open(infile, "r")
		lines = f.read().strip().split('\n')
		f.close()
	
		if len(lines) <20:
			print "Warning: mutation"+' can not be found in '+ infile+'.'
			return None,None

		ddG = {}
		for i in range(len(lines)):
			cells = lines[i].strip().split()
			ddG[to1letter[cells[0]]]=float(cells[1])
	return ddG[aa2]			

def evoEnergy(infile,aa2,pos):
	if not os.path.isfile(infile):
		print 'Error: '+infile+' is defective.'
		return None
	else:
		f = open(infile,'r')
		lines = f.read().split('\n')
		f.close()
	if len(lines) < 1:
		print "Warning: mutation "+str(pos)+aa2+' can not be found in '+ infile+'.'
		return None
	return float(lines[len(lines)-1].strip())

def cisEnergy(infile,aa2,pos):
	if not os.path.isfile(infile):
		print 'Error: '+infile+' is defective.'
		return None, None, None, None, None, None
	else:
		f = open(infile,'r')
		lines = f.read().strip().split('\n')
		f.close()
	if len(lines) < 13:
		print "Warning: mutation "+str(pos)+aa2+' can not be found in '+ infile+'.'
		return None, None, None, None, None, None
	cells = lines[9].strip().split()
	wClap = float(cells[2])
	wSelf = float(cells[4])
	wTotal = float(cells[6])
	cells = lines[10].strip().split()
	mClap = float(cells[2])
	mSelf = float(cells[4])
	mTotal = float(cells[6])
	return wClap, wSelf, wTotal, mClap, mSelf, mTotal

def contactScore(infile):
	if not os.path.isfile(infile):
		print 'Error: '+infile+' is defective.'
		return None, None, None, None, None, None, None, None, None, None, None, None
	else:
		f = open(infile,'r')
		lines = f.read().strip().split('\n')
		f.close()
		if len(lines) <= 1:
			return 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 
		
		cells = lines[1].split('\t')
		wRes = 0
		for i in range(2,28):
			wRes += int(cells[i])

		#contact residue
		sefNum_intra = 0
		sefFunNum_intra = 0
		sefConNum_intra = 0
		sefConFunNum_intra = 0

		sefNum_inter = 0
		sefFunNum_inter = 0
		sefConNum_inter = 0
		sefConFunNum_inter = 0

		sefNum_het = 0
		sefFunNum_het = 0
		sefConNum_het = 0
		sefConFunNum_het = 0


		intra_contact = 6
		inter_contact = 8
		het_contact = 8

		#print len(lines)
		i = 0
		tmp_resNum = {}
		while i < len(lines):
			#print lines[i]
			if re.match('^intraresidue',lines[i]):
				intraFun = {}
				j = i+1
				while j < len(lines):
					#print lines[j]
					cells = lines[j].split('\t')
					if cells[0] == 'interresidue':
						i = j
						tmp_resNum = {}
						break
					elif cells[0] != '':
						if float(cells[0]) <= intra_contact:
							sefNum_intra += 1 
							#for k in range(3,29):
							tmpFun = {}
							for e in range(26):
								if int(cells[e+3]) == 1:
									intraFun[e] = int(cells[e+3])
									tmpFun[e] = int(cells[e+3])
							if len(tmpFun)>0:
								sefFunNum_intra += 1
							k = j+1
							tmp_contact = 0.0
							while k < len(lines):
								cells = lines[k].split('\t')
								if cells[0] != '':
									j = k
									break
								elif cells[1] == '*intraresidue':
									tmp_contact = intra_contact
								elif cells[1] == '*interresidue':
									tmp_contact = inter_contact
								elif cells[1] == '*het':
									tmp_contact = het_contact
								elif cells[1] == '-': 
									if float(cells[2]) <= tmp_contact:
										if not tmp_resNum.has_key(cells[3]+cells[4]):
											tmp_resNum[cells[3]+cells[4]]=1
											sefConNum_intra += 1
											tmpConFun = {}
											for e in range(26):
												if int(cells[e+5]) == 1:
													tmpConFun[e] = int(cells[e+5])
											if len(tmpConFun)>0:
												sefConFunNum_intra += 1
								k += 1
						else:
							j += 1
					else:
						j += 1
				#print "******************************************"						
			elif re.match('^interresidue',lines[i]):
				interFun = {}
				j = i+1
				while j < len(lines):
					cells = lines[j].split('\t')
					if cells[0] == 'het':
						#tmp_resNum = {}
						i = j
						break
					elif cells[0] != '':
						if float(cells[0]) <= inter_contact:
							sefNum_inter += 1 
							#for k in range(3,29):
							tmpFun = {}
							for e in range(26):
								if int(cells[e+3]) == 1:
									interFun[e] = int(cells[e+3])
									tmpFun[e] = int(cells[e+3])
							if len(tmpFun)>0:
								sefFunNum_inter += 1
							k = j+1
							tmp_contact = 0.0
							while k < len(lines):
								cells = lines[k].split('\t')
								if cells[0] != '':
									j = k
									break
								elif cells[1] == '*intraresidue':
									tmp_contact = intra_contact
								elif cells[1] == '*interresidue':
									tmp_contact = inter_contact
								elif cells[1] == '*het':
									tmp_contact = het_contact
								elif cells[1] == '-': 
									if float(cells[2]) <= tmp_contact:
										if not tmp_resNum.has_key(cells[3]+cells[4]):
											tmp_resNum[cells[3]+cells[4]]=1
											sefConNum_inter += 1
											tmpConFun = {}
											for e in range(26):
												if int(cells[e+5]) == 1:
													tmpConFun[e] = int(cells[e+5])
											if len(tmpConFun)>0:
												sefConFunNum_inter += 1
								k += 1
						else:
							j += 1
					else:
						j += 1
			
			elif re.match('^het',lines[i]):
				hetFun = {}
				j = i+1
				while j < len(lines):
					cells = lines[j].split('\t')
					if cells[0] != '':
						if float(cells[0]) <= het_contact:
							sefNum_het += 1 
							#for k in range(3,29):
							tmpFun = {}
							for e in range(26):
								if int(cells[e+3]) == 1:
									hetFun[e] = int(cells[e+3])
									tmpFun[e] = int(cells[e+3])
							if len(tmpFun)>0:
								sefFunNum_het += 1
							k = j+1
							tmp_contact = 0.0
							while k < len(lines):
								cells = lines[k].split('\t')
								if cells[1] == '*intraresidue':
									tmp_contact = intra_contact
								elif cells[1] == '*interresidue':
									tmp_contact = inter_contact
								elif cells[1] == '*het':
									tmp_contact = het_contact
								elif cells[1] == '-': 
									if float(cells[2]) <= tmp_contact:
										if not tmp_resNum.has_key(cells[3]+cells[4]):
											tmp_resNum[cells[3]+cells[4]]=1
											sefConNum_het += 1
											tmpConFun = {}
											for e in range(26):
												if int(cells[e+5]) == 1:
													tmpConFun[e] = int(cells[e+5])
											if len(tmpConFun)>0:
												sefConFunNum_het += 1
								k += 1
								if k >= len(lines):
									j = k
						else:
							j += 1
					else:
						j += 1
				i = j
				#print "^^^^^^^^^^^^^^^^"+str(i)
			else:
				i += 1
		#print sefNum_intra, sefFunNum_intra, sefConNum_intra, sefConFunNum_intra, sefNum_inter, sefFunNum_inter, sefConNum_inter, sefConFunNum_inter, sefNum_het, sefFunNum_het, sefConNum_het, sefConFunNum_het
		return sefNum_intra, sefFunNum_intra, sefConNum_intra, sefConFunNum_intra, sefNum_inter, sefFunNum_inter, sefConNum_inter, sefConFunNum_inter, sefNum_het, sefFunNum_het, sefConNum_het, sefConFunNum_het


	

def get_all_features(featuresDir,aa1,aa2,pos):
	features = []
	#amino acid featrues
	#wVol, mVol, dV, wWeight, mWeight, dW, wHydro, mHydro, dH, wIso, mIso, dI = 	AAinf(aa1,aa2)
	wVol, mVol, dV, wWeight, mWeight, dW = AAinf(aa1,aa2)
	features.append(wVol)#1
	features.append(mVol)#2
	features.append(dV)#3
	features.append(wWeight)#4
	features.append(mWeight)#5
	features.append(dW)#6
	"""
	features.append(wHydro)
	features.append(mHydro)
	features.append(dH)
	features.append(wIso)
	features.append(mIso)
	features.append(dI)
	"""
	#psic
	psicFile = featuresDir+'seqFeature/psic/seq.psic'
	wPsic, mPsic = psicScore(psicFile,aa1,aa2,pos)
	if wPsic == None:
		features.append('#')
		features.append('#')
		features.append('#')
	else:
		features.append(wPsic) #7
		features.append(mPsic) #8
		features.append(mPsic-wPsic)#9
	
	#tpsic
	tpsicFile = featuresDir+'templateFeature/tpsic/template.psic'
	wTpsic, mTpsic = tpsicScore(tpsicFile,aa1,aa2,pos)
	if wTpsic == None:
		features.append('#')
		features.append('#')
		features.append('#')
	else:
		features.append(wTpsic) #10
		features.append(mTpsic) #11
		features.append(mTpsic-wTpsic) #12

	#pharmacophore
	wpharmFile = featuresDir+'itasserFeature/pharm/'+pos+aa2+'_w.pharm'
	wVector, wneighborVector = pharmacophore(wpharmFile)
	mpharmFile = featuresDir+'itasserFeature/pharm/'+pos+aa2+'_m.pharm'
	mVector, mneighborVector = pharmacophore(mpharmFile)

	if wVector == None:
		for i in range(0,12):
			features.append("#")
	else:
		for i in range(0,len(wneighborVector)):
			features.append(wneighborVector[i])#13~24
	if mVector == None:
		print mpharmFile
		for i in range(0,12):
			features.append("#")
	else:
		for i in range(0,len(mneighborVector)):
			features.append(mneighborVector[i])#25~36
	if wVector == None or mVector == None:
		features.append("#")
		features.append("#")
		features.append("#")
		features.append("#")
		features.append("#")
		features.append("#")
		features.append("#")
		features.append("#")
	else:
		cosVector,rmsVector = cosine_rms(wVector,mVector)
		cosNeighborVector,rmsNeighborVector = cosine_rms(wneighborVector,mneighborVector)
		cosNeighborVectorSingle,rmsNeighborVectorSingle = cosine_rms(wneighborVector[0:10],mneighborVector[0:10])
		cosNeighborVectorPair, rmsNeighborVectorPair = cosine_rms(wneighborVector[10:12],mneighborVector[10:12])
		features.append(cosVector)#37	
		features.append(cosNeighborVector)	#38
		features.append(cosNeighborVectorSingle)#39	
		features.append(cosNeighborVectorPair)	#40
		features.append(rmsVector)	#41
		features.append(rmsNeighborVector)	#42
		features.append(rmsNeighborVectorSingle)	#43
		features.append(rmsNeighborVectorPair)	#44

	#pfamscore
	pfamFile = featuresDir+'seqFeature/pfam/'+pos+aa2+'.pfam'
	wPfam,mPfam,dPfam = pfamScore(pfamFile)
	if wPfam == None:
		features.append("#")
		features.append("#")
		features.append("#")
	else:
		features.append(wPfam)#45
		features.append(mPfam)#46
		features.append(dPfam)#47

	#concavity
	concavityFile = featuresDir+'itasserFeature/concavity/itasser/concavity/itasser_v2_residue.pdb'
	concavityScore = concavity(concavityFile,pos)
	if concavityScore == None:
		print concavityFile
		features.append('#')
	else:
		features.append(concavityScore)#48
		#print concavityScore

	#JSD score
	jsdAaFile = featuresDir+'itasserFeature/concavity/JSD_aa_concavity.dat'
	wJsd, mJsd = JSD_aa_score(jsdAaFile,aa1,aa2,pos)
	if wJsd == None:
		features.append('#')
		features.append('#')
		features.append('#')
	else:
		features.append(wJsd) #49
		features.append(mJsd) #50
		features.append(mJsd-wJsd)#51

	jsdFile = featuresDir+'itasserFeature/concavity/JSD_concavity.dat'
	jsdScore= JSD_score(jsdFile,pos)
	if jsdScore == None:
		features.append('#')
	else:
		features.append(jsdScore) #52

	#Depth
	depthFile = featuresDir+'itasserFeature/depth/pdb-residue.depth'
	depthScore = depth(depthFile, pos)
	if depthScore == None:
		features.append('#')
	else:
		features.append(depthScore)#53

	#SEF ddG 
	sefFile = featuresDir+'itasserFeature/sef/'+aa1+pos+aa2+'.ddG'
	ddG = ddGSef(sefFile,aa2)
	if ddG == None:
		features.append('#')
	else:
		features.append(ddG)#54

	#the energy of evoDesign
	evoFile = featuresDir+'itasserFeature/evodesign/thdseqali/'+pos+aa2+'/energy.txt'
	evoScore= evoEnergy(evoFile,aa2,pos)
	if evoScore == None:
		features.append('#')
	else:
		features.append(evoScore)#55

	#mutcis energy
	cisFile = featuresDir+'itasserFeature/mutcis/'+pos+aa2+'.cis'
	wClap, wSelf, wTotal, mClap, mSelf, mTotal = cisEnergy(cisFile,aa2,pos)
	if wClap == None:
		features.append('#')
		features.append('#')
		features.append('#')
		features.append('#')
		features.append('#')
		features.append('#')
		features.append('#')
		features.append('#')
		features.append('#')
	else:
		features.append(wClap)#56
		features.append(mClap)#57
		features.append(mClap - wClap)#58
		features.append(wSelf)#59
		features.append(mSelf)#60
		features.append(mSelf - wSelf)#61
		features.append(wTotal)#62
		features.append(mTotal)#63
		features.append(mTotal - wTotal)#64
	#contact score
	contactFile = featuresDir+'templateFeature/contact/'+pos+aa1+aa2+'_pdb.contact'
	#print contactFile	
	sefNum_intra, sefFunNum_intra, sefConNum_intra, sefConFunNum_intra, sefNum_inter, sefFunNum_inter, sefConNum_inter, sefConFunNum_inter, sefNum_het, sefFunNum_het, sefConNum_het, sefConFunNum_het = contactScore(contactFile)
	if contactFile == None:
		features.append('#')
		features.append('#')
		features.append('#')
		features.append('#')
		features.append('#')
		features.append('#')
		features.append('#')
		features.append('#')
		#features.append('#')
		#features.append('#')
		#features.append('#')
		#features.append('#')
	else:
		features.append(sefNum_intra)#65
		features.append(sefFunNum_intra)#66
		features.append(sefConNum_intra)#67
		features.append(sefConFunNum_intra)#68
		features.append(sefNum_inter+sefNum_het)#69
		features.append(sefFunNum_inter+sefFunNum_het)#70
		features.append(sefConNum_inter+sefConNum_het)#71
		features.append(sefConFunNum_inter+sefConFunNum_het)#72
		#features.append(sefNum_het)#73
		#features.append(sefFunNum_het)#74
		#features.append(sefConNum_het)#75
		#features.append(sefConFunNum_het)#76

	return features


def cosine_rms(v1,v2):
	#compute cosine similarity of v1 to v2: (v1 dot v2)/{||v1||*||v2||)
	sumxx, sumxy, sumyy = 0, 0, 0
	rms = 0
	for i in range(len(v1)):
		x = v1[i]; y = v2[i]
		sumxx += x*x
		sumyy += y*y
		sumxy += x*y
		rms += (x-y)**2
	rms = math.sqrt(float(rms)/len(v1))
	if sumxx == 0 or sumyy == 0:
		return 0.0,rms 
	return sumxy/math.sqrt(sumxx*sumyy),rms





if __name__ == '__main__':
        mutations = sys.argv[1]
	outDir = sys.argv[2]

        mutations = mutations.strip().split('-')
	fout = open(preparation.check_slash(outDir)+'features','w')
	for mutation in mutations:
                cells = mutation.strip().split('_')
		pos = cells[1]
		aa1 = cells[0]
		aa2 = cells[2]

		features = get_all_features(preparation.check_slash(outDir),aa1,aa2,pos)
		fout.write(' '.join(map(str,features)))
		fout.write('\n')
