#!/usr/bin/env python

import sys,math
import numpy as np
import matplotlib
matplotlib.use('agg')
import matplotlib.pyplot as plt
from sklearn.neighbors import KernelDensity


def ReadFeatures(file):
	templist = []
	f = open(file,"r")
	line = f.readline()
	NumColumn = len(line.strip().split())
	print NumColumn
	while line:
		cells = line.strip().split()
		#if cells[0] == "NOT":
		#	line = f.readline()
		#	continue
		#templist.append(map(float, cells))
		if len(cells) != NumColumn:
			print 'the number of columns is different'
			sys.exit()
		templist.append(cells)
		line = f.readline()
	featurelist = np.array(templist)
	return featurelist

def filter(data):
	tempdata = []
	for i in range(0,len(data)):
		if data[i] == "#":
			continue
		elif data[i] == "nan":
			tempdata.append(0.0)
		else:
			tempdata.append(float(data[i]))
	return np.array(tempdata)	
	
	
def KDE(data1,data2,output,allbin=10):
	data1 = data1[:,None]
	#min1 = np.min(data1)
	#max1 = np.max(data1)

	data2 = data2[:,None]
	#min2 = np.min(data2)
	#max2 = np.max(data2)

	#Min = min(min1,min2)
	#Max = max(max1,max2)
	#points = np.linspace(Min,Max,1000)[:,None]
	points = np.linspace(0,1,1000)[:,None]

	kde1 = KernelDensity(kernel='gaussian', bandwidth=0.5/allbin).fit(data1)
	kde2 = KernelDensity(kernel='gaussian', bandwidth=0.5/allbin).fit(data2)
	return kde1, kde2

def Histogram2(data1,data2,output):
	Min = min(data1)
	Max = max(data1)

	data1 = data1[:,None]
	sum1 = len(data1)
	data1p = {}
	for i in data1:
		key = i[0]
		if data1p.has_key(key):
			data1p[key]+=1
		else:
			data1p[key]=1

	data2 = data2[:,None]
	sum2 = len(data2)
	data2p = {}
	for i in data2:
		key = i[0]
		if data2p.has_key(key):
			data2p[key]+=1
		else:
			data2p[key]=1
	
	for i in data1p:
		if not data2p.has_key(i):
			data2p[i]=0
	for i in data2p:
		if not data1p.has_key(i):
			data1p[i]=0

	for i in data1p:
		data1p[i]=float(data1p[i])/sum1
		data2p[i]=float(data2p[i])/sum2
		#print i, data1p[i], data2p[i]

	return data1p,data2p

def Histogram3(data1,data2,output,allbin=10):
	data1 = data1[:,None]
	sum1 = len(data1)

	data2 = data2[:,None]
	sum2 = len(data2)

	num_bin = allbin
	bins = np.linspace(0,1,num_bin+1)
	
        n1, bins1, patches = plt.hist(data1,bins,range=(0,1),normed=1)
	n2, bins2, patches2 = plt.hist(data2,bins,range=(0,1),normed=1)
	plt.close()

	hist1 = {}
	hist2 = {}
	for i in range(0,num_bin):
		temp1 = []
		temp1.append(bins[i+1])
		temp1.append(n1[i])
		hist1[i]=temp1
		
		temp2 = []
		temp2.append(bins[i+1])
		temp2.append(n2[i])
		hist2[i]=temp2
	return hist1,hist2


def pKDE(kde,data):
	return math.exp(kde.score_samples(data))

def pHist(hist,data):
	for key, value in hist.iteritems():
		if data < value[0]:
			return value[1]
	return hist[len(hist)-1][1]

def pHist2(hist,data):
	for key, value in hist.iteritems():
		if data == key:
			return value
	return 0.0

def pList(data,model,allbin,lable):
	#lable = 0 means KDE, lable = 1 means hist
	tempdata = []
	tempP = []
	for i in range(0,len(data)):
		if data[i] == "#":
			tempdata.append("#")
			tempP.append(0.0)
		elif data[i] == "nan":
			tempdata.append(0)
			if lable == 0:
				tempP.append(pKDE(model,0.0)/allbin)
			elif lable == 1:
				tempP.append(pHist(model,0.0)/allbin)
			else:
				print "no this lable: "+str(lable)
		else:
			tempdata.append(float(data[i]))
			if lable == 0:
				tempP.append(pKDE(model,float(data[i]))/allbin)
			elif lable == 1:
				tempP.append(pHist(model,float(data[i]))/allbin)
			else:
				print "no this lable: "+str(lable)
	return np.array(tempdata).reshape((len(data),1)),np.array(tempP).reshape((len(data),1))	

def pList2(data,model,allbin,lable):
	#lable = 0 means KDE, lable = 1 means hist
	tempdata = []
	tempP = []
	for i in range(0,len(data)):
		if data[i] == "#":
			tempdata.append("#")
			tempP.append(0.0)
		elif data[i] == "nan":
			tempdata.append(0)
			if lable == 0:
				tempP.append(pKDE(model,0.0)/allbin)
			elif lable == 1:
				tempP.append(pHist2(model,0.0))
			else:
				print "no this lable: "+str(lable)
		else:
			tempdata.append(float(data[i]))
			if lable == 0:
				tempP.append(pKDE(model,float(data[i])/allbin))
			elif lable == 1:
				tempP.append(pHist2(model,float(data[i])))
			else:
				print "no this lable: "+str(lable)

	return np.array(tempdata).reshape((len(data),1)),np.array(tempP).reshape((len(data),1))	


def printFile(data,output):
	fout = open(output,"w")
	for i in range(data.shape[0]):
		temp = ""
		for j in range(data.shape[1]):
			temp += str(data[i,j])+" "
		fout.write(temp+"\n")
	fout.close()



def DrawKDE_Hist(list1,list2,testlist,soutput,col,allbin=10):
	data1 = filter(list1)
	data2 = filter(list2)
	# Histogram must be ahead of KDE
	kdeDisease,kdeNeutral = KDE( data1,data2,output,allbin)
		
	fDisease,pDisease = pList(testlist,kdeDisease,allbin,0)
	fNeutral,pNeutral = pList(testlist,kdeNeutral,allbin,0)
	dpDN = pDisease-pNeutral
		
	print np.min(pDisease),np.max(pDisease),np.min(pNeutral),np.max(pNeutral),'********'

	fDiseasefile = soutput+"diseasef"+col
	pDiseasefile = soutput+"diseasep"+col
	fNeutralfile = soutput+"neutralf"+col
	pNeutralfile = soutput+"neutralp"+col
	dpDiseasefile = soutput+"diseasedp"+col
		
	printFile(fDisease,fDiseasefile)
	printFile(pDisease,pDiseasefile)
	printFile(fNeutral,fNeutralfile)
	printFile(pNeutral,pNeutralfile)
	printFile(dpDN,dpDiseasefile)


def DrawHist(list1,list2,testlist,soutput,col,allbin=10,lable=0):
	data1 = filter(list1)
	data2 = filter(list2)

	if lable == 0:
		histDisease,histNeutral = Histogram3(data1,data2,output,allbin)
	elif lable ==1:
		histDisease,histNeutral = Histogram2(data1,data2,output)
	#plt.savefig(output,dpi=300)
	
	if lable == 0:
		fDisease,pDisease = pList(testlist,histDisease,allbin,1)
		fNeutral,pNeutral = pList(testlist,histNeutral,allbin,1)
	elif lable == 1:
		fDisease,pDisease = pList2(testlist,histDisease,allbin,1)
		fNeutral,pNeutral = pList2(testlist,histNeutral,allbin,1)

	dpDN = pDisease-pNeutral
		
	print np.min(pDisease),np.max(pDisease),np.min(pNeutral),np.max(pNeutral),'********'

	fDiseasefile = soutput+"diseasef"+col
	pDiseasefile = soutput+"diseasep"+col
	fNeutralfile = soutput+"neutralf"+col
	pNeutralfile = soutput+"neutralp"+col
	dpDiseasefile = soutput+"diseasedp"+col
		
	printFile(fDisease,fDiseasefile)
	printFile(pDisease,pDiseasefile)
	printFile(fNeutral,fNeutralfile)
	printFile(pNeutral,pNeutralfile)
	printFile(dpDN,dpDiseasefile)

if __name__ =='__main__':
	diseasefile = sys.argv[1]
	neutralfile = sys.argv[2]
	testfile = sys.argv[3]
	output = sys.argv[4]

	diseaselist = ReadFeatures(diseasefile)
	neutrallist = ReadFeatures(neutralfile)
	testlist = ReadFeatures(testfile)

	for i in range(0,2):
		DrawHist(diseaselist[:,i],neutrallist[:,i],testlist[:,i],output,str(i),10,1)

	DrawHist(diseaselist[:,2],neutrallist[:,2],testlist[:,2],output,str(2),15)
	
	for i in range(3,5):
		DrawHist(diseaselist[:,i],neutrallist[:,i],testlist[:,i],output,str(i),10,1)

	DrawHist(diseaselist[:,5],neutrallist[:,5],testlist[:,5],output,str(5),15)

	for i in range(6,12):
		DrawKDE_Hist(diseaselist[:,i],neutrallist[:,i],testlist[:,i],output,str(i),15)

	for i in range(12,37):
		DrawHist(diseaselist[:,i],neutrallist[:,i],testlist[:,i],output,str(i),10,1)

	for i in range(37,39):
		DrawKDE_Hist(diseaselist[:,i],neutrallist[:,i],testlist[:,i],output,str(i),20)

	for i in range(39,44):
		DrawKDE_Hist(diseaselist[:,i],neutrallist[:,i],testlist[:,i],output,str(i),10)

	for i in range(44,47):
		DrawHist(diseaselist[:,i],neutrallist[:,i],testlist[:,i],output,str(i),25)

	for i in range(47,51):
		DrawHist(diseaselist[:,i],neutrallist[:,i],testlist[:,i],output,str(i),20)

	for i in range(51,54):
		DrawKDE_Hist(diseaselist[:,i],neutrallist[:,i],testlist[:,i],output,str(i),20)

	DrawHist(diseaselist[:,54],neutrallist[:,54],testlist[:,54],output,str(54),20)

	for i in range(55,57):
		DrawKDE_Hist(diseaselist[:,i],neutrallist[:,i],testlist[:,i],output,str(i),10)

	DrawKDE_Hist(diseaselist[:,57],neutrallist[:,57],testlist[:,57],output,str(57),20)

	for i in range(58,61):
		DrawKDE_Hist(diseaselist[:,i],neutrallist[:,i],testlist[:,i],output,str(i),15)

	for i in range(61,64):
		DrawKDE_Hist(diseaselist[:,i],neutrallist[:,i],testlist[:,i],output,str(i),20)

	for i in range(64,66):
		DrawHist(diseaselist[:,i],neutrallist[:,i],testlist[:,i],output,str(i),10,1)

	for i in range(66,68):
		DrawHist(diseaselist[:,i],neutrallist[:,i],testlist[:,i],output,str(i),40)

	for i in range(68,70):
		DrawHist(diseaselist[:,i],neutrallist[:,i],testlist[:,i],output,str(i),10,1)

	for i in range(70,72):
		DrawHist(diseaselist[:,i],neutrallist[:,i],testlist[:,i],output,str(i),40)


