#!/usr/bin/perl
################################################################################
#                                                                              #
#                             GenerateProfile.pl                               #
#                                 Yang Cao                                     #
#                                2016.04.25                                    #
# Prepares Profile file, secondary structure file, phi-psi file, surface file  #
# of a given target protein for the following design process                   #
#                                                                              #
# Example:                                                                     #
# GenerateProfile.pl [target protein pdb file]                                 #
#                                                                              #
################################################################################

#program path
#my $bin_path="../";
use File::Basename;
use Cwd 'abs_path';
my $bin_path=dirname(abs_path(__FILE__));


my $querypdb=$ARGV[0];
#cutoff for TMalign
my $cutoff=$ARGV[1];



#PDB data set path
my $pdbset= "/nfs/amino-library/PDB";
#my $pdbset= "/nfs/amino-home/yacao/test/psudopdb2"; 






####  Generate seq profile ####
#system "perl $bin_path/bin_profile/tma-tgt-ali.pl $querypdb $pdbset/ >msa.txt";
@decoys = glob("$pdbset/*.pdb");
$ntmp = scalar @decoys;
printf "Num of PDB Templates to generate sequence profile: %d\n", $ntmp;
my $homology=0;
my $mark=0;
open TMFILE, ">msa.txt" or die "Error in creating msa.txt\n";
for($i=0; $i<@decoys; ++$i) {

  chomp($dec = $decoys[$i]);
  @tmo = `$bin_path/bin_profile/TMalign $dec $querypdb `;
  #print "$bin_path/bin_profile/TMalign $dec $querypdb\n";
  
  for($j=0; $j<@tmo; $j++) {
    if($tmo[$j] =~ /TM-score=/) {
      my @dat1=split(/,/, $tmo[$j]);
      my @dat2=split(/=/, $dat1[2]);
      print "$dat2[1]  $dec\n";
      
      if($dat2[1]>$cutoff)
      {
      	$homology=1;
      }
      else
      {
      	$homology=0;
      }
      last;
    }
  }

if($homology==0) {next;}

  for($j=0; $j<@tmo; $j++) {
    if($tmo[$j] =~ /denotes residue/) {
      chomp($dali = $tmo[$j+1]);
      chomp($tali = $tmo[$j+3]);
      last;
    }
  }
  
  # save aligned decoy
  $ti = 0;
  $lena = length $tali;
  for($j=0; $j<$lena; $j++) {
    if(substr($tali, $j, 1) ne '-') {
      $ali[$i][$ti++] = substr($dali, $j, 1);
    }      
  }
  # print target sequence
  if($mark==0)
  {
  for($j=0; $j<$lena; $j++) {
    $tr = substr($tali, $j, 1);
    if($tr ne '-') {
      print TMFILE $tr;  
    }
  }
  print TMFILE "\n";
  }

  
  for($j=0; $j<$ti; $j++) {
    print TMFILE $ali[$i][$j];
  }
  $mark++;
  print TMFILE "\n";
}


=pod
# print aligned decoys
for($i=0; $i<@decoys; $i++) {
  for($j=0; $j<$ti; $j++) {
    print TMFILE $ali[$i][$j];
  }
  if($j>0)
  {
  print TMFILE "$j\n";
  }
}
=cut

close TMFILE;

system "$bin_path/bin_profile/mkprf msa.txt >prf.txt";

#system "rm msa.txt";

#### Generate secondary structure, surface, phi-psi files ####
$DSSP = "$bin_path/bin_profile/dsspcmbi";
$DSSPV2H = "$bin_path/bin_profile/dssp-ver2hor";
$CONVSS = "$bin_path/bin_profile/convSS.pl";

`$DSSP $querypdb > dssp.txt`;
`$DSSPV2H dssp.txt >dssph.txt`;
`$CONVSS dssph.txt >ss.txt`;

system "rm dssp.txt dssph.txt";




