#include "AminoName.h"

//
//
//  FUNCTION    :   Protein::charToAmino()
//
//  ARGUMENT    :   ---> name: one-letter representation of an amino acid type.
//
//  RETURN VALUE:   integer representation of that type.
//
//  COMPLEXITY  :   constant.
//
//
Protein::AminoName Protein::charToAmino(const char name) {

    switch(name) {

        case 'A': return ALA; 
        case 'V': return VAL;
        case 'F': return PHE;
        case 'P': return PRO;
        case 'M': return MET;
        case 'I': return ILE;
        case 'L': return LEU;
        case 'D': return ASP;
        case 'E': return GLU;
        case 'K': return LYS;
        case 'R': return ARG;
        case 'S': return SER;
        case 'T': return THR;
        case 'Y': return TYR;
        case 'H': return HIS;
        case 'C': return CYS;
        case 'N': return ASN;
        case 'Q': return GLN;
        case 'W': return TRP;
        case 'G': return GLY;
    }
}


//
//
//  FUNCTION    :   Protein::aminoToChar()
//
//  ARGUMENT    :   ---> name: integer representation of an amino acid type.
//
//  RETURN VALUE:   one-letter representation of that type.
//
//  COMPLEXITY  :   constant.
//
//
char Protein::aminoToChar(const AminoName name) {

    switch(name) {

        case ALA: return 'A'; 
        case VAL: return 'V';
        case PHE: return 'F';
        case PRO: return 'P';
        case MET: return 'M';
        case ILE: return 'I';
        case LEU: return 'L';
        case ASP: return 'D';
        case GLU: return 'E';
        case LYS: return 'K';
        case ARG: return 'R';
        case SER: return 'S';
        case THR: return 'T';
        case TYR: return 'Y';
        case HIS: return 'H';
        case CYS: return 'C';
        case ASN: return 'N';
        case GLN: return 'Q';
        case TRP: return 'W';
        case GLY: return 'G';
    }
}
