/*----------------------------------------------------------------------------*/
/*                                                                            */
/*                                   Text.cpp                                 */
/*                                                                            */
/*----------------------------------------------------------------------------*/


#include <stdio.h>


namespace Text {

  long int nnewline(FILE* fp);
  int ncharLine(FILE* fp);
}


// -*- C++ -*-------------------------------------------------------------------
//
//  Function:       Text::nnewline()
//
//  Author:         Andrea Bazzoli
//
//  Date:           20/09/2007
//
//  Interface:
//    returns the number of "\n" characters in a text file from a start 
//    position.
// 
//  Arguments:
//    position in the file from which to start counting.
//
//  Complexity: O(N), where N is the number of characters from the start 
//              position to the end of the file.  
//
// -----------------------------------------------------------------------------
long int Text::nnewline(FILE* fp) 
{
  long int count = 0;
  char tmp;
  while(fscanf(fp, "%c", &tmp) != EOF)
    if(tmp == '\n')
      count++;
  
  return count; 
}


// -*- C++ -*-------------------------------------------------------------------
//
//  Function:       Text::ncharLine()
//
//  Author:         Andrea Bazzoli
//
//  Date:           10/05/2008
//
//  Interface:
//    Argument:
//      fp: pointer to a given position in an ASCII text file.
//
//    Return value:
//      number of characters up to the end of the line. The given position is
//      included in the count while the \n line terminator, or the end-of-file 
//      terminator if the last line is considered, is not.
// 
//  Complexity: O(N), where N is the number of characters from the given 
//              position to the end of the line.
//
//  Notes: 
//    although this function works IN THEORY even when 'fp' points to the 
//    start of a final empty line, IN PRACTICE it does not. I guess the 
//    function was tested with 'fp' pointing elsewhere, probably due to 
//    positioning errors by the fseek() function in the main() function. 
//
// -----------------------------------------------------------------------------
int Text::ncharLine(FILE* fp) {

  int count = 0;
  while(!feof(fp)) {
  
    char tmp;
    fscanf(fp, "%c", &tmp);
    if(tmp != '\n')
      count++;
    else
      break;
  }

  return count;
}

