#!/usr/bin/perl -w
use strict;


`hostname`=~/(\S+)/;
my $node=$1;
my $datetime=`date`;
my $path=`pwd`;
my $start=time();
print "Hostname: $node\n";
print "Started : $datetime\n";
print "Path    : $path\n";

chomp($path);
my $libdir = "/nfs/amino-library";


my $user      = $ARGV[0];              # user name
my $s         = $ARGV[1];                 # name of sequence
my $pdbid     = $ARGV[2];             # PDBID for saving JSD socre to run concavity
my $pdbname   = $ARGV[3];             # name of pdb file
my $datadir   = $ARGV[4];           # directory for data
my $bindir    = $ARGV[5];           
my $blastdir  = $ARGV[6];   # Directory containing  BLAST executables
my $nrdb      = $ARGV[7];   # Non-redundant sequence database for runing PSI-BLAST



=pod
my $user      = "yangji";              # user name
my $s         = "seq.txt";            # name of sequence
my $pdbid     = "13gs";             # PDBID for saving JSD socre to run concavity
my $pdbname   = "model1.pdb";              # name of pdb file
my $datadir   = "/home/yangji/COACH/output/13gsA";           # directory for data
my $bindir    = "/home/yangji/COACH/bin";
=cut





my $blast_parser = "parse_blast.pl";
my $resultag  = "v2";          # tag for saving result
######################################################

##check if necessary data exist
if(!-s "$datadir/$pdbname")
{
    die "$pdbname is missed!\n";
}

if(!-s "$bindir/concavity")
{
    die "concavity is missed!\n";
}

if(!-s "$bindir/$blast_parser")
{
    die "$blast_parser is missed!\n";
}

my $modelname = $pdbname; $modelname=~s/\.pdb//;
my $outdir    = "$datadir/$modelname/concavity";
print "$outdir\n";
`mkdir -p $outdir`;


#####temp dir for worling ##########
my $workdir       = "/tmp/$user/CONCAVITY_$pdbid\_$modelname";
`/bin/mkdir -p $workdir`;
chdir "$workdir";
`/bin/rm -f $workdir/*`;


###copy data for processing#########
print "$workdir\n";
print "/bin/cp $datadir/$pdbname .\n";
`/bin/cp $datadir/$pdbname .`;
print "/bin/cp $bindir/$blast_parser .\n";
`/bin/cp $bindir/$blast_parser .`;
if(!-s "$datadir/$s")
{
    my $fasta=&getfasta("$datadir/$pdbname");
    open(FST, ">$s");
    print FST ">$pdbid\n$fasta\n";
    close(FST); 
    print "/bin/cp $s $datadir/";
    `/bin/cp $s $datadir/`;
}
else
{
    `/bin/cp $datadir/$s .`;
}
`/bin/cp $bindir/concavity .`;
my $chain = &get_chain($pdbname);


#####################################################
# SETTINGS (Do not change below this line)
my @AA=
    (
     "ALA", "ARG", "ASN", "ASP", "CYS", "GLN", "GLU", "GLY", "HIS", "ILE", "LEU", "LYS", "MET","PHE",
     "PRO", "SER", "THR", "TRP", "TYR", "VAL", "ASX", "GLX", "UNK");

my %AA2index=
    (
     'A'=>'1', 'R'=>'2', 'N'=>'3', 'D'=>'4', 'C'=>'5', 'Q'=>'6', 'E'=>'7', 'G'=>'8', 'H'=>'9', 'I'=>'10', 'L'=>'11', 'K'=>'12', 
     'M'=>'13', 'F'=>'14', 'P'=>'15', 'S'=>'16', 'T'=>'17', 'W'=>'18', 'Y'=>'19', 'V'=>'20', 'B'=>'21', 'Z'=>'22', '-'=>'23');	
my %index2AA=
    (
     '1'=>'A','2'=>'R','3'=>'N','4'=>'D','5'=>'C','6'=>'Q','7'=>'E','8'=>'G','9'=>'H','10'=>'I','11'=>'L','12'=>'K', 
     '13'=>'M','14'=>'F','15'=>'P','16' =>'S','17'=>'T','18'=>'W','19'=>'Y','20'=>'V','21'=>'B','22'=>'Z','23'=>'-');	
	
my %ts=
    (
     'ALA'=>'A', 'ARG'=>'R', 'ASN'=>'N', 'ASP'=>'D', 'CYS'=>'C', 'GLN'=>'Q', 'GLU'=>'E', 'GLY'=>'G',
     'HIS'=>'H', 'ILE'=>'I', 'LEU'=>'L', 'LYS'=>'K', 'MET'=>'M', 'PHE'=>'F', 'PRO'=>'P', 'SER'=>'S', 
     'THR'=>'T', 'TRP'=>'W', 'TYR'=>'Y', 'VAL'=>'V', 'ASX'=>'N', 'GLX'=>'Q', 'UNK'=>'G',
     'A'=>'ALA', 'R'=>'ARG', 'N'=>'ASN', 'D'=>'ASP', 'C'=>'CYS', 'Q'=>'GLN', 'E'=>'GLU', 'G'=>'GLY',
     'H'=>'HIS', 'I'=>'ILE', 'L'=>'LEU', 'K'=>'LYS', 'M'=>'MET', 'F'=>'PHE', 'P'=>'PRO', 'S'=>'SER', 
     'T'=>'THR', 'W'=>'TRP', 'Y'=>'TYR', 'V'=>'VAL', 'B'=>'ASX', 'Z'=>'GLX', 'X'=>'UNK');
	
     # A  R  N  D  C  Q  E  G  H  I  L  K  M  F  P  S  T  W  Y  V  B  Z  X
my @BLOSUM62 = 
    (
     [ 4,-1,-2,-2, 0,-1,-1, 0,-2,-1,-1,-1,-1,-2,-1, 1, 0,-3,-2, 0,-2,-1, 0],
     [-1, 5, 0,-2,-3, 1, 0,-2, 0,-3,-2, 2,-1,-3,-2,-1,-1,-3,-2,-3, 0, 1,-1],
     [-2, 0, 6, 1,-3, 0, 0, 0, 1,-3,-3, 0,-2,-3,-2, 1, 0,-4,-2,-3, 6, 0,-1],
     [-2,-2, 1, 6,-3, 0, 2,-1,-1,-3,-4,-1,-3,-3,-1, 0,-1,-4,-3,-3, 1, 0,-1],
     [ 0,-3,-3,-3, 9,-3,-4,-3,-3,-1,-1,-3,-1,-2,-3,-1,-1,-2,-2,-1,-3,-3,-2],
     [-1, 1, 0, 0,-3, 5, 2,-2, 0,-3,-2, 1, 0,-3,-1, 0,-1,-2,-1,-2, 0, 5,-1],
     [-1, 0, 0, 2,-4, 2, 5,-2, 0,-3,-3, 1,-2,-3,-1, 0,-1,-3,-2,-2, 0, 2,-1],
     [ 0,-2, 0,-1,-3,-2,-2, 6,-2,-4,-4,-2,-3,-3,-2, 0,-2,-2,-3,-3, 0,-2,-1],
     [-2, 0, 1,-1,-3, 0, 0,-2, 8,-3,-3,-1,-2,-1,-2,-1,-2,-2, 2,-3, 1, 0,-1],
     [-1,-3,-3,-3,-1,-3,-3,-4,-3, 4, 2,-3, 1, 0,-3,-2,-1,-3,-1, 3,-3,-3,-1],
     [-1,-2,-3,-4,-1,-2,-3,-4,-3, 2, 4,-2, 2, 0,-3,-2,-1,-2,-1, 1,-3,-2,-1],
     [-1, 2, 0,-1,-3, 1, 1,-2,-1,-3,-2, 5,-1,-3,-1, 0,-1,-3,-2,-2, 0, 1,-1],
     [-1,-1,-2,-3,-1, 0,-2,-3,-2, 1, 2,-1, 5, 0,-2,-1,-1,-1,-1, 1,-2, 0,-1],
     [-2,-3,-3,-3,-2,-3,-3,-3,-1, 0, 0,-3, 0, 6,-4,-2,-2, 1, 3,-1,-3,-3,-1],
     [-1,-2,-2,-1,-3,-1,-1,-2,-2,-3,-3,-1,-2,-4, 7,-1,-1,-4,-3,-2,-2,-1,-2],
     [ 1,-1, 1, 0,-1, 0, 0, 0,-1,-2,-2, 0,-1,-2,-1, 4, 1,-3,-2,-2, 1, 0, 0],
     [ 0,-1, 0,-1,-1,-1,-1,-2,-2,-1,-1,-1,-1,-2,-1, 1, 5,-2,-2, 0, 0,-1, 0],
     [-3,-3,-4,-4,-2,-2,-3,-2,-2,-3,-2,-3,-1, 1,-4,-3,-2,11, 2,-3,-4,-2,-2],
     [-2,-2,-2,-3,-2,-1,-2,-3, 2,-1,-1,-2,-1, 3,-3,-2,-2, 2, 7,-1,-2,-1,-1],
     [ 0,-3,-3,-3,-1,-2,-2,-3,-3, 3, 1,-2, 1,-1,-2,-2, 0,-3,-1, 4,-3,-2,-1],
     [-2, 0, 6, 1,-3, 0, 0, 0, 1,-3,-3, 0,-2,-3,-2, 1, 0,-4,-2,-3, 6, 0,-1],
     [-1, 1, 0, 0,-3, 5, 2,-2, 0,-3,-2, 1, 0,-3,-1, 0,-1,-2,-1,-2, 0, 5,-1],
     [ 0,-1,-1,-1,-2,-1,-1,-1,-1,-1,-1,-1,-1,-1,-2, 0, 0,-2,-1,-1,-1,-1,-1]);

# This is the BLOSUM62 background distribution.
my %blsm_bck=
    (
     'A'=>0.078, 'R'=>0.051, 'N'=>0.041, 'D'=>0.052, 'C'=>0.024, 'Q'=>0.034, 'E'=>0.059, 'G'=>0.083, 
     'H'=>0.025, 'I'=>0.062, 'L'=>0.092, 'K'=>0.056, 'M'=>0.024, 'F'=>0.044, 'P'=>0.043, 'S'=>0.059, 
     'T'=>0.055, 'W'=>0.014, 'Y'=>0.034, 'V'=>0.072, 'B'=>0.041, 'Z'=>0.034, 'X'=>0.083,);
	 
my @BLOSM_FREQ = 
    ( [0.0215,0.0023,0.0019,0.0022,0.0016,0.0019,0.0030,0.0058,0.0011,0.0032,0.0044,0.0033,0.0013,0.0016,0.0022,0.0063,0.0037,0.0004,0.0013,0.0051,0.0019,0.0019,0.0058],
      [0.0023,0.0178,0.0020,0.0016,0.0004,0.0025,0.0027,0.0017,0.0012,0.0012,0.0024,0.0062,0.0008,0.0009,0.0010,0.0023,0.0018,0.0003,0.0009,0.0016,0.0020,0.0025,0.0017],
      [0.0019,0.0020,0.0141,0.0037,0.0004,0.0015,0.0022,0.0029,0.0014,0.0010,0.0014,0.0024,0.0005,0.0008,0.0009,0.0031,0.0022,0.0002,0.0007,0.0012,0.0141,0.0015,0.0029],
      [0.0022,0.0016,0.0037,0.0213,0.0004,0.0016,0.0049,0.0025,0.0010,0.0012,0.0015,0.0024,0.0005,0.0008,0.0012,0.0028,0.0019,0.0002,0.0006,0.0013,0.0037,0.0016,0.0025],
      [0.0016,0.0004,0.0004,0.0004,0.0119,0.0003,0.0004,0.0008,0.0002,0.0011,0.0016,0.0005,0.0004,0.0005,0.0004,0.0010,0.0009,0.0001,0.0003,0.0014,0.0004,0.0003,0.0008],
      [0.0019,0.0025,0.0015,0.0016,0.0003,0.0073,0.0035,0.0014,0.0010,0.0009,0.0016,0.0031,0.0007,0.0005,0.0008,0.0019,0.0014,0.0002,0.0007,0.0012,0.0015,0.0073,0.0014],
      [0.0030,0.0027,0.0022,0.0049,0.0004,0.0035,0.0161,0.0019,0.0014,0.0012,0.0020,0.0041,0.0007,0.0009,0.0014,0.0030,0.0020,0.0003,0.0009,0.0017,0.0022,0.0035,0.0019],
      [0.0058,0.0017,0.0029,0.0025,0.0008,0.0014,0.0019,0.0378,0.0010,0.0014,0.0021,0.0025,0.0007,0.0012,0.0014,0.0038,0.0022,0.0004,0.0008,0.0018,0.0029,0.0014,0.0378],
      [0.0011,0.0012,0.0014,0.0010,0.0002,0.0010,0.0014,0.0010,0.0093,0.0006,0.0010,0.0012,0.0004,0.0008,0.0005,0.0011,0.0007,0.0002,0.0015,0.0006,0.0014,0.0010,0.0010],
      [0.0032,0.0012,0.0010,0.0012,0.0011,0.0009,0.0012,0.0014,0.0006,0.0184,0.0114,0.0016,0.0025,0.0030,0.0010,0.0017,0.0027,0.0004,0.0014,0.0120,0.0010,0.0009,0.0014],
      [0.0044,0.0024,0.0014,0.0015,0.0016,0.0016,0.0020,0.0021,0.0010,0.0114,0.0371,0.0025,0.0049,0.0054,0.0014,0.0024,0.0033,0.0007,0.0022,0.0095,0.0014,0.0016,0.0021],
      [0.0033,0.0062,0.0024,0.0024,0.0005,0.0031,0.0041,0.0025,0.0012,0.0016,0.0025,0.0161,0.0009,0.0009,0.0016,0.0031,0.0023,0.0003,0.0010,0.0019,0.0024,0.0031,0.0025],
      [0.0013,0.0008,0.0005,0.0005,0.0004,0.0007,0.0007,0.0007,0.0004,0.0025,0.0049,0.0009,0.0040,0.0012,0.0004,0.0009,0.0010,0.0002,0.0006,0.0023,0.0005,0.0007,0.0007],
      [0.0016,0.0009,0.0008,0.0008,0.0005,0.0005,0.0009,0.0012,0.0008,0.0030,0.0054,0.0009,0.0012,0.0183,0.0005,0.0012,0.0012,0.0008,0.0042,0.0026,0.0008,0.0005,0.0012],
      [0.0022,0.0010,0.0009,0.0012,0.0004,0.0008,0.0014,0.0014,0.0005,0.0010,0.0014,0.0016,0.0004,0.0005,0.0191,0.0017,0.0014,0.0001,0.0005,0.0012,0.0009,0.0008,0.0014],
      [0.0063,0.0023,0.0031,0.0028,0.0010,0.0019,0.0030,0.0038,0.0011,0.0017,0.0024,0.0031,0.0009,0.0012,0.0017,0.0126,0.0047,0.0003,0.0010,0.0024,0.0031,0.0019,0.0038],
      [0.0037,0.0018,0.0022,0.0019,0.0009,0.0014,0.0020,0.0022,0.0007,0.0027,0.0033,0.0023,0.0010,0.0012,0.0014,0.0047,0.0125,0.0003,0.0009,0.0036,0.0022,0.0014,0.0022],
      [0.0004,0.0003,0.0002,0.0002,0.0001,0.0002,0.0003,0.0004,0.0002,0.0004,0.0007,0.0003,0.0002,0.0008,0.0001,0.0003,0.0003,0.0065,0.0009,0.0004,0.0002,0.0002,0.0004],
      [0.0013,0.0009,0.0007,0.0006,0.0003,0.0007,0.0009,0.0008,0.0015,0.0014,0.0022,0.0010,0.0006,0.0042,0.0005,0.0010,0.0009,0.0009,0.0102,0.0015,0.0007,0.0007,0.0008],
      [0.0051,0.0016,0.0012,0.0013,0.0014,0.0012,0.0017,0.0018,0.0006,0.0120,0.0095,0.0019,0.0023,0.0026,0.0012,0.0024,0.0036,0.0004,0.0015,0.0196,0.0012,0.0012,0.0018],
      [0.0019,0.0020,0.0141,0.0037,0.0004,0.0015,0.0022,0.0029,0.0014,0.0010,0.0014,0.0024,0.0005,0.0008,0.0009,0.0031,0.0022,0.0002,0.0007,0.0012,0.0141,0.0015,0.0029],
      [0.0019,0.0025,0.0015,0.0016,0.0003,0.0073,0.0035,0.0014,0.0010,0.0009,0.0016,0.0031,0.0007,0.0005,0.0008,0.0019,0.0014,0.0002,0.0007,0.0012,0.0015,0.0073,0.0014],
      [0.0058,0.0017,0.0029,0.0025,0.0008,0.0014,0.0019,0.0378,0.0010,0.0014,0.0021,0.0025,0.0007,0.0012,0.0014,0.0038,0.0022,0.0004,0.0008,0.0018,0.0029,0.0014,0.0378]);



#####################################################
my %AA2sim=();
for(my $i=0;$i<23;$i++)
{
    for(my $j=0;$j<23;$j++)
    {
	my $x= $i+1;my $y= $j+1;
#	$AA2sim{$index2AA{$x},$index2AA{$y}}= $BLOSM_FREQ[$i][$j];
	$AA2sim{$index2AA{$x},$index2AA{$y}}= $BLOSUM62[$i][$j];
    }
}
my $PSEUDOCOUNT= 0.0000001;
my $lambda     = 0.5;
my $window     = 3;

###################################################################################################


my @seqtxts=`cat $s`;
my $sequence="";    
my %seqQ=();
foreach my $seqtxt(@seqtxts)
{
    if($seqtxt =~ /\>/)
    {
	next;
    }
    $seqtxt   =~ s/\s//mg;
    $seqtxt   =~ s/\n//mg;
    $sequence = $sequence.$seqtxt;
}
my $Lch=length $sequence;  


open(FASTA, ">protein.fasta");
printf FASTA ">$s\n";
for(my $i=1;$i<=$Lch;$i++)
{
    my $seq1=substr($sequence, $i-1, 1);
    $seqQ{$i}=$seq1;
    printf FASTA "$seq1";
    if(int($i/60)*60==$i)
    {
	printf FASTA "\n";
    }
}
close(FASTA);


if(!-s "$datadir/blast.out")
{
    print "running PSI-BLAST...\n";
    system ("$blastdir/bin/blastpgp  -b 1000 -j 3 -h 0.001 -d $nrdb -i protein.fasta > blast.out");
    `/bin/cp blast.out $datadir/`;
}
else
{    
    #print "Blast result exists, copy directly...\n";
    `/bin/cp $datadir/blast.out .`;
    my $str=`cat blast.out`;
    if($str !~ /round/)
    {
	print "running PSI-BLAST...\n";
	system ("$blastdir/bin/blastpgp  -b 1000 -j 3 -h 0.001 -d $nrdb -i protein.fasta > blast.out");
	`/bin/cp blast.out $datadir/`;
    }
}

`$workdir/$blast_parser -fas blast.out -q protein.fasta blast.fasta`;
print "$workdir/$blast_parser -fas blast.out -q protein.fasta blast.fasta\n";

my %ALN=();
my $Pcount=0;
open(ALN,"<$workdir/blast.fasta") || die "Cant open blast.fasta";
while(my $line=<ALN>)
{
    chomp($line);
    if($line =~ /^>(\S+)/)
    {
	my $Pname=$1;
	$Pcount++;
	my $Evalue= $1 if($line =~ /E=(\S+)/);	
	$ALN{$Pcount, 0}=$Pname;
	$ALN{$Pcount, 1}=$Evalue;
    }
    else
    {
	$line =~ s/X/-/g;  ###replace X by gap
	$ALN{$Pcount, 2}=$line;	
    }    
}
close(ALN);






############################### compute JSD score ##################################
printf "computing JSD socre...\n";
open(OUT1,">$pdbid\_$chain\_jsd.scores");
open(OUT2,">$pdbid\_$chain\_jsd_aa.scores");
#open(OUT2,">$workdir/SOPM.dat");
if($Pcount > 1)
{
    $Pcount=1000 if ($Pcount > 1000);
    my %seq_weights = ();
    my $tot_weights =   &load_sequence_weights(\%ALN, \$Pcount,\%AA2index,\%seq_weights); 
    
    my %jsd_scores = ();my %zscore1=(); my %wscore1=(); my %jsd_aa_scores=();     
    my %res_count = &js_divergence_score(\%ALN, \$Pcount, \%blsm_bck, \%AA2index, \%seq_weights, $PSEUDOCOUNT, $lambda, \%jsd_scores, \%jsd_aa_scores, $tot_weights);
   
#   &z_scores(\%jsd_scores,\$Lch,\%zscore1);
    
    #### Window removed because of low performance
#    &w_scores(\%jsd_scores,\$Lch,\$window,\$lambda,\%wscore1);
    #&z_scores(\%wscore1,\$Lch,\%zscore1);
    
#   my %SOP=();my %zscore2=();my %wscore2=();
#   &sum_of_pairs(\%ALN,\$Pcount,\%AA2sim,\%seq_weights,\%SOP);
#   &z_scores(\%SOP,\$Lch,\%zscore2);

   #### Window removed because of low performance
   #&w_scores(\%SOP,\$Lch,\$window,\$lambda,\%wscore2);
   #&z_scores(\%wscore2,\$Lch,\%zscore2);


    my @Qres   =split(//, $ALN{1, 2});
    my $Qresno=0;
	
	printf OUT2 "#";
	foreach my $key (sort {$AA2index{$a} <=> $AA2index{$b}} keys %AA2index)
	{
		if ($key ne '-')
		{
			printf OUT2 "\t%s",$key;
		}
	}
	printf OUT2 "\n";
    for(my $j=0; $j<=$#Qres; $j++)
    {
	if($Qres[$j] ne '-')
	{
	    $Qresno++;
	    #printf OUT1 "%-4d\t%s\t%.6f\t%d/%d\n",$Qresno-1, $seqQ{$Qresno}, $wscore1{$Qresno}, $res_count{$j}, $Pcount;
	    printf OUT1 "%-4d\t%s\t%.6f\t%d/%d\n",$Qresno-1, $seqQ{$Qresno}, $jsd_scores{$Qresno}, $res_count{$j}, $Pcount;

		printf OUT2	"%-4d",$Qresno-1;
		foreach my $key (sort {$AA2index{$a} <=> $AA2index{$b}} keys %AA2index)
	    {
			if ($key ne '-')
			{
				printf OUT2 "\t%.6f",$jsd_aa_scores{$Qresno,$key};
			}
		}
		printf OUT2	"\n";
	}
    }

    if($Qresno!=$Lch)
    {
	print "warning: length is not equal ($Qresno!=$Lch)";
    }

}
else
{
   for(my $j=1; $j<=$Lch; $j++)
   {
       printf OUT1 "%-4d\t%s\t%.6f\t%4d/%4d\n",$j, $seqQ{$j}, 1.0, 1, 1;
 #      printf OUT2 "%-4d  %5s\n",$j,'1.00';
   }
}
close(OUT1);
close(OUT2);
#close(OUT2);
`/bin/cp "$pdbid\_$chain\_jsd.scores" $datadir/JSD_concavity.dat`;  ##copy results back
`/bin/cp "$pdbid\_$chain\_jsd_aa.scores" $datadir/JSD_aa_concavity.dat`;  ##copy results back


######################################### run concavity with conservation score ####################################
print "runing concavity...\n";
print "./concavity -print_grid_pdb 1 -conservation $pdbid $pdbname $resultag";
`./concavity -print_grid_pdb 1 -conservation $pdbid $pdbname $resultag`;

&bsites("$modelname\_$resultag\_pocket.pdb", "$modelname\_$resultag\_residue.pdb");

$pdbname =~ s/\.pdb//;
`/bin/cp $pdbname\_$chain\_$resultag.scores $pdbname\_$resultag.bsr`;
`/bin/cp $pdbname\_$chain\_$resultag.scores $outdir/$pdbname.bsr`;
`/bin/cp -r *$resultag*    $outdir/`;
`/bin/cp -r Bsites*    $outdir/`;
`/bin/cp blast.fasta $outdir/`;

################# endding procedure ######################
my $time=`date`;
printf "ending time: $time";
`sync`;
sleep(5);
`rm -fr $workdir`;










sub sum_of_pairs
{
    my ($ALN_ref,$Nseq_ref,$blossum_ref,$SW_ref,$SOP)=@_;
    my %align       =%$ALN_ref;
    my $Nseq        =$$Nseq_ref;
    my %sim_matrix  =%$blossum_ref;
    my %weights     =%$SW_ref;  
   
    my @Qres        =split(//,$align{1,2});
    my $Ncol        =$#Qres;    
    my $Qresno=0;my %Qmapping=();
    for(my $j=0;$j<=$#Qres;$j++)
    {
	if($Qres[$j] ne '-'){$Qresno++;$Qmapping{$Qresno}=$j;} 	
    }
    my @ARR=();my $sum_seq_weights = 0;
    for(my $i=1;$i<=$Nseq;$i++)
    {
	my @res=split(//,$align{$i,2});
	for(my $j=0;$j<=$#res;$j++)
	{
	    $ARR[$i][$j]=$res[$j];
	}
	$sum_seq_weights += $weights{$i};
    }
    my %SOP_scores=();
    for(my $C=0;$C<=$Ncol;$C++)
    {
	my $sum   = 0;
	my $maxsum= 0;
	for(my $i=1;$i<=$Nseq;$i++)
	{
	    my $aa1=$ARR[$i][$C];
	    for(my $j=$i+1;$j<=$Nseq;$j++)
	    {
		my $aa2=$ARR[$j][$C];
		if(($ARR[$i][$C] eq '-')||($ARR[$j][$C] eq '-')){next;}
		$maxsum += $weights{$i} * $weights{$j};
		$sum    += $weights{$i} * $weights{$j} * $sim_matrix{$aa1,$aa2};	
	    }	    
	}
	if ($maxsum !=0)
	{
	    $sum   /=$maxsum;
	}
	else
	{
	    $sum=0;
	}
	my $gap_sum=0;
	for(my $i=1;$i<=$Nseq;$i++)
	{
	  if($ARR[$i][$C] eq '-')
	  {
	      $gap_sum +=   $weights{$i}; 
	  }
	}
	my $weighted_gap_penalty = (1-  ($gap_sum/$sum_seq_weights));
	$SOP_scores{$C}=($sum*$weighted_gap_penalty);	
    }
    for(my $j=1;$j<=$Qresno;$j++)
    {
	$$SOP{$j}= sprintf("%5.4f",$SOP_scores{$Qmapping{$j}});
    }
    return;
    
}
sub js_divergence_score
{
    my ($ALN_ref, $Nseq_ref, $dist_ref, $AA_ref, $SW_ref, $pseudocount, $lambda, $JSD_ref, $JSD_AA, $tot_weights)=@_;
    my %align   = %$ALN_ref;
    my $Nseq    = $$Nseq_ref;
    my %distr   = %$dist_ref;
    my %weights = %$SW_ref;  
    my %AA2in   = %$AA_ref;    

    my @Qres   =split(//,$align{1, 2});
    my $Ncol   =$#Qres;
    my %res_count=();
    
    my $Qresno=0;
    my %Qmapping=();
    for(my $j=0; $j<=$#Qres; $j++)
    {
	$res_count{$j}=0;
	if($Qres[$j] ne '-')
	{
	    $Qresno++;
	    $Qmapping{$Qresno}=$j;
	}
    }

    my @ARR=();
    for(my $i=1; $i<=$Nseq; $i++)
    {
	my @res=split(//, $align{$i, 2});
	for(my $j=0; $j<=$#res; $j++)
	{
	    $ARR[$i][$j]=$res[$j];
	    if($res[$j] eq $Qres[$j])
	    {
		$res_count{$j}++;
	    }
	}
    }
    my $AAcount = keys %AA2in;
    my %AA_freq=();my %sum_seq_weights=();my %R=();my %JSD_scores=();
    for(my $j=0; $j<=$Ncol; $j++)
    {
	foreach my $key (sort {$AA2in{$a} <=> $AA2in{$b}} keys %AA2in)
	{
	    $AA_freq{$j, $key}=0;
	    $R{$j, $key}      =0;
	}
	for(my $i=1; $i<=$Nseq; $i++)
	{
	    $AA_freq{$j, $ARR[$i][$j]} += $weights{$i}; ##weighted frequency in clolumn $j
	}
	foreach my $key (sort {$AA2in{$a} <=> $AA2in{$b}} keys %AA2in)
	{	   
	    $AA_freq{$j, $key} = ($AA_freq{$j, $key}+$pseudocount)/($tot_weights + $AAcount * $pseudocount); ##I change it here
	    if($key ne '-')
	    {
		$R{$j, $key}=($lambda*$AA_freq{$j,$key}) + ((1-$lambda)*$distr{$key});	   
	    }
	}
    }
    for(my $j=0;$j<=$Ncol;$j++)
    {
	my $JSD_score=0;my $weighted_gap_penalty=0;my $gap_sum=0;
	foreach my $key (sort {$AA2in{$a} <=> $AA2in{$b}} keys %AA2in)
	{
	    if($key eq '-'){next;} ##I add $lambda below
	    if($R{$j, $key} != 0.0)
	    {
		if($AA_freq{$j, $key} == 0.0)
		{
		    print "warning AA frequency = 0!\n";
		    $JSD_score += (1-$lambda)*($distr{$key} * log2($distr{$key}/$R{$j,$key}));
			$$JSD_AA{$j,$key} = (1-$lambda)*($distr{$key} * log2($distr{$key}/$R{$j,$key})); 
		}
		elsif($distr{$key} == 0.0)
		{
		    print "warning backround frequency = 0!\n";
		    $JSD_score += $lambda * ($AA_freq{$key} * log2($AA_freq{$key}/$R{$j,$key}));
			$$JSD_AA{$j,$key} = $lambda * ($AA_freq{$key} * log2($AA_freq{$key}/$R{$j,$key}));
		}
		else
		{
		    $JSD_score += $lambda * ($AA_freq{$j,$key} * log2($AA_freq{$j,$key}/$R{$j,$key})) + (1-$lambda) * ($distr{$key} * log2($distr{$key}/$R{$j,$key}));
			$$JSD_AA{$j,$key} = $lambda * ($AA_freq{$j,$key} * log2($AA_freq{$j,$key}/$R{$j,$key})) + (1-$lambda) * ($distr{$key} * log2($distr{$key}/$R{$j,$key}));
		}
	    }
	}	
##	$JSD_score = $JSD_score/2;
	for(my $i=1; $i<=$Nseq; $i++)
	{
	    if($ARR[$i][$j] eq '-')
	    {
		$gap_sum +=   $weights{$i}; 
	    }
	}
	$weighted_gap_penalty = (1-  ($gap_sum/$tot_weights));
	$JSD_scores{$j}=($JSD_score*$weighted_gap_penalty);	 ##a simple process of gap penalty
    }
    for(my $j=1;$j<=$Qresno;$j++)
    {
	$$JSD_ref{$j}= sprintf("%5.4f",$JSD_scores{$Qmapping{$j}});
    }

 
    return %res_count;
}

# Get Seqeuence Weights based on Henikoff-Henikoff schema
sub load_sequence_weights
{
    my ($ALN_ref, $Nseq_ref, $AA_ref, $seq_weights_ref)=@_;

    my %align  = %$ALN_ref;
    my $Nseq   = $$Nseq_ref;
    my %AA2in  = %$AA_ref;

    my $weights = 0;

    my %NRC=();my %RC=();my %seen=();
    for(my $i=1; $i<=$Nseq; $i++)
    {
	my @res=split(//, $align{$i, 2});

	for(my $j=0; $j<=$#res; $j++)
	{
	    my $AAN=$AA2in{$res[$j]};
	    $RC{$j, $AAN}++;	  #number of times $AAN appears at the jth column of the alignment
	    if(exists $seen{$j, $AAN}) 
	    {
		next;
	    }
	    else
	    {
		$seen{$j, $AAN}=1; 
		$NRC{$j}++;  #total number of AA type in the jth column
	    }
	}	
    }   

    for(my $i=1; $i<=$Nseq; $i++)
    {
	my @res=split(//,$align{$i,2});
	
	my $ali_L=@res;
	for(my $j=0; $j<=$#res; $j++)
	{
	    my $AAN=$AA2in{$res[$j]};
	    $$seq_weights_ref{$i} += 1/($NRC{$j} * $RC{$j, $AAN});    	   
	}
	$$seq_weights_ref{$i} = sprintf("%6.4f", $$seq_weights_ref{$i}/$ali_L); ##normalization
	$weights += $$seq_weights_ref{$i};
    }
    return $weights;  
}
	
sub log2 
{
    my $n = shift;
    return log($n)/log(2);
}

sub w_scores
{# This subroutine takes a list of scores and a length and transforms them so that each position is a weighted average of the surrounding positions. 
     my($score_ref,$L,$W,$lambda,$w_score_ref)=@_;
     my %r_score  =%$score_ref;
     my $Lch   =$$L;
     my $window   =$$W;
     for(my $i=1;$i<=$Lch;$i++)
     {
	 my $sum     = 0;
	 my $n_terms = 0;
	 for(my $j=$i-$window;$j<=$i+$window;$j++)
	 {
	     if(($i != $j) && (exists $r_score{$j}))
	     {
		 $sum +=  $r_score{$j};
		 $n_terms++;
	     }
	 }
	 $$w_score_ref{$i}= sprintf("%5.4f",$sum/$n_terms);
     }
}
    
sub z_scores
{# This subroutine calculates Z-scores for a set of scores (NOTE: hash index starts from 1)   
     my($score_ref,$L,$z_score_ref)=@_;
     my %r_score  =%$score_ref;
     my $Lch   =$$L;
     my $z1_a  = 0; my $z1_a2=0; my %zscore=();

     for(my $i=1; $i<=$Lch; $i++)
     {	 
	 $z1_a  += $r_score{$i}; 
	 $z1_a2 += $r_score{$i}**2;
     }
     $z1_a/=$Lch;
     $z1_a2/=$Lch;
     my $z1_sd=sqrt($z1_a2-$z1_a**2);
     for(my $i=1; $i<=$Lch; $i++)
     {
	 $zscore{$i}=($r_score{$i}-$z1_a)/$z1_sd;
     }
     my @zscore_keys=sort{$zscore{$b}<=>$zscore{$a}} keys %zscore;	
     my $Zmax = $zscore{$zscore_keys[0]};
     for(my $i=1; $i<=$Lch; $i++)
     {
	 $$z_score_ref{$i}= sprintf("%3.2f", $zscore{$i}/$Zmax);	
     }
}


sub get_chain
{
    my ($name)=@_;
    my $chain="";
    open(FIN, "$name");
    my $count=0;
    while(my $line = <FIN>)
    {
	if($line =~ /^ATOM/)
	{
	     my $chain1 = substr($line, 21, 1);
	     if(length $chain >0)
	     {
		 if($chain ne $chain1)
		 {
		     print "more than one chain!\n";
		 }
	     }
	     else
	     {
		 $chain=$chain1;
	     }
	}
    }

    if($chain eq " ")
    {
	$chain="A";
    }
    return $chain;
}

sub getfasta
{   
 
    my ($pdb)=@_;


    my %AA=(
            'ALA'=>'A', 'ARG'=>'R', 'ASN'=>'N', 'ASP'=>'D', 'CYS'=>'C', 'GLN'=>'Q', 'GLU'=>'E', 'GLY'=>'G',
            'HIS'=>'H', 'ILE'=>'I', 'LEU'=>'L', 'LYS'=>'K', 'MET'=>'M', 'PHE'=>'F', 'PRO'=>'P', 'SER'=>'S',
            'THR'=>'T', 'TRP'=>'W', 'TYR'=>'Y', 'VAL'=>'V', 'ASX'=>'N', 'GLX'=>'Q', 'UNK'=>'G', 
            'A'=>'ALA', 'R'=>'ARG', 'N'=>'ASN', 'D'=>'ASP', 'C'=>'CYS', 'Q'=>'GLN', 'E'=>'GLU', 'G'=>'GLY',
            'H'=>'HIS', 'I'=>'ILE', 'L'=>'LEU', 'K'=>'LYS', 'M'=>'MET', 'F'=>'PHE', 'P'=>'PRO', 'S'=>'SER',
            'T'=>'THR', 'W'=>'TRP', 'Y'=>'TYR', 'V'=>'VAL', 'B'=>'ASX', 'Z'=>'GLX', 'X'=>'UNK');

    my $seq="";
    open(FPDB, $pdb) or die "$pdb is missed!\n";
    my %resatoms=(); 
    while(my $line = <FPDB>)
    {
        if($line =~ /^ATOM/i)
        {
            my $res_name  = substr($line, 17, 3);
            my $res_no    = substr($line, 22, 5);
            my $check     = $res_no .  substr($line, 12, 4); #atom

            if(!exists $resatoms{$check}) ##check for alternative position atoms
            {
                $resatoms{$check}=1;

                my $atom_name=substr($line, 12, 4);
                if($atom_name eq ' CA ')
                {
                    $seq = $seq. $AA{$res_name};
                }
            }
        }
    }
    close(FPDB);

    return $seq;
}


sub bsites
{
    
    my ($pocPDB, $pocRes)=@_;

    if(!-s "$pocPDB")
    {
	printf "warning: no prediction!\n";	
    }
    open(FIN, "$pocPDB");
    my %score=();
    my @Pred_bsres=();

    ###get pocket centers
    my %pocket=();  my $poc_no = 0;  my %pocketc=();
    while(my $line1 = <FIN>)
    {	
        if($line1 =~ /^HETATM/)
        {
	    $poc_no    = substr($line1, 22, 4); $poc_no =~ s/\s+//;
	    $poc_no += 1;
            my $x = substr($line1, 30, 8);
            my $y = substr($line1, 38, 8);
            my $z = substr($line1, 46, 8);
            push(@{$pocket{$poc_no, 1}}, $x);
	    push(@{$pocket{$poc_no, 2}}, $y);
	    push(@{$pocket{$poc_no, 3}}, $z);
        }
    }
    close(FIN);

    my %cs=();
    my %pred=();
    for(my $i=1; $i<=$poc_no; $i++)
    {
	my @gridx = @{$pocket{$i, 1}};
	my @gridy = @{$pocket{$i, 2}};
	my @gridz = @{$pocket{$i, 3}};
	my %predictions=();

	open(PDB, "$pocRes");	
	my $cscore=0;
	while(my $line4=<PDB>)
	{
	    if($line4=~/^ATOM/)
	    {
		my $x        = substr($line4, 30, 8);
		my $y        = substr($line4, 38, 8);
		my $z        = substr($line4, 46, 8);
		my $resscore = substr($line4, 62, 4);
		my $res_no   = substr($line4, 22, 4); $res_no =~ s/\s+//;
		
		if(($resscore > 0.0) && !exists $predictions{$res_no})
		{		    
		    my $flag  = &grid_contact($x, $y, $z, \@gridx, \@gridy, \@gridz);
		    if($flag==1)
		    {
			$cscore += $resscore;
			$predictions{$res_no}=1;
		    }		    
		}
	    }            
        }
	close(PDB);

	my @keys= sort {$a <=> $b} keys %predictions;
	$cscore = $cscore/@keys if(@keys>0);
	$cs{$i} = sprintf "%.2f", $cscore;
	push(@{$pred{$i}}, @keys);
	
	#print "$cscore\t@keys\n";
	#last;
    }

    my @keys=sort {$cs{$b}<=>$cs{$a}} keys %cs;
    #my @keys=keys %cs;

    open(OUT, ">Bsites_$modelname.dat");
    foreach my $k(@keys)
    {
	if(@{$pred{$k}}>0)
	{
	    print OUT "$cs{$k}\t@{$pred{$k}}\n";
	}
    }
    close(OUT);
}


sub grid_contact
{       
    my ($x, $y, $z, $gridx_ref, $gridy_ref, $gridz_ref)=@_;

    my @gridx=@$gridx_ref;
    my @gridy=@$gridy_ref;
    my @gridz=@$gridz_ref;

    my $n=@gridx;
    my $flag=0;
    for(my $i=0; $i<$n; $i++)
    {
	if(&dist($x, $y, $z, $gridx[$i], $gridy[$i], $gridz[$i]) < 2.5)
	{
	    $flag = 1;
	    last;
	}
    }
    return $flag;
}
      

sub dist
{
    my ($x, $y, $z, $x1, $y1, $z1)=@_;

    my $dist=sqrt( ($x-$x1)**2 + ($y-$y1)**2 +($z-$z1)**2 );
    return $dist;
}




