#!/usr/bin/env python
import sys,re,os,sqlite3
from xml.etree import ElementTree
import userConfig,preparation,uniprotAnnotation

ceiling = 0.9
floor = 0.3

def read_human_blast_out(blastOut, query_pos):
	max_num = 5
	with open(blastOut,'r') as f:
		tree = ElementTree.parse(f)
	
	hit_nodes = tree.getiterator('Hit')
	hit_accessions = []
	hit_positions = []
	hit_q2index2q1s = []
	hit_q1index2q2s = []
	if len(hit_nodes)<max_num:
		max_num = len(hit_nodes)
	lable = 0
	for i in range(len(hit_nodes)):
		for j in hit_nodes[i].getchildren():
			if j.tag == 'Hit_accession':
				hit_accession = hit_nodes[i].getchildren()[3].text
			elif j.tag == 'Hit_hsps':
				hsp_dic = {}
				for k in j[0].getchildren():
					hsp_dic[k.tag] = k.text
		l_q1 = int(hsp_dic['Hsp_query-from'])-1
		l_q2 = int(hsp_dic['Hsp_hit-from'])-1
		if query_pos< (l_q1+1) or query_pos>int(hsp_dic['Hsp_query-to']):
			continue
		q1 = hsp_dic['Hsp_qseq']
		q2 = hsp_dic['Hsp_hseq']
		q2Index2q1 = {}
		q1Index2q2 = {}
		hit_pos = 0
		for index in range(len(q1)):
			if q1[index] != '-':
				l_q1 += 1
			if q2[index] != '-':
				l_q2 += 1
				if q1[index] == '-':
					q2Index2q1[l_q2] = 0
				else:
					q2Index2q1[l_q2]=l_q1
			if l_q1 == query_pos and q2[index]!= '-':
				hit_pos = l_q2
		for index in range(len(q2)):
			if q2[index] != '-':
				l_q2 += 1
			if q1[index] != '-':
				l_q1 += 1
				if q2[index] == '-':
					q1Index2q2[l_q1] = 0
				else:
					q1Index2q2[l_q1]=l_q2
		
		identity = float(hsp_dic['Hsp_identity'])
		alignLen = float(hsp_dic['Hsp_align-len'])
		identity = identity/alignLen

		if hit_pos != 0:
			"""
			if identity >= ceiling:
				max_num = 1
			elif identity < floor:
				break
			"""
			if identity < floor:
				break
			lable += 1
			hit_accessions.append(hit_accession)
			hit_positions.append(hit_pos)
			hit_q2index2q1s.append(q2Index2q1) 
			hit_q1index2q2s.append(q1Index2q2) 
			#print hit_accession, hit_pos
			#print q2Index2q1
			if lable >= max_num:
				break
	#http://www.jb51.net/article/17687.htm
	return hit_accessions, hit_positions, hit_q2index2q1s, hit_q1index2q2s

def get_homologous_protein(blastOut, query_pos):
	max_num = 1 
	select = ['SELECT * FROM resMap WHERE acc = ?1 and resolution >=0 ORDER BY resolution',
	'SELECT * FROM resMap WHERE acc = ?1 and resolution <0']
	dbfile = userConfig.humanDB+'.sqlite'
	if (not os.path.exists(dbfile)) or (not os.path.isfile(dbfile)):
		print 'ERROR: '+dbfile+' does not exist.'
		sys.exit()
	conn = sqlite3.connect(dbfile)

	with open(blastOut,'r') as f:
		tree = ElementTree.parse(f)
	
	hit_nodes = tree.getiterator('Hit')
	hit_accessions = []
	hit_positions = []
	pdb_positions = []
	pdb_units = []
	hit_q2index2q1s = []
	hit_q1index2q2s = []
	hit_pdbs = []
	hit_identity = []
	if len(hit_nodes)<max_num:
		max_num = len(hit_nodes)
	lable = 0
	for i in range(len(hit_nodes)):
		for j in hit_nodes[i].getchildren():
			if j.tag == 'Hit_accession':
				hit_accession = hit_nodes[i].getchildren()[3].text
			elif j.tag == 'Hit_hsps':
				hsp_dic = {}
				for k in j[0].getchildren():
					hsp_dic[k.tag] = k.text
		l_q1 = int(hsp_dic['Hsp_query-from'])-1
		l_q2 = int(hsp_dic['Hsp_hit-from'])-1
		if query_pos < (l_q1+1) or query_pos>int(hsp_dic['Hsp_query-to']):
			continue
		q1 = hsp_dic['Hsp_qseq']
		q2 = hsp_dic['Hsp_hseq']
		q2Index2q1 = {}
		q1Index2q2 = {}
		hit_pos = 0
		for index in range(len(q1)):
			if q1[index] != '-':
				l_q1 += 1
			if q2[index] != '-':
				l_q2 += 1
				if q1[index] == '-':
					q2Index2q1[l_q2] = 0
				else:
					q2Index2q1[l_q2]=l_q1
			if l_q1 == query_pos and q2[index]!= '-':
				hit_pos = l_q2
		for index in range(len(q2)):
			if q2[index] != '-':
				l_q2 += 1
			if q1[index] != '-':
				l_q1 += 1
				if q2[index] == '-':
					q1Index2q2[l_q1] = 0
				else:
					q1Index2q2[l_q1]=l_q2
		
		identity = float(hsp_dic['Hsp_identity'])
		alignLen = float(hsp_dic['Hsp_align-len'])
		identity = identity/alignLen

		if hit_pos != 0:
			"""
			if identity >= ceiling:
				max_num = 1
			elif identity < floor:
				break
			"""
			if identity > 0.8:
				continue
			if identity < floor:
				break

			pdbId = None
			with conn:	
				pdbs = uniprotAnnotation.fetchone(conn,select[0],(hit_accession,),1)
				if len(pdbs)< 1:
					pdbs = uniprotAnnotation.fetchone(conn,select[1],(hit_accession,),1)
				for entry in pdbs:
					if str(entry[5]) == 'Null' or str(entry[6]) == 'Null':
						break
					temp6 = int(filter(str.isdigit, str(entry[6])))
					temp5 = int(filter(str.isdigit, str(entry[5])))
					#pdbLen = entry[6]-entry[5]+1
					pdbLen = temp6-temp5+1
					off = entry[2]-entry[4]
					pdb_pos = hit_pos+off
					if pdb_pos < entry[5] or pdb_pos > entry[6]:
						continue
					pdbId = entry[1].strip().split(':')[0] 
					pdb_unit = userConfig.biounitDB+'/'+pdbId+'.pdb'
					if len(entry[1].strip().split(':') )>1:
						chain = entry[1].strip().split(':')[1]  
					else:
						chain = None
					if int(entry[8]) == 1:
						bioUnits = entry[9].strip().split(',')
						for unit in bioUnits:
							firstModel = unit.strip().split('.')[0]
							if re.search(chain,firstModel):
								pdb_unit = userConfig.biounitDB+'/biounit/'+pdbId+'.'+unit
								break
					break
			if pdbId is None:
				continue 
			lable += 1

			hit_accessions.append(hit_accession)
			hit_positions.append(hit_pos)
			pdb_positions.append(pdb_pos)
			pdb_units.append(pdb_unit)
			hit_q2index2q1s.append(q2Index2q1) 
			hit_q1index2q2s.append(q1Index2q2) 
			if chain == None:
				hit_pdbs.append(pdbId+':')
			else:
				hit_pdbs.append(pdbId+':'+chain)
			hit_identity.append(identity)
			if lable >= max_num:
				break
	return hit_accessions, hit_positions, pdb_positions, pdb_units, hit_q2index2q1s, hit_q1index2q2s, hit_pdbs, hit_identity

def run_human_blast(seqfile,outRoute):
	#cmd = check_slash(userConfig.ncbiBlast)+'blastp -seg yes -evalue 1e-3 -num_threads 1 -max_target_seqs 1000 -outfmt 5 -db '+check_slash(userConfig.sprotDB)+'sprot -query '+seqfile+' -out seqSprot.blast'
	cmd = preparation.check_slash(userConfig.ncbiBlast)+'blastp -seg yes -evalue 1e-3 -num_threads 1 -max_target_seqs 1000 -outfmt 5 -db '+userConfig.humanDB+' -query '+seqfile+' -out '+preparation.check_slash(outRoute)+'seqSprot.blast'
	print cmd
	preparation.run_cmd(cmd)
	
def run_human_psiblast(seqfile,outRoute):
	#cmd = check_slash(userConfig.ncbiBlast)+'blastp -seg yes -evalue 1e-3 -num_threads 1 -max_target_seqs 1000 -outfmt 5 -db '+check_slash(userConfig.sprotDB)+'sprot -query '+seqfile+' -out seqSprot.blast'
	cmd = preparation.check_slash(userConfig.ncbiBlast)+'psiblast -seg yes -evalue 1 -num_threads 2 -num_iteration 3 -num_alignments 3000 -inclusion_ethresh 1e-3 -outfmt 5 -db '+userConfig.humanDB+' -query '+seqfile+' -out '+preparation.check_slash(outRoute)+'seqHuman.blast'
	#print cmd
	preparation.run_cmd(cmd)

def run_uniprot_blastpgp(seqfile,outRoute):
	outRoute = preparation.check_slash(outRoute)
	#cmd = preparation.check_slash(userConfig.ncbiBlastL)+'blastpgp -F F -a 1 -j 3 -b 3000 -e 1 -h 1e-3 -d '+userConfig.uniref90+' -i '+seqfile+' -o '+outRoute+'seqPsi.blast -C '+outRoute+'seq.chk -Q '+outRoute+'seq.blastPsiMat'
	cmd = preparation.check_slash(userConfig.ncbiBlast)+'psiblast -seg yes -num_threads 1 -num_iterations 3 -num_alignments 3000 -evalue 1 -inclusion_ethresh 1e-3 -db '+userConfig.uniref90+' -query '+seqfile+' -out '+outRoute+'seqPsi.blast -out_pssm '+outRoute+'seq.chk -out_ascii_pssm '+outRoute+'seq.blastPsiMat'
	print cmd
	preparation.run_cmd(cmd,1)
	
	
def blastmatrix(matrixFile):
	fin = open(matrixFile,'r')
	line = fin.readline()
	matrix = {}
	while line:
		if re.match('^\#',line):
			line = fin.readline()
			continue
		if re.match('^\s+(\w.*$)',line):
			alpha = line.strip().split()
		elif re.match('^[A-Z\*]',line):
			cells = line.strip().split()
			let = cells[0]
			row = {}
			for i in range(1,len(cells)):
				row[alpha[i-1]]=cells[i]
			matrix[let]=row
		line = fin.readline()
	return matrix

def get_uniprot_seq(query):
		cmd = 'fgrep -s -m 1 -B 1 \''+query+'\' '+userConfig.humanSeq 	
		entry = run_cmd(cmd)
		if entry == '':
			return ('','')
		head = entry.split('\n')[0]
		seq = entry.split('\n')[1]
		
		if re.match('^>',head):
			print 'ERROR: incorrect fasta defined line format: '+head
			sys.exit()
		return head, seq
		
#def get_structural_alignment()
#def run_blat(query, db):
#	cmd = $CONFIG{blat}+" -prot -minIdentity= "+$pident+" -out"			

