#!/usr/bin/env python

import re,os,urllib,sys,sqlite3
from subprocess import Popen,PIPE
import userConfig

fttokens = ['ACT_SITE', 'BINDING', 'CARBOHYD', 'CA_BIND', 'COILED', 'COMPBIAS', 'CROSSLNK', 'DISULFID', 'DNA_BIND', 'INIT_MET', 'INTRAMEM', 'LIPID', 'METAL', 'MOD_RES', 'MOTIF', 'NON_STD', 'NP_BIND', 'PEPTIDE', 'PROPEP', 'REGION', 'REPEAT', 'SIGNAL', 'SITE', 'TRANSIT', 'TRANSMEM', 'ZN_FING']

def rmfile(myfile):
	if os.path.isfile(myfile):
		os.remove(myfile)

def download(url,local):
	urllib.urlretrieve(url,local)

def checkSlash(route):
	if route.strip()[-1] != '/':
		route = route.strip()+"/"
	return route

def isExe(fpath):
	return os.path.isfile(fpath) and os.access(fpath, os.X_OK)

def createdb(dbfile, id2accfile, ftfile, snp2accfile, idmapfile):
	tableTitles = ['id2acc', 'features', 'snp2acc', 'xdb2acc']
	tCols =  [       3,          5,         7,         3]
	#Database schema
	tables = {
		'id2acc' : 'CREATE TABLE id2acc (acc VARCHAR(12) NOT NULL, sprot BOOLEAN NOT NULL, name VARCHAR(24) NOT NULL)',
		'features' : 'CREATE TABLE features (acc VARCHAR(12) NOT NULL, name VARCHAR(10) NOT NULL, start INTEGER NOT NULL,end INTEGER NOT NULL, desc TEXT)',
		'snp2acc' : 'CREATE TABLE snp2acc (acc VARCHAR(12) NOT NULL, rsid VARCHAR(16) NOT NULL, aapair CHAR(2) NOT NULL, pos INTEGER NOT NULL, aa1 CHAR(1) NOT NULL, aa2 CHAR(1) NOT NULL, desc TEXT)',
		'xdb2acc' : 'CREATE TABLE xdb2acc (acc VARCHAR(12) NOT NULL, type VARCHAR(16) NOT NULL COLLATE NOCASE, name VARCHAR(24) NOT NULL COLLATE NOCASE)'
		}	
	inserts = {
		'id2acc' : 'INSERT INTO id2acc VALUES ('+', '.join(('?')*tCols[0])+')',
		'features' : 'INSERT INTO features VALUES ('+', '.join(('?')*tCols[1])+')',
		'snp2acc' : 'INSERT INTO snp2acc  VALUES ('+', '.join(('?')*tCols[2])+')',
		'xdb2acc' : 'INSERT INTO xdb2acc  VALUES ('+', '.join(('?')*tCols[3])+')'
		}
	indices = {
		'id2acc' : ['CREATE INDEX x_idacc ON id2acc (acc)',
			'CREATE INDEX x_idname ON id2acc (name)'],
		'features' : ['CREATE INDEX x_ftacc  ON features (acc)',
			'CREATE INDEX x_ftname ON features (name)',
			'CREATE INDEX x_ftacnm ON features (acc, name)',
			'CREATE INDEX x_ftacst ON features (acc, start)',
			'CREATE INDEX x_ftacen ON features (acc, end)'],
		'snp2acc' : ['CREATE INDEX x_snpacc ON snp2acc (acc)',
			'CREATE INDEX x_snprs  ON snp2acc (rsid)',
			'CREATE INDEX x_snppos ON snp2acc (pos)',
			'CREATE INDEX x_snpapp ON snp2acc (acc, pos, aapair)'],
		'xdb2acc' : ['CREATE INDEX x_xdbacc  ON xdb2acc (acc)',
			'CREATE INDEX x_xdbtype ON xdb2acc (type)',
			'CREATE INDEX x_xdbname ON xdb2acc (name)',
			'CREATE INDEX x_xdbtyna ON xdb2acc (type, name)']
		}
	typenames = ['UniProtKB-AC', 'UniProtKB-ID', 'GeneID', 'RefSeq', 'GI', 'PDB', 'GO', 'UniRef100', 'UniRef90', 'UniRef50', 'UniParc', 'PIR', 'NCBI-taxon', 'MIM', 'UniGene', 'PubMed', 'EMBL', 'EMBL-CDS', 'Ensembl', 'Ensembl_TRS', 'Ensembl_PRO', 'PubMed-Extra']
	#typenames = ['UniProtKB-AC', 'UniProtKB-ID', 'GeneID', 'RefSeq', 'GI', 'PDB', 'GO', 'IPI', 'UniRef100', 'UniRef90', 'UniRef50', 'UniParc', 'PIR', 'NCBI-taxon', 'MIM', 'UniGene', 'PubMed', 'EMBL', 'EMBL-CDS', 'Ensembl', 'Ensembl_TRS', 'Ensembl_PRO', 'PubMed-Extra']
	print 'Creating SQLite database ...'
	rmfile(dbfile)

	conn = sqlite3.connect(dbfile)
	with conn:
		#Opened database sucessfully'
		cur = conn.cursor()
		#Create tables
		for table in tableTitles:
			cur.execute(tables[table])
	
		dataFiles = [id2accfile, ftfile, snp2accfile, idmapfile]
		print 'Populating database (this will take a while):'
		for i in range(0,len(dataFiles)):
			dataFile = dataFiles[i]
			tableTitle = tableTitles[i]
			print 'Loading data into '+tableTitle
			fin = open(dataFile,'r')
			if re.match('^xdb',tableTitle):
				batchSize = 50000
			else:
				batchSize = 5000
			count = 0
			line = fin.readline()
			while line:
				if re.match('^#',line) or re.match('^\s*$',line):
					line = fin.readline()
					continue
				line = re.sub('\n$','',line)
				if re.match('^xdb',tableTitle):
					orgtag = ''
					if human['spname']:
						orgtag = '_'+human['spname']
					#For format description see: ftp://ftp.uniprot.org/pub/databases/uniprot/current_release/knowledgebase/idmapping/README
					aCells = re.split('\t',line)
					if (orgtag != '') and not (re.match('^\w+'+orgtag+'$',aCells[1])):
						line = fin.readline()
						continue
					acc = aCells[0]
					#Load the following ID types: GeneID RefSeq GI PDB GO UniRef100 UniParc PIR UniGene EMBL EMBL-CDS Ensembl Ensembl_TRS Ensembl_PRO	
					idtypes = [2,3,4,5,6,7,10,11,14,16,17,18,19,20]
					#idtypes = [2,3,4,5,6,7,11,12,15,17,18,19,20,21]
					for idtype in idtypes:
						rCells = re.split(';\s*',aCells[idtype])
						for cell in rCells:
							if len(cell) != 0:
								cell = re.sub('^(ENSG[0-9]+)\.\s*\[[A-Z0-9-]+\]\s*','\\1',cell)
								try:
									cur.execute(inserts[tableTitle],(acc,typenames[idtype],cell))
									if (count % batchSize) == 0:
										#sys.stdout.write('.')
										conn.commit()
								except sqlite3.Error, e:
									if conn:
										conn.rollback()
									print "Error %s:" % e.args[0]
									sys.exit(1)
								count += 1
				else:
					rCells = re.split('\t',line,tCols[i])
					try:
						cur.execute(inserts[tableTitle],rCells)
						if (count % batchSize) == 0:
							#sys.stdout.write('.')
							conn.commit()
					except sqlite3.Error, e:
						if conn:
							conn.rollback()
						print "Error %s:" % e.args[0]
						sys.exit(1)
					count += 1
				line = fin.readline()
			fin.close()
			if count:
				conn.commit()
			print 'Indexing '+tableTitle+' ...'
			for index in indices[tableTitle]:
				cur.execute(index) 
			print 'Optimizing '+tableTitle+' ...'
			cur.execute('ANALYZE '+tableTitle)
			if re.match('^xdb', tableTitle):
				print 'Compressing '+dataFile+' ...'
				cmd = 'gzip -f '+dataFile
				s = Popen(cmd,shell = True, stdout=PIPE,stderr=PIPE)
				stdout, stderr = s.communicate()
				if stderr != '':
					print stderr
					sys.exit()
				rmfile(dataFile)	
			else:
				rmfile(dataFile)
	conn.close()


#-------------------------------------------------------------------------------------
#-------------------------------------------------------------------------------------

outfile =sys.argv[1]
outfile = checkSlash(outfile)
if not os.path.exists(outfile):
	print "the "+outfile+" does not exitst!"
	sys.exit()
species = 'human'
human = {'os':'Homo sapiens', 'spname':'HUMAN', 'taxn':9606, 'file':'human'}
aa = '[ACDEFGHIKLMNPQRSTVWY]'
seqfile = outfile+species+".seq"
ftfile = outfile+species+".ft"
swfile = outfile+species+".dat"
id2accfile = outfile+species+".id2acc"
snp2accfile = outfile+species+".snp2acc"
xdb2accfile = outfile+species+".xdb2acc"
dbfile = outfile+species+".sqlite"
names = ['uniprot_sprot_'+human['file']+'.dat','uniprot_trembl_'+human['file']+'.dat',human['spname']+'_'+str(human['taxn'])+'_idmapping_selected.tab']
uniprot_ftp = 'ftp://ftp.uniprot.org/pub/databases/uniprot/current_release/knowledgebase'

for name in names:
	infile = ''
	match = re.search('idmapping_selected',name) 
	if match:
		infile = uniprot_ftp+'/idmapping/by_organism/'+name+'.gz'
	else:
		infile = uniprot_ftp+'/taxonomic_divisions/'+name+'.gz'

	#--------------------------------------------------------------------------
	print "Downloading "+infile+" ..."
	local = infile.split('/')[-1]
	rmfile(outfile+local)
	download(infile,outfile+local)

	#--------------------------------------------------------------------------
	print "Decompressing "+local	

	cmd = 'gunzip -f '+outfile+local
	s = Popen(cmd,shell = True, stdout=PIPE,stderr=PIPE)
	stdout, stderr = s.communicate()
	if stderr != '':
		print stderr
		sys.exit()

	print "Done."


#--------------------------------------------------------------------------
swall = open(swfile,'w')
seq = open(seqfile,'w')
ft = open(ftfile,'w')
id2acc = open(id2accfile,'w')
snp2acc = open(snp2accfile,'w')
dbase_len = 0
total_filtered = 0

for name in names:
	match = re.search('idmapping_selected',name) 
	if match:
		continue
	
	#--------------------------------------------------------------------------
	print "Processing "+name+" ..."
	num_filtered = 0
	
	infile = open(outfile+name,'r')	
	line = infile.readline()
	while line:
		lable = 0
		#line = line.strip()
		match = re.match('^ID.*\\b(\d+) AA',line)
		if match:
			osNum = 0#
			accession = []#
			genes = []#
			entryLines = []#
			snps = []#
			ftLines = []#
			seqLines = []#
			entryId = line.split()[1]#
			lenth = int(match.group(1))#
			match = re.match('^FT   (\w+)\s',line)
			entryLines.append(line)
			de = ""
		elif re.match('^OS   ',line):
			entryLines.append(line)
			match = re.match('^FT   (\w+)\s',line)
			if not human['os'] or re.search('^OS   (?:'+human['os']+')(?:\s+\(.+\))?\.*$',line): 
				osNum += 1
		elif re.match('^AC   ',line):
			entryLines.append(line)
			accsTmp = line.strip().split()
			for i in range(1,len(accsTmp)):
				accession.append(accsTmp[i].strip().split(';')[0])
			#print accession
		elif re.match('^GN   ',line):
			entryLines.append(line)
			match = re.search('Name=([^;]+)',line)	
			if match:
				genes.append(match.group(1).strip()) 
			match = re.search('Synonyms=([^;]+)',line)	
			if match:
				tmps = match.group(1).strip().split(',')	
				for tmp in tmps:
					if tmp.strip() != '':
						genes.append(tmp.strip())
			#print genes
		elif re.match('^FT   \w+\s',line):
			#print line
			entryLines.append(line)
			match = re.match('^FT   (\w+)\s',line)
			feature = match.group(1)
			###ftLine = line.strip()
			ftLine = re.sub('\n$','',line)
			line = infile.readline()
			while line:
				#line = line.strip()
				cont = line[34:]
				cont = re.sub('\n$','',cont)
				if line[2:34] != ' '*32:
					lable = 1
					break
				if not re.match('^/FTId=\w+',cont):
					ftLine = ftLine + ' ' + cont
				entryLines.append(line)
				line = infile.readline()
			#print feature
			ftLine = re.sub('\.\s*$','',ftLine)
			#print ftLine
			if feature == 'VARIANT':
				start = ftLine[14:20].strip()
				end = ftLine[21:27].strip()
				desc = ftLine[34:]
				#print start, end, desc
				if re.match('^\d+$',start) and re.match('^\d+$',end) and (int(end)-int(start) == 0) and re.match('^'+aa+' -> '+aa+'\\b\s*',desc):
					pattern = re.compile('^('+aa+') -> ('+aa+')\\b\s*')
					match = pattern.match(desc)
					aa1 = match.group(1)
					aa2 = match.group(2)
					desc = pattern.sub('',desc)
					desc = re.sub('^\((.+)\)\.?','\\1',desc)
					#print desc
					tmpSnps = re.findall('dbSNP:(rs\d+)',desc)
					dbcount = 0
					for snp in tmpSnps:
						#print snp
						snps.append([snp,''.join(sorted([aa1,aa2])),start, aa1, aa2,desc])
						dbcount += 1
					#if dbcount != 0:
					#	print snps
					#else:
					#	print "*"
			else:
				ftLine = re.sub('^FT   ','',ftLine)
				if feature in fttokens:
					tmpList = re.split('\s+',ftLine,3)
					#print len(tmpList)
					for tmpi in range(len(tmpList),4):
						tmpList.append('')
					#tmpList = re.split('\s*',ftLine)
					#tmpftLine = (tmpList[0:3])
					#if len(tmpList) > 3:
					#	tmpftLine.append(ftLine[ftLine.find(tmpList[3]):])
					#else: tmpftLine.append('')
					#ftLines.append(tmpftLine)
					ftLines.append(tmpList)
		elif re.match('^DE\s+\w.*\n$',line):
			entryLines.append(line)
			match = re.match('^DE\s+(\w.*)\n$',line)
			tmpinf = match.group(1)
			if tmpinf == 'Contains:':
				line = infile.readline()
				while line:
					if not re.match('^DE\s{5}',line):
						lable = 1
						break
					entryLines.append(line)
					line = infile.readline()
			else:
				tmpinf = re.sub('\[','{',tmpinf)
				tmpinf = re.sub('\]','}',tmpinf)
				de += " "+tmpinf
			#print de
		elif re.match('^SQ   ',line):
			entryLines.append(line)
			#print line+"\n"
			line = infile.readline()
			while line:
				if re.match('^ ',line):
					entryLines.append(line)
					sequence = line
					sequence = re.sub('\s+','',sequence)
					#print seqquence
					seqLines.append(sequence)
				else:
					lable = 1
					break
				line = infile.readline()
		elif re.match('^\/\/',line):
			if osNum:
				for tmpLine in entryLines:
					swall.write(tmpLine)
				swall.write(line)
				primacc = accession[0]
				canonical = 0
				if re.search('uniprot_sprot', name):
					canonical = 1
					de = ' Canonical;'+de 
				lenSeq = " Length: "+str(lenth)
				maxLen = 1000 - len(lenSeq)
				if len(de) > maxLen:
					de = de[0:maxLen]
				seq.write(">sp|"+primacc+"|"+entryId+de+lenSeq+'\n')
				seq.write(''.join(seqLines)+'\n')
				if len(ftLines)>0:
					for tmpLine in ftLines:
						ft.write(primacc+'\t'+'\t'.join(tmpLine)+'\n')
				id2acc.write(primacc+'\t'+str(canonical)+'\t'+entryId+'\n')
				for tmpLine in accession:
					id2acc.write(primacc+'\t'+str(canonical)+'\t'+tmpLine+'\n')
				if len(genes) >0:
					for tmpLine in genes:
						id2acc.write(primacc+'\t'+str(canonical)+'\t'+tmpLine+'\n')
				if len(snps) >0:
					for tmpLine in snps:
						snp2acc.write(primacc+'\t'+'\t'.join(tmpLine)+'\n')
				num_filtered += 1
				dbase_len += lenth	
		else:
			entryLines.append(line)
		if lable == 0:
			line = infile.readline()
	infile.close()
	print "Fount "+str(num_filtered)+" known "+species+" proteins."
	rmfile(outfile+name)
	total_filtered += num_filtered

swall.close()
seq.close()
ft.close()
id2acc.close()
snp2acc.close()
print 'Compressing '+swfile+' ...'
cmd = 'gzip -f '+swfile
s = Popen(cmd,shell = True, stdout=PIPE,stderr=PIPE)
stdout, stderr = s.communicate()
if stderr != '':
	print stderr
	sys.exit()


#--------------------------------------------------------------------------
varsplic = 0
if varsplic != 0:
	gzfile  = 'uniprot_sprot_varsplic.fasta.gz'
	fafile = re.sub('\.gz$','',gzfile)
	#"""
	infile = uniprot_ftp+'/complete/'+gzfile

	print "Downloading "+infile+" ..."
	rmfile(outfile+gzfile)
	download(infile,outfile+gzfile)

	#--------------------------------------------------------------------------
	print "Decompressing "+gzfile
	cmd = 'gunzip -f '+outfile+gzfile
	s = Popen(cmd,shell = True, stdout=PIPE,stderr=PIPE)
	stdout, stderr = s.communicate()
	if stderr != '':
		print stderr
		sys.exit()
	print "Done."

	varcount = 0
	print "Processing "+fafile+" ..."
	orgtag = ''
	if human['spname']:
		orgtag = '_'+human['spname']
	seq = open(seqfile,'a')
	id2acc = open(id2accfile,'a')
	seqLine = ''
	defLine = ''

	readfile = open(outfile+fafile,'r')
	line = readfile.readline()
	while line:
		#line = line.strip()
		line = re.sub('\n$','',line)
		if re.match('^>',line):
			if len(defLine) > 0:
				definition = re.split('\s+',defLine,1)[0]
				if (orgtag == '') or re.search('\w+'+orgtag+'$',definition):	
					seq.write(defLine+'\n')
					if len(seqLine)>0:
						seq.write(seqLine+'\n')
					varcount += 1
					dbase_len += len(seqLine)
					acc = re.split('\|',defLine,2)[1]
					if len(acc)>0 and re.search('-\d+$',acc):
						id2acc.write(acc+'\t0\t'+acc+'\n')
					else:
						print 'Missing/malformed varsplic accession: '+acc
			defLine = line
			seqLine = ''
		else:
			seqLine += line
		line = readfile.readline()
	if len(defLine) > 0:
		definition = re.split('\s+',defLine,1)[0]
		if (orgtag == '') or re.search('\w+'+orgtag+'$',definition):	
			seq.write(defLine+'\n')
			if len(seqLine)>0:
				seq.write(seqLine+'\n')
			varcount += 1
			dbase_len += len(seqLine)
			acc = re.split('\|',defLine,2)[1]
			if len(acc)>0 and re.search('-\d+$',acc):
				id2acc.write(acc+'\t0\t'+acc+'\n')
			else:
				print 'Missing/malformed varsplic accession: '+acc
	seq.close()
	id2acc.close()
	readfile.close()
	if varcount >0:
		print 'Appended '+str(varcount)+' known '+species+' protein isoforms.'
	rmfile(outfile+fafile)

	print 'Total number of proteins in '+species+' database: '+str(total_filtered + varcount);
	print 'Total length of protein sequences in '+species+' database: '+str(dbase_len)+' residues'
#"""

#--------------------------------------------------------------------------
mkBlastdb = 'makeblastdb'
mkdbLog = species+'.makeblastdb.log'
mkdbArgs = '-in '+seqfile+' -dbtype prot -out '+outfile+species+' -title '+species+' -parse_seqids -logfile '+outfile+mkdbLog
print 'Running '+mkBlastdb+' ...'
mkdbPath = checkSlash(userConfig.ncbiBlast)+mkBlastdb
if not isExe(mkdbPath): 
	print 'Failed to locate'+mkdpPath+'executable file!'
	sys.exit()
cmd = mkdbPath+' '+mkdbArgs
s = Popen(cmd,shell = True, stdout=PIPE,stderr=PIPE)
stdout, stderr = s.communicate()
if stderr != '':
	print stderr
	sys.exit()
print "Done."


#--------------------------------------------------------------------------
idmapfile = outfile+human['spname']+'_'+str(human['taxn'])+'_idmapping_selected.tab'
createdb(dbfile, id2accfile, ftfile, snp2accfile, idmapfile)
print 'All done.'



