#!/usr/bin/env python
import sys,os,sqlite3
import userConfig,uniprotAnnotation,structure,alignment

def get_contact_uniprot_annotation(pdbId,residues,acc,outLines,pdbUnit=None,lable=0):
	select = ['SELECT * FROM resMap WHERE acc = ?1 AND name = ?2',
			'SELECT * FROM resMap WHERE name = ?'] 

	dbfile = userConfig.humanDB+'.sqlite'
	for chain in residues:
		conn = sqlite3.connect(dbfile)
		with conn:
			row = uniprotAnnotation.fetchone(conn, select[0],(acc,pdbId+':'+chain,),1)
			if len(row) < 1:
				row = uniprotAnnotation.fetchone(conn, select[1],(pdbId+':'+chain,),1)
		conn.close()
			
		if len(row)>0:
			off = row[0][4]-row[0][2]
			for i in range(len(residues[chain])):
				unitpPos = residues[chain][i][0]+off
				contactFt = uniprotAnnotation.get_residue_uniprot_annotation([row[0][0]],[unitpPos])		
				fttokens = {'ACT_SITE':0, 'BINDING':0, 'CARBOHYD':0, 'CA_BIND':0, 'COILED':0, 'COMPBIAS':0, 'CROSSLNK':0, 'DISULFID':0, 'DNA_BIND':0, 'INIT_MET':0, 'INTRAMEM':0, 'LIPID':0, 'METAL':0, 'MOD_RES':0, 'MOTIF':0, 'NON_STD':0, 'NP_BIND':0, 'PEPTIDE':0, 'PROPEP':0, 'REGION':0, 'REPEAT':0, 'SIGNAL':0, 'SITE':0, 'TRANSIT':0, 'TRANSMEM':0, 'ZN_FING':0}
				outline = str(residues[chain][i][1])+'\t'+chain+'\t'+str(residues[chain][i][0])+'\t'
				for ftname in contactFt:		
					fttokens[ftname]=1
				for fttoken in fttokens:
					outline += str(fttokens[fttoken])+'\t'
				if lable == 2:
					outLines.append('\t-\t'+outline.strip()+'\n')
				else:
					outLines.append(outline.strip()+'\n')
				
				if lable == 1:	
					intras, inters, hets = structure.struct_analysis(pdbUnit,chain,int(residues[chain][i][0]),6.0,5.0,6.0)
					outLines.append('\t*intraresidue\n')
					get_contact_uniprot_annotation(pdbId,intras,acc,outLines,None,2)
					outLines.append('\t*interresidue\n')
					get_contact_uniprot_annotation(pdbId,inters,acc,outLines,None,2)
					outLines.append('\t*het\n')
					get_contact_uniprot_annotation(pdbId,hets,acc,outLines,None,2)


def analysis_contact(acc, pdbname,pdbUnit,pdbPos,outLines):
	dbfile = userConfig.humanDB+'.sqlite'
	if (not os.path.exists(dbfile)) or (not os.path.isfile(dbfile)):
		print 'ERROR: '+dbfile+'does not exist.'
		sys.exit()

	outLines.append('biounit\n')
	chain = pdbname.strip().split(':')[1]
	pdbId = pdbname.strip().split(':')[0]
	print '************************'
	print pdbUnit,chain, pdbPos
	intras, inters, hets = structure.struct_analysis(pdbUnit,chain,int(pdbPos))
	if intras == None:
		return None
	
	outLines.append('intraresidue\n')
	get_contact_uniprot_annotation(pdbId,intras,acc,outLines,pdbUnit,1)
	outLines.append('interresidue\n')
	get_contact_uniprot_annotation(pdbId,inters,acc,outLines,pdbUnit,1)
	outLines.append('het\n')
	get_contact_uniprot_annotation(pdbId,hets,acc,outLines,pdbUnit,1)


def get_native_contact_uniprot_annotation(residues,outLines,pdbName,dataType,itasserfile=None,lable=0):
	for chain in residues:
		for i in range(len(residues[chain])):
			fttokens = {'ACT_SITE':0, 'BINDING':0, 'CARBOHYD':0, 'CA_BIND':0, 'COILED':0, 'COMPBIAS':0, 'CROSSLNK':0, 'DISULFID':0, 'DNA_BIND':0, 'INIT_MET':0, 'INTRAMEM':0, 'LIPID':0, 'METAL':0, 'MOD_RES':0, 'MOTIF':0, 'NON_STD':0, 'NP_BIND':0, 'PEPTIDE':0, 'PROPEP':0, 'REGION':0, 'REPEAT':0, 'SIGNAL':0, 'SITE':0, 'TRANSIT':0, 'TRANSMEM':0, 'ZN_FING':0}
			seqAccessions, seqPositions,seqIndex2query,seqQuery2uniprot = alignment.read_human_blast_out('/nfs/amino-home/liquan/CancerDriver/myprogram/experiment/features/'+dataType+'/contact/'+pdbName+'/seqSprot.blast',int(residues[chain][i][0]))
			contactFt = uniprotAnnotation.get_residue_uniprot_annotation(seqAccessions,seqPositions)
			outline = str(residues[chain][i][1])+'\t'+chain+'\t'+str(residues[chain][i][0])+'\t'
			for ftname in contactFt:		
				fttokens[ftname]=1
			for fttoken in fttokens:
				outline += str(fttokens[fttoken])+'\t'
			if lable == 2:
				outLines.append('\t-\t'+outline.strip()+'\n')
			else:
				outLines.append(outline.strip()+'\n')

			if lable == 1:	
				intras, inters, hets = structure.struct_analysis(itasserfile,chain,int(residues[chain][i][0]),6.0,5.0,6.0)
				outLines.append('\t*intraresidue\n')
				get_native_contact_uniprot_annotation(intras,outLines,pdbName,dataType,None,2)
				outLines.append('\t*interresidue\n')
				get_native_contact_uniprot_annotation(inters,outLines,pdbName,dataType,None,2)
				outLines.append('\t*het\n')
				get_native_contact_uniprot_annotation(hets,outLines,pdbName,dataType,None,2)


def analysis_contact_itasser(pdbname,itasserfile,pdbPos,dataType,outLines):
	outLines.append('I-TASSER\n')
	intras, inters, hets = structure.struct_analysis(itasserfile,'A',int(pdbPos))
	if intras == None:
		return None
	
	outLines.append('intraresidue\n')
	get_native_contact_uniprot_annotation(intras,outLines,pdbname,dataType,itasserfile,1)
	outLines.append('interresidue\n')
	get_native_contact_uniprot_annotation(inters,outLines,pdbname,dataType,itasserfile,1)
	outLines.append('het\n')
	get_native_contact_uniprot_annotation(hets,outLines,pdbname,dataType,itasserfile,1)

def get_spring_contact_uniprot_annotation(residues,outLines,pdbName,dataType,itasserfile=None,seqL=0,lable=0):
	for chain in residues:
		for i in range(len(residues[chain])):
			fttokens = {'ACT_SITE':0, 'BINDING':0, 'CARBOHYD':0, 'CA_BIND':0, 'COILED':0, 'COMPBIAS':0, 'CROSSLNK':0, 'DISULFID':0, 'DNA_BIND':0, 'INIT_MET':0, 'INTRAMEM':0, 'LIPID':0, 'METAL':0, 'MOD_RES':0, 'MOTIF':0, 'NON_STD':0, 'NP_BIND':0, 'PEPTIDE':0, 'PROPEP':0, 'REGION':0, 'REPEAT':0, 'SIGNAL':0, 'SITE':0, 'TRANSIT':0, 'TRANSMEM':0, 'ZN_FING':0}
			if int(residues[chain][i][0]) > seqL:
				#seqAccessions, seqPositions,seqIndex2query,seqQuery2uniprot = alignment.read_human_blast_out('/nfs/amino-home/liquan/CancerDriver/myprogram/experiment/features/'+dataType+'/contact/'+pdbName+'/seqSprot.blast',(int(residues[chain][i][0])-seqL))
				#seqAccessions, seqPositions,seqIndex2query,seqQuery2uniprot = alignment.read_human_blast_out('/nfs/amino-home/liquan/CancerDriver/myprogram/experiment/addNewData/experiment/features/'+dataType+'/contact/'+pdbName+'/seqSprot.blast',(int(residues[chain][i][0])-seqL))
				#seqAccessions, seqPositions,seqIndex2query,seqQuery2uniprot = alignment.read_human_blast_out('/nfs/amino-home/liquan/CancerDriver/myprogram/experiment/addNewData/H3N2HA/features/'+dataType+'/contact/'+pdbName+'/seqSprot.blast',(int(residues[chain][i][0])-seqL))
				seqAccessions, seqPositions,seqIndex2query,seqQuery2uniprot = alignment.read_human_blast_out('/nfs/amino-home/liquan/CancerDriver/myprogram/experiment/TP53/features/'+dataType+'/contact/'+pdbName+'/seqSprot.blast',(int(residues[chain][i][0])-seqL))
			else:
				#seqAccessions, seqPositions,seqIndex2query,seqQuery2uniprot = alignment.read_human_blast_out('/nfs/amino-home/liquan/CancerDriver/myprogram/experiment/features/'+dataType+'/contact/'+pdbName+'/seqSprot.blast',int(residues[chain][i][0]))
				#seqAccessions, seqPositions,seqIndex2query,seqQuery2uniprot = alignment.read_human_blast_out('/nfs/amino-home/liquan/CancerDriver/myprogram/experiment/addNewData/experiment/features/'+dataType+'/contact/'+pdbName+'/seqSprot.blast',int(residues[chain][i][0]))
				#seqAccessions, seqPositions,seqIndex2query,seqQuery2uniprot = alignment.read_human_blast_out('/nfs/amino-home/liquan/CancerDriver/myprogram/experiment/addNewData/H3N2HA/features/'+dataType+'/contact/'+pdbName+'/seqSprot.blast',int(residues[chain][i][0]))
				seqAccessions, seqPositions,seqIndex2query,seqQuery2uniprot = alignment.read_human_blast_out('/nfs/amino-home/liquan/CancerDriver/myprogram/experiment/TP53/features/'+dataType+'/contact/'+pdbName+'/seqSprot.blast',int(residues[chain][i][0]))

			contactFt = uniprotAnnotation.get_residue_uniprot_annotation(seqAccessions,seqPositions)
			outline = str(residues[chain][i][1])+'\t'+chain+'\t'+str(residues[chain][i][0])+'\t'
			for ftname in contactFt:		
				fttokens[ftname]=1
			for fttoken in fttokens:
				outline += str(fttokens[fttoken])+'\t'
			if lable == 2:
				outLines.append('\t-\t'+outline.strip()+'\n')
			else:
				outLines.append(outline.strip()+'\n')

			if lable == 1:	
				intras, inters, hets = structure.struct_analysis(itasserfile,chain,int(residues[chain][i][0]),6.0,5.0,6.0)
				outLines.append('\t*intraresidue\n')
				get_spring_contact_uniprot_annotation(intras,outLines,pdbName,dataType,None,seqL,2)
				outLines.append('\t*interresidue\n')
				get_spring_contact_uniprot_annotation(inters,outLines,pdbName,dataType,None,seqL,2)
				outLines.append('\t*het\n')
				get_spring_contact_uniprot_annotation(hets,outLines,pdbName,dataType,None,seqL,2)


def analysis_contact_spring(pdbname,itasserfile,pdbPos,dataType,outLines,seqL):
	outLines.append('I-TASSER\n')
	intras, inters, hets = structure.struct_analysis(itasserfile,'A',int(pdbPos))
	if intras == None:
		return None
	
	outLines.append('intraresidue\n')
	get_spring_contact_uniprot_annotation(intras,outLines,pdbname,dataType,itasserfile,seqL,1)
	outLines.append('interresidue\n')
	get_spring_contact_uniprot_annotation(inters,outLines,pdbname,dataType,itasserfile,seqL,1)
	outLines.append('het\n')
	get_spring_contact_uniprot_annotation(hets,outLines,pdbname,dataType,itasserfile,seqL,1)
