#!/usr/bin/env python
import re,sys,os
import preparation, userConfig

minSeqLen = 17


def get_msa(query, blastfile,outFile):
	outFile = preparation.check_slash(outFile)
	fin = open(blastfile,'r')	
	data = fin.read()
	fin.close()
	hits = data.split('\n>')
	count = 0
	seq = {}
	
	match = re.search('Query=[^\n]+\n\nLength=(\d+)',hits[0])
	if not match:
		match = re.search('(\d+)\s+letters',hits[0])
		if not match:
			print 'ERROR: syntax error in '+blastfile
			sys.exit()
	seqLength = int(match.group(1))

	if len(query) != seqLength:
		print 'ERROR: sequence length in blast '+str(seqLength)+' is not equal the query length '+str(len(query))+'.'	
		sys.exit()
	
	preClustFile = outFile+'seq.msa'
	preClust = open(preClustFile,'w')
	preClust.write(query+'\n')
	large = 0
	for i in range(1,len(hits)):
		hit = hits[i]
		match = re.search('\n Identities = \d+\/(\d+) \((\d+)%',hit)
		if not match:
			print 'ERROR: no Identities in '+blastfile
			sys.exit()
		length = int(match.group(1))
		identity = int(match.group(2))
		if length < minSeqLen:
			continue
		if identity > 98 or identity < 30:
			large += 1 
			continue
		sub = re.split('Positives.+?\n',hit)

		querys = re.findall('Query:?\s+\d+\s+([A-Z-]+)\s+\d+\s*?\n',sub[1])
		starts = re.findall('Query:?\s+(\d+)\s+[A-Z-]+\s+\d+\s*?\n',sub[1])

		sbjects = re.findall('Sbjct:?\s+\d+\s+([A-Z-]+)\s+\d+\s*?\n',sub[1])
		tempSeq = ['-']*seqLength
		for i in range(len(querys)):
			index = int(starts[i])-1
			sbjects[i]=sbjects[i].replace('J','-').replace('U','-').replace('Z','-').replace('B','-').replace('O','-').replace('X','-')
			for j in range(len(querys[i])):
				if querys[i][j] != '-':
					tempSeq[index] = sbjects[i][j]
					index += 1

		preClust.write(''.join(tempSeq)+'\n')
		count += 1

		#if count > 300:
		#	break
	
	preClust.close()
	if count == 0:
		print 'ERROR No alignments exist for '+preClustFile
		sys.exit()
	return preClustFile

def run_evodesign(query, blastfile, outFile, model):
	outFile = preparation.check_slash(outFile)
	msaFile = get_msa(query, blastfile, outFile)	
	
	cmd = preparation.check_slash(userConfig.evoBin)+'bin_profile/mkprf '+msaFile+' '+preparation.check_slash(userConfig.evoBin)+'mkprf/blosum-62.txt >'+outFile+'prf.txt'
	print cmd+"\n"
	preparation.run_cmd(cmd)
	cmd = preparation.check_slash(userConfig.evoBin)+'bin_profile/dsspcmbi '+model+' >'+outFile+'dssp.txt'
	print cmd+"\n"
	preparation.run_cmd(cmd,1)
	cmd = preparation.check_slash(userConfig.evoBin)+'bin_profile/dssp-ver2hor '+outFile+'dssp.txt >'+outFile+'dssph.txt'
	print cmd+"\n"
	preparation.run_cmd(cmd)
	cmd = preparation.check_slash(userConfig.evoBin)+'bin_profile/convSS.pl '+outFile+'dssph.txt '+outFile+'dssp.txt '+outFile+' >'+outFile+'ss.txt'
	print cmd+"\n"
	preparation.run_cmd(cmd)

	cmd = '../bin/xtdseqali seq.txt prf.txt ss.txt sa.txt phi-psi.txt 1.0 1.0 1.0'
        print cmd
	preparation.run_cmd(cmd)
	energy = preparation.run_cmd(cmd)
	return energy

