#!/usr/bin/env python
import os, sys, math
#sys.path.append('/nfs/amino-home/liquan/bin/anaconda/bin/python')
from pdbchains import PDBchains
import numpy as np
import preparation,userConfig

concut = userConfig.itasser+"/common/contact.comm"
AA = ['GLY','ALA','SER','CYS','VAL','THR','ILE','PRO','MET','ASP','ASN','LEU','LYS','GLU','GLN','ARG','HIS','PHE','TYR','TRP']
mass = {'N':132,
	'Q':146,
	'S':105,
	'T':119,
	'Y':181,
	'A':89,
	'C':121,
	'G':75,
	'I':131,
	'L':131,
	'M':149,
	'F':165,
	'P':115,
	'W':204,
	'V':117,
	'R':174,
	'H':155,
	'K':146,
	'D':133,
	'E':147
}
HBcutoffRadiue = 3.5 #Hydrogen-Acceptor
HBcutoffAngle = 35.0 #Hydrogen-Acceptor


def ReadConcut(contact):
	"""
	read cut-off from concut.comm
	"""
	f_contact = open(contact,"r")
	line = f_contact.readline()

	cutoff = {}
	for Aa in AA:
		line = f_contact.readline()
		cell = line.strip().split()
		i = 0
		cut = {}
		for Ab in AA:
			i += 1
			cut[Ab] = float(cell[i])
		cutoff[Aa] = cut
	#read delta
	line = f_contact.readline()
	for Aa in AA:
		line = f_contact.readline()
		cell = line.strip().split()
		i = 0
		for Ab in AA:
			i += 1
			cutoff[Aa][Ab] += 2.5*float(cell[i])
			cutoff[Aa][Ab] = round(cutoff[Aa][Ab],4)
	return cutoff		


def SCList(pose):
	"""
	Read pdb and record SG aotms for every residue
	output is a dictionary recording the side chain of all residues (residueList).
	eg:
	sideChainList[1] will give you a dictionary as below, which means the side chain of residue 1 has 4 atoms, and their atom NO. in coord[0] of pose.
	sideChainList[1] = [4, 5, 6, 7]
	"""
	num_atoms = len(pose.coord[0])

	sideChainList = {}
	sideChainAtom = []
	for i in range(0, num_atoms):
		if pose.atom_info[0][i].atom_name != "C" and pose.atom_info[0][i].atom_name != "CA" and pose.atom_info[0][i].atom_name != "N" and pose.atom_info[0][i].atom_name != "O" and pose.atom_info[0][i].atom_name[0] != "H" and pose.atom_info[0][i].atom_name != "OXT":
			sideChainAtom.append(i)
		if i+1 == num_atoms:
			sideChainList[pose.atom_info[0][i].res_num] = sideChainAtom
			break
		if pose.atom_info[0][i].res_num != pose.atom_info[0][i+1].res_num:
			sideChainList[pose.atom_info[0][i].res_num] = sideChainAtom
			sideChainAtom = []
	return sideChainList


def SCcenter(pose, sideChainList):
        """
        Calculate Side chain center of mass
	Output (sideChainCenter) is a dictionary recording the coordinates of the center of each side chain residue, eg: sideChainCenter[1]=[ 128.20325,   36.33525,  -13.04575]) 
        """
	sideChainCenter = {}
	num_residues = len(pose.sequence[0])
	for i in range(1,num_residues+1):
		atoms_center = np.zeros(3,float)
		#print str(num_residues)+" "+str(i)
		#print sideChainList[i]
		if sideChainList[i]:
			for j in sideChainList[i]:
				atoms_center += pose.coord[0][j]
			atoms_center = atoms_center/len(sideChainList[i])
		else:
			atoms_center = pose.coord[0][pose.ca_pos[0][i-1]]
		sideChainCenter[i] = atoms_center
	return sideChainCenter

def standarAA(AA):
	if AA == 'ASX':
		AA = 'ASN'
	elif AA == 'MSE':
		AA = 'MST'
	elif AA == 'GLX':
		AA = 'GLN'
	elif AA == 'UNK':
		AA = 'GLY'
	return AA

def SCcontacts(pose, sideChainCenter, mpos, cutoff):
	"""
	collect side chain contacts for mutant position
	Output (sideChainContacts) is a list recording the residues' NO. contacting with mpos residue 
	"""
	sequence = pose.sequence[0]

	sideChainContacts = []
	num_residues = len(sequence)
	if not sideChainCenter.has_key(mpos):
		#print "******************************"
		#print num_residues
		return []
	for i in range(1,num_residues+1):
		if i == mpos:
			continue
		#print sideChainCenter
		#print len(sideChainCenter)
		#print mpos
		#print sideChainCenter[mpos]
		#print sideChainCenter[i]
		dis = np.sqrt(np.sum((sideChainCenter[mpos]-sideChainCenter[i])**2))
		#print str(dis) + sequence[mpos-1]+sequence[i-1]
		#print pose.codeToabrev[sequence[mpos-1]]
		#print pose.codeToabrev[sequence[i-1]]+" "+sequence[i-1]
		#mpos_residue = standarAA(pose.codeToabrev[sequence[mpos-1]])
		#i_residue = standarAA(pose.codeToabrev[sequence[i-1]])
		mpos_residue = pose.codeToabrev[sequence[mpos-1]]
		i_residue = pose.codeToabrev[sequence[i-1]]
		if dis < cutoff[mpos_residue][i_residue]:
			sideChainContacts.append(i)	
	return sideChainContacts

def RMapA(pose):
	num_atoms = len(pose.coord[0])

	residueMapAtoms = {}
	r_size = [0,-1]
	for i in range(0, num_atoms):
		r_size[1] += 1	
		if i+1 == num_atoms:
			residueMapAtoms[pose.atom_info[0][i].res_num] = r_size
			break
		if pose.atom_info[0][i].res_num != pose.atom_info[0][i+1].res_num:
			residueMapAtoms[pose.atom_info[0][i].res_num] = r_size
			r_size = [i+1,i]
	return residueMapAtoms


def NResidue(pose, residueMapAtoms, pos):
	"""
	backbone:N
	Arg
		side chain: NE,NH1,NH2
	Trp
		side chain:NE1,
	Gln
		side chain: NE2
	Asn
		side chain: ND2
	Lys
		side chain: NZ
	His
		side chain: ND1, NE2
	"""
	H_HNResidue = {}
	N_HNResidue = {} 
	N_noHResidue = {}
	N_Residue = {}
	num_H = 0
	num_noH = 0
	num = 0
	for j in range(residueMapAtoms[pos][0],residueMapAtoms[pos][1]+1):
		atomA = pose.atom_info[0][j].atom_name
		if atomA == "N": 
			lable = 1
			num += 1
			N_Residue[num] = pose.coord[0][j]
			for k in range(residueMapAtoms[pos][0],residueMapAtoms[pos][1]+1):
				atomB = pose.atom_info[0][k].atom_name
				if atomB == "H" or (len(atomB)==2 and atomB[1].isdigit()):
					lable = 0
					num_H += 1
					H_HNResidue[num_H] = pose.coord[0][k]
					N_HNResidue[num_H] = pose.coord[0][j]
			if lable == 1:
				num_noH += 1
				N_noHResidue[num_noH] = pose.coord[0][j]
						
		elif atomA[0] == "N": 
			lable = 1
			num += 1
			N_Residue[num] = pose.coord[0][j]
			for k in range(residueMapAtoms[pos][0],residueMapAtoms[pos][1]+1):
				atomB = pose.atom_info[0][k].atom_name
				if atomB[0] == "H" and atomB[1:len(atomA)] == atomA[1:]:
					lable = 0
					num_H += 1
					H_HNResidue[num_H] = pose.coord[0][k]
					N_HNResidue[num_H] = pose.coord[0][j]
			if lable == 1:
				num_noH += 1
				N_noHResidue[num_noH] = pose.coord[0][j]
	return H_HNResidue, N_HNResidue, N_Residue, N_noHResidue


def OResidue(pose, residueMapAtoms, pos):
	H_HOResidue = {}
	O_HOResidue = {}
	O_noHResidue = {}
	O_Residue = {}
	num_H = 0
	num_noH = 0
	num = 0
	for j in range(residueMapAtoms[pos][0],residueMapAtoms[pos][1]+1):
		atomA = pose.atom_info[0][j].atom_name
		if atomA == "O":
			num += 1
			O_Residue[num] = pose.coord[0][j]
			num_noH += 1
			O_noHResidue[num_noH] = pose.coord[0][j] 
		elif atomA[0] == "O": 
			lable = 1
			num += 1
			O_Residue[num] = pose.coord[0][j]
			for k in range(residueMapAtoms[pos][0],residueMapAtoms[pos][1]+1):
				atomB = pose.atom_info[0][k].atom_name
				if atomB[0] == "H" and atomB[1:len(atomA)] == atomA[1:]:
					lable = 0
					num_H += 1
					H_HOResidue[num_H] = pose.coord[0][k]
					O_HOResidue[num_H] = pose.coord[0][j]
			if lable == 1:
				num_noH += 1
				O_noHResidue[num_noH] = pose.coord[0][j]
			
	return H_HOResidue, O_HOResidue, O_Residue, O_noHResidue

def SearchHN_N(pose, residueMapAtoms):
	num_residues = len(pose.sequence[0])
	
	H_HNList = {}
	N_HNList = {}
	N_noHList = {}
	N_List = {}
	for i in range(1,num_residues+1):
		H_HNResidue, N_HNResidue, N_Residue, N_noHResidue= NResidue(pose, residueMapAtoms, i)
		if H_HNResidue:
			H_HNList[i] = H_HNResidue
			N_HNList[i] = N_HNResidue
		if N_noHResidue:
			N_noHList[i] = N_noHResidue
		if N_Residue:
			N_List[i] = N_Residue
	return H_HNList, N_HNList, N_List, N_noHList


def SearchHO_O(pose, residueMapAtoms):
	num_residues = len(pose.sequence[0])
	
	H_HOList = {}
	O_HOList = {}
	O_noHList = {}
	O_List = {}
	for i in range(1,num_residues+1):
		H_HOResidue, O_HOResidue, O_Residue, O_noHResidue = OResidue(pose, residueMapAtoms, i)
		if H_HOResidue:
			H_HOList[i] = H_HOResidue
			O_HOList[i] = O_HOResidue
		if O_noHResidue:
			O_noHList[i] = O_noHResidue
		if O_Residue:
			O_List[i] = O_Residue
	return H_HOList, O_HOList, O_List, O_noHList


def Hydrogen4Residue(pose, pos, residueMapAtoms, H_HNList, N_HNList, N_List, H_HOList, O_HOList, O_List):
	num_residue = len(pose.coord[0])
	
	#######################N-H: N as donor
	H_HNResidue, N_HNResidue, N_Residue, N_noHResidue = NResidue(pose, residueMapAtoms,pos)
	HN = {}
	
	#######################N-H...O
	if H_HNResidue:
		dis_O_H = {}
		for key_i in H_HNResidue:
			num_residue = 0
			dis_residue = {}
			
			for key_j in O_List:
				if key_j == pos:
					continue
				num_atom = 0
				dis_tmp = []
				for key_k in O_List[key_j]:
					dis = np.sqrt(np.sum((H_HNResidue[key_i]-O_List[key_j][key_k])**2))
					if dis < HBcutoffRadiue:
						num_atom += 1
						dis_tmp.append(key_k)
				
				if dis_tmp:
					dis_residue[key_j] = dis_tmp
			if dis_residue:
				dis_O_H[key_i] = dis_residue
			
		for key_i in dis_O_H:
			HNvec = N_HNResidue[key_i] - H_HNResidue[key_i]
			dis_HN = np.sqrt(np.sum(N_HNResidue[key_i]-H_HNResidue[key_i])**2)
			for key_j in dis_O_H[key_i]:
				for k in dis_O_H[key_i][key_j]:
					ONvec = N_HNResidue[key_i] - O_List[key_j][k]
					dis_ON = np.sqrt(np.sum(N_HNResidue[key_i]-O_List[key_j][k])**2)
					angle = np.arccos(np.dot(HNvec,ONvec)/(dis_HN*dis_ON))
					if angle < HBcutoffAngle:
						HN[key_i] = 1

	#######################N-H...N
	if H_HNResidue:
		dis_N_H = {}
		for key_i in H_HNResidue:
			num_residue = 0
			dis_residue = {}
			
			for key_j in N_List:
				if key_j == pos:
					continue
				num_atom = 0
				dis_tmp = []
				for key_k in N_List[key_j]:
					dis = np.sqrt(np.sum((H_HNResidue[key_i]-N_List[key_j][key_k])**2))
					if dis < HBcutoffRadiue:
						num_atom += 1
						dis_tmp.append(key_k)
				
				if dis_tmp:
					dis_residue[key_j] = dis_tmp
			if dis_residue:
				dis_N_H[key_i] = dis_residue
			
		for key_i in dis_N_H:
			HNvec = N_HNResidue[key_i] - H_HNResidue[key_i]
			dis_HN = np.sqrt(np.sum(N_HNResidue[key_i]-H_HNResidue[key_i])**2)
			for key_j in dis_N_H[key_i]:
				for k in dis_N_H[key_i][key_j]:
					NNvec = N_HNResidue[key_i] - N_List[key_j][k]
					dis_NN = np.sqrt(np.sum(N_HNResidue[key_i]-N_List[key_j][k])**2)
					angle = np.arccos(np.dot(HNvec,NNvec)/(dis_HN*dis_NN))
					if angle < HBcutoffAngle:
						HN[key_i] = 1
	#######################N as acceptor
	N = {}
	#######################N...HN
	if N_Residue:
		dis_N_H = {}
		for key_i in N_Residue:
			num_residue = 0
			dis_residue = {}
			
			for key_j in H_HNList:
				if key_j == pos:
					continue
				num_atom = 0
				dis_tmp = []
				for key_k in H_HNList[key_j]:
					dis = np.sqrt(np.sum((N_Residue[key_i]-H_HNList[key_j][key_k])**2))
					if dis < HBcutoffRadiue:
						num_atom += 1
						dis_tmp.append(key_k)
				
				if dis_tmp:
					dis_residue[key_j] = dis_tmp
			if dis_residue:
				dis_N_H[key_i] = dis_residue
			
		for key_i in dis_N_H:
			for key_j in dis_N_H[key_i]:
				for k in dis_N_H[key_i][key_j]:
					HNvec = N_HNList[key_j][k] - H_HNList[key_j][k]
					dis_HN = np.sqrt(np.sum(N_HNList[key_j][k]-H_HNList[key_j][k])**2)

					NNvec = N_HNList[key_j][k] - N_Residue[key_i]
					dis_NN = np.sqrt(np.sum(N_HNList[key_j][k]-N_Residue[key_i])**2)
					angle = np.arccos(np.dot(HNvec,NNvec)/(dis_HN*dis_NN))
					if angle < HBcutoffAngle:
						N[key_i] = 1
	
	#######################N...HO
	if N_Residue:
		dis_N_H = {}
		for key_i in N_Residue:
			num_residue = 0
			dis_residue = {}
			
			for key_j in H_HOList:
				if key_j == pos:
					continue
				num_atom = 0
				dis_tmp = []
				for key_k in H_HOList[key_j]:
					dis = np.sqrt(np.sum((N_Residue[key_i]-H_HOList[key_j][key_k])**2))
					if dis < HBcutoffRadiue:
						num_atom += 1
						dis_tmp.append(key_k)
				
				if dis_tmp:
					dis_residue[key_j] = dis_tmp
			if dis_residue:
				dis_N_H[key_i] = dis_residue
			
		for key_i in dis_N_H:
			for key_j in dis_N_H[key_i]:
				for k in dis_N_H[key_i][key_j]:
					HOvec = O_HOList[key_j][k] - H_HOList[key_j][k]
					dis_HO = np.sqrt(np.sum(O_HOList[key_j][k]-H_HOList[key_j][k])**2)

					NOvec = O_HOList[key_j][k] - N_Residue[key_i]
					dis_NO = np.sqrt(np.sum(O_HOList[key_j][k]-N_Residue[key_i])**2)
					angle = np.arccos(np.dot(HOvec,NOvec)/(dis_HO*dis_NO))
					if angle < HBcutoffAngle:
						N[key_i] = 1
	


	#######################O-H: O as donor
	H_HOResidue, O_HOResidue, O_Residue, O_noHResidue = OResidue(pose, residueMapAtoms,pos)
	HO = {}
	
	#######################O-H...O
	if H_HOResidue:
		dis_O_H = {}
		for key_i in H_HOResidue:
			num_residue = 0
			dis_residue = {}
			
			for key_j in O_List:
				if key_j == pos:
					continue
				num_atom = 0
				dis_tmp = []
				for key_k in O_List[key_j]:
					dis = np.sqrt(np.sum((H_HOResidue[key_i]-O_List[key_j][key_k])**2))
					if dis < HBcutoffRadiue:
						num_atom += 1
						dis_tmp.append(key_k)
				
				if dis_tmp:
					dis_residue[key_j] = dis_tmp
			if dis_residue:
				dis_O_H[key_i] = dis_residue
			
		for key_i in dis_O_H:
			HOvec = O_HOResidue[key_i] - H_HOResidue[key_i]
			dis_HO = np.sqrt(np.sum(O_HOResidue[key_i]-H_HOResidue[key_i])**2)
			for key_j in dis_O_H[key_i]:
				for k in dis_O_H[key_i][key_j]:
					ONvec = O_HOResidue[key_i] - O_List[key_j][k]
					dis_ON = np.sqrt(np.sum(O_HOResidue[key_i]-O_List[key_j][k])**2)
					angle = np.arccos(np.dot(HOvec,ONvec)/(dis_HO*dis_ON))
					if angle < HBcutoffAngle:
						HO[key_i] = 1

	#######################O-H...N
	if H_HOResidue:
		dis_N_H = {}
		for key_i in H_HOResidue:
			num_residue = 0
			dis_residue = {}
			
			for key_j in N_List:
				if key_j == pos:
					continue
				num_atom = 0
				dis_tmp = []
				for key_k in N_List[key_j]:
					dis = np.sqrt(np.sum((H_HOResidue[key_i]-N_List[key_j][key_k])**2))
					if dis < HBcutoffRadiue:
						num_atom += 1
						dis_tmp.append(key_k)
				
				if dis_tmp:
					dis_residue[key_j] = dis_tmp
			if dis_residue:
				dis_N_H[key_i] = dis_residue
			
		for key_i in dis_N_H:
			HOvec = O_HOResidue[key_i] - H_HOResidue[key_i]
			dis_HO = np.sqrt(np.sum(O_HOResidue[key_i]-H_HOResidue[key_i])**2)
			for key_j in dis_N_H[key_i]:
				for k in dis_N_H[key_i][key_j]:
					NOvec = O_HOResidue[key_i] - N_List[key_j][k]
					dis_NO = np.sqrt(np.sum(O_HOResidue[key_i]-N_List[key_j][k])**2)
					angle = np.arccos(np.dot(HOvec,NOvec)/(dis_HO*dis_NO))
					if angle < HBcutoffAngle:
						HO[key_i] = 1
		
	#######################O as acceptor
	O = {}
	#######################O...HN
	if O_Residue:
		dis_O_H = {}
		for key_i in O_Residue:
			num_residue = 0
			dis_residue = {}
			
			for key_j in H_HNList:
				if key_j == pos:
					continue
				num_atom = 0
				dis_tmp = []
				for key_k in H_HNList[key_j]:
					dis = np.sqrt(np.sum((O_Residue[key_i]-H_HNList[key_j][key_k])**2))
					if dis < HBcutoffRadiue:
						num_atom += 1
						dis_tmp.append(key_k)
				
				if dis_tmp:
					dis_residue[key_j] = dis_tmp
			if dis_residue:
				dis_O_H[key_i] = dis_residue
			
		for key_i in dis_O_H:
			for key_j in dis_O_H[key_i]:
				for k in dis_O_H[key_i][key_j]:
					HNvec = N_HNList[key_j][k] - H_HNList[key_j][k]
					dis_HN = np.sqrt(np.sum(N_HNList[key_j][k]-H_HNList[key_j][k])**2)

					ONvec = N_HNList[key_j][k] - O_Residue[key_i]
					dis_ON = np.sqrt(np.sum(N_HNList[key_j][k]-O_Residue[key_i])**2)
					angle = np.arccos(np.dot(HNvec,ONvec)/(dis_HN*dis_ON))
					if angle < HBcutoffAngle:
						O[key_i] = 1
	
	#######################O...HO
	if O_Residue:
		dis_O_H = {}
		for key_i in O_Residue:
			num_residue = 0
			dis_residue = {}
			
			for key_j in H_HOList:
				if key_j == pos:
					continue
				num_atom = 0
				dis_tmp = []
				for key_k in H_HOList[key_j]:
					dis = np.sqrt(np.sum((O_Residue[key_i]-H_HOList[key_j][key_k])**2))
					if dis < HBcutoffRadiue:
						num_atom += 1
						dis_tmp.append(key_k)
				
				if dis_tmp:
					dis_residue[key_j] = dis_tmp
			if dis_residue:
				dis_O_H[key_i] = dis_residue
			
		for key_i in dis_O_H:
			for key_j in dis_O_H[key_i]:
				for k in dis_O_H[key_i][key_j]:
					HOvec = O_HOList[key_j][k] - H_HOList[key_j][k]
					dis_HO = np.sqrt(np.sum(O_HOList[key_j][k]-H_HOList[key_j][k])**2)

					OOvec = O_HOList[key_j][k] - O_Residue[key_i]
					dis_OO = np.sqrt(np.sum(O_HOList[key_j][k]-O_Residue[key_i])**2)
					angle = np.arccos(np.dot(HOvec,OOvec)/(dis_HO*dis_OO))
					if angle < HBcutoffAngle:
						O[key_i] = 1

	acceptor = 0
	donor = 0
	for i in N:
		acceptor += 1	
	for i in O:
		acceptor += 1
	for i in HN:
		donor += 1
	for i in HO:
		donor += 1
	
	return acceptor, donor

def Charge(residue):
	charge = 100
	if residue == "R" or residue == "K":
		charge = 1
	elif residue == "D" or residue == "E":
		charge = -1
	else:
		charge = 0			
	return charge

"""
def Hydrophobic(residue):
	hydrophobic = 100
	if residue == "G" or residue == "A" or residue == "L" or residue == "I" or residue == "V" or residue == "P" or residue == "F" or residue == "M" or residue == "W":
		hydrophobic = 1
	else:
		hydrophobic = 0
	return hydrophobic
"""
def Hydrophobic(residue):
	hydrophobic = 100
	if residue == "C" or residue == "L" or residue == "I" or residue == "V" or residue == "F" or residue == "M" or residue == "W":
		hydrophobic = 1
	else:
		hydrophobic = 0
	return hydrophobic

"""
def Pair_polar(residue1, residue2):
	hydrophobic1 = 100
	hydrophobic2 = 100
	classes = 100

	if residue1 == "G" or residue1 == "A" or residue1 == "L" or residue1 == "I" or residue1 == "V" or residue1 == "P" or residue1 == "F" or residue1 == "M" or residue1 == "W":
		hydrophobic1 = 1
	elif residue1 == "S" or residue1 == "Q" or residue1 == "T" or residue1 == "C" or residue1 == "N" or residue1 == "Y":
		hydrophobic1 = -1
	else:
		hydrophobic1 = 0

	if residue2 == "G" or residue2 == "A" or residue2 == "L" or residue2 == "I" or residue2 == "V" or residue2 == "P" or residue2 == "F" or residue2 == "M" or residue2 == "W":
		hydrophobic2 = 1
	elif residue2 == "S" or residue2 == "Q" or residue2 == "T" or residue2 == "C" or residue2 == "N" or residue2 == "Y":
		hydrophobic2 = -1
	else:
		hydrophobic2 = 0 
 
	if hydrophobic1 == 1 and hydrophobic2 ==1:
		classes = 1
	elif hydrophobic1 == 1 and hydrophobic2 == -1:
		classes = 2
	elif hydrophobic1 == -1 and hydrophobic2 == 1:
		classes = 2
	elif hydrophobic1 == -1 and hydrophobic2 == -1:
		classes = 3
	else:
		classes = 0
	return classes
"""
def Pair_polar(residue1, residue2):
	hydrophobic1 = 100
	hydrophobic2 = 100
	classes = 100

	if residue1 == "G" or residue1 == "A" or residue1 == "L" or residue1 == "I" or residue1 == "V" or residue1 == "P" or residue1 == "F" or residue1 == "M" or residue1 == "W":
		hydrophobic1 = 1
	else:
		hydrophobic1 = 0

	if residue2 == "G" or residue2 == "A" or residue2 == "L" or residue2 == "I" or residue2 == "V" or residue2 == "P" or residue2 == "F" or residue2 == "M" or residue2 == "W":
		hydrophobic2 = 1
	else:
		hydrophobic2 = 0 
 
	if hydrophobic1 == 1 and hydrophobic2 ==1:
		classes = 1
	elif hydrophobic1 == 1 and hydrophobic2 == 0:
		classes = 2
	elif hydrophobic1 == 0 and hydrophobic2 == 1:
		classes = 2
	elif hydrophobic1 == 0 and hydrophobic2 == 0:
		classes = 3
	return classes

def Aromatic(residue):
	aromatic = 100
	if residue == "W" or residue == "Y" or residue == "F" or residue == "H":
		aromatic = 1
	else:
		aromatic = 0
	return aromatic	


def Pharmacophore(pdbdir, mpos, output):
	pose = PDBchains()
	pose.Read(pdbdir, allow_alt_loc = False, allow_insert = False)

	cutoff = ReadConcut(concut)
	sideChainList = SCList(pose)
	sideChainCenter = SCcenter(pose, sideChainList)
	sideChainContacts = SCcontacts(pose, sideChainCenter, mpos, cutoff)

	residueMapAtoms = RMapA(pose)
	H_HNList, N_HNList, N_List, N_noHList = SearchHN_N(pose, residueMapAtoms)
	H_HOList, O_HOList, O_List, O_noHList = SearchHO_O(pose, residueMapAtoms)
	
	acceptor = []
	donor = []
	ac, do = Hydrogen4Residue(pose, mpos, residueMapAtoms, H_HNList, N_HNList, N_List, H_HOList, O_HOList, O_List)
	acceptor.append(ac)
	donor.append(do)
	for i in sideChainContacts:
		ac, do = Hydrogen4Residue(pose, i, residueMapAtoms, H_HNList, N_HNList, N_List, H_HOList, O_HOList, O_List)
		acceptor.append(ac)
		donor.append(do)

	sequence = pose.sequence[0]
	charge = []
	hydrophobic = []
	pair_polar = []
	aromatic = []
	charge.append(Charge(sequence[mpos-1]))
	hydrophobic.append(Hydrophobic(sequence[mpos-1]))
	pair_polar.append(0)
	aromatic.append(Aromatic(sequence[mpos-1]))
	#print sequence[mpos-1]+" "+str(mpos)
	for i in sideChainContacts:
		#print sequence[i-1]+" "+str(i)
		charge.append(Charge(sequence[i-1]))
		hydrophobic.append(Hydrophobic(sequence[i-1]))
		pair_polar.append(Pair_polar(sequence[mpos-1], sequence[i-1]))
		aromatic.append(Aromatic(sequence[i-1]))

	f_out = open(output,"w")
	
	for i in range(0,1+len(sideChainContacts)):
		line = [] 
		if hydrophobic[i] == 1:
			line.append(1)
			line.append(0)
		else:
			line.append(0)
			line.append(1)
		if charge[i] == 1:
			line.append(1)
			line.append(0)
			line.append(0)
		elif charge[i] == -1:
			line.append(0)
			line.append(1)
			line.append(0)
		else:
			line.append(0)
			line.append(0)
			line.append(1)
		if pair_polar[i] == 1:
			line.append(1)
			line.append(0)
			line.append(0)
		elif pair_polar[i] == 2:
			line.append(0)
			line.append(1)
			line.append(0)
		elif pair_polar[i] == 3:
			line.append(0)
			line.append(0)
			line.append(1)
		else:
			line.append(0)
			line.append(0)
			line.append(0)
		if aromatic[i] == 1:
			line.append(1)
			line.append(0)
		else:
			line.append(0)
			line.append(1)
		line.append(acceptor[i])
		line.append(donor[i])
		f_out.write('\t'.join(map(str,line)))
		f_out.write('\n')
	f_out.close()

def run_pharmacophore(model,mpos,outdir,outName):
	outdir = preparation.check_slash(outdir)	
	primaryDir = os.getcwd()
	os.chdir(outdir)

	mpos = int(mpos)	
	Pharmacophore(model, mpos, outdir+outName) 
	os.chdir(primaryDir)
	return	outdir+outName
