#!/usr/bin/env python
import re,sys,os
import preparation, userConfig

minSeqLen = 17


def prepare_clust(query, blastfile,outFile):
	outFile = preparation.check_slash(outFile)
	fin = open(blastfile,'r')	
	data = fin.read()
	fin.close()
	hits = data.split('\n>')
	count = 1
	seq = {}
	
	match = re.search('Query=[^\n]+\n\nLength=(\d+)',hits[0])
	if not match:
		match = re.search('(\d+)\s+letters',hits[0])
		if not match:
			print 'ERROR: syntax error in '+blastfile
			sys.exit()
	seqLength = int(match.group(1))

	if len(query) != seqLength:
		print 'ERROR: sequence length in blast '+str(seqLength)+' is not equal the query length '+str(len(query))+'.'	
		sys.exit()
	
	preClustFile = outFile+'seq.preclust'
	preClust = open(preClustFile,'w')
	preClust.write('>query\n'+query+'\n')
	large = 0
	for i in range(1,len(hits)):
		hit = hits[i]
		match = re.search('\n Identities = \d+\/(\d+) \((\d+)%',hit)
		if not match:
			print 'ERROR: no Identities in '+blastfile
			sys.exit()
		length = int(match.group(1))
		identity = int(match.group(2))
		if length < minSeqLen:
			continue
		if identity > 94 or identity < 30:
			large += 1 
			continue
		sub = re.split('Positives.+?\n',hit)
		sbjects = re.findall('Sbjct:?\s+\d+\s+([A-Z-]+)\s+\d+\s*?\n',sub[1])
		for sbject in sbjects:
			if seq.has_key(count):
				seq[count] += sbject	
			else:
				seq[count] = sbject
		if not seq[count]:
			print 'ERROR: no Sbject line in '+sub[0]
			sys.exit()
		seq[count] = re.sub('\-','',seq[count])
		preClust.write('>name'+str(count)+'\n'+seq[count]+'\n')
		count += 1
		if count > 300:
			break
	
	total = len(seq.keys())
	if total != (count-1):
		print 'ERROR: total '+str(total)+' is not equal to count '+(count-1)
		sys.exit()
	preClust.close()
	if total == 0:
		print 'ERROR No alignments exist for '+preClustFile
		sys.exit()
	return preClustFile

def run_clustalOmega(preClustFile,outFile):
	outFile = preparation.check_slash(outFile)
	if not os.path.isfile(preClustFile):
		print 'ERROR: '+preClustFile+' does not exist!'
		sys.exit()
	cmd = userConfig.clustalO+' -i '+preClustFile	
	#print cmd
	msa = preparation.run_cmd(cmd)	
	if msa == '':
		print 'ERROR: psic does not work.'
		sys.exit()
	n_seq = 0
	name = ''
	seq = {}
	lines = msa.strip().split('\n')
	for line in lines:
		match = re.match('^>(\S+)',line)
		if match:
			name = match.group(1)
			if seq.has_key(name):
				print 'ERROR: clustal Omega genegrated a non-standard allignment file.'
				sys.exit()
			seq[name] = ''
			n_seq+=1
		else:
			if line.strip() == '':
				print 'ERROR: unexpected line from clustal Omega.'
				sys.exit()
			seq[name] += line.strip()
			
	psicInFile = outFile + 'psicIn'
	fin = open(psicInFile,'w')
	fin.write('CLUSTAL\n\n')
	for k, v in seq.iteritems():
		fin.write('_'*(67-len(k))+k+' '+v+'\n')
	fin.close()
	return psicInFile, seq		

def get_query_psic(psicOutFile, seq, outFile):
	outFile = preparation.check_slash(outFile)
	if not os.path.isfile(psicOutFile):
		print 'ERROR: '+psicOutFile+' does not exist!'
		sys.exit()

	fin = open(psicOutFile,'r')
	lines = fin.read().strip().split('\n')
	fin.close()

	queryPsicFile = outFile+'seq.psic'	
	fout = open(queryPsicFile,'w')
	count = 1
	if not seq.has_key('query') or seq['query']=='':
		print 'ERROR: can not fine auery line in the input file of clustal Omega'
		sys.exit()

	query = [i for i in seq['query']]
	lable = -2
	for line in lines:
		if lable == -2:
			lable += 1
			continue
		elif lable == -1:
			fout.write(line+'\n') 
			lable += 1
			continue
		elif re.match('[A-Z]',query[lable]):
			line = re.sub('^\d+\s+','',line) 
			fout.write(str(count)+' '+line+'\n')
			lable += 1
			count += 1
		else:
			lable += 1
	fout.close()
	return queryPsicFile, count-1

def run_psic(query, blastfile, outFile):
	outFile = preparation.check_slash(outFile)
	size = 0
	cmd = 'du -m '+blastfile+' 2>/dev/null';
	print cmd
	size = preparation.run_cmd(cmd)	
	size = re.match('^(\d+)',size).group(1)
	print size

	preClustFile = prepare_clust(query, blastfile, outFile)	
	psicInFile, seq = run_clustalOmega(preClustFile,outFile)	
	if not os.path.isfile(psicInFile):
		print 'ERROR: '+psicInFile+' does not exist!'
		sys.exit()
	psicOutFile = outFile+'psicOut'
	cmd = userConfig.newPsic+' '+psicInFile+' '+userConfig.psicMatrix+' '+psicOutFile
	print cmd
	preparation.run_cmd(cmd)	
	queryPsicFile, count = get_query_psic(psicOutFile,seq,outFile)
	if count != len(query):
		print 'ERROR: query psic length $count is not equal with the sequence length.' 
		sys.exit()
	return queryPsicFile

def run_template_psic(psicInFile, outFile):
	outFile = preparation.check_slash(outFile)
	psicOutFile = outFile+'template.psic'
	cmd = userConfig.newPsic+' '+psicInFile+' '+userConfig.psicMatrix+' '+psicOutFile
	print cmd
	preparation.run_cmd(cmd)	
	return psicOutFile
