#!/usr/bin/env python
import sys,re,math,os
import userConfig,preparation

class atom:
	def __init__(self):
		self.atomIndex = None
		self.atomName = None
		self.resName = None
		self.resIndex = None
		self.chain = None
		self.coord = [None]*3
		self.isMainchain = 0
		

	def read_atom(self,atomLine):
		mainChain = ['N','CA','C','O']
		if len(atomLine) >= 54:
			self.atomIndex = int(atomLine[6:11])
			self.atomName = atomLine[12:16].strip()
			self.resName = atomLine[17:20].strip()
			self.resIndex = int(atomLine[22:26])
			if atomLine[21] == ' ':
				self.chain = '_'
			else:
				self.chain = atomLine[21]
			self.coord[0] = float(atomLine[30:38])
			self.coord[1] = float(atomLine[38:46])
			self.coord[2] = float(atomLine[46:54])
			if self.atomName in mainChain:
				self.isMainchain = 1 
			return True
		else:
			return False
	
class residue:

	def __init__(self):
		self.resIndex = None
		self.resName = None
		self.chain = None
		self.numAtom = 0
		self.atoms = []
		self.het = 0 
		self.ter = 0

	'''
	def __init__(self,resIndex, resName, chain, numAtom, atoms, het, ter):
		self.resIndex = resIndex
		self.resName = resName
		self.chain = chain
		self.numAtom = numAtom
		self.atoms = atoms
		self.het = het 
		self.ter = ter
	'''
class pdb_struct:
	def __init__(self):
		self.skipH = 1
		self.skipOxt = 1
		self.numHet = 0
		self.numRes = 0
		self.struct = []
	
	def read_pdb(self,pdbfile):
		fin = open(pdbfile,'r')
		lines = fin.read().strip().split('\n')	
		r = residue()
		tempNumRes = 0
		tempNumAtom = 0
		tempNumHet = 0
		for line in lines:
			if not (re.search('^ATOM',line)):
				if not (re.search('^HETATM',line)):
					if re.search('^TER',line):
						r.ter = 1
					elif re.search('^ENDMDL',line):
						break
					continue
				elif re.search('HOH',line) or re.search('DIS',line):
					continue
			a = atom()
			if not a.read_atom(line):
				print 'ERROR: can\'t parse '+line
				continue

			if self.skipH and a.atomName[0] == 'H':
				continue
			if self.skipOxt and a.atomName == 'Oxt':
				continue
			
			if r.resIndex == None:
				r.resIndex = a.resIndex
				r.chain = a.chain
			if (tempNumAtom > 0 and a.resIndex != r.resIndex) or (a.chain != r.chain):
				r.numAtom = tempNumAtom
				tempNumRes += 1 
				self.struct.append(r)	
				if r.het == 1:
					tempNumHet += 1
				r = residue()
				tempNumAtom = 0
			
			tempNumAtom += 1
			r.atoms.append(a)
			r.resIndex = a.resIndex
			r.resName = a.resName
			r.chain =  a.chain
			if re.search('HETATM',line):
				r.het = 1
			
		if tempNumAtom > 0:
			r.numAtom = tempNumAtom
			tempNumRes += 1 
			#print r.resIndex,r.resName,r.chain, r.numAtom ,r.het,r.ter
			self.struct.append(r)	
			if r.het == 1:
				tempNumHet += 1
	
		self.numHet = tempNumHet
		self.numRes = tempNumRes

def count_contacts(residue1,residue2,cutoff):
	n1 = residue1.numAtom
	n2 = residue2.numAtom
	mindis = 1000.0
	minAtom1 = None
	minAtom2 = None
	if n1 <1:
		print 'Error, '+residue1.chain+' '+residue1.resIndex+' '+ residue1.resName+' no side-chain.'
		sys.exit()
	if n2 <1:
		print 'Error, '+residue2.chain+' '+residue2.resIndex+' '+ residue2.resName+' no side-chain.'
		sys.exit()

	for i1 in range(n1):
		x1 = residue1.atoms[i1].coord[0]  
		y1 = residue1.atoms[i1].coord[1]  
		z1 = residue1.atoms[i1].coord[2]  

		for i2 in range(n2):
			x2 = residue2.atoms[i2].coord[0]  
			y2 = residue2.atoms[i2].coord[1]  
			z2 = residue2.atoms[i2].coord[2]  

			dis = math.sqrt((x1-x2)**2+(y1-y2)**2+(z1-z2)**2)
			if dis < mindis:
				mindis = dis
				minAtom1 = i1
				minAtom2 = i2
	if mindis < cutoff:
		return True, minAtom1, minAtom2, mindis
	else:
		return False, minAtom1, minAtom2, mindis
	

def count_contacts_center(residue1,residue2,cutoff):
	n1 = residue1.numAtom
	n2 = residue2.numAtom
	x1=0.0
	y1=0.0
	z1=0.0
	x2=0.0
	y2=0.0
	z2=0.0
	CA1 = None
	CA2 = None
	num1 = 0
	num2 = 0
	for i1 in range(n1):
		if residue1.atoms[i1].isMainchain:
			if residue1.atoms[i1].atomName == 'CA':
				CA1 = i1
			continue
		num1 += 1
		x1 += residue1.atoms[i1].coord[0]  
		y1 += residue1.atoms[i1].coord[1]  
		z1 += residue1.atoms[i1].coord[2]  
	if num1 < 1 and CA1 != None:
		num1 += 1
		x1 += residue1.atoms[CA1].coord[0]  
		y1 += residue1.atoms[CA1].coord[1]  
		z1 += residue1.atoms[CA1].coord[2]  
	elif num1 < 1 and CA1 == None:
		print 'Error, '+residue1.resIndex+' '+ residue1.resName+' no side-chain and no CA.'
		sys.exit()

	for i2 in range(n2):
		if residue2.atoms[i2].isMainchain:
			if residue2.atoms[i2].atomName == 'CA':
				CA2 = i2
			continue
		num2 += 1
		x2 += residue2.atoms[i2].coord[0]  
		y2 += residue2.atoms[i2].coord[1]  
		z2 += residue2.atoms[i2].coord[2]  
	if num2 < 1 and CA2 != None:
		num2 += 1
		x2 += residue2.atoms[CA2].coord[0]  
		y2 += residue2.atoms[CA2].coord[1]  
		z2 += residue2.atoms[CA2].coord[2]  
	elif num2 < 1 and CA2 == None:
		print 'Error, '+residue2.resIndex+' '+ residue2.resName+' no side-chain and no CA.'
		sys.exit()

	x1 /= num1
	y1 /= num1
	z1 /= num1	
	x2 /= num2
	y2 /= num2
	z2 /= num2 	

	dis = math.sqrt((x1-x2)**2+(y1-y2)**2+(z1-z2)**2)
	if dis < cutoff:
		return True, dis
	else:
		return False, dis

def count_contacts(residue1,residue2,cutoff):
	n1 = residue1.numAtom
	n2 = residue2.numAtom
	mindis = 1000.0
	minAtom1 = None
	minAtom2 = None
	mainchain = 1
	if n1 <1:
		print 'Error, '+residue1.chain+' '+residue1.resIndex+' '+ residue1.resName+' no side-chain.'
		sys.exit()
	if n2 <1:
		print 'Error, '+residue2.chain+' '+residue2.resIndex+' '+ residue2.resName+' no side-chain.'
		sys.exit()

	for i1 in range(n1):
		x1 = residue1.atoms[i1].coord[0]  
		y1 = residue1.atoms[i1].coord[1]  
		z1 = residue1.atoms[i1].coord[2]  

		for i2 in range(n2):
			if not mainchain and residue1.atoms[i1].isMainchain and residue2.atoms[i2].isMainchain:
				continue
			x2 = residue2.atoms[i2].coord[0]  
			y2 = residue2.atoms[i2].coord[1]  
			z2 = residue2.atoms[i2].coord[2]  

			dis = math.sqrt((x1-x2)**2+(y1-y2)**2+(z1-z2)**2)
			if dis < mindis:
				mindis = dis
				minAtom1 = i1
				minAtom2 = i2
	if mindis < cutoff:
		return True, minAtom1, minAtom2, mindis
	else:
		return False, minAtom1, minAtom2, mindis
	

def concut_sc(concutFile):
	fin = open(concutFile,'r')	
	AA = fin.readline().strip().split()
	cutoff = {}
	for a1 in AA:
		cut = fin.readline().strip().split()
		i = 0
		for a2 in AA:
			i += 1
			cutoff[a1+'_'+a2] =float(cut[i])

	fin.readline()
	for a1 in AA:
		dev = fin.readline().strip().split()
		i = 0
		for a2 in AA:
			i += 1
			cutoff[a1+'_'+a2] += 2.5*float(dev[i]) 
	return cutoff

def struct_analysis(biounitFile, chain, resIndex, intra_thresh=10.0, inter_thresh=10.0, het_thresh=10.0):	
	pdb = pdb_struct()
	pdb.read_pdb(biounitFile)

	residue1 = None
	residue2 = None

	intra = {}
	inter = {}
	het = {}
	for i in range(pdb.numRes):
		residue1 = pdb.struct[i]
		if resIndex == int(residue1.resIndex) and chain == residue1.chain:
			break
		if i == (pdb.numRes-1):
			print 'ERROR: '+biounitFile+' does not have the residue index ('+str(resIndex)+').'
			return None, None, None
			#sys.exit()
	for i in range(pdb.numRes):
		residue2 = pdb.struct[i]
		if resIndex == int(residue2.resIndex) and chain == residue2.chain:
			continue
		if chain == residue2.chain and not residue2.het:
			#intra_thresh = 6.0
			#intra_thresh = 10.0
			
			lable, atom1, atom2,dis = count_contacts(residue1,residue2,intra_thresh)
			if lable == True:
				if intra.has_key(residue2.chain):
					intra[residue2.chain].append((residue2.resIndex, dis))
				else:
					intra[residue2.chain]=[(residue2.resIndex,dis)]
		elif chain != residue2.chain and not residue2.het:	
			#inter_thresh = 5.0
			#inter_thresh = 10.0
			lable, atom1,atom2,dis = count_contacts(residue1,residue2,inter_thresh)
			if lable == True:
				if inter.has_key(residue2.chain):
					inter[residue2.chain].append((residue2.resIndex, dis))
				else:
					inter[residue2.chain]=[(residue2.resIndex,dis)]
		elif residue2.het:	
			#het_thresh = 6.0
			#het_thresh = 10.0
			lable, atom1,atom2,dis = count_contacts(residue1,residue2,het_thresh)
			if lable == True:
				if het.has_key(residue2.chain):
					het[residue2.chain].append((residue2.resIndex, dis))
				else:
					het[residue2.chain]=[(residue2.resIndex,dis)]
	return intra, inter, het

def mutant_struct(aa1, pos, aa2, query, outDir, model):
	pos = int(pos)
	outName = str(pos)+aa2
	outDir = preparation.check_slash(outDir)	
	mutantSeq = ''
	for i in range(0,len(query)):
		if i == (pos-1):
			mutantSeq += aa2.upper()
			continue
		mutantSeq += query[i].upper()
	
	mutSeqFile = open(outDir+outName+'.seq','w')
	mutSeqFile.write(mutantSeq)
	mutSeqFile.flush()
	os.fsync(mutSeqFile)
	mutSeqFile.close()
	
	cmd = userConfig.scwrl+' -i ' +model+ ' -s '+outDir+outName+ '.seq -o '+outDir+outName+'.pdb'
        print cmd
	preparation.run_cmd(cmd)
	return outDir+outName+'.pdb'

	
	

if __name__ == "__main__":
	pdb = pdb_struct()
	'''
	pdb.read_pdb('/nfs/amino-home/liquan/CancerDriver/myprogram/database/pdb/biounit/5TTR.GH_GH.pdb3')
	chain = 'G'
	resIndex = 116
	'''
	pdb.read_pdb('/nfs/amino-home/liquan/CancerDriver/myprogram/database/pdb/biounit/1A3B.LHI.pdb1')
	chain = 'H'
	resIndex = 215

	residue1 = None
	residue2 = None

	for i in range(pdb.numRes):
		residue1 = pdb.struct[i]
		if resIndex == int(residue1.resIndex) and chain == residue1.chain:
			break
	for i in range(pdb.numRes):
		residue2 = pdb.struct[i]
		if resIndex == int(residue2.resIndex) and chain == residue2.chain:
			continue
		if chain == residue2.chain and not residue2.het:
			#intra_thresh = 6.0
			intra_thresh = 10.0
			
			lable, atom1, atom2,dis = count_contacts(residue1,residue2,intra_thresh)
			if lable == True:
				print 'Intramolecular:', residue1.chain, residue1.resIndex, residue1.resName, residue1.atoms[atom1].atomIndex,residue1.atoms[atom1].atomName,'->',residue2.chain, residue2.resIndex, residue2.resName,residue2.atoms[atom2].atomIndex,residue2.atoms[atom2].atomName,'=',dis 
		elif chain != residue2.chain and not residue2.het:	
			#inter_thresh = 5.0
			inter_thresh = 10.0
			lable, atom1,atom2,dis = count_contacts(residue1,residue2,inter_thresh)
			if lable == True:
				print 'Intermolecular:', residue1.chain, residue1.resIndex, residue1.resName,residue1.atoms[atom1].atomIndex,residue1.atoms[atom1].atomName,'->',residue2.chain, residue2.resIndex, residue2.resName,residue2.atoms[atom2].atomIndex,residue2.atoms[atom2].atomName,'=',dis 
		elif residue2.het:	
			#het_thresh = 6.0
			het_thresh = 10.0
			lable, atom1,atom2,dis = count_contacts(residue1,residue2,het_thresh)
			if lable == True:
				print 'Het:', residue1.chain, residue1.resIndex, residue1.resName,residue1.atoms[atom1].atomIndex,residue1.atoms[atom1].atomName,'->',residue2.chain, residue2.resIndex, residue2.resName,residue2.atoms[atom2].atomIndex,residue2.atoms[atom2].atomName,'=',dis 


	"""
	cutoff=concut_sc('/nfs/amino-library/common/contact.comm')
	for i in range(pdb.numRes):
		residue1 = pdb.struct[i]
		if resIndex == int(residue1.resIndex) and chain == residue1.chain:
			break
	for i in range(pdb.numRes):
		residue2 = pdb.struct[i]
		if resIndex == int(residue2.resIndex) and chain == residue2.chain:
			continue
		if chain == residue2.chain:
			thresh = cutoff[residue1.resName+'_'+residue2.resName]
			lable, dis = count_contacts_center(residue1,residue2,thresh)
			if lable == True:
				print 'intramolecular',residue1.chain, residue1.resIndex, residue1.resName,'->',residue2.chain, residue2.resIndex, residue2.resName,':',dis 
		if chain != residue2.chain:	
			thresh = cutoff[residue1.resName+'_'+residue2.resName]
			lable, dis = count_contacts_center(residue1,residue2,thresh)
			if lable == True:
				print 'intermolecular',residue1.chain, residue1.resIndex, residue1.resName,'->',residue2.chain, residue2.resIndex, residue2.resName,':',dis 


	print pdb.numHet
	print pdb.numRes 
	for i in range(pdb.numRes):
		print "********************"
		print pdb.struct[i].resIndex,pdb.struct[i].resName,pdb.struct[i].chain,pdb.struct[i].numAtom,pdb.struct[i].het,pdb.struct[i].ter
		for j in range(pdb.struct[i].numAtom):
			print pdb.struct[i].atoms[j].atomIndex,pdb.struct[i].atoms[j].atomName,pdb.struct[i].atoms[j].resName,pdb.struct[i].atoms[j].resIndex,pdb.struct[i].atoms[j].chain,pdb.struct[i].atoms[j].coord[0],pdb.struct[i].atoms[j].coord[1],pdb.struct[i].atoms[j].coord[2],pdb.struct[i].atoms[j].isMainchain

	"""
 
