#!/usr/bin/perl
use Math::Trig;
use Cwd 'abs_path';

#########################################################################setpath
$usrname=getlogin() || $ENV{'USER'};               #useful when making tmp dir.;
$dampred = "/home/jian.wu/shared/workDirQlj/DAMpred/DAMpred";
$bindir = "$dampred/bin";
$libdir = "$dampred/lib";

$i_tasser = "$bindir/I-TASSER5.1";
$java_home = "/usr";             #java bin will be located at $java_home/bin/java

$inputdir   = "/home/$usrname/data/1a2bA"; #at least has 1a2bA.fasta
$runstyle  = "serial";                       #parallel or serial
#########################################################################setpath

$nres_cut1     = 10;
$nres_cut2     = 1500;
$seq   = "";

$system    = $^O;
unless (lc($system) eq "linux")
{
    printf("Your operating system $system is unsupported at this time\n");
    printf("Currently only Linux is supported\n");
    exit();
}



################parse arguments
$totarg=0;
foreach $w(@ARGV)
{
	$totarg++;
}

$flagmand[0]=0;
$flagmand[1]=0;
$flagmand[2]=0;
for($i=0;$i<$totarg-1;$i++)
{
	$str1=substr($ARGV[$i+1],0,1);
	if($str1 eq "-")
	{
		next;
	}
	if($ARGV[$i] eq "-inputdir")
    	{	
		$inputdir=$ARGV[$i+1];
		$flagmand[0]=1;
    	}
	elsif($ARGV[$i] eq "-seq")
	{
		$seq = $ARGV[$i+1];
		$flagmand[1]=1;	
	}
	elsif($ARGV[$i] eq "-mutation")
	{
		$mutationfile = $ARGV[$i+1];
		$flagmand[2]=1;
	}
	elsif($ARGV[$i] eq "-runstyle")
    {
		$runstyle=$ARGV[$i+1];
    }
}


if($flagmand[0]==0)
{
    &print_help();
    print "please set -inputdir\n\n";    
    exit();
}
if($flagmand[1]==0)
{
    &print_help();
    print "please set -seq\n\n";    
    exit();
}
if($flagmand[2]==0)
{
    &print_help();
    print "please set -mutation\n\n";    
    exit();
}

print "$inputdir/$mutationfile";
open (FM1, "$inputdir/$mutationfile") or die "YOU DIDNT OPEN $mutationfile";
@mutations=<FM1>;
close(FM1);
foreach my $mutation(@mutations){
	chomp($mutation);
	if($mutation =~/^([A-Za-z])([1-9]\d*)([A-Za-z]$)/)
	{
		$wild=$1;
		$position=$2;
		$mutant=$3;
	}
	else
	{
		&print_help();
		print "please enter right mutation: Wild Position Mutation.\n eg: -mutation A32R\n\n";
		exit();		
	}

	$key = "$wild\_$position\_$mutant";
	$mutation = $key;
}




printf "\nYour setting for running I-TASSER is:\n";
printf "-libdir    = $libdir\n";
printf "-inputdir   = $inputdir\n";
printf "-runstyle  = $runstyle\n";
printf "-seq     = $seq\n";
printf "-mutation    = $mutationfile\n";

%ISAA=(
		'A'=>'1',
		'H'=>'1',
		'T'=>'1',
		'R'=>'1',
		'I'=>'1',
		'W'=>'1',
		'N'=>'1',
		'L'=>'1',
		'Y'=>'1',
		'D'=>'1',
		'K'=>'1',
		'V'=>'1',
		'C'=>'1',
		'M'=>'1',
		'Q'=>'1',
		'F'=>'1',
		'E'=>'1',
		'P'=>'1',
		'G'=>'1',
		'S'=>'1',
		);



########################check the input########################
$infile = "$inputdir/$seq";
@array = split(/\./,$seq);
$seqname= @array[0];
$outputdir = "$inputdir/$seqname";
if(!(-e $outputdir))
{
	mkdir $outputdir;
}

@PDBfile  = ();
%LCH = ();
$LCH{1} = 0;
$Lmessage = '';
$message = '';


if(not -e $infile)
{
	print "\nError: $infile is not exist\n\n";
	exit();
}
@lines=();
open(FIN,"<$infile");
while(my $line=<FIN>)
{
	push(@lines, $line);
}
close(FIN);


$n_ch1=0;
$n_ch++;
$nnn=@lines;
for(my $i=0; $i<@lines; $i++)
{
	my $line=$lines[$i];
	if($line=~/^>/)
	{	
		$n_ch1++;	    
		if($n_ch1>1)
		{
			last;
		}	  
		push(@PDBfile, $line);	    
	}	
	else
	{
		$line =~ s/\s+//g;
		my @seqline=split(//, $line);
		$line="";
		foreach my $ch (@seqline)
		{
			if(exists $ISAA{$ch})
			{
				$line .= $ch;
				$sequence .= $ch;
			}
			else
			{
				$Lmessage .="LOG: $ch is not a standard amino acid. It was removed automatically.\n";
			}
		}
		if(length $line >0)	    
		{	
			$LCH{$n_ch} += length $line;
			push(@PDBfile, $line);
		}
	}	
}	

if($n_ch1>1)
{
	$n_ch=$n_ch1;
}	
if($n_ch > 1)
{  
	$Lmessage .='LOG:';
	$Lmessage .="Submitted protein contained multiple chains. Currently, STRUM cannot be used for oligomeric proteins, hence it will use only the first protein chain from the provided file\n\n";   	
}
if($LCH{1} < $nres_cut1)
{	
	$message .='Error: ';	
	$message .="The protein contains less than $nres_cut1 (Lch=$LCH{1}) residues and is too short for obtaining any meaningful prediction using this server.\n\n";	
} 
if($LCH{1} > $nres_cut2)
{
	$message .='Error: ';	$message .="The protein is too large for our server (Lch=$LCH{1}),  maximum length of protein accepted by the server is $nres_cut2. You may want to split the protein into domains and submit individual domains.\n\n";
} 
$protein_length=$LCH{1};

foreach my $mutation(@mutations){
	@cells = split(/_/,$mutation);
	$wild = $cells[0];
	$position = $cells[1];	
	$mutant = $cells[2];	
	if($position>$protein_length)
	{
		$message .='Error: ';	
		$message .="The position $position in $mutation is greater than length of the input protein.\n\n";
	}

	if($wild ne substr($sequence,$position-1,1))
	{
		my $res_index=substr($sequence,$position-1,1);
		$message .='Error: ';
		$message .="Amino acid $wild in $mutation doesn't correspond to the $position th residue $res_index of the input protein.\n\n";
	}
	if($wild eq $mutant)
	{
		$message .='Error: ';
		$message .="mutant amino acid $mutant is equal to wild-type amino acid.\n\n";
	}
}

if($Lmessage ne '')
{
	print "\n\n$Lmessage";
}
if($message ne '')
{
	print "\n\n$message";
	exit(-1);
}

open(OUTPDB,">$inputdir/protein.fasta");
foreach my $line(@PDBfile)
{
	chomp($line);
	print OUTPDB "$line\n";	       
}
close(OUTPDB);
print "\n\nNote: The pudated sequence is $inputdir/protein.fasta\n";     
########################run DAMpred########################
$nanosecond=`date +\%N`;
chomp($nanosecond);
$unicode="$seqname$nanosecond";            #useful when making tmp dir and setting unique jobname

$tmp = "$inputdir/tmp";
`mkdir -p $tmp`;


##################################################
if(!-s "$inputdir/protein.fasta")
{
	die "No structure or sequence information is provided!\n";
}    

my $length=0;
my @ttt=`cat $inputdir/protein.fasta`;
chomp(@ttt);
foreach my $t(@ttt)
{
	next if($t =~ /^>/);
	$t =~ s/\s+//g;
	$length += length $t;
}

    
my $mutations = join('-',@mutations);


########################################sequence based features
print "sequence-based features is running......\n";  	
my $seqOutput="$outputdir/seqFeature";
`mkdir -p $seqOutput`;
$seqtag   = "DAM\_SEQ\_$seqname";
$mod      =`cat $bindir/DAMpred/RunSeqMod`;
$mod =~ s/\!SEQ\!/$inputdir\/protein.fasta/mg;
$mod =~ s/\!MUTATION\!/$mutations/mg;   
$mod =~ s/\!OUTDIR\!/$tmp\/seqFeature/mg;
open(QSF1,">$seqOutput/$seqtag.py");
print QSF1 "$mod";
close(QSF1);
`$seqOutput/$seqtag.py >>$inputdir/running.log 2>&1`;

########################################run I-TASSER

print "I-TASSER is running......\n";  	
if(-s "$itasserOutput/model1.pdb")
{
	print "I-TASSER has been completed!";
}
else
{
	$itasserOutput="$tmp/$seqname";
	`mkdir -p $itasserOutput`;
	`cp $inputdir/protein.fasta $itasserOutput/seq.fasta`;
	`$i_tasser/I-TASSERmod/runI-TASSER.pl -pkgdir $i_tasser -libdir $libdir -java_home $java_home -seqname $unicode -datadir $itasserOutput -runstyle $runstyle >> $inputdir/running.log 2>&1`; 	
}


if(-s "$itasserOutput/model1.pdb")
{
	`cp $itasserOutput/model1.pdb $inputdir/itasser.pdb`;
}
else
{
	print "Structure was not predicted by I-TASSER......\n";
}

########################################template based features
print "template-based features is running......\n";  	
$templateOutput="$outputdir/templateFeature";
`mkdir -p $templateOutput`;
$templatetag   = "DAM\_TEMPLATE\_$seqname";
$mod      =`cat $bindir/DAMpred/RunTemplateMod`;
$mod =~ s/\!SEQ\!/$inputdir\/protein.fasta/mg;
$mod =~ s/\!MUTATION\!/$mutations/mg;   
$mod =~ s/\!TEMPLATE\!/$itasserOutput\/init.dat/mg;   
$mod =~ s/\!OUTDIR\!/$tmp\/templateFeature/mg;
open(QSF1,">$templateOutput/$templatetag.py");
print QSF1 "$mod";
close(QSF1);
`$templateOutput/$templatetag.py >> $inputdir/running.log 2>&1`;


########################################itasser based features
print "itasser-based features is running......\n";  	
$itasserOutput="$outputdir/itasserFeature";
`mkdir -p $itasserOutput`;
$itassertag   = "DAM\_ITASSER\_$seqname";
$mod      =`cat $bindir/DAMpred/RunItasserMod`;
$mod =~ s/\!SEQ\!/$inputdir\/protein.fasta/mg;
$mod =~ s/\!MUTATION\!/$mutations/mg;   
$mod =~ s/\!MODEL\!/$inputdir\/itasser.pdb/mg;   
$mod  =~ s/\!PSICOUT\!/$tmp\/seqFeature\/psic/mg;
$mod =~ s/\!OUTDIR\!/$tmp\/itasserFeature/mg;
open(QSF1,">$itasserOutput/$itassertag.py");
print QSF1 "$mod";
close(QSF1);
`$itasserOutput/$itassertag.py >> $inputdir/running.log 2>&1`;

#-------------------Extrcting features and predicting------------------->
#### step-9: extract features
`$bindir/DAMpred/extractFeatures.py $mutations $tmp >> $inputdir/running.log 2>&1`;

`$bindir/DAMpred/normalizeFeature.py $tmp/features $bindir/DAMpred/MinMax.txt $tmp/normFeatures >> $inputdir/running.log 2>&1`;

`mkdir -p $tmp/probability`;
`$bindir/DAMpred/getProbability.py $bindir/DAMpred/dFeaturesNew $bindir/DAMpred/nFeaturesNew $tmp/normFeatures $tmp/probability/ >> $inputdir/running.log 2>&1`;

`mkdir -p $tmp/top70`;
`$bindir/DAMpred/integrateFeautre.py $tmp/probability/ $tmp/top70/all 0 0 0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 61 62 63 64 65 66 67 68 69 70 71 >> $inputdir/running.log 2>&1`;

`$bindir/DAMpred/BANNprediction.py $tmp/top70/alldiseasef $tmp/top70/alldiseasep $bindir/DAMpred/parameterDbest.txt 0.5844 >> $inputdir/running.log 2>&1`;
`$bindir/DAMpred/BANNprediction.py $tmp/top70/allneutralf $tmp/top70/allneutralp $bindir/DAMpred/parameterNbest.txt 0.4156 >> $inputdir/running.log 2>&1`;
`$bindir/DAMpred/BANNprediction.py $tmp/top70/alldiseasef $tmp/top70/alldiseasedp $bindir/DAMpred/parameterDNbest.txt 0.1688 >> $inputdir/running.log 2>&1`;
`$bindir/DAMpred/voting.py $tmp/top70/alldiseasep_results $tmp/top70/allneutralp_results $tmp/top70/alldiseasedp_results $tmp/final.result $mutations >>$inputdir/running.log 2>&1`;

`$bindir/DAMpred/integrateFeautre4user.py $tmp/features $tmp/featuresPvalueTop20 $mutations 6 7 8 9 11 16 28 33 36 41 42 44 45 46 51 52 57 62 64 66 >> $inputdir/running.log 2>&1`;

if(!-s "$tmp/final.result")
{
    print "$tmp/final.result doesn't exist or is empty!\n";
    exit;
}

`cp $tmp/final.result $inputdir/`;
`cp $tmp/featuresPvalueTop20 $inputdir/`;


#~~~~~~~~~~~~~~~~~~~~~~~ End procedure  ~~~~~~~~~~~~~~~~~~~~~~~~~#
print "Ending...\n";

open(DDG, "<$inputdir/final.result");
print "\n";
print "\n";
print "\n";
print "#####################################################################################################\n";
print "#                                    DAMpred    Result                                              #\n";
print "#                                                                                                   #\n"; 
print "# protein     position     wild-type     mutant-type     Sd     Sn     Sdiff     prediction         #\n";
 
while(<DDG>)
{
	$final_line="$_";
	@final=split(/\t/, $final_line);
	$prediction="";
	if(int($final[6])>1)
	{
		$prediction="disease";
	}
	else
	{
		$prediction="neutral";
	}
	printf "# %-12s%-13s%-14s%-16s%-7.3f%-7.3f%-10.3f%-10s         #\n", $seqname, $final[0],$final[1],$final[2],$final[3],$final[4],$final[5],$prediction;
}

print "#                                                                                                   #\n";
print "# the result is saved in final.result                                                               #\n";
print "#                                                                                                   #\n";
print "#####################################################################################################\n";
print "\n";
print "\n";
print "\n";

sub print_help
{
    printf 
	"DAMpred USAGE:
=====================
Mandatory arguments:
=====================
./runDAMpred.pl  -inputdir datadir -seq seqname.fasta -mutation mutation 

    -datadir:   this is the directory where your input sequence \"proteinname.fasta\" and inoput mutation \"mutation.txt\" are located, and the 
                final results should be saved
    -seq:       the name of protein sequence in raw format and one letter code (the protein sequence code must be saved in 
                datadir)
    -mutation:  the name of mutationfile filled by the single-point muation information including: the wild-type residue, the position number of the residue that 
                undergoes mutation and the mutant residue, eg, A44T

==================
Optional arguments:
==================
    -runstyle   default value is \"serial\" which means running I-TASSER simulation sequentially, 
                \"parallel\" means running I-TASSER simulations in parallel, distributed on cluster nodes
    
    
    
     We suggest testing your installation first with a short sequence (e.g., about 40 residues) before running production jobs for your proteins.
     An example command for running I-TASSER using a seq \"example.fasta\" under the folder /home/XXX/DAMpred/example

     $bindir/runDAMpred.pl -inputdir $dampred/example -seq example.fasta -mutation mutation.txt

";
}



