package buildDataFile;
import java.io.*;
import similarSearch.*;
import java.util.*;

class PairFileLine implements Comparable<PairFileLine>
{
	private String line;
	private double sasaA;
	
	public PairFileLine(String line)
	{
		this.line=line;
		float sA=Float.parseFloat(line.substring(92,97));
		this.sasaA=ResCharacter.sasaAreaNormalize(sA);
	}
	
	public int compareTo(PairFileLine other)
	{
		if(this.sasaA > other.sasaA)
			return 1;
		if(this.sasaA < other.sasaA)
			return -1;
		return 0;
	}
	
	public String getLine()
	{
		return this.line;
	}
}

public class SplitPairFile 
{
	public static void splitByDistance() throws IOException
	{
		BufferedReader in = new BufferedReader(new FileReader("E:\\pairwiseDesign\\pairs\\contactPairs8_0.txt"));
		PrintStream out=new PrintStream("E:\\pairwiseDesign\\pairs\\splitByECogDistance\\pairs8_0.txt");
		String line;
		while((line=in.readLine())!=null)
		{
			double eCogDistance=Double.parseDouble(line.substring(64,70).trim());
			if(eCogDistance<8.0&&eCogDistance>=7.8)
			{
				out.println(line);
			}
		}
		out.close();
	}
	
	private static char ssTypeTri(char c)
	{
		if(c == 'H' || c== 'G' || c=='I')
			return 'H';
		if(c == 'E')
			return 'E';
		return 'C';
	}
	
	public static void splitBySSAndSasa() throws IOException
	{
//H		
		char[] ss={'H','E','C'};
		
		for(int i=0;i<3;i++)
		for(int j=0;j<3;j++)
		for(double sasaI=0.0;sasaI<0.95;sasaI+=0.1)
		for(double sasaJ=0.0;sasaJ<0.95;sasaJ+=0.1)
		{
			BufferedReader in = new BufferedReader(new FileReader("/mnt/0a/xiongpeng/java/data/pairs2/rotPairs8_0psd.txt"));
			String fileName=""+ss[i]+ss[j]+String.format("%3.1f", sasaI)+"_"+String.format("%3.1f", sasaJ)+".txt";
			System.out.println(fileName);
			PrintStream out = new PrintStream("/mnt/0a/xiongpeng/java/data/pairs2/rotSplitBySSandSasa/"+fileName);
			String line;
//			ArrayList<PairFileLine> lineList= new ArrayList<PairFileLine>();
			while((line=in.readLine())!=null)
			{
				int resAID=Integer.parseInt(line.substring(7,12).trim());
				int resBID=Integer.parseInt(line.substring(21,25).trim());
				if(Math.abs(resBID-resAID)<4)
					continue;
				char ssA=ssTypeTri(line.charAt(202));
				char ssB=ssTypeTri(line.charAt(229));
				if(ssA != ss[i] || ssB != ss[j])
					continue;
				float sasaA=Float.parseFloat(line.substring(223,228).trim());
				float sasaAN=ResCharacter.sasaAreaNormalize(sasaA);
				float sasaB=Float.parseFloat(line.substring(250,255).trim());
				float sasaBN=ResCharacter.sasaAreaNormalize(sasaB);
				if(sasaAN<sasaI || sasaAN>=sasaI+0.1)
					continue;
				if(sasaBN<sasaJ || sasaBN>=sasaJ+0.1)
					continue;
				out.println(line);
//				lineList.add(new PairFileLine(line));
			}
			
/*			for(PairFileLine ln : lineList)
			{
				out.println(ln.getLine());
			}
*/	
			in.close();
			out.close();
		}
		
	}
	
	public static void splitBySSAndSasaRes() throws IOException
	{
//H		
		char[] ss={'H','E','C'};
		
		for(int i=0;i<3;i++)
		for(int j=0;j<3;j++)
		for(double sasaI=0.0;sasaI<0.95;sasaI+=0.1)
		for(double sasaJ=0.0;sasaJ<0.95;sasaJ+=0.1)
		{
			BufferedReader in = new BufferedReader(new FileReader("/mnt/0a/xiongpeng/java/data/pairs2/resPair8_0med.txt"));
			String fileName=""+ss[i]+ss[j]+String.format("%3.1f", sasaI)+"_"+String.format("%3.1f", sasaJ)+".txt";
			System.out.println(fileName);
			PrintStream out = new PrintStream("/mnt/0a/xiongpeng/java/data/pairs2/splitBySSandSasa/"+fileName);
			String line;
//			ArrayList<PairFileLine> lineList= new ArrayList<PairFileLine>();
			while((line=in.readLine())!=null)
			{
				int resAID=Integer.parseInt(line.substring(7,12).trim());
				int resBID=Integer.parseInt(line.substring(17,21).trim());
				if(Math.abs(resBID-resAID)<4)
					continue;
				char ssA=ssTypeTri(line.charAt(71));
				char ssB=ssTypeTri(line.charAt(98));
				if(ssA != ss[i] || ssB != ss[j])
					continue;
				float sasaA=Float.parseFloat(line.substring(92,97).trim());
				float sasaAN=ResCharacter.sasaAreaNormalize(sasaA);
				float sasaB=Float.parseFloat(line.substring(119,124).trim());
				float sasaBN=ResCharacter.sasaAreaNormalize(sasaB);
				if(sasaAN<sasaI || sasaAN>=sasaI+0.1)
					continue;
				if(sasaBN<sasaJ || sasaBN>=sasaJ+0.1)
					continue;
				out.println(line);
//				lineList.add(new PairFileLine(line));
			}
			
/*			for(PairFileLine ln : lineList)
			{
				out.println(ln.getLine());
			}
*/
			out.close();
			in.close();
		}
		
	}
	
	public static void neighborPair() throws IOException
	{

		for(double sasaI=0.0;sasaI<0.95;sasaI+=0.1)
		for(double sasaJ=0.0;sasaJ<0.95;sasaJ+=0.1)
		for(int dis = 1;dis <4;dis++)
		{
			BufferedReader in = new BufferedReader(new FileReader("/mnt/0a/xiongpeng/java/data/pairs2/rotPairs8_0psd.txt"));
			String fileName="nb"+dis+"-"+String.format("%3.1f", sasaI)+"_"+String.format("%3.1f", sasaJ)+".txt";
			System.out.println(fileName);
			PrintStream out = new PrintStream("/mnt/0a/xiongpeng/java/data/pairs2/rotNeighbor/"+fileName);
			String line;
//			ArrayList<PairFileLine> lineList= new ArrayList<PairFileLine>();
			while((line=in.readLine())!=null)
			{
				int resAID=Integer.parseInt(line.substring(7,12).trim());
				int resBID=Integer.parseInt(line.substring(21,25).trim());
				if((resBID-resAID) != dis)
					continue;
				
				float sasaA=Float.parseFloat(line.substring(223,228).trim());
				float sasaAN=ResCharacter.sasaAreaNormalize(sasaA);
				float sasaB=Float.parseFloat(line.substring(250,255).trim());
				float sasaBN=ResCharacter.sasaAreaNormalize(sasaB);
				if(sasaAN<sasaI || sasaAN>=sasaI+0.1)
					continue;
				if(sasaBN<sasaJ || sasaBN>=sasaJ+0.1)
					continue;
//				lineList.add(new PairFileLine(line));
				out.println(line);
			}
			in.close();
/*			
			for(PairFileLine ln : lineList)
			{
				out.println(ln.getLine());
			}
*/			
			out.close();
		}
	}
	
	public static void neighborPairRes() throws IOException
	{

		for(double sasaI=0.0;sasaI<0.95;sasaI+=0.1)
		for(double sasaJ=0.0;sasaJ<0.95;sasaJ+=0.1)
		for(int dis = 1;dis <4;dis++)
		{
			BufferedReader in = new BufferedReader(new FileReader("/mnt/0a/xiongpeng/java/data/pairs2/resPair8_0med.txt"));
			String fileName="nb"+dis+"-"+String.format("%3.1f", sasaI)+"_"+String.format("%3.1f", sasaJ)+".txt";
			System.out.println(fileName);
			PrintStream out = new PrintStream("/mnt/0a/xiongpeng/java/data/pairs2/neighbor/"+fileName);
			String line;
//			ArrayList<PairFileLine> lineList= new ArrayList<PairFileLine>();
			while((line=in.readLine())!=null)
			{
				int resAID=Integer.parseInt(line.substring(7,12).trim());
				int resBID=Integer.parseInt(line.substring(17,21).trim());
				if((resBID-resAID) != dis)
					continue;
				
				float sasaA=Float.parseFloat(line.substring(92,97).trim());
				float sasaAN=ResCharacter.sasaAreaNormalize(sasaA);
				float sasaB=Float.parseFloat(line.substring(119,124).trim());
				float sasaBN=ResCharacter.sasaAreaNormalize(sasaB);
				if(sasaAN<sasaI || sasaAN>=sasaI+0.1)
					continue;
				if(sasaBN<sasaJ || sasaBN>=sasaJ+0.1)
					continue;
//				lineList.add(new PairFileLine(line));
				out.println(line);
			}
			in.close();
/*			
			for(PairFileLine ln : lineList)
			{
				out.println(ln.getLine());
			}
*/			
			out.close();
		}
	}
	
	public static void main(String[] args) throws IOException
	{
//		splitBySSAndSasa();
		neighborPair();
	}
}
