package burial;

import java.io.IOException;
import java.util.ArrayList;

import math.Matrix;
import model.Tern;

public class Rotation {
	public static Matrix rotByX(double theta)
	{
		
		double sinTheta = Math.sin(theta);
		double cosTheta = Math.cos(theta);
		double[][] x = { {1,0,0},
				{0, cosTheta, sinTheta},
				{0, -1.0*sinTheta, cosTheta}};
		
		Matrix m = new Matrix(x);
	
		return m;
	}
	
	public static Matrix rotByY(double theta)
	{
		double sinTheta = Math.sin(theta);
		double cosTheta = Math.cos(theta);
		double[][] x = { {cosTheta, 0, -1.0*sinTheta},
				{0, 1, 0},
				{sinTheta, 0, cosTheta}};
		
		Matrix m = new Matrix(x);
	
		return m;
	}
	
	public static Matrix rotByZ(double theta)
	{
		double sinTheta = Math.sin(theta);
		double cosTheta = Math.cos(theta);
		double[][] x = { {cosTheta, sinTheta, 0},
				{-1.0*sinTheta , cosTheta, 0},
				{0, 0, 1}};
		
		Matrix m = new Matrix(x);
	
		return m;
	}
	
	public static Tern buildTern(Tern A, Tern B, Tern C, double bondLen, double ang, double dihed)
	{
		Tern CB = B.subtract(C);
		Tern AB = B.subtract(A);
		Tern x = CB.divide(CB.length());
		
		Tern z = CB.cross(AB);
		z = z.divide(z.length());
		Tern y = z.cross(x);
		double[][] values = {
				{x.x(),y.x(),z.x()},
				{x.y(),y.y(),z.y()},
				{x.z(),y.z(),z.z()}
		};
		Matrix m = new Matrix(values);
		Matrix mx = Rotation.rotByX(dihed/180.0*Math.PI);
		Matrix mz = Rotation.rotByZ(ang/180.0*Math.PI);
		m = m.multiply(mx).multiply(mz);
		Tern DInner = new Tern(bondLen,0,0);
		Tern d = Tern.transformation(m, DInner).add(C);
		return d;
	}
	
	public static void main(String[] args) throws IOException
	{
		ArrayList<Tern> coorList = new ArrayList<Tern>();
		coorList.add(new Tern(0.828, -1.203, 0.0));
		coorList.add(new Tern(0.0, 0.0, 0.0));
		coorList.add(new Tern( -0.967,  -0.040  , 1.189));
		double len = 1.5;
		double angle = 109;
		double dihed = -60;
		Tern d = buildTern(coorList.get(0), coorList.get(1), coorList.get(2), len, angle, dihed);
		System.out.printf("%8.3f%8.3f%8.3f",d.x(),d.y(),d.z());
	}
}
