package math;

public class CredibleProbLikelyhood 
{
	private double pLow;
	private double pHigh;
	
	public CredibleProbLikelyhood(int N, int K, double deltaL)
	{
		if(K==0||K==1)
		{ 
			this.pLow = 0.0;
			double p2 = 1.0/N;
			double targetlnLHigh = lnBinomial(N, 1, p2) - Math.log(deltaL);
			this.pHigh = findP(N,1,p2,1.0,targetlnLHigh,false);
		}
		else 
		{
			double p1 = 1.0*(K)/N;
			double targetlnLLow = lnBinomial(N, K, p1) - Math.log(deltaL);
			this.pLow = findP(N, K, 0.0, p1, targetlnLLow, true);
			double p2 = 1.0*(K)/N;
			double targetlnLHigh = lnBinomial(N, K, p2) - Math.log(deltaL);
			this.pHigh = findP(N,K,p2,1.0,targetlnLHigh,false);
		}
	}

	private static double lnPermutation(int N,int K)
	{
		if(K*2 > N)
			return lnPermutation(N, N-K);
		double lnC = 0.0;
		for(int i=0;i<K;i++)
		{
			lnC += Math.log(N-i)-Math.log(K-i);
		}
		return lnC;
	}
	
	private static double lnBinomial(int N,int K,double p)
	{
		if(p == 0 && K ==0)
			return 1.0;
		else if(p == 0 )
			return 0.0;
		else if(p == 1.0 && K == N)
			return 1.0;
		else if(p == 1.0)
			return 0.0;
		
		if(K>60 && N>2*K)
			return lnNor(N,K,p);
		
		double lnL = lnPermutation(N,K) + 1.0*K*Math.log(p) + 1.0*(N-K)*Math.log(1-p);
		return lnL;
	}
	
	private static double lnNor(int N,int K, double p)
	{
		double x = K;
		double miu = N*p;
		double sigmaSquare = p*(1-p)*N;
	
		double l = -1.0*(x-miu)*(x-miu)/2/sigmaSquare-0.5*Math.log(sigmaSquare)-0.5*Math.log(2*Math.PI);
		return l;
	}
	
	private static double findP(int N, int K, double from, double to, double targetlnL, boolean increase) 
	{
		double cutoff = 1.0*K/N*0.001;
		double mid = (from+to)/2.0;
		while(to-from > cutoff && to - from > 0.000000001)
		{
			mid = (from+to)/2.0;
			double lnLMid = lnBinomial(N, K, mid);
			if(lnLMid == targetlnL)
				return mid;
			else if(lnLMid > targetlnL && increase || lnLMid < targetlnL && !increase)
				to = mid;
			else
				from = mid;
		}
		return mid;
	}
	
	public double low()
	{
		return this.pLow;
	}
	
	public double high()
	{
		return this.pHigh;
	}
	
	
	
	public static void main(String[] args)
	{
		int N =100;
		int K =1;
		CredibleProbLikelyhood cpl = new CredibleProbLikelyhood(N,K,3.0);
		
		double pLow = cpl.pLow;
		double pHigh = cpl.pHigh;
		System.out.printf("%7.5f %7.5f\n",pLow*N,pHigh*N);
		
		double p1 = 1.0*(K)/N;
		double targetlnLLow = lnBinomial(N, K, p1) - Math.log(5.0);
		pLow = findP(N, K, 0.0, p1, targetlnLLow, true);
		System.out.println(pLow);
		
		
	}
	
}
