package math;

import java.util.Arrays;

public class CubicEquation {

	private double a;
	private double b;
	private double c;
	private double d;
	
	private double A;
	private double B;
	private double C;
	private double delta;
	
	
	public CubicEquation(double a, double b, double c, double d)
	{
		this.a = a;
		this.b = b;
		this.c = c;
		this.d = d;
		this.A = b*b-3*a*c;
		this.B = b*c-9*a*d;
		this.C = c*c-3*b*d;
		
		this.delta = B*B-4*A*C;
//		System.out.println(A + " " + B + " "  + C +" " +delta);
		
	}
	
	
	
	public double[] solve()
	{
		double[] s = this.solveEquation();
		if(s == null) return null;
		Arrays.sort(s);
		
		double t = s[0];
		s[0] = s[2];
		s[2] = t;
		return s;
	}
	
	private double[] solveEquation()
	{
		double[] s = new double[3];
		if(A==0 && B==0)
		{
			s[0]=-1*b/a/3.0;
			s[1] = s[0];
			s[2] = s[1];
			return s;
		}
		if(this.delta ==0)
		{
			if(this.A == 0)
				return null;
			double K = this.B/this.A;
			s[0] = -1.0*b/a + K;
			s[1] = -0.5*K;
			s[2] = s[1];
			return s;
		}
		if(this.delta < 0)
		{
			if(A <=0 ) return null;
			double T = (2*A*b-3*a*B)/2.0/Math.pow(A, 1.5);
			if(Math.abs(T) > 1.0) return null;
			double theta = Math.acos(T);
			double cosT3 = Math.cos(theta/3.0);
			double sinT3 = Math.sin(theta/3.0);
			double sqA = Math.sqrt(A);
			s[0] = (-1.0*b-2*sqA*cosT3)/3.0/a;
			s[1] = (-1.0*b+sqA*(cosT3+Math.sqrt(3.0)*sinT3))/3.0/a;
			s[2] = (-1.0*b+sqA*(cosT3-Math.sqrt(3.0)*sinT3))/3.0/a;
			return s;
		}
		
		return null;
	}
	
	public static void main(String[] args)
	{
		double[] ce1 = {1,-0.001834,0.000000259,-0.00000000000961};
		CubicEquation ce = new CubicEquation(ce1[0],ce1[1],ce1[2],ce1[3]);
		double[] s = ce.solve();
		if(s == null) System.out.println("no solution");
		else System.out.println(s[0] + " " + s[1] + " " + s[2]);
	
	}
}
