package math;

import java.io.IOException;

public class FitQuadraticFunction {
	
	private double[] x;
	private double[] y;
	private int sampleNum;
	double a0;
	double a1;
	double a2;
	/*
	 * fit: y=a0+a1*x+a2*x^2
	 */
	
	public FitQuadraticFunction(double[] x, double[] y)
	{
		this.x = x;
		this.y = y;
		if(x.length != y.length)
			throw new RuntimeException("x, y length not equal");
		this.sampleNum = x.length;
		fit();
	}
	
	public double[] getFitResult()
	{
		double[] s = {a0,a1,a2};
		return s;
	}
	
	public double getExtremumX()
	{
		double maxX = x[0];
		double minX = x[0];
		for(int i=0;i<x.length;i++)
		{
			if(x[i] > maxX) maxX = x[i];
			if(x[i] < minX) minX = x[i];
		}
		double mid = -0.5*a1/a2;
		if(mid > maxX) return maxX;
		else if(mid < minX) return minX;
		
		return mid;
	}
	
	public void fit()
	{
		
		double[][] matrix = new double[3][4];
		matrix[0][0] = this.sampleNum;
		
		double sumx = 0;
		double sumxx = 0;
		double sumxxx = 0;
		double sumxxxx = 0;
		double sumy = 0;
		double sumxy = 0;
		double sumxxy = 0;
		
		for(int i=0;i<this.sampleNum;i++)
		{
			sumx += x[i];
			sumxx += x[i]*x[i];
			sumy += y[i];
			sumxxx += x[i]*x[i]*x[i];
			sumxxxx += x[i]*x[i]*x[i]*x[i];
			sumxy += x[i]*y[i];
			sumxxy += x[i]*x[i]*y[i];
		}
		
		matrix[0][1] = sumx;
		matrix[0][2] = sumxx;
		matrix[0][3] = sumy;
		matrix[1][0] = sumx;
		matrix[1][1] = sumxx;
		matrix[1][2] = sumxxx;
		matrix[1][3] = sumxy;
		matrix[2][0] = sumxx;
		matrix[2][1] = sumxxx;
		matrix[2][2] = sumxxxx;
		matrix[2][3] = sumxxy;
		
		Matrix m = new Matrix(matrix);
		double[] solution = m.solveLinearEquation();
		this.a0 = solution[0];
		this.a1 = solution[1];
		this.a2 = solution[2];
	}
	
	public static void main(String[] args) throws IOException
	{
	//	double[] x = {0.2,0.22,0.24,0.26,0.28,0.3,0.32,0.34,0.36,0.38,0.4};
	//	double[] y = {3.9766,3.9766,3.9762,3.9759,3.9756,3.9757,3.9757,3.9757,3.9758,3.9759,3.9762};
		double[] x = {3.0, 3.5, 4.0, 4.5,5.0, 5.5, 4.4};
		double[] y = {0.1701,0.1735,0.1750, 0.1746, 0.1733, 0.1725, 0.1749};
	//	double[] x = {120, 135, 150, 165, 180, 195};
	//	double[] y = {0.1944, 0.1970 , 0.2006, 0.2007,  0.2004, 0.1988 };
		
		FitQuadraticFunction fq = new FitQuadraticFunction(x, y);
		double[] s = fq.getFitResult();
		double mid= fq.getExtremumX();
		System.out.printf("%6.3f %6.3f %6.3f\n",s[0],s[1],s[2]);
		System.out.println(mid);
	}
}
