package math;
import java.util.*;
import java.io.*;

import sequence.SubstitutionMatrix;

public class ProbabilityArray 
{
	private ArrayList<Double> pArray=new ArrayList<Double>();
	private int arrayLength=0;
	private int sampleNum=-1;
	
	public ProbabilityArray(ArrayList<Double> pArray)
	{
		for(double   p : pArray)
		{
			this.pArray.add(p);
		}
		this.arrayLength=this.pArray.size();
		normalize();
	}
	
	public ProbabilityArray(ArrayList<Double> pArray,int sampleNum)
	{
		this(pArray);
		this.sampleNum=sampleNum;
	}
	
	public ProbabilityArray(ArrayList<Double> pArray,int sampleNum,ProbabilityArray Pb)
	{
		this(pArray,sampleNum);
		addPseudocount(Pb);
	}


	
	public ProbabilityArray(double[] pArray)
	{
		for(double p:pArray)
		{
			this.pArray.add(p);
		}
		this.arrayLength=this.pArray.size();
		normalize();
	}
	
	public ProbabilityArray(double[] pArray,int sampleNum)
	{
		this(pArray);
		this.sampleNum=sampleNum;
	}
	
	public ProbabilityArray(double[] pArray,int sampleNum,ProbabilityArray Pb)
	{
		this(pArray,sampleNum);
		addPseudocount(Pb);
	}
	
	
	public ProbabilityArray(String fileName) throws IOException
	{
		BufferedReader in = new BufferedReader(new FileReader(fileName));
		String line;
		while((line=in.readLine())!=null)
		{
			if(line.length()>0)
			{
				this.pArray.add(Double.parseDouble(line.trim()));
			}
		}
		in.close();
		this.arrayLength=this.pArray.size();
	}
	
	public int sampleNum()
	{
		return this.sampleNum;
	}
	
	public ProbabilityArray clone()
	{
		ArrayList<Double> pList = new ArrayList<Double>();
		for(int i=0;i<this.arrayLength;i++)
		{
			pList.add(this.pArray.get(i));
		}
		return new ProbabilityArray(pList,this.sampleNum);
	}
	
	public ArrayList<Double> getArray()
	{
		return this.pArray;
	}
	
	public int size()
	{
		return this.arrayLength;
	}
	
	public void normalize()
	{
		double sum=0;
		for(Double p : this.pArray)
		{
			sum+=p;
			if(p<0)
			{
				throw new RuntimeException("negative probability");
			}
		}
		if(sum==0)
		{
			for(int i=0;i<this.arrayLength;i++)
			{
				this.pArray.set(i,0.0);
			}
			this.sampleNum=0;
		}
		else
		{
			for(int i=0;i<this.pArray.size();i++)
			{
				this.pArray.set(i, this.pArray.get(i)/sum);
			}
		}
	}
	
	
	
	public void addPseudocount(ProbabilityArray Pb)
	{
		if(this.sampleNum < 0) throw new RuntimeException("can't add pseudo count, sample Number unknow!");
		int N = this.sampleNum;
		for(int i=0;i<this.arrayLength;i++)
		{
			int K = (int)Math.round(N*this.pArray.get(i));
			double newP = PseudoCount.adjustedProbabilityPAOld(N, K, Pb.getArray().get(i));
			this.pArray.set(i, newP);
		}
		normalize();
	}
	
	public void addPseudocount(ProbabilityArray Pb,double lamda)
	{
		if(this.sampleNum < 0) throw new RuntimeException("can't add pseudo count, sample Number unknown!");
		int N = this.sampleNum;
		for(int i=0;i<this.arrayLength;i++)
		{
			int K = (int)Math.round(N*this.pArray.get(i));
			double newP = PseudoCount.adjustedProbabilityPA2(this.pArray.get(i),N, K, Pb.getArray().get(i), lamda);
			this.pArray.set(i, newP);
		}
		normalize();
	}
	
	public void addPseudocount(ProbabilityArray Pb,double bgErr,double obErr)
	{
		if(this.sampleNum < 0) throw new RuntimeException("can't add pseudo count, sample Number unknown!");
		int N = this.sampleNum;
		for(int i=0;i<this.arrayLength;i++)
		{
			int K = (int)Math.round(N*this.pArray.get(i));
			double newP = PseudoCount.adjustedProbabilityPA3(this.pArray.get(i),N, K, Pb.getArray().get(i),bgErr,obErr);
			this.pArray.set(i, newP);
		}
		normalize();
	}
	
	public void addPseudocountFixNum(ProbabilityArray Pb, int Bc)
	{
		if(this.sampleNum < 0) throw new RuntimeException("can't add pseudo count, sample Number unknow!");
		int N = this.sampleNum;
		for(int i=0;i<this.arrayLength;i++)
		{
			double p = this.pArray.get(i);
			double pNew = (p*N + Pb.getArray().get(i)*Bc)/(N + Bc);
			this.pArray.set(i, pNew);
		}
		normalize();
	}
	
	public void addPseudocountFixNum(ProbabilityArray Pb, double Bc)
	{
		if(this.sampleNum < 0) throw new RuntimeException("can't add pseudo count, sample Number unknow!");
		int N = this.sampleNum;
		for(int i=0;i<this.arrayLength;i++)
		{
			double p = this.pArray.get(i);
			double pNew = (p*N + Pb.getArray().get(i)*Bc)/(N + Bc);
			this.pArray.set(i, pNew);
		}
		normalize();
	}
	
	public double entropy()
	{
		double ent=0;
		for(double p : this.pArray)
		{
			if(p==0)
			{
				continue;
			}
			ent=ent-p*Math.log(p)/Math.log(2.0);
		}
		return ent;
	}
	
	public double entropyWithoutGly()
	{
		double ent = 0.0;
		for(int i=0;i<20;i++)
		{
			if(i==5) continue;
			double p = this.pArray.get(i);
			if(p==0) continue;
			ent=ent-p*Math.log(p)/Math.log(2.0);
		}
		return ent;
	}
	
	public double entropyWithoutAlaGly()
	{
		double ent = 0.0;
		for(int i=1;i<20;i++)
		{
			if(i==5) continue;
			double p = this.pArray.get(i);
			if(p==0) continue;
			ent=ent-p*Math.log(p)/Math.log(2.0);
		}
		return ent;
	}
	
	public double relativeInformation(ProbabilityArray other)
	{
		double ri=0;
		if(this.size()!=other.size())
		{
			throw new RuntimeException("array size not equal");
		}
		for(int i=0;i<this.size();i++)
		{
			double p=this.pArray.get(i);
			double q=other.getArray().get(i);
			if(p==0)
			{
				continue;
			}
			else if(q==0)
			{
				ri=Double.POSITIVE_INFINITY;
			}
			else
			{
				ri=ri+p*(Math.log(p)-Math.log(q));
			}
		}
		return ri;
	}
	
	public double distance(ProbabilityArray other)
	{
		if(this.size()!=other.size())
		{
			throw new RuntimeException("array size not equal");
		}
		double D=0;
		for(int i=0;i<this.size();i++)
		{
			double p=this.pArray.get(i);
			double q=other.pArray.get(i);
			if(p!=0)
			{
				D+=p*Math.log(2*p/(p+q));
			}

			if(q!=0)
			{
				D+=q*Math.log(2*q/(p+q));
			}
		}
		return D;
	}
	
	public void setSampleNum(int sampleNum)
	{
		this.sampleNum=sampleNum;
	}
	
	public ProbabilityArray merge(ProbabilityArray other)
	{
		if(this.sampleNum==-1||other.sampleNum==-1)
		{
			throw new RuntimeException("can't not merge probabilityArray, sample number unknown");
		}
		
		double[] newArray=new double[this.arrayLength];
		for(int i=0;i<this.arrayLength;i++)
		{
			newArray[i]=this.pArray.get(i)*this.sampleNum+other.pArray.get(i)*other.sampleNum;
		}
		int newSampleNum=this.sampleNum+other.sampleNum;
		return new ProbabilityArray(newArray,newSampleNum);
	}
	
	public ProbabilityMatrix cross(ProbabilityArray other)
	{
		if(this.size()!=20 || other.size()!=20)
			throw new RuntimeException("probability array size not fit");
		
		double[][] pm=new double[20][20];
		for(int i=0;i<20;i++)
		{
			for(int j=0;j<20;j++)
			{
				pm[i][j]=this.pArray.get(i)*other.pArray.get(j);
			}
		}
		return new ProbabilityMatrix(pm);
	}
	
	public void printArray(PrintStream out)
	{
		out.printf("%-7d ",this.sampleNum);
		for(int i=0;i<this.arrayLength;i++)
		{
			out.printf("%-9.7f ", this.pArray.get(i));
		}
		out.println();
	}
	
	public static ArrayList<ProbabilityArray> getPaListFromFile(String file) throws IOException
	{
		BufferedReader in = new BufferedReader(new FileReader(file));
		String line;
		ArrayList<ProbabilityArray> paList = new ArrayList<ProbabilityArray>();
		while((line = in.readLine())!=null)
		{
			ArrayList<Double> pa = new ArrayList<Double>();
			String[] spt = line.trim().split(" +");
			if(spt.length != 21)
			{
				in.close();
				throw new RuntimeException("pa size is not 20: " + spt.length);
			}
			int sampleNum = Integer.parseInt(spt[0]);
			for(int i=1;i<21;i++)
			{
				pa.add(Double.parseDouble(spt[i]));
			}
			paList.add(new ProbabilityArray(pa,sampleNum));
		}
		in.close();
		return paList;
	}
	
	public ProbabilityArray dotProduct(TransMatrix tm)
	{
		if(tm.getSize()!=this.arrayLength)
			throw new RuntimeException("size not equal " + tm.getSize() + " " + this.arrayLength);
		double[] pa=new double[this.arrayLength];
		for(int i=0;i<this.arrayLength;i++)
		{
			for(int j=0;j<this.arrayLength;j++)
			{
				pa[i]+=this.pArray.get(j)*tm.getMatrix()[j][i];
			}
		}
		return new ProbabilityArray(pa);
	}
	
	public ProbabilityArray applySubstitution(SubstitutionMatrix subM)
	{
		if(this.arrayLength != 20)
			throw new RuntimeException("size not equal ");
		double[] pa=new double[this.arrayLength];
		for(int i=0;i<this.arrayLength;i++)
		{
			for(int j=0;j<this.arrayLength;j++)
			{
				pa[i]+=this.pArray.get(j)*subM.subM[i][j];
			}
		}
		return new ProbabilityArray(pa);
	}
}
