package math;

import input.InputFile;

import java.util.ArrayList;
import java.io.*;

public class ProbabilityArrayRT 
{
	private ArrayList<Double> pArray=new ArrayList<Double>();
	private int arrayLength=0;
	private int sampleNum=-1;
	
	public ProbabilityArrayRT(ArrayList<Double> pArray)
	{
		this.pArray = pArray;
		this.arrayLength = pArray.size();
		normalize();
	}
	
	public ProbabilityArrayRT(double[] pa)
	{
		for(int i=0;i<pa.length;i++)
		{
			this.pArray.add(pa[i]);
		}
		this.arrayLength = this.pArray.size();
		normalize();
	}
	
	public ProbabilityArrayRT(ArrayList<Double> pArray, int sampleNum)
	{
		this.pArray = pArray;
		this.arrayLength = pArray.size();
		this.sampleNum = sampleNum;
		normalize();
	}
	
	public ProbabilityArrayRT(double[] pa, int sampleNum)
	{
		for(int i=0;i<pa.length;i++)
		{
			this.pArray.add(pa[i]);
		}
		this.sampleNum = sampleNum;
		this.arrayLength = this.pArray.size();
		normalize();
	}
	
	public void normalize()
	{
		double sum = 0.0;
		for(int i=0;i<this.arrayLength;i++)
		{
			sum += this.pArray.get(i);
		}
		
		if(sum == 0.0)
		{
			this.sampleNum = 0;
			return;
		}
		
		for(int i=0;i<this.arrayLength;i++)
		{
			this.pArray.set(i, this.pArray.get(i)/sum);
		}
	}
	
	public int getSampleNum()
	{
		return this.sampleNum;
	}
	
	public int maxElementSampleNum()
	{
		int max = 0;
		for(int i=0;i<this.arrayLength;i++)
		{
			int num = (int)Math.floor(this.sampleNum*this.pArray.get(i));
			if(num>max) max = num;
		}
		return max;
	}
	
	public static ProbabilityArrayRT getPB(int i) throws IOException
	{
		InputFile inputFiles = new InputFile();
		
		String path = inputFiles.get("rotamerBackgroundPA")+ i + ".txt";
		BufferedReader in = new BufferedReader(new FileReader(path));
		String line;
		ArrayList<Double> list = new ArrayList<Double>();
		while((line = in.readLine())!=null)
		{
			if(line.length() < 3)
				continue;
			double p = Double.parseDouble(line.trim());
			list.add(p);
		}
		in.close();
		return new ProbabilityArrayRT(list);
	}
	
	public double sum(double[] pa)
	{
		double sum = 0.0;
		for(int i=0;i<pa.length;i++)
		{
			sum += pa[i];
		}
		return sum;
	}
	
	
	public void addPseudoCount(ProbabilityArrayRT pb)
	{
		if(this.sampleNum < 0) throw new RuntimeException("can't add pseudo count, sample Number unknow!");
		int N = this.sampleNum;
		for(int i=0;i<this.arrayLength;i++)
		{
			int K = (int)Math.round(N*this.pArray.get(i));
			double newP = PseudoCount.adjustedProbabilityPAOld(N, K,pb.getArray().get(i));
			this.pArray.set(i, newP);
		}
		normalize();
	}
	
	public void addPseudoCountTest(ProbabilityArrayRT pb,  double lamda, double sig)
	{
		if(this.sampleNum < 0) throw new RuntimeException("can't add pseudo count, sample Number unknow!");
		int N = this.sampleNum;
		for(int i=0;i<this.arrayLength;i++)
		{
			int K = (int)Math.round(N*this.pArray.get(i));
			double newP = PseudoCount.adjustedProbabilityTest(N, K, pb.getArray().get(i), lamda, sig);
			this.pArray.set(i, newP);
		}
		normalize();
	}
	
	public int sampleNum()
	{
		return this.sampleNum;
	}
	
	public ArrayList<Double> getArray()
	{
		return this.pArray;
	}
	
	public int size()
	{
		return this.arrayLength;
	}
	
	public void setSampleNum(int n)
	{
		this.sampleNum = n;
	}
	
	public ProbabilityArrayRT merge(ProbabilityArrayRT other)
	{
		if(this.arrayLength != other.arrayLength)
		{
			throw new RuntimeException("array length not equal");
		}
		if(this.sampleNum < 0 || other.sampleNum <0)
		{
			throw new RuntimeException("sample number unknown");
		}
		
		ArrayList<Double> pa = new ArrayList<Double>();
		for(int i=0;i<this.arrayLength;i++)
		{
			double p = this.pArray.get(i)*this.sampleNum + other.pArray.get(i)*other.sampleNum;
			pa.add(p);
		}
		return new ProbabilityArrayRT(pa,this.sampleNum+other.sampleNum);
	}
	
	public ProbabilityMatrixRT cross(ProbabilityArrayRT other)
	{
		if(this.arrayLength != other.arrayLength)
		{
			throw new RuntimeException("array length not equal");
		}
		double[][] pm = new double[this.arrayLength][this.arrayLength];
		for(int i=0;i<this.arrayLength;i++)
		{
			for(int j=0;j<this.arrayLength;j++)
			{
				pm[i][j] = this.pArray.get(i)*other.pArray.get(j);
			}
		}
		return new ProbabilityMatrixRT(pm,-1);
	}
	
	public static ArrayList<ProbabilityArrayRT> getPaListFromFile(String file,int rotamerNum) throws IOException
	{
		BufferedReader in = new BufferedReader(new FileReader(file));
		String line;
		ArrayList<ProbabilityArrayRT> list = new ArrayList<ProbabilityArrayRT>();
		while((line = in.readLine())!=null)
		{
			if(line.length() < 5)
				continue;
			String[] spt = line.trim().split(" +");
			if(spt.length != rotamerNum+1)
			{	
				in.close();
				throw new RuntimeException("file error at: " + file + " line: "+ line);
			}
			int sampleNum = Integer.parseInt(spt[0].trim());
			double[] pa = new double[rotamerNum];
			for(int i=0;i<rotamerNum;i++)
			{
				pa[i] = Double.parseDouble(spt[i+1].trim());
			}
			list.add(new ProbabilityArrayRT(pa,sampleNum));
		}
		in.close();
		return list;
	}
	
	public void printArray(PrintStream out)
	{
		out.printf("%-7d ",this.sampleNum);
		for(int i=0;i<this.arrayLength;i++)
		{
			out.printf("%-9.7f ", this.pArray.get(i));
		}
		out.println();
	}
	
	public static void main(String[] args) throws IOException
	{
		ArrayList<ProbabilityArrayRT> list = getPaListFromFile("F:\\135lpaList.txt",55);
		for(int i=0;i<10;i++)
		{
			list.get(i).printArray(System.out);
		}
	}
	
}
