package math;
import java.io.*;
import java.util.ArrayList;

public class ProbabilityMatrix
//20*20 probabilityMatrix
{
	
	private double[][] pm=new double[20][20];
	private int sampleNum=-1;
	// sampleNum == -1 means number unknown 
	public ProbabilityMatrix(double[][] pm)
	{
		this.pm=pm;
		normalize();
	}
	
	public ProbabilityMatrix(double[][] pm, int sampleNum)
	{
		this.pm=pm;
		this.sampleNum=sampleNum;
		normalize();
	}
	
	public ProbabilityMatrix(double[][] pm,int sampleNum,ProbabilityMatrix Pb)
	{
		this(pm,sampleNum);
		addPseudoCount(Pb);
	}
	
	public ProbabilityMatrix(String fileName) throws IOException
	{
		String line;
		BufferedReader in = new BufferedReader(new FileReader(fileName));
		int lineNum=0;
		while((line=in.readLine())!=null)
		{
			if(lineNum>=20)
				continue;
			if(line.length()<10)
				continue;
			if(line.charAt(0)=='#')
				continue;
			String[] spt=line.split(" +");
			if(spt.length!=20)
				continue;
			for(int i=0;i<20;i++)
			{
				this.pm[lineNum][i]=Double.parseDouble(spt[i]);
			}
			lineNum++;
		}
		in.close();
		normalize();
	}
	
	public void normalize()
	{
		double sum=0.0;
		for(int i=0;i<20;i++)
		{
			for(int j=0;j<20;j++)
			{
				double p=this.pm[i][j];
				if(p<0)
					throw new RuntimeException("negative probability");
				sum+=this.pm[i][j];
			}
		}
		if(sum==0)
		{
			this.sampleNum=0;
			return;
		}
		
		for(int i=0;i<20;i++)
			for(int j=0;j<20;j++)
				this.pm[i][j]=this.pm[i][j]/sum;	
	}
	
	public void addPseudoCount(ProbabilityMatrix Pb)
	{
		if(this.sampleNum==-1)
			throw new RuntimeException("sampleNum unknown");
		if(this.sampleNum<2)
		{
			for(int i=0;i<20;i++)
				for(int j=0;j<20;j++)
					this.pm[i][j]=Pb.pm[i][j];
			this.sampleNum=-1;
			return;
		}
		
		
		
		for(int i=0;i<20;i++)
		{
			for(int j=0;j<20;j++)
			{
				int K=(int)Math.round(this.sampleNum*this.pm[i][j]);
				int N=this.sampleNum;
				this.pm[i][j] = PseudoCount.adjustedProbabilityPM(N, K, Pb.pm[i][j]);
			}
		}
		normalize();
	}

	public double[][] getPM()
	{
		return this.pm;
	}
	
	public int getSampleNum()
	{
		return this.sampleNum;
	}

	public void setSampleNum(int sampleNum)
	{
		this.sampleNum=sampleNum;
	}
	
	public double entropy()
	{
		double ent=0;
		for(int i=0;i<20;i++)
		{
			for(int j=0;j<20;j++)
			{
				double p=this.pm[i][j];
				if(p==0)
					continue;
				else ent-=p*Math.log(p);
			}
		}
		return ent;
	}
	
	public double relativeInformation(ProbabilityMatrix Pb)
	{
		double ri=0;
		for(int i=0;i<20;i++)
		{
			for(int j=0;j<20;j++)
			{
				double p=this.pm[i][j];
				double q=Pb.pm[i][j];
				if(p==0)
					continue;
				if(q==0)
					return Double.POSITIVE_INFINITY;
				
				ri+=p*(Math.log(p)-Math.log(q));
			}
		}

		return ri;
	}
	
	public double distance(ProbabilityMatrix other)
	{
		double dis=0;
		for(int i=0;i<20;i++)
		{
			for(int j=0;j<20;j++)
			{
				double p=this.pm[i][j];
				double q=other.pm[i][j];

				double mean=(p+q)/2.0;

				if(p!=0)
				{
					dis+=p*(Math.log(p)-Math.log(mean));
				}
				
				if(q!=0)
				{
					dis+=q*(Math.log(q)-Math.log(mean));
				}
			}
		}
		return dis;
	}
	
	public ProbabilityArray xMarginal()
	{
		double[] xM=new double[20];
		for(int i=0;i<20;i++)
		{
			for(int j=0;j<20;j++)
			{
				xM[i]+=this.pm[i][j];
			}
		}
		return new ProbabilityArray(xM);
	}
	
	public ProbabilityArray yMarginal()
	{
		double[] yM=new double[20];
		for(int i=0;i<20;i++)
		{
			for(int j=0;j<20;j++)
			{
				yM[j]+=this.pm[i][j];
			}
		}
		return new ProbabilityArray(yM);
	}
	
	public double mutualInformation()
	{
		double mi=0;
		ProbabilityArray xM=this.xMarginal();
		ProbabilityArray yM=this.yMarginal();
		for(int i=0;i<20;i++)
		{
			for(int j=0;j<20;j++)
			{
				
				if(this.pm[i][j]==0)
					continue;
				mi+=pm[i][j]*(Math.log(pm[i][j]/xM.getArray().get(i)/yM.getArray().get(j)));
			}
		}
		return mi;
	}
	
	public double mutualInformation(ProbabilityArray pX, ProbabilityArray pY)
	{
		double mi=0;
		for(int i=0;i<20;i++)
		{
			for(int j=0;j<20;j++)
			{
				
				if(this.pm[i][j]==0)
					continue;
				mi+=pm[i][j]*(Math.log(pm[i][j]/pX.getArray().get(i)/pY.getArray().get(j)));
			}
		}
		return mi;
	}
	
	public ProbabilityMatrix merge(ProbabilityMatrix other)
	{
		double[][] newPM=new double[20][20]; 
		for(int i=0;i<20;i++)
		{
			for(int j=0;j<20;j++)
			{
				newPM[i][j]=this.pm[i][j]*this.sampleNum+other.pm[i][j]*other.sampleNum;
			}
		}
		return new ProbabilityMatrix(newPM,this.sampleNum+other.sampleNum);	
	}
	
	public void transpose()
	{
		for(int i=0;i<20;i++)
		{
			for(int j=0;j<i;j++)
			{
				double p = this.pm[i][j];
				this.pm[i][j] = this.pm[j][i];
				this.pm[j][i] = p;
			}
		}
	}
	
	public int maxFq()
	{
		int maxFq = 0;
		for(int i=0;i<20;i++)
		{
			for(int j=0;j<20;j++)
			{
				if(this.pm[i][j]*this.sampleNum > maxFq)
				{
					maxFq = (int)Math.round(this.pm[i][j]*this.sampleNum);
				}
			}
		}
		return maxFq;
	}
	
	public void printMatrix(PrintStream out)
	{
		for(int i=0;i<20;i++)
		{
			for(int j=0;j<20;j++)
			{
				out.printf("%10.8f ", this.pm[i][j]);
			}
			out.println();
		}
	}
	
	public static ArrayList<ProbabilityMatrix> getPmListFromFile(String file) throws IOException
	{
		ArrayList<ProbabilityMatrix> pmList = new ArrayList<ProbabilityMatrix>();
		BufferedReader in = new BufferedReader(new FileReader(file));
		String line;
		int count = -2;
		int sampleNum = -1;
		double[][] pm = new double[20][20];
		while((line = in.readLine())!=null)
		{
			if(line.length()<1)
				continue;
			if(count == -1)
			{
				sampleNum = Integer.parseInt(line.trim());
			}
			
			if(count == 0)
				pm = new double[20][20];
			if(count>=0 && count <20)
			{
				String[] spt= line.trim().split(" +");
				if(spt.length!=20)
				{
					in.close();
					throw new RuntimeException("error at line:\nline\n");
				}
				for(int i=0;i<20;i++)
					pm[count][i] = Double.parseDouble(spt[i]);
			}
			if(count == 19)
			{
				pmList.add(new ProbabilityMatrix(pm,sampleNum));
				sampleNum = -1;
				count = -3;
			}
			count++;
			
		}
		in.close();
		return pmList;
	}
	
	
}
