package math;

import java.util.ArrayList;
import java.io.*;

public class ProbabilityMatrixRT 
{
	private double[][] pm;
	private int rotamerNum;
	private int sampleNum = -1;
	
	public ProbabilityMatrixRT(double[][] pm, int sampleNum)
	{
		this.pm = pm;
		this.rotamerNum = pm.length;
		this.sampleNum = sampleNum;
		normalize();
	}
	
	public void normalize()
	{
		double sum = 0.0;
		for(int i=0;i<this.rotamerNum;i++)
		{
			for(int j=0;j<this.rotamerNum;j++)
			{
				sum += pm[i][j];
			}
		}
		if(sum == 0)
		{
			this.sampleNum = 0;
			return;
		}
		for(int i=0;i<this.rotamerNum;i++)
		{
			for(int j=0;j<this.rotamerNum;j++)
			{
				pm[i][j] = pm[i][j]/sum;
			}
		}
	}
	
	public double sum(double[][] pm)
	{
		double sum = 0.0;
		for(int i=0;i<this.rotamerNum;i++)
		{
			for(int j=0;j<this.rotamerNum;j++)
			{
				sum += pm[i][j];
			}
		}
		return sum;
	}
	
	
	
	
	
	public void addPsudoCount(ProbabilityMatrixRT pb)
	{
		if(this.sampleNum==-1)
			throw new RuntimeException("sampleNum unknown");
		if(this.sampleNum<2)
		{
			for(int i=0;i<this.rotamerNum;i++)
			{
				for(int j=0;j<this.rotamerNum;j++)
				{
					pm[i][j] = pb.pm[i][j];
				}
			}
			this.sampleNum=-1;
			return;
		}
		
		for(int i=0;i<this.rotamerNum;i++)
		for(int j=0;j<this.rotamerNum;j++)
		{
			int K=(int)Math.round(this.sampleNum*this.pm[i][j]);
			int N=this.sampleNum;
			
			this.pm[i][j] = PseudoCount.adjustedProbabilityPM(N, K, pb.pm[i][j]);		
		}
		normalize();
	}
	
	
	public int getSampleNum()
	{
		return this.sampleNum;
	}
	
	public double[][] getPM()
	{
		return this.pm;
	}
	public int rotamerNum()
	{
		return this.rotamerNum;
	}
	
	public void setSampleNum(int n)
	{
		this.sampleNum = n;
	}
	
	public int maxFq()
	{
		int maxFq = 0;
		for(int i=0;i<this.rotamerNum;i++)
		{
			for(int j=0;j<this.rotamerNum;j++)
			{
				if(this.pm[i][j]*this.sampleNum > maxFq)
				{
					maxFq = (int)Math.round(this.pm[i][j]*this.sampleNum);
				}
			}
		}
		return maxFq;
	}
	
	public ProbabilityMatrixRT merge(ProbabilityMatrixRT other)
	{
		if(this.rotamerNum != other.rotamerNum)
		{
			throw new RuntimeException("array length not equal");
		}
		if(this.sampleNum < 0 || other.sampleNum <0)
		{
			throw new RuntimeException("sample number unknown");
		}
		
		double[][] newPm = new double[this.rotamerNum][this.rotamerNum];
		for(int i=0;i<this.rotamerNum;i++)
		for(int j=0;j<this.rotamerNum;j++)
		{
			newPm[i][j] = this.pm[i][j]*this.sampleNum + other.pm[i][j]*other.sampleNum;
			
		}
		return new ProbabilityMatrixRT(newPm,this.sampleNum+other.sampleNum);
	}
	
	public void printMatrix(PrintStream out)
	{
		for(int i=0;i<this.rotamerNum;i++)
		{
			for(int j=0;j<this.rotamerNum;j++)
			{
				out.printf("%9.7f ",this.pm[i][j]);
			}
			out.println();
		}
	}
	
	public void transpose()
	{
		for(int i=0;i<this.rotamerNum;i++)
		{
			for(int j=0;j<i;j++)
			{
				double tmp = this.pm[i][j];
				this.pm[i][j] = this.pm[j][i];
				this.pm[j][i] = tmp;
			}
		}
	}
	
	public static ArrayList<ProbabilityMatrixRT> getPmListFromFile(String file, int rotamerNum) throws IOException
	{
		ArrayList<ProbabilityMatrixRT> pmList = new ArrayList<ProbabilityMatrixRT>();
		BufferedReader in = new BufferedReader(new FileReader(file));
		String line;
		int count = -2;
		int sampleNum = -1;
		double[][] pm = new double[rotamerNum][rotamerNum];
		while((line = in.readLine())!=null)
		{
			if(line.length()<1)
				continue;
			if(count == -1)
			{
				sampleNum = Integer.parseInt(line.trim());
			}
			
			if(count == 0)
				pm = new double[rotamerNum][rotamerNum];
			if(count>=0 && count <rotamerNum)
			{
				String[] spt= line.trim().split(" +");
				if(spt.length!=rotamerNum)
				{
					in.close();
					throw new RuntimeException("error at line:\n" + line + "\n");
				}
				for(int i=0;i<rotamerNum;i++)
					pm[count][i] = Double.parseDouble(spt[i]);
			}
			if(count == rotamerNum-1)
			{
				pmList.add(new ProbabilityMatrixRT(pm,sampleNum));
				sampleNum = -1;
				count = -3;
			}
			count++;
			
		}
		in.close();
		return pmList;
	}
	

}
