package math;

import java.io.IOException;

public class PseudoCount {


	public static double adjustedProbabilityPM(int N, int K, double pBg)
	{
		double lamda = 1.0;
		if(N == 0)
			return pBg;
		
		CredibleProbLikelyhood cp = new CredibleProbLikelyhood(N,K,5.0);
		double pOb = 1.0*K/N;

		double pHigh = cp.high();
		double pLow = cp.low();
		
		double m;
		if(pBg == 0)
			return 0;
		else if(pBg > pOb)
			m = (pHigh-pOb)/(pHigh-pOb+lamda/(1+lamda)*pBg);
		else
			m = (pOb - pLow)/(pOb-pLow+lamda*pBg);
		
		double p = m*pBg + (1-m)*pOb;
		if(p>pHigh)
			p = pHigh;
		else if(p<pLow)
			p = pLow;
		return p;
	}
	
	
	public static double adjustedProbabilityPA(int N, int K, double pBg)
    {
	
        double lamda = 2.0;
        if(N == 0)
            return pBg;
       
        CredibleProbLikelyhood cp = new CredibleProbLikelyhood(N,K,5.0);
        double pOb = 1.0*K/N;

        double pHigh = cp.high();
        double pLow = cp.low();
       
        double m;
        if(pBg == 0)
            return 0;
        else if(pBg > pOb)
            m = (pHigh-pOb)*(pHigh-pOb)/((pHigh-pOb)*(pHigh-pOb)+lamda*lamda/(1+lamda)*(1+lamda)*pBg*pBg);
        else
            m = (pOb - pLow)*(pOb-pLow)/((pOb-pLow)*(pOb-pLow)+lamda*lamda*pBg*pBg);
       
        double p = m*pBg + (1-m)*pOb;
        if(p>pHigh)
        {
        	p = pHigh;
        }
        else if(p<pLow)
        {
        	p = pLow;
        }
        return p;
    }
	
	public static double adjustedProbabilityPA(double pOb, int N, int K, double pBg)
    {
		
        double lamda = 2.0;
        if(N == 0)
            return pBg;
       
        CredibleProbLikelyhood cp = new CredibleProbLikelyhood(N,K,5.0);
       
        double pHigh = cp.high();
        double pLow = cp.low();
       
        double m;
        if(pBg == 0)
            return 0;
        else if(pBg > pOb)
            m = (pHigh-pOb)*(pHigh-pOb)/((pHigh-pOb)*(pHigh-pOb)+lamda*lamda/(1+lamda)*(1+lamda)*pBg*pBg);
        else
            m = (pOb - pLow)*(pOb-pLow)/((pOb-pLow)*(pOb-pLow)+lamda*lamda*pBg*pBg);
       
        double p = m*pBg + (1-m)*pOb;
        if(p>pHigh)
        {
        	p = pHigh;
        }
        else if(p<pLow)
        {
        	p = pLow;
        }
        return p;
    }
	
	public static double adjustedProbabilityPA2(double pOb, int N, int K, double pBg, double lamda)
    {
        if(N == 0)
            return pBg;
       
        CredibleProbLikelyhood cp = new CredibleProbLikelyhood(N,K,3.0);
       
        double pHigh = cp.high();
        double pLow = cp.low();
       
        double x = lamda*lamda/(pHigh-pLow)/(pHigh-pLow);
        double m = 1.0/(1+x);
        
        double p = m*pBg + (1-m)*pOb;
        if(p>pHigh)
        {
        	p = pHigh;
        }
        else if(p<pLow)
        {
        	p = pLow;
        }
        return p;
    }
	
	
	
	public static double adjustedProbabilityPA3(double pOb, int N, int K, double pBg, double bgErr, double obErr)
    {
        if(N == 0)
            return pBg;
       
        CredibleProbLikelyhood cp = new CredibleProbLikelyhood(N,K,obErr);
       
        double pHigh = cp.high();
        double pLow = cp.low();
       
        double x = bgErr*bgErr/(pHigh-pLow)/(pHigh-pLow);
        double m = 1.0/(1+x);
        
        double p = m*pBg + (1-m)*pOb;
        if(p>pHigh)
        {
        	p = pHigh;
        }
        else if(p<pLow)
        {
        	p = pLow;
        }
        return p;
    }
	
	
	public static double adjustedProbabilityPAOld(int N, int K, double pBg)
	{
		double lamda = 2.0;
		if(N == 0)
			return pBg;
		
		CredibleProbLikelyhood cp = new CredibleProbLikelyhood(N,K,5.0);
		double pOb = 1.0*K/N;

		double pHigh = cp.high();
		double pLow = cp.low();
		
		double m;
		if(pBg == 0)
			return 0;
		else if(pBg > pOb)
			m = (pHigh-pOb)/(pHigh-pOb+lamda/(1+lamda)*pBg);
		else
			m = (pOb - pLow)/(pOb-pLow+lamda*pBg);
		
		double p = m*pBg + (1-m)*pOb;
		if(p>pHigh)
			p = pHigh;
		else if(p<pLow)
			p = pLow;
		return p;
	}
	
	public static double adjustedProbabilityPAFixNum0(int N, int K, double pBg, int Bc)
	{
		return (K+Bc*pBg)/(N+Bc);
	}
	
	
	
	public static double adjustedProbabilityTest(int N, int K, double pBg, double lamda, double sig)
	{
	
		if(N == 0)
			return pBg;
		
		
		CredibleProbLikelyhood cp = new CredibleProbLikelyhood(N,K,sig);
		double pOb = 1.0*K/N;

		double pHigh = cp.high();
		double pLow = cp.low();
		
		double m;
		if(pBg == 0)
			return 0;
		else if(pBg > pOb)
			m = (pHigh-pOb)/(pHigh-pOb+lamda/(1+lamda)*pBg);
		else
			m = (pOb - pLow)/(pOb-pLow+lamda*pBg);
		
		double p = m*pBg + (1-m)*pOb;
		if(p>pHigh)
			p = pHigh;
		else if(p<pLow)
			p = pLow;
		return p;
	}
	
	public static void main(String[] args) throws IOException
	{
		double p = adjustedProbabilityPM(850, 7, 0.0001295);
		System.out.println(p);
	}

	
}
