package math;

import java.util.ArrayList;
import model.ProteinChain;
import model.Tern;
import java.io.*;

public class RMSD 
{
	public static double simpleRmsd(ArrayList<Tern> listA,ArrayList<Tern> listB)
	{
		if(listA.size()!=listB.size())
			throw new RuntimeException("can't calculate rmsd, atom number not equal");
		int listLength=listA.size();
		double m=0;
		for(int i=0;i<listLength;i++)
		{
			m += listA.get(i).distSquare(listB.get(i));
		}
		m=m/listLength;
		m=Math.sqrt(m);
		return m;
	}
	
	public static Tern findCOG(ArrayList<Tern> list)
	{
		Tern sum=new Tern();
		for(Tern tn : list)
		{
			sum=sum.add(tn);
		}
		sum=sum.divide(1.0f*list.size());
		return sum;
	}
	
	public static double rmsd(ArrayList<Tern> listA,ArrayList<Tern> listB)
	{
		//we use the least squares alignment to build a Transform that 
		//superimpose that two vector together, then calculate the simple rmsd

		Tern Acog=findCOG(listA);
		Tern Bcog=findCOG(listB);
		
		ArrayList<Tern> listA2=new ArrayList<Tern>();
		for(Tern tn : listA)
			listA2.add(tn.subtract(Acog));
		ArrayList<Tern> listB2=new ArrayList<Tern>();
		for(Tern tn : listB)
			listB2.add(tn.subtract(Bcog));

		Tern[] ternsA=new Tern[listA.size()];
		for(int k=0;k<listA.size();k++) 
			ternsA[k]=listA.get(k).subtract(Acog);
		
		Tern[] ternsB=new Tern[listB.size()];
		for(int k=0;k<listB.size();k++) 
			ternsB[k]=listB.get(k).subtract(Bcog);

		Transform tf=Transform.buildRotation(ternsB, ternsA);
		
//		tf.print();
		ArrayList<Tern> newListA=new ArrayList<Tern>();
		for(Tern tn : listA2)
		{ 
			newListA.add(tf.transform(tn));
		}
//		System.out.println("listSize :" + listA2.size());
		
		return simpleRmsd(newListA,listB2);
	}
	
	public static double CaRmsd(ProteinChain pcA, ProteinChain pcB)
	{
		if(pcA.getResidue().size() != pcB.getResidue().size())
			throw new RuntimeException("residue number not equal: "+ pcA.getResidue().size() + " " + pcB.getResidue().size());
		
		ArrayList<Tern> aList = new ArrayList<Tern>();
		ArrayList<Tern> bList = new ArrayList<Tern>();
		
		for(int i=0;i<pcA.getResidue().size();i++)
		{
			Tern ta = pcA.getResidue().get(i).getAtom("CA").getCoordinate();
			Tern tb = pcB.getResidue().get(i).getAtom("CA").getCoordinate();
//			System.out.println(ta.distance(tb));
			if(ta == null || tb == null)
				throw new RuntimeException("error at get ca atom : res" + (i+1));
			aList.add(ta);
			bList.add(tb);
		}
		
		return rmsd(aList,bList);
		
	}
	
	private static ArrayList<Tern> getTernList(String file) throws IOException
	{
		BufferedReader in = new BufferedReader(new FileReader(file));
		String line;
		ArrayList<Tern> tList = new ArrayList<Tern>();
		while((line = in.readLine())!=null)
		{
			String[] spt = line.trim().split(" +");
			if(spt.length != 3)
				throw new RuntimeException("file read error");
			double x = Double.parseDouble(spt[0]);
			double y = Double.parseDouble(spt[1]);
			double z = Double.parseDouble(spt[2]);
			Tern t = new Tern(x,y,z);
			tList.add(t);
		}
		return tList;
	}
	
	
	
	public static void main(String[] args) throws IOException
	{
		
		
		
	}
	
}
