package math;
import java.io.*;
import java.util.ArrayList;



public class ScoreMatrix 
{
	private double[][] sm;
	private int sampleNum=-1;
	
	public ScoreMatrix(double[][] sm, int sampleNum)
	{
		this.sm=sm;
		this.sampleNum=sampleNum;
	}
	
	
	public ScoreMatrix(ProbabilityMatrix pm, ProbabilityMatrix pmB)
	{
		this.sampleNum = pm.getSampleNum();
		double[][] sm = new double[20][20];
		double[][] pClone = new double[20][20];
		for(int i=0;i<20;i++)
		for(int j=0;j<20;j++)
		{
			pClone[i][j] = pm.getPM()[i][j];
		}
		ProbabilityMatrix pmClone = new ProbabilityMatrix(pClone,pm.getSampleNum());
		pmClone.addPseudoCount(pmB);
		for(int i=0;i<20;i++)
		{
			for(int j=0;j<20;j++)
			{
				double p = pmClone.getPM()[i][j];
				double pBg = pmB.getPM()[i][j];
				
				if(p == 0 && pBg ==0)
					sm[i][j] = 0;
				else if(p ==0)
					sm[i][j] = 10000;
				else if(pBg == 0)
					sm[i][j] = -10000;
				else
					sm[i][j] = -1.0*Math.log(p/pBg);
			}
		}
		this.sm = sm;
	}
	
	
	public double[][] getScoreMatrix()
	{
		return this.sm;
	}
	
	public void multiply(double f)
	{
		for(int i=0;i<20;i++)
		{
			for(int j=0;j<20;j++)
			{
				this.sm[i][j] = this.sm[i][j]*f;
			}
		}
	}
	
	
	
	
	public int getSampleNum()
	{
		return this.sampleNum;
	}
	
	
	
	public void printMatrix(PrintStream out)
	{
		for(int i=0;i<20;i++)
		{
			for(int j=0;j<20;j++)
			{
				out.printf("%10.5f ", this.sm[i][j]);
			}
			out.println();
		}
	}
	
	public static ArrayList<ScoreMatrix> getScoreMatrixListFromFile(String file) throws IOException
	{
		ArrayList<ScoreMatrix> smList = new ArrayList<ScoreMatrix>();
		BufferedReader in = new BufferedReader(new FileReader(file));
		String line;
		int count = -2;
		int sampleNum = -1;
		double[][] sm = new double[20][20];
		while((line = in.readLine())!=null)
		{
			if(line.length()<1)
				continue;
			if(count == -1)
			{
				sampleNum = Integer.parseInt(line.trim());
			}
			
			if(count == 0)
				sm = new double[20][20];
			if(count>=0 && count <20)
			{
				String[] spt= line.trim().split(" +");
				if(spt.length!=20)
				{
					in.close();
					throw new RuntimeException("error at line:\n" + line + "\n");
				}
				for(int i=0;i<20;i++)
					sm[count][i] = Double.parseDouble(spt[i]);
			}
			if(count == 19)
			{
				smList.add(new ScoreMatrix(sm,sampleNum));
				sampleNum = -1;
				count = -3;
			}
			count++;
			
		}
		in.close();
		return smList;
	}
	


	
	
}
