package math;
import java.io.*;
import java.util.*;

import model.Rotamers;

public class ScoreMatrixRT 
{
	private double[][] sm;
	private int rotamerNum;
	private int sampleNum;
	
	public ScoreMatrixRT(double[][] sm, int sampleNum)
	{
		this.sm=sm;
		this.sampleNum=sampleNum;
		this.rotamerNum = sm.length;
	}
	
	public ScoreMatrixRT(ProbabilityMatrixRT pm, ProbabilityMatrixRT pmB) throws IOException
	{
		this.sampleNum = pm.getSampleNum();
		this.rotamerNum = pm.rotamerNum();
		double[][] sm = new double[pm.rotamerNum()][pm.rotamerNum()];
		double[][] pClone = new double[pm.rotamerNum()][pm.rotamerNum()];
		for(int i=0;i<pm.rotamerNum();i++)
		for(int j=0;j<pm.rotamerNum();j++)
		{
			pClone[i][j] = pm.getPM()[i][j];
		}
		ProbabilityMatrixRT pmClone = new ProbabilityMatrixRT(pClone,pm.getSampleNum());
		
		pmClone.addPsudoCount(pmB);
	
		
		for(int i=0;i<pm.rotamerNum();i++)
		{
			for(int j=0;j<pm.rotamerNum();j++)
			{
				double p = pmClone.getPM()[i][j];
				double pBg = pmB.getPM()[i][j];
				if(p<0||pBg<0)
					throw new RuntimeException("negative probability\np = " + p + "\npBg = " +pBg );
				
				if(p == 0 && pBg ==0)
					sm[i][j] = 0;
				else if(p ==0)
					sm[i][j] = 10;
				else if(pBg == 0)
					sm[i][j] = -10;
				else
				{
					sm[i][j] = -1.0*Math.log(p/pBg);
				}
			}
		}
		this.sm = sm;
	}
	
	private double minScore(int K)
	{
		if(K < 20)
			return -0.2 * K - 1.0;
		else 
			return -10;
	}
	
	public void adjustment(ProbabilityMatrixRT pm)
	{
		int N = pm.getSampleNum();
		for(int i=0;i<this.rotamerNum;i++)
		{
			for(int j=0;j<this.rotamerNum;j++)
			{
				if(i>0 && i<4 && j > 0 && j < 4) continue;
				
				int K = (int)Math.round(N*pm.getPM()[i][j]);
				double minS = minScore(K);
				if(this.sm[i][j] < minS)
				{
					this.sm[i][j] = minS;
				}
			}
		}
	}
	
	public double[][] getScoreMatrix()
	{
		return this.sm;
	}
	
	public void multiply(double f)
	{
		for(int i=0;i<this.rotamerNum;i++)
		{
			for(int j=0;j<this.rotamerNum;j++)
			{
				this.sm[i][j] = this.sm[i][j]*f;
			}
		}
	}
	
	public int getSampleNum()
	{
		return this.sampleNum;
	}
	
	public int getRotamerNum()
	{
		return this.rotamerNum;
	}
	
	public static ArrayList<ScoreMatrixRT> getScoreMatrixListFromFile(String file,int rotamerNum) throws IOException
	{
		ArrayList<ScoreMatrixRT> smList = new ArrayList<ScoreMatrixRT>();
		BufferedReader in = new BufferedReader(new FileReader(file));
		String line;
		int count = -2;
		int sampleNum = -1;
		double[][] sm = new double[rotamerNum][rotamerNum];
		while((line = in.readLine())!=null)
		{
			if(line.length()<1)
				continue;
			if(count == -1)
			{
				sampleNum = Integer.parseInt(line.trim());
			}
			
			if(count == 0)
				sm = new double[rotamerNum][rotamerNum];
			if(count>=0 && count <rotamerNum)
			{
				String[] spt= line.trim().split(" +");
				if(spt.length!=rotamerNum)
				{
					in.close();
					throw new RuntimeException("error at line:\n" + line + "\n" + spt.length + " !=" +rotamerNum);
				}
				
				for(int i=0;i<rotamerNum;i++)
					sm[count][i] = Double.parseDouble(spt[i]);
			}
			if(count == rotamerNum-1)
			{
				smList.add(new ScoreMatrixRT(sm,sampleNum));
				sampleNum = -1;
				count = -3;
			}
			count++;
			
		}
		in.close();
		return smList;
	}
	
	public static ScoreMatrixRT getScoreMatrixFromRandomAccessFile(RandomAccessFile raFile,int i,Rotamers rots) throws IOException
	{
		int singleSMLength = 126+8+(11*rots.rotamerNum()+2)*rots.rotamerNum();
		System.out.println(singleSMLength);
		int fileLength = (int)(raFile.length());
		if((i+1)*singleSMLength > fileLength)
			throw new RuntimeException("out of file boundary");
		raFile.seek(singleSMLength*i+126);
		String line = raFile.readLine();
		int sampleNum = Integer.parseInt(line.trim());
		double[][] matrix = new double[rots.rotamerNum()][rots.rotamerNum()];
		for(int k=0;k<rots.rotamerNum();k++)
		{
			line = raFile.readLine();
			String[] spt = line.trim().split(" +");
			for(int j=0;j<rots.rotamerNum();j++)
			{
				matrix[k][j] = Double.parseDouble(spt[j]);
			}
		}
		return new ScoreMatrixRT(matrix,sampleNum);
	}
	
	public void printMatrix(PrintStream out)
	{
		for(int i=0;i<this.rotamerNum;i++)
		{
			for(int j=0;j<this.rotamerNum;j++)
			{
				out.printf("%10.5f ", this.sm[i][j]);
			}
			out.println();
		}
	}
}
