package math;
import java.util.*;

import model.Tern;

public class Stat 
{
	public static double mean(ArrayList<Double> numList)
	{
		double sum = 0.0;
		for(Double num : numList)
		{
			sum += num;
		}
		if(numList.size() == 0)
			return 0;
		else 
			return sum/numList.size();
	}
	
	public static double var(ArrayList<Double> numList)
	{
		ArrayList<Double> xx = new ArrayList<Double>();
		for(double num : numList)
		{
			xx.add(num*num);
		}
		double xMean = mean(numList);
		return mean(xx)-xMean*xMean;
	}
	
	public static double sd(ArrayList<Double> numList)
	{
		return Math.sqrt(var(numList));
	}
	
	public static double mid(ArrayList<Double> numList)
	{
		double[] nums = new double[numList.size()];
		for(int i=0;i<numList.size();i++)
		{
			nums[i] = numList.get(i);
		}
		Arrays.sort(nums);
		
		int midIndex = numList.size()/2;
		return nums[midIndex];
	}
	
	public static double top(ArrayList<Double> numList, double proportion)
	{
		if(proportion>1.0 || proportion<0)
			throw new RuntimeException("bad input "+proportion);
		double[] nums = new double[numList.size()];
		for(int i=0;i<numList.size();i++)
		{
			nums[i] = numList.get(i);
		}
		Arrays.sort(nums);
		int selectIndex = (int)Math.floor(numList.size()*proportion);
		return nums[selectIndex];
	}
	
	
	
	public static Double[] meanList(ArrayList<Double[]> x)
	{
		int arraySize = x.get(0).length;
		Double[] mean = new Double[arraySize];
		for(int i=0;i<arraySize;i++)
		{
			ArrayList<Double> sub = new ArrayList<Double>();
			for(Double[] xSingle : x)
			{
				sub.add(xSingle[i]);
			}
			mean[i] = mean(sub);
		}
		return mean;
	}
	
	
	public static Double[] sdList(ArrayList<Double[]> x)
	{
		int arraySize = x.get(0).length;
		Double[] sd = new Double[arraySize];
		for(int i=0;i<arraySize;i++)
		{
			ArrayList<Double> sub = new ArrayList<Double>();
			for(Double[] xSingle : x)
			{
				sub.add(xSingle[i]);
			}
			sd[i] = sd(sub);
		}
		return sd;
	}
	
	
	public static Double[][] hist(ArrayList<Double> numList, int bins)
	{
		double max = numList.get(0);
		double min = max;
		for(int i=0;i<numList.size();i++)
		{
			if(numList.get(i) > max) max = numList.get(i);
			if(numList.get(i) < min) min = numList.get(i);
		}
		double binLen = (max-min)/bins;
//		System.out.printf("binLen = %6.3f max = %6.3f min = %6.3f\n",binLen,max,min	);
		Double[][] histList = new Double[2][bins];
		for(int i=0;i<bins;i++)
		{
			double mid = (i+0.5)*binLen + min;
			histList[0][i] = mid;
			histList[1][i] = 0.0;
		}
		for(int i=0;i<numList.size();i++)
		{
			double num = numList.get(i);
			int binID = (int)Math.floor((num-min)/binLen);
			if(binID == bins) binID = bins-1;
			histList[1][binID] += 1.0;
		}
		return histList;
	}
	
	public static Tern meanTern(ArrayList<Tern> terns)
	{
		double sx = 0.0;
		double sy = 0.0;
		double sz = 0.0;
		for(Tern t : terns)
		{
			sx += t.x();
			sy += t.y();
			sz += t.z();
		}
		
		sx = sx / terns.size();
		sy = sy / terns.size();
		sz = sz / terns.size();
		return new Tern(sx,sy,sz);
	}
	
	public static double sdTern(ArrayList<Tern> terns)
	{
		Tern mean = meanTern(terns);
		double ds = 0;
		for(Tern t : terns)
		{
			ds += t.distSquare(mean);
		}
		ds = ds/terns.size();
		return Math.sqrt(ds);
	}
	
}
