package math;
import java.io.*;

public class TransMatrix 
{
	private double[][] matrix;
	private int size;
	
	public TransMatrix(double[][] matrix)
	{
		this.matrix = matrix;
		this.size = matrix.length;
	}
	
	public TransMatrix(String fileName) throws IOException
	{
		BufferedReader in = new BufferedReader(new FileReader(fileName));
		String line;
		in.readLine();
		this.size=-1;
		int j=0;
		while((line = in.readLine())!=null)
		{
			if(line.length()<2)
				continue;
			String[] spt=line.trim().split(" +");
			if(size<0)
			{
				this.size = spt.length-1;
				this.matrix=new double[this.size][this.size];
			}
			for(int i=0;i<this.size;i++)
			{
				this.matrix[j][i]=Double.parseDouble(spt[i+1]);
			}
			j++;
		}
		if(j!=this.size)
		{
			in.close();
			throw new RuntimeException("read matrix error: rows:" + this.size + " arrays:" + j );
		}
		in.close();
		
	}
	
	public int getSize()
	{
		return this.size;
	}
	
	public double[][] getMatrix()
	{
		return this.matrix;
	}
	
	public double[][] toSeqScoreMatrix()
	{
		double[] pBackground={0.0793,0.0132,0.0656,0.0722,0.0390,
				0.0619,0.0213,0.0565,0.0718,0.0916,
				0.0250,0.0409,0.0400,0.0402,0.0475,
				0.0642,0.0473,0.0737,0.0107,0.0382};
		ProbabilityArray Pb=new ProbabilityArray(pBackground);
		
		double[][] sm = new double[20][20];
		for(int i=0;i<20;i++)
		{
			for(int j=0;j<20;j++)
			{
				double sij = Math.log(this.matrix[i][j]/Pb.getArray().get(j))/Math.log(2.0);
				double sji = Math.log(this.matrix[j][i]/Pb.getArray().get(i))/Math.log(2.0);
				if(this.matrix[i][j] == 0)
				{
					sij = Math.log(0.0001/Pb.getArray().get(j))/Math.log(2.0);
				}
				if(this.matrix[j][i]==0)
				{
					sji = Math.log(0.0001/Pb.getArray().get(i))/Math.log(2.0);
				}
				sm[i][j] = (sij+sji);
				if(sm[i][j]<-4)
					sm[i][j]=-4;
				sm[j][i] = sm[i][j];
				
			}
		}
		return sm;
	}
	
	public void printSeqScoreMatrix(PrintStream out)
	{
		String order = "ARNDCQEGHILKMFPSTWYV";
		double[][] sm = toSeqScoreMatrix();
		for(int i=0;i<20;i++)
		{
			out.print("   "+order.charAt(i));
		}
		out.print("\n");
		for(int i=0;i<20;i++)
		{
			out.print(order.charAt(i)+" ");
			for(int j=0;j<20;j++)
			{
				double score =  Math.floor(sm[i][j]+0.5);
				out.printf("%3.0f ",score);
			}
			out.print("\n");
		}
		
		
	}
}
