package model;

import java.io.Serializable;
import java.util.Scanner;

import similarSearch.ResCharacter;
import similarSearch.ResPara;

public class InterfaceResidue implements Serializable{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1058859558038030512L;
	public final String pdbID;
	public final char chainID;
	private Residue res;
	public final String dunbrackCode;
	public final char ss;
	public final float phi;
	public final float psi;
	public final float saiM;
	public final float saiD;
	public final float minDistanceToOtherChain;
	
	public InterfaceResidue(Residue res,  StrideRes srM, StrideRes srD, String pdbID, float d)
	{
		if(pdbID.length() != 4)
			throw new RuntimeException("pdbID length must be 4 :" + pdbID);
		this.pdbID = pdbID;
		this.chainID = res.getChainID();
		this.res = res;
		
		
		this.ss = srM.ssType;
		this.phi = srM.phi;
		this.psi = srM.psi;
		this.saiM = srM.getAreaNormalized();
		this.saiD = srD.getAreaNormalized();
		this.minDistanceToOtherChain = d;
		SideChain sc = new SideChain(res);
		this.dunbrackCode = sc.getDunbrackCode();
	}
	
	public InterfaceResidue(String line)
	{
		Scanner sc = new Scanner(line);
		this.pdbID = sc.next();
		this.chainID = sc.next().charAt(0);
		int resID = sc.nextInt();
		this.dunbrackCode = sc.next();
		this.res = new Residue(this.dunbrackCode.substring(0, 3), resID);
		this.res.setChainID(this.chainID);
		this.ss = sc.next().charAt(0);
		this.phi = sc.nextFloat();
		this.psi = sc.nextFloat();
		this.saiM = sc.nextFloat();
		this.saiD = sc.nextFloat();
		this.minDistanceToOtherChain = sc.nextFloat();
		sc.close();
	}
	
	public InterfaceResidue(Residue res,  StrideRes srM, StrideRes srD,float d)
	{
		this(res,srM,srD,"xxxx",d);
	}
	
	public Residue getResidue()
	{
		return this.res;
	}
	
	public String getDunbrackCode()
	{
		return this.dunbrackCode;
	}
	
	public String getResType()
	{
		return this.dunbrackCode.substring(0, 3);
	}
	
	public char getChainID()
	{
		return this.res.getChainID();
	}
	
	public boolean similarTo(InterfaceResidue irB, ResPara rpr)
	{
		if(!ResCharacter.simSecondaryStructure(this.ss, irB.ss, rpr.ssLevel())) return false;
		if(!ResCharacter.simSasaN(this.saiD, irB.saiD, rpr.sasaLevel())) return false;
		if(!ResCharacter.simSasaN(this.saiM, irB.saiM, rpr.sasaLevel())) return false;
		if(!ResCharacter.simSasaN(this.saiM-this.saiD, irB.saiM-irB.saiD, rpr.sasaLevel())) return false;
		if(!ResCharacter.simPhiPsi(this.phi, this.psi, irB.phi, irB.psi, rpr.ppLevel())) return false;
		return true;
	}
	
	public String toString()
	{
		String s = String.format("%4s %c %4d %7s %c %7.2f %7.2f %5.3f %5.3f %5.3f", pdbID,chainID,res.getID(),dunbrackCode,ss,phi,psi,saiM,saiD,minDistanceToOtherChain);
		return s;
	}
}
