package model;
import java.util.*;
import java.io.*;
import math.ProbabilityArray;

public class MSA 
{
	private ArrayList<String> seqs;
	private ArrayList<ProbabilityArray> profiles;
	
	public MSA(ArrayList<String> seqs)
	{
		this.seqs = seqs;
		creatProfile();
	}
	
	public MSA(String fileName) throws IOException
	{
		BufferedReader in = new BufferedReader(new FileReader(fileName));
		String line;
		int lastSeqLength = -1;
		this.seqs = new ArrayList<String>();
		while((line = in.readLine())!=null)
		{
			if(lastSeqLength<0)
				lastSeqLength = line.trim().length();
			if(lastSeqLength != line.trim().length())
				throw new RuntimeException("msa length not equal");
			lastSeqLength = line.trim().length();
			this.seqs.add(line.trim());
		}
		in.close();
		creatProfile();
	}
	
	public MSA(String fileName,int seqNum) throws IOException
	{
		BufferedReader in = new BufferedReader(new FileReader(fileName));
		String line;
		int lastSeqLength = -1;
		this.seqs = new ArrayList<String>();
		while((line = in.readLine())!=null)
		{
			if(lastSeqLength<0)
				lastSeqLength = line.trim().length();
			if(lastSeqLength != line.trim().length())
				throw new RuntimeException("msa length not equal");
			lastSeqLength = line.trim().length();
			this.seqs.add(line.trim());
			if(this.seqs.size()==seqNum)
				break;
		}
		in.close();
		creatProfile();
	}
	
	public MSA(String fileName, String fileType) throws IOException
	{
		if(fileType.equals("fa"))
		{
			BufferedReader in = new BufferedReader(new FileReader(fileName));	
			String line;
			String seqFrag = "";
			this.seqs = new ArrayList<String>();
			while((line = in.readLine())!=null)
			{
				if(line.charAt(0) == '>')
				{
					if(seqFrag.length() > 1)
						seqs.add(seqFrag);
					seqFrag = "";
					continue;
				}
				seqFrag = seqFrag + line.trim();
			}
			seqs.add(seqFrag);
			in.close();
			int len = seqs.get(0).length();
			for(String s : seqs)
			{
				if(s.length()!=len)
					throw new RuntimeException("sequence length not equal!");
			}
			this.seqs = getTruncatedMSA(seqs);
			creatProfile();
		}
		
		else if(fileType.equals("dali"))
		{
			this.seqs = getMSASeqsFromDaliResult(fileName);
			creatProfile();
		}
		else if(fileType.equals("designLog"))
		{
			this.seqs = new ArrayList<String>();
			BufferedReader in = new BufferedReader(new FileReader(fileName));	
			String line;
			while((line = in.readLine())!=null)
			{
				if(!line.startsWith("sequence")) continue;
				String[] spt = line.split(" +");
				if(spt.length < 2) continue;
				this.seqs.add(spt[1]);
			}
			in.close();
			if(this.seqs.size() > 0)
				creatProfile();
		}
	}
	
	public int seqLength()
	{
		return this.seqs.get(0).length();
	}
	
	public ArrayList<String> getSeqs()
	{
		return this.seqs;
	}
	
	public static String truck(String seq)
	{
		String newSeq = "";
		for(int i=0;i<seq.length();i++)
		{
			if(ResName.isResName(seq.charAt(i)))
			{
				newSeq = newSeq+seq.charAt(i);
			}
		}
		return newSeq;
	}
	
	public static String truckBySeqA(String seqA, String seqB)
	{
		String newSeq = "";
		for(int i=0;i<seqA.length();i++)
		{
			if(ResName.isResName(seqA.charAt(i)))
			{
				newSeq = newSeq+seqB.charAt(i);
			}
		}
		return newSeq;
	}
	
	public ArrayList<String> getMSASeqsFromDaliResult(String file) throws IOException
	{
		ArrayList<String> curSeqs = new ArrayList<String>();
		BufferedReader in = new BufferedReader(new FileReader(file));
		String line;
		String seqA="";
		String seqB="";
		while((line = in.readLine())!=null)
		{
			if(line.length() < 5)
				continue;
			if(line.trim().substring(0,3).equals("No "))
			{
				if(seqA.length()>0)
				{
					curSeqs.add(truckBySeqA(seqA,seqB));
				}
				seqA = "";
				seqB = "";
				continue;
			}
			if(line.trim().substring(0, 5).equals("Query"))
			{
				seqA = seqA+line.trim().split(" +")[1].toUpperCase();
			}
			else if(line.trim().substring(0,5).equals("Sbjct"))
			{
				seqB = seqB + line.trim().split(" +")[1].toUpperCase();
			}
		}
		curSeqs.add(truckBySeqA(seqA,seqB));
		curSeqs.add(truck(seqA));
		in.close();
		return curSeqs;
	}
	
	public static double identity(String seqA, String seqB)
	{
		if(seqA.length()!= seqB.length())
			throw new RuntimeException("length not equal: "+seqA.length()+" != "+seqB.length());
		
		int same = 0;
		for(int i=0;i<seqA.length();i++)
		{
			if(seqA.charAt(i)==seqB.charAt(i))
				same++;
		}
		return 1.0*same/seqA.length();
	}
	
	public void creatProfile()
	{
		this.profiles = new ArrayList<ProbabilityArray>();
		
		
		for(int i=0;i<seqs.get(0).length();i++)
		{
			double[] prof = new double[20];
			for(int j=0;j<seqs.size();j++)
			{
				if(!ResName.isResName(seqs.get(j).charAt(i)))
					continue;
				int resType = ResName.sinToInt(seqs.get(j).charAt(i));
				prof[resType] += 1.0;
			}
			this.profiles.add(new ProbabilityArray(prof));
		}
	}
	
	public ArrayList<ProbabilityArray> getProfile()
	{
		return this.profiles;
	}
	
	public double meanIdentity()
	{
		ArrayList<Double> idts = new ArrayList<Double>();
		if(this.seqs.size()==1)
			return 1.0;
		for(int i=0;i<this.seqs.size();i++)
		{
			for(int j=0;j<i;j++)
			{
				idts.add(identity(this.seqs.get(i),this.seqs.get(j)));
			}
		}
		return math.Stat.mean(idts);
	}
	
	private static int maxPType(ProbabilityArray pa)
	{
		double p = 0;
		int type = -1;
		for(int i=0;i<20;i++)
		{
			if(pa.getArray().get(i) > p)
			{
				p = pa.getArray().get(i);
				type = i;
			}
		}
		return type;
	}

	public ArrayList<Double> idtToNat(String seq)
	{
		ArrayList<Double> idtList = new ArrayList<Double>();
		for(int i=0;i<seq.length();i++)
		{
			int natType = ResName.sinToInt(seq.charAt(i));
			idtList.add(this.profiles.get(i).getArray().get(natType));
		}
		return idtList;
	}
	
	public ArrayList<Double> similarityToNat(String natSeq) throws IOException
	{
		ArrayList<Double> simList = new ArrayList<Double>();
		for(String s : this.seqs)
		{
			simList.add(sequence.SeqAlign.similarity(s, natSeq));
		}
		
		return simList;
	}
	
	public double meanSimToNat(String seq) throws IOException
	{
		return math.Stat.mean(similarityToNat(seq));
	}
	
	public double meanIdtToNat(String seq)
	{
		return math.Stat.mean(idtToNat(seq));
	}
	
	public ArrayList<Double> getMaxPList()
	{
		ArrayList<Double> idtList = new ArrayList<Double>();
		int len = this.seqs.get(0).length();
		for(int i=0;i<len;i++)
		{
			int maxPType = maxPType(this.profiles.get(i));
			idtList.add(this.profiles.get(i).getArray().get(maxPType));
		}
		return idtList;
	}
	
	public ArrayList<String> getMaxPTypeList()
	{
		ArrayList<String> maxPTypeList = new ArrayList<String>();
		int len = this.seqs.get(0).length();
		for(int i=0;i<len;i++)
		{
			int maxPType = maxPType(this.profiles.get(i));
			maxPTypeList.add(ResName.intToTri(maxPType));
		}
		return maxPTypeList;
	}
	
	public static ArrayList<String> getTruncatedMSA(ArrayList<String> msa)
	{
		ArrayList<String> tMSA=new ArrayList<String>();
		boolean firstRes=true;
		{
			for(int i=0;i<msa.get(0).length();i++)
			{
				if(msa.get(0).charAt(i) != '-')
				{
					if(firstRes)
					{
						for(int j=0;j<msa.size();j++)
							tMSA.add(msa.get(j).charAt(i)+"");
						firstRes=false;
					}
					else 
					{
						for(int j=0;j<msa.size();j++)
							tMSA.set(j, tMSA.get(j)+msa.get(j).charAt(i));
					}
					
				}
			}
		}
		return tMSA;
	}
	
	public HashMap<Integer,String> conservedSites(double cutoff)
	{
		int seqNum = this.seqs.size();
		int cutOffNum = (int)Math.floor(seqNum*cutoff);
		HashMap<Integer,String> csSites = new HashMap<Integer,String>();
		for(int i=0;i<this.seqLength();i++)
		{
			int[] pa = new int[20];
			for(int j=0;j<seqNum;j++)
			{
				int resType = ResName.sinToInt(this.seqs.get(j).charAt(i));
				pa[resType] ++;
			}
			for(int k=0;k<20;k++)
			{
				if(pa[k] > cutOffNum)
					csSites.put(i, ResName.intToTri(k));
			}
		}
		return csSites;
	}
	
	public HashMap<Integer,String> conservedNatSites(double cutoff,String natSeq)
	{
		HashMap<Integer,String> csSites = conservedSites(cutoff);
		HashMap<Integer,String> csNatSite = new HashMap<Integer,String>();
		
		for(Integer i : csSites.keySet())
		{
			if(ResName.triToSin(csSites.get(i)) == natSeq.charAt(i))
			{
				csNatSite.put(i, csSites.get(i));
			}
		}
		return csNatSite;
		
	}
	
	public MSA subMSA(ArrayList<Integer> sites)
	{
		int seqNum = this.seqs.size();
		ArrayList<String> subSeqs = new ArrayList<String>();
		for(int i=0;i<seqNum;i++)
		{
			
			String s = "";
			for(int j=0;j<sites.size();j++)
			{
				s = s+this.seqs.get(i).charAt(sites.get(j));
			}
			subSeqs.add(s);
		}
		return new MSA(subSeqs);
	}
	
	public static void main(String[] args) throws IOException
	{
		PrintStream out = new PrintStream("E:\\pairwiseDesign\\testPDB4\\1fhl\\daliMSA.txt");
		MSA msa = new MSA("E:\\pairwiseDesign\\testPDB4\\1fhl\\dali.txt","dali");
		ArrayList<String> seqs = msa.seqs;
		for(String s : seqs)
		{
			out.println(s);
		}
		out.close();
	}
}
