package model;
import java.io.*;
import java.util.*;
import java.util.regex.Pattern;

public class PDB 
{
	private String PDBid;
	private ArrayList<ProteinChain> chains;
	private ArrayList<Atom> DNAatoms;
	private ArrayList<Atom> HETatoms;
	private String expData="unknow";
	
	public static Atom lineToAtom(String line)
	{
		Atom atom=new Atom();
		atom.setName(line.substring(12,16).trim());
		if(line.charAt(12)=='H')
			atom.setType("H");
		else
			atom.setType(line.substring(12,14).trim());
		float x=Float.parseFloat(line.substring(30,38).trim());
		float y=Float.parseFloat(line.substring(38,46).trim());
		float z=Float.parseFloat(line.substring(46,54).trim());
		atom.setCoordinate(x,y,z);
		return atom;
	}
	
	public PDB(String filename) throws IOException
	{
		this(filename, filename.substring(filename.length()-8,filename.length()-4));
	}
	
	public PDB(String filename,String pdbId) throws IOException
	{
		BufferedReader in=new BufferedReader(new FileReader(filename));
		this.PDBid=pdbId;
		this.chains=new ArrayList<ProteinChain>();
		this.DNAatoms=new ArrayList<Atom>();
		this.HETatoms=new ArrayList<Atom>();
		String line=null;
		char lastChainId='@';
		int lastResId=-10;
		int models=0;
		ProteinChain thisChain=new ProteinChain();
		Residue thisResidue=new Residue();
		while((line=in.readLine())!=null)
		{
			if(line.length()<54) 
				continue;
			String head=line.substring(0,6).trim();
			if(head.equals("MODEL"))
				{
//when dealing with NMR structure, only use the first model
					if(models!=0)
						break;
					else 
						models=1;
				}
			
			if(head.equals("EXPDTA"))
			{
				if(Pattern.matches(".*NMR.*", line.toUpperCase()))
				{
					this.expData="NMR";
				}
				else if(Pattern.matches(".*RAY.*",line.toUpperCase()))
				{
					this.expData="XRAY";
				}
			}

			if(!head.equals("ATOM") && !head.equals("HETATM"))
				continue;
//			System.out.println(line);
			String resType=line.substring(17,20).trim();
			if(resType.equals("HOH"))
				continue;
			
			if(resType.equals("DA")||resType.equals("DT")||resType.equals("DG")||resType.equals("DC"))
			{	
				DNAatoms.add(lineToAtom(line));
				continue;
			}
			if(resType.equals("A")||resType.equals("T")||resType.equals("G")||resType.equals("C"))
			{	
				DNAatoms.add(lineToAtom(line));
				continue;
			}
			
//			if(head.equals("HETATM"))	
//			{
//				HETatoms.add(lineToAtom(line));
//				continue;
//			}
////			
			if(!ResName.isResName(resType))
			{
				if(resType.equals("MSE")) 
				{
		//			System.err.println("Warning! MSE in PDB");
		//			continue;
				}
				else 
				{
					HETatoms.add(lineToAtom(line));
					continue;
				}
			}
			
//			System.out.println(line);
			char thisChainId=line.charAt(21);
//			System.out.println(thisChainId);
			if(thisChainId!=lastChainId)
			{
				thisChain=new ProteinChain(this.PDBid,thisChainId);
				this.chains.add(thisChain);
				lastChainId=thisChainId;
				lastResId=-10;
			}
			
			char altLoc = line.charAt(16);
			if(altLoc != ' ' && altLoc != 'A' && altLoc != '1')
				continue;
			int thisResId;
			try
			{
				thisResId=Integer.parseInt(line.substring(22,26).trim());
			}
			catch(NumberFormatException exception)
			{
				continue;
			}
			
			if(thisResId!=lastResId)
			{
				thisResidue=new Residue(resType,thisResId);
				thisResidue.setChainID(thisChainId);
				thisChain.addResidue(thisResidue);
				lastResId=thisResId;
			}
			
			thisResidue.addAtom(lineToAtom(line));
			
		}	
		
		for(ProteinChain pc : this.chains)
		{
			pc.computeSideChain();
		}
		in.close();
	}
	
	public ArrayList<Atom> getHetAtoms()
	{
		return this.HETatoms;
	}
	
	public String getName()
	{
		return this.PDBid;
	}
	
	public ArrayList<ProteinChain> getChains()
	{
		return this.chains;
	}
	
	public ArrayList<Atom> getIons()
	{
		ArrayList<Atom> ions = new ArrayList<Atom>();
		HashSet<String> allIons = new HashSet<String>();
		allIons.add("GD");
		allIons.add("F");
		allIons.add("LU");
		allIons.add("HO");
		allIons.add("BR");
		allIons.add("O");
		allIons.add("CA");
		allIons.add("K");
		allIons.add("MG");
		allIons.add("ZN");
		allIons.add("W");
		allIons.add("XE");
		allIons.add("HG");
		allIons.add("SE");
		allIons.add("FE");
		allIons.add("CU");
		allIons.add("CS");
		allIons.add("AU");
		allIons.add("MN");
		allIons.add("IR");
		allIons.add("MO");
		allIons.add("SM");
		allIons.add("BA");
		allIons.add("SR");
		allIons.add("YB");
		allIons.add("LA");
		allIons.add("PT");
		allIons.add("NI");
		allIons.add("AG");
		allIons.add("PR");
		allIons.add("CD");
		allIons.add("LI");
		allIons.add("CO");
		allIons.add("NA");
		allIons.add("PB");
		allIons.add("CL");
		
		for(Atom s : this.HETatoms)
		{
			if(allIons.contains(s.getType()))
				ions.add(s);
		}
		
		return ions;
	}
	
	public ProteinChain getChain(char c)
	{
		for(ProteinChain pc : this.chains)
		{
			if(pc.getChainId() == c)
				return pc;
		}
		return null;
	}
	
	public ProteinChain getFirstChain()
	{
		return this.chains.get(0);
	}
	
	public String getFirstSeq()
	{
		return getFirstChain().getSequence();
	}
	
	public boolean containsDNA()
	{
		return(!this.DNAatoms.isEmpty());
	}
	
	public boolean containsHETATOM()
	{
		return (!this.HETatoms.isEmpty());
	}
	
	public ArrayList<String> getSequence()
	{
		ArrayList<String> seqs=new ArrayList<String>();
		for(ProteinChain chain : this.getChains())
		{
			String s="";
			for(Residue res : chain.getResidue())
			{
				s=s+ResName.triToSin(res.getName());
			}
			seqs.add(s);
		}
		return seqs;
	}
	
//return the atom list of amino acid
	public ArrayList<Atom> getAtoms()
	{
		ArrayList<Atom> atoms=new ArrayList<Atom>();
		for(ProteinChain chain : this.chains)
		{
			for(Residue res : chain.getResidue())
			{
				atoms.addAll(res.getAtoms().values());
			}
		}
		return atoms;
			
	}
	
	public String expDateType()
	{
		return this.expData;
	}

	public static void main(String[] args) throws IOException
	{
		PDB pdb = new PDB(args[0]);
		String seq = pdb.getSequence().get(0);
		System.out.println(seq);
/*
		Atom n = PDB.lineToAtom("ATOM    136  N   MED A  20      19.348  11.060  48.060");
		Atom ca = PDB.lineToAtom("ATOM    135  CA  MED A  20      20.359  10.160  47.526");
		Atom cb = PDB.lineToAtom("ATOM    137  CB  MED A  20      21.434   9.409  49.031");
		Atom cg = PDB.lineToAtom("ATOM    138  CG  MED A  20      23.355   9.917  48.799");
		
		double len = cb.distance(cg);
		System.out.println(len);
		Tern cacb = cb.getCoordinate().subtract(ca.getCoordinate());
		Tern cbcg = cg.getCoordinate().subtract(cb.getCoordinate());
		double ang = cacb.angle(cbcg);
		double dihedral = math.Dihedral.dihedral(n.getCoordinate(), ca.getCoordinate(), cb.getCoordinate(), cg.getCoordinate());
		
		System.out.println(ang + " " + dihedral);
*/
	}
	
	
	

}
