package model;
import java.util.*;
import java.io.*;

public class PPInterface {
	

	
	private ArrayList<InterfaceResidue> piResListA = new ArrayList<InterfaceResidue>();
	private ArrayList<InterfaceResidue> piResListB = new ArrayList<InterfaceResidue>();
	private ArrayList<InterfaceResidue> piResList = new ArrayList<InterfaceResidue>();
	private ArrayList<InterfacePair> interChainPairs = new ArrayList<InterfacePair>();
	private ArrayList<InterfacePair> intraChainPairs = new ArrayList<InterfacePair>();

	

	
	public PPInterface(ProteinChain pcA, ProteinChain pcB, StrideProtein spMonomerA, StrideProtein spMonomerB,StrideProtein spDimer, double cutoff)
	{
		int lenA = pcA.getChainLength();
		int lenB = pcB.getChainLength();
		float[][] distanceMatrix = new float[lenA][lenB];
		for(int i=0;i<lenA;i++)
		{
			Residue resA = pcA.getResidue().get(i);
			boolean resAOK = resA.cbInfoSufficient() && ResName.isResName(resA.getName());
			for(int j=0;j<lenB;j++)
			{
				Residue resB = pcB.getResidue().get(j);
				boolean resBOK = resB.cbInfoSufficient()  && ResName.isResName(resB.getName());
				if(!resAOK || !resBOK)
					distanceMatrix[i][j] = 100.0f;
				else
					distanceMatrix[i][j] = resA.getExpectedCOG2().distance(resB.getExpectedCOG2());
			}
		}
		
		
		for(int i=0;i<lenA;i++)
		{
			float minD = 100.0f;
			for(int j=0;j<lenB;j++)
			{
				if(distanceMatrix[i][j] < minD)
					minD = distanceMatrix[i][j];
			}
			if(minD < cutoff)
			{
				Residue res = pcA.getResidue().get(i);
				StrideRes srM = spMonomerA.getStrideResidue(""+res.getChainID()+res.getID());
				StrideRes srD = spDimer.getStrideResidue("" + res.getChainID() + res.getID());
				if(srM == null || srD == null)
					continue;
				InterfaceResidue irA = new InterfaceResidue(res,srM,srD,pcA.getPDBId(),minD);
				this.piResList.add(irA);
				this.piResListA.add(irA);
			}
		}
		
		for(int j=0;j<lenB;j++)
		{
			float minD = 100.0f;
			for(int i=0;i<lenA;i++)
			{
				if(distanceMatrix[i][j] < minD)
					minD = distanceMatrix[i][j];
			}
			if(minD < cutoff)
			{
				Residue res = pcB.getResidue().get(j);
				StrideRes srM = spMonomerB.getStrideResidue(""+res.getChainID()+res.getID());
				StrideRes srD = spDimer.getStrideResidue("" + res.getChainID() + res.getID());
				if(srM == null || srD == null)
					continue;
				InterfaceResidue irB = new InterfaceResidue(res,srM,srD,pcB.getPDBId(),minD);
				this.piResListB.add(irB);
				this.piResList.add(irB);
			}
		}
		
		for(int i=0;i<piResListA.size();i++)
		{
			for(int j=0;j<piResListB.size();j++)
			{
				Residue resA = piResListA.get(i).getResidue();
				Residue resB = piResListB.get(j).getResidue();
				if(!resA.infoSufficient() || ! resB.infoSufficient()) continue;
				double d = resA.getExpectedCOG2().distance(resB.getExpectedCOG2());
				if(d < cutoff)
					this.interChainPairs.add(new InterfacePair(piResListA.get(i),piResListB.get(j)));
			}
		}
		
		for(int i=0;i<piResListA.size();i++)
		{
			for(int j=i+1;j<piResListA.size();j++)
			{
				Residue resA = piResListA.get(i).getResidue();
				Residue resB = piResListA.get(j).getResidue();
				if(!resA.infoSufficient() || ! resB.infoSufficient()) continue;
				double d = resA.getExpectedCOG2().distance(resB.getExpectedCOG2());
				if(d < cutoff)
					this.intraChainPairs.add(new InterfacePair(piResListA.get(i),piResListA.get(j)));
			}
		}
		
		for(int i=0;i<piResListB.size();i++)
		{
			for(int j=i+1;j<piResListB.size();j++)
			{
				Residue resA = piResListB.get(i).getResidue();
				Residue resB = piResListB.get(j).getResidue();
				if(!resA.infoSufficient() || ! resB.infoSufficient()) continue;
				double d = resA.getExpectedCOG2().distance(resB.getExpectedCOG2());
				if(d < cutoff)
					this.intraChainPairs.add(new InterfacePair(piResListB.get(i),piResListB.get(j)));
			}
		}
		
	}
	
	public ArrayList<InterfaceResidue> getResList()
	{
		return this.piResList;
	}
	
	public ArrayList<InterfaceResidue> getResListA()
	{
		return this.piResListA;
	}
	
	public ArrayList<InterfaceResidue> getResListB()
	{
		return this.piResListB;
	}
	
	public static void showWhatsMore(ArrayList<InterfaceResidue> irA, ArrayList<InterfaceResidue> irB)
	{
		HashSet<String> setA = new HashSet<String>();
		for(InterfaceResidue ir : irA)
		{
			setA.add(ir.toString());
		}
		
		for(InterfaceResidue ir : irB)
		{
			String s = ir.toString();
			if(setA.contains(s)) continue;
			System.out.println(s);
		}
	}
	
	public ArrayList<InterfacePair> getInterPairs()
	{
		return this.interChainPairs;
	}
	
	public ArrayList<InterfacePair> getIntraPairs()
	{
		return this.intraChainPairs;
	}
	
	public ArrayList<InterfacePair> getPairs()
	{
		ArrayList<InterfacePair> ips = new ArrayList<InterfacePair>();
		for(InterfacePair ip : this.interChainPairs)
			ips.add(ip);
		for(InterfacePair ip : this.intraChainPairs)
			ips.add(ip);
		return ips;
	}
	
	public static void main(String[] args) throws IOException
	{
		String dir = "E:/Git/java/tmp/";
		PDB pdb = new PDB(dir + "1c03.pdb");
		ProteinChain pcA = pdb.getChain('B');
		ProteinChain pcB = pdb.getChain('D');
		StrideProtein spMA = new StrideProtein(dir + "1c03B.str");
		StrideProtein spMB = new StrideProtein(dir + "1c03D.str");
		StrideProtein spD = new StrideProtein(dir + "1c03.str");
		
		PPInterface ppi6 = new PPInterface(pcA,pcB,spMA,spMB,spD,6.0);
		ArrayList<InterfaceResidue> irs6 = ppi6.piResList;
		
		
		PPInterface ppi7 = new PPInterface(pcA,pcB,spMA,spMB,spD,7.0);
		ArrayList<InterfaceResidue> irs7 = ppi7.piResList;
		
		PPInterface ppi8 = new PPInterface(pcA,pcB,spMA,spMB,spD,8.0);
		ArrayList<InterfaceResidue> irs8 = ppi8.piResList;
		
		PPInterface ppi9 = new PPInterface(pcA,pcB,spMA,spMB,spD,9.0);
		ArrayList<InterfaceResidue> irs9 = ppi9.piResList;
		
		showWhatsMore(irs8,irs9);
		
		
	} 
	
}
