package model;
import java.util.*;
import java.io.*;

public class SideChain 
{
	private ArrayList<Double> chiList = new ArrayList<Double>();
	private ArrayList<Integer> dunClassList = new ArrayList<Integer>();
	private HashMap<String,Atom> atoms=new HashMap<String,Atom>();
	private String triName;
	private boolean atomSufficient = false;
	
	public SideChain(Residue res)
	{
		this.atoms = res.getAtoms();
		this.triName = res.getName();
		this.atomSufficient = checkAtoms();

		computeChi();
		getDunbrackClass();
	}
	
	public boolean checkAtoms()
	{
		if(this.triName.equals("GLY"))
		{
			return atoms.containsKey("N")&&atoms.containsKey("CA");
		}
		
		if(!(atoms.containsKey("N")&&atoms.containsKey("CA")&&atoms.containsKey("CB")))
			return false;
		
		if(this.triName.equals("VAL"))
		{
			return atoms.containsKey("CG1");
		}
		else if(this.triName.equals("LEU"))
		{
			return atoms.containsKey("CG")&&atoms.containsKey("CD1");
		}
		else if(this.triName.equals("ILE"))
		{
			return atoms.containsKey("CG1")&&atoms.containsKey("CD1");
		}
		else if(this.triName.equals("MET"))
		{
			return atoms.containsKey("CG")&&atoms.containsKey("SD")&&atoms.containsKey("CE");
		}
		else if(this.triName.equals("PHE"))
		{
			return atoms.containsKey("CG")&&atoms.containsKey("CD1");
		}
		else if(this.triName.equals("ASN"))
		{
			return atoms.containsKey("CG")&&atoms.containsKey("OD1");
		}
		else if(this.triName.equals("GLU"))
		{
			return atoms.containsKey("CG")&&atoms.containsKey("CD")&&atoms.containsKey("OE1");
		}
		else if(this.triName.equals("GLN"))
		{
			return atoms.containsKey("CG")&&atoms.containsKey("CD")&&atoms.containsKey("OE1");
		}
		else if(this.triName.equals("HIS"))
		{
			return atoms.containsKey("CG")&&atoms.containsKey("ND1");
		}
		else if(this.triName.equals("LYS"))
		{
			return atoms.containsKey("CG")&&atoms.containsKey("CD")&&atoms.containsKey("CE")&&atoms.containsKey("NZ");
		}
		else if(this.triName.equals("ARG"))
		{
			return atoms.containsKey("CG")&&atoms.containsKey("CD")&&atoms.containsKey("NE")&&atoms.containsKey("CZ");
		}
		else if(this.triName.equals("ASP"))
		{
			return atoms.containsKey("CG")&&atoms.containsKey("OD1");
		}
		else if(this.triName.equals("ALA"))
		{
			return true;
		}
		else if(this.triName.equals("SER"))
		{
			return atoms.containsKey("OG");
		}
		else if(this.triName.equals("THR"))
		{
			return atoms.containsKey("OG1");
		}
		else if(this.triName.equals("TYR"))
		{
			return atoms.containsKey("CG")&&atoms.containsKey("CD1");
		}
		else if(this.triName.equals("TRP"))
		{
			return atoms.containsKey("CG")&&atoms.containsKey("CD1");
		}
		else if(this.triName.equals("CYS"))
		{
			return atoms.containsKey("SG");
		}
		else if(this.triName.equals("PRO"))
		{
			return atoms.containsKey("CG");
		}
		throw new RuntimeException("unknown residue name: " + this.triName);
	}
	
	public ArrayList<Double> getChiList()
	{
		return this.chiList;
	}
	

	
	public void computeChi()
	{

		if(!atoms.containsKey("N") || !atoms.containsKey("CA") || !atoms.containsKey("CB"))
			return;
		if(this.triName.equals("VAL"))
		{
			if(!atoms.containsKey("CG1"))
				return;
			double chi1 = math.Dihedral.dihedral(atoms.get("N").getCoordinate(), atoms.get("CA").getCoordinate(), atoms.get("CB").getCoordinate(), atoms.get("CG1").getCoordinate());
			this.chiList.add(chi1);
			
		}
		else if(this.triName.equals("LEU"))
		{
			if(!atoms.containsKey("CG"))
				return;
			double chi1 = math.Dihedral.dihedral(atoms.get("N").getCoordinate(), atoms.get("CA").getCoordinate(), atoms.get("CB").getCoordinate(), atoms.get("CG").getCoordinate());
			this.chiList.add(chi1);
			if(!atoms.containsKey("CD1"))
				return;
			double chi2 = math.Dihedral.dihedral(atoms.get("CA").getCoordinate(), atoms.get("CB").getCoordinate(), atoms.get("CG").getCoordinate(), atoms.get("CD1").getCoordinate());
			this.chiList.add(chi2);
		}
		else if(this.triName.equals("ILE"))
		{
			if(!atoms.containsKey("CG1"))
				return;
			double chi1 = math.Dihedral.dihedral(atoms.get("N").getCoordinate(), atoms.get("CA").getCoordinate(), atoms.get("CB").getCoordinate(), atoms.get("CG1").getCoordinate());
			this.chiList.add(chi1);
			if(!atoms.containsKey("CD1"))
				return;
			double chi2 = math.Dihedral.dihedral(atoms.get("CA").getCoordinate(), atoms.get("CB").getCoordinate(), atoms.get("CG1").getCoordinate(), atoms.get("CD1").getCoordinate());
			this.chiList.add(chi2);
			
		}
		else if(this.triName.equals("MET"))
		{
			if(!atoms.containsKey("CG"))
				return;
			double chi1 = math.Dihedral.dihedral(atoms.get("N").getCoordinate(), atoms.get("CA").getCoordinate(), atoms.get("CB").getCoordinate(), atoms.get("CG").getCoordinate());
			this.chiList.add(chi1);
			if(!atoms.containsKey("SD"))
				return;
			double chi2 = math.Dihedral.dihedral(atoms.get("CA").getCoordinate(), atoms.get("CB").getCoordinate(), atoms.get("CG").getCoordinate(), atoms.get("SD").getCoordinate());
			this.chiList.add(chi2);
			if(!atoms.containsKey("CE"))
				return;
			double chi3 = math.Dihedral.dihedral(atoms.get("CB").getCoordinate(), atoms.get("CG").getCoordinate(), atoms.get("SD").getCoordinate(), atoms.get("CE").getCoordinate());
			this.chiList.add(chi3);
		}
		else if(this.triName.equals("PHE"))
		{
			if(!atoms.containsKey("CG"))
				return;
			double chi1 = math.Dihedral.dihedral(atoms.get("N").getCoordinate(), atoms.get("CA").getCoordinate(), atoms.get("CB").getCoordinate(), atoms.get("CG").getCoordinate());
			this.chiList.add(chi1);
			if(!atoms.containsKey("CD1"))
				return;
			double chi2 = math.Dihedral.dihedral(atoms.get("CA").getCoordinate(), atoms.get("CB").getCoordinate(), atoms.get("CG").getCoordinate(), atoms.get("CD1").getCoordinate());
			this.chiList.add(chi2);
		}
		else if(this.triName.equals("ASN"))
		{
			if(!atoms.containsKey("CG"))
				return;
			double chi1 = math.Dihedral.dihedral(atoms.get("N").getCoordinate(), atoms.get("CA").getCoordinate(), atoms.get("CB").getCoordinate(), atoms.get("CG").getCoordinate());
			this.chiList.add(chi1);
			if(!atoms.containsKey("OD1"))
				return;
			double chi2 = math.Dihedral.dihedral(atoms.get("CA").getCoordinate(), atoms.get("CB").getCoordinate(), atoms.get("CG").getCoordinate(), atoms.get("OD1").getCoordinate());
			this.chiList.add(chi2);
		}
		else if(this.triName.equals("GLU"))
		{
			if(!atoms.containsKey("CG"))
				return;
			double chi1 = math.Dihedral.dihedral(atoms.get("N").getCoordinate(), atoms.get("CA").getCoordinate(), atoms.get("CB").getCoordinate(), atoms.get("CG").getCoordinate());
			this.chiList.add(chi1);
			if(!atoms.containsKey("CD"))
				return;
			double chi2 = math.Dihedral.dihedral(atoms.get("CA").getCoordinate(), atoms.get("CB").getCoordinate(), atoms.get("CG").getCoordinate(), atoms.get("CD").getCoordinate());
			this.chiList.add(chi2);
			if(!atoms.containsKey("OE1"))
				return;
			double chi3 = math.Dihedral.dihedral(atoms.get("CB").getCoordinate(), atoms.get("CG").getCoordinate(), atoms.get("CD").getCoordinate(), atoms.get("OE1").getCoordinate());
			this.chiList.add(chi3);
		}
		else if(this.triName.equals("GLN"))
		{
			if(!atoms.containsKey("CG"))
				return;
			double chi1 = math.Dihedral.dihedral(atoms.get("N").getCoordinate(), atoms.get("CA").getCoordinate(), atoms.get("CB").getCoordinate(), atoms.get("CG").getCoordinate());
			this.chiList.add(chi1);
			if(!atoms.containsKey("CD"))
				return;
			double chi2 = math.Dihedral.dihedral(atoms.get("CA").getCoordinate(), atoms.get("CB").getCoordinate(), atoms.get("CG").getCoordinate(), atoms.get("CD").getCoordinate());
			this.chiList.add(chi2);
			if(!atoms.containsKey("OE1"))
				return;
			double chi3 = math.Dihedral.dihedral(atoms.get("CB").getCoordinate(), atoms.get("CG").getCoordinate(), atoms.get("CD").getCoordinate(), atoms.get("OE1").getCoordinate());
			this.chiList.add(chi3);
		}
		else if(this.triName.equals("HIS"))
		{
			if(!atoms.containsKey("CG"))
				return;
			double chi1 = math.Dihedral.dihedral(atoms.get("N").getCoordinate(), atoms.get("CA").getCoordinate(), atoms.get("CB").getCoordinate(), atoms.get("CG").getCoordinate());
			this.chiList.add(chi1);
			if(!atoms.containsKey("ND1"))
				return;
			double chi2 = math.Dihedral.dihedral(atoms.get("CA").getCoordinate(), atoms.get("CB").getCoordinate(), atoms.get("CG").getCoordinate(), atoms.get("ND1").getCoordinate());
			this.chiList.add(chi2);
		}
		else if(this.triName.equals("LYS"))
		{
			if(!atoms.containsKey("CG"))
				return;
			double chi1 = math.Dihedral.dihedral(atoms.get("N").getCoordinate(), atoms.get("CA").getCoordinate(), atoms.get("CB").getCoordinate(), atoms.get("CG").getCoordinate());
			this.chiList.add(chi1);
			if(!atoms.containsKey("CD"))
				return;
			double chi2 = math.Dihedral.dihedral(atoms.get("CA").getCoordinate(), atoms.get("CB").getCoordinate(), atoms.get("CG").getCoordinate(), atoms.get("CD").getCoordinate());
			this.chiList.add(chi2);
			if(!atoms.containsKey("CE"))
				return;
			double chi3 = math.Dihedral.dihedral(atoms.get("CB").getCoordinate(), atoms.get("CG").getCoordinate(), atoms.get("CD").getCoordinate(), atoms.get("CE").getCoordinate());
			this.chiList.add(chi3);
			if(!atoms.containsKey("NZ"))
				return;
			double chi4 = math.Dihedral.dihedral(atoms.get("CG").getCoordinate(), atoms.get("CD").getCoordinate(), atoms.get("CE").getCoordinate(), atoms.get("NZ").getCoordinate());
			this.chiList.add(chi4);
		}
		else if(this.triName.equals("ARG"))
		{
			if(!atoms.containsKey("CG"))
				return;
			double chi1 = math.Dihedral.dihedral(atoms.get("N").getCoordinate(), atoms.get("CA").getCoordinate(), atoms.get("CB").getCoordinate(), atoms.get("CG").getCoordinate());
			this.chiList.add(chi1);
			if(!atoms.containsKey("CD"))
				return;
			double chi2 = math.Dihedral.dihedral(atoms.get("CA").getCoordinate(), atoms.get("CB").getCoordinate(), atoms.get("CG").getCoordinate(), atoms.get("CD").getCoordinate());
			this.chiList.add(chi2);
			if(!atoms.containsKey("NE"))
				return;
			double chi3 = math.Dihedral.dihedral(atoms.get("CB").getCoordinate(), atoms.get("CG").getCoordinate(), atoms.get("CD").getCoordinate(), atoms.get("NE").getCoordinate());
			this.chiList.add(chi3);
			if(!atoms.containsKey("CZ"))
				return;
			double chi4 = math.Dihedral.dihedral(atoms.get("CG").getCoordinate(), atoms.get("CD").getCoordinate(), atoms.get("NE").getCoordinate(), atoms.get("CZ").getCoordinate());
			this.chiList.add(chi4);
		}
		else if(this.triName.equals("ASP"))
		{
			if(!atoms.containsKey("CG"))
				return;
			double chi1 = math.Dihedral.dihedral(atoms.get("N").getCoordinate(), atoms.get("CA").getCoordinate(), atoms.get("CB").getCoordinate(), atoms.get("CG").getCoordinate());
			this.chiList.add(chi1);
			if(!atoms.containsKey("OD1"))
				return;
			double chi2 = math.Dihedral.dihedral(atoms.get("CA").getCoordinate(), atoms.get("CB").getCoordinate(), atoms.get("CG").getCoordinate(), atoms.get("OD1").getCoordinate());
			this.chiList.add(chi2);
		}
		else if(this.triName.equals("SER"))
		{
			if(!atoms.containsKey("OG"))
				return;
			double chi1 = math.Dihedral.dihedral(atoms.get("N").getCoordinate(), atoms.get("CA").getCoordinate(), atoms.get("CB").getCoordinate(), atoms.get("OG").getCoordinate());
			this.chiList.add(chi1);
		}
		else if(this.triName.equals("THR"))
		{
			if(!atoms.containsKey("OG1"))
				return;
			double chi1 = math.Dihedral.dihedral(atoms.get("N").getCoordinate(), atoms.get("CA").getCoordinate(), atoms.get("CB").getCoordinate(), atoms.get("OG1").getCoordinate());
			this.chiList.add(chi1);
		}
		else if(this.triName.equals("TYR"))
		{
			if(!atoms.containsKey("CG"))
				return;
			double chi1 = math.Dihedral.dihedral(atoms.get("N").getCoordinate(), atoms.get("CA").getCoordinate(), atoms.get("CB").getCoordinate(), atoms.get("CG").getCoordinate());
			this.chiList.add(chi1);
			if(!atoms.containsKey("CD1"))
				return;
			double chi2 = math.Dihedral.dihedral(atoms.get("CA").getCoordinate(), atoms.get("CB").getCoordinate(), atoms.get("CG").getCoordinate(), atoms.get("CD1").getCoordinate());
			this.chiList.add(chi2);
		}
		else if(this.triName.equals("TRP"))
		{
			if(!atoms.containsKey("CG"))
				return;
			double chi1 = math.Dihedral.dihedral(atoms.get("N").getCoordinate(), atoms.get("CA").getCoordinate(), atoms.get("CB").getCoordinate(), atoms.get("CG").getCoordinate());
			this.chiList.add(chi1);
			if(!atoms.containsKey("CD1"))
				return;
			double chi2 = math.Dihedral.dihedral(atoms.get("CA").getCoordinate(), atoms.get("CB").getCoordinate(), atoms.get("CG").getCoordinate(), atoms.get("CD1").getCoordinate());
			this.chiList.add(chi2);
		}
		else if(this.triName.equals("CYS"))
		{
			if(!atoms.containsKey("SG"))
				return;
			double chi1 = math.Dihedral.dihedral(atoms.get("N").getCoordinate(), atoms.get("CA").getCoordinate(), atoms.get("CB").getCoordinate(), atoms.get("SG").getCoordinate());
			this.chiList.add(chi1);
		}
		else if(this.triName.equals("PRO"))
		{
			if(!atoms.containsKey("CG"))
				return;
			double chi1 = math.Dihedral.dihedral(atoms.get("N").getCoordinate(), atoms.get("CA").getCoordinate(), atoms.get("CB").getCoordinate(), atoms.get("CG").getCoordinate());
			this.chiList.add(chi1);
		}
	}
	
	public int dunClass(double chi)
	{
		if(chi >=0 && chi < 120)
			return 1;
		if(chi <0 && chi >= -120)
			return 3;
		return 2;
	}
	
	public void getDunbrackClass()
	{
		if(this.triName.equals("ASN") || this.triName.equals("ASP"))
		{
			if(chiList.size()==0)
				return;
			dunClassList.add(dunClass(chiList.get(0)));
			if(chiList.size()==1)
				return;
			if(chiList.get(1) >= 30 && chiList.get(1) < 90)
				dunClassList.add(1);
			else if(chiList.get(1) >= -30 && chiList.get(1) < 30)
				dunClassList.add(2);
			else if(chiList.get(1) >= -90 && chiList.get(1) < -30)
				dunClassList.add(3);
			else if(chiList.get(1) >= -150 && chiList.get(1) < -90)
				dunClassList.add(4);
			else if(chiList.get(1) <-150 || chiList.get(1) >= 150)
				dunClassList.add(5);
			else if(chiList.get(1) >= 90 && chiList.get(1) < 150)
				dunClassList.add(6);
		}
		
		else if(this.triName.equals("GLN") || this.triName.equals("GLU"))
		{
			if(chiList.size()==0)
				return;
			dunClassList.add(dunClass(chiList.get(0)));
			if(chiList.size()==1)
				return;
			dunClassList.add(dunClass(chiList.get(1)));
			if(chiList.size()==2)
				return;
			if(chiList.get(2) >= 30 && chiList.get(2) < 90)
				dunClassList.add(1);
			else if(chiList.get(2) >= -30 && chiList.get(2) < 30)
				dunClassList.add(2);
			else if(chiList.get(2) >= -90 && chiList.get(2) < -30)
				dunClassList.add(3);
			else if(chiList.get(2) >= -150 && chiList.get(2) < -90)
				dunClassList.add(4);
			else if(chiList.get(2) <-150 || chiList.get(2) >= 150)
				dunClassList.add(5);
			else if(chiList.get(2) >= 90 && chiList.get(2) < 150)
				dunClassList.add(6);
		}
		else if(this.triName.equals("PHE") || this.triName.equals("TYR") || this.triName.equals("HIS"))
		{
			if(chiList.size()==0)
				return;
			dunClassList.add(dunClass(chiList.get(0)));
			if(chiList.size()==1)
				return;
			if(chiList.get(1)>=30 && chiList.get(1)<150)
				dunClassList.add(1);
			else if(chiList.get(1)>=-30 && chiList.get(1) <30)
				dunClassList.add(2);
			else 
				dunClassList.add(3);
		}
		else if(this.triName.equals("TRP"))
		{
			if(chiList.size()==0)
				return;
			dunClassList.add(dunClass(chiList.get(0)));
			if(chiList.size()==1)
				return;
			if(chiList.get(1)>= -180 && chiList.get(1)<-60)
				dunClassList.add(1);
			else if(chiList.get(1)>= -60 && chiList.get(1) < 60)
				dunClassList.add(2);
			else 
				dunClassList.add(3);
		}
		else if(this.triName.equals("PRO"))
		{
			if(chiList.size()==0)
				return;
			if(chiList.get(0)>=0 && chiList.get(0) < 90)
				dunClassList.add(1);
			else if(chiList.get(0) >= -90 && chiList.get(0) < 0)
				dunClassList.add(2);
		}
		else if(this.triName.equals("ALA") || this.triName.equals("GLY"))
		{
			dunClassList.add(1);
		}
		else 
		{
			for(int i=0;i<chiList.size();i++)
			{
				dunClassList.add(dunClass(chiList.get(i)));
			}
		}
	}
	
	public static void test() throws IOException
	{
		PDB pdb1 = new PDB("E:\\pairwiseDesign\\testPDB2\\mod2\\native.pdb");
 		System.out.println(pdb1.getSequence().get(0));
 		
 		Residue res = pdb1.getChains().get(0).getResidue().get(3);
 		SideChain sc = new SideChain(res);
 		ArrayList<Double> chis = sc.chiList;
 		for(int i=0;i<chis.size();i++)
 		{
 			System.out.printf("%7.2f\n",chis.get(i));
 		}
	}
	
	public ArrayList<Integer> getClassID()
	{
		return this.dunClassList;
	}
	
	public String getTriName()
	{
		return this.triName;
	}
	
	public String getDunbrackCode()
	{
		String s = this.triName;
		for(int i=0;i<this.dunClassList.size();i++)
		{
			s = s + this.dunClassList.get(i);
		}
		for(int i=this.dunClassList.size();i<4;i++)
		{
			s = s + "0";
		}
		return s;
	}
	

	public static void main(String[] args) throws IOException
	{
/*
 * 		PDB pdb = new PDB("D:\\download\\2JAL.pdb");
		for(Residue res : pdb.getChains().get(0).getResidue())
		{
			SideChain sdc = new SideChain(res);
			if(sdc.atomSufficient)
			{
				if(sdc.dumClassList.size() == 0)
				{
					System.out.println(res.getID() + res.getName());
					continue;
				}
				for(int i=0;i<sdc.dumClassList.size();i++)
				{
//					System.out.print(sdc.dumClassList.get(i)+" ");
				}
//				System.out.println();
			}	
		}
*/		
		test();
	}
}
