package model;
import similarSearch.ResCharacter;
import similarSearch.ResPara;

public class StrideRes
{
	public String resName;
	public char chainID;
	public int resID;
	public int resIDSD=-1;
	public int resNewID;
	public char ssType;
	public float phi;
	public float psi;
	public float area;
	private float areaN;
	
	public StrideRes()
	{}
	
	public StrideRes(char ssType,float phi,float psi,float area)
	{
		this.ssType=ssType;
		this.phi=phi;
		this.psi=psi;
		this.area=area;
		this.areaN = similarSearch.ResCharacter.sasaAreaNormalize(this.area);
	}
	
	public StrideRes(float areaN, char ssType,float phi,float psi )
	{
		this.ssType=ssType;
		this.phi=phi;
		this.psi=psi;
		this.areaN=areaN;
	}
	
	public StrideRes(String resName,char chainID,int resID,char ssType,float phi,float psi,float area)
	{
		this.resName=resName;
		this.chainID=chainID;
		this.resID=resID;
		this.resNewID=resID;
		this.ssType=ssType;
		this.phi=phi;
		this.psi=psi;
		this.area=area;
		this.areaN = similarSearch.ResCharacter.sasaAreaNormalize(this.area);
	}
	
	public StrideRes(String line)
	{
		this.resName=line.substring(5,8);
		this.chainID=line.charAt(9);
		String resIDString = line.substring(11,15).trim();
		if(resIDString.charAt(resIDString.length()-1) >= 'A' && resIDString.charAt(resIDString.length()-1) <= 'Z')
			resIDString = resIDString.substring(0,resIDString.length()-1);
		this.resID=Integer.parseInt(resIDString);
		this.resNewID=Integer.parseInt(line.substring(16,20).trim());
		this.ssType=line.toUpperCase().charAt(24);
		this.phi=Float.parseFloat(line.substring(41,49).trim());
		this.psi=Float.parseFloat(line.substring(51,59).trim());
		this.area=Float.parseFloat(line.substring(63,69));
		this.areaN = similarSearch.ResCharacter.sasaAreaNormalize(this.area);
	}
	
	public static StrideRes buildFromResidueFileLine(String line)
	{
		StrideRes sr=new StrideRes();
		sr.resName=line.substring(12,15);
		sr.resID=Integer.parseInt(line.substring(7,11).trim());
		sr.chainID=line.charAt(5);
		sr.ssType=line.charAt(17);
		sr.phi=Float.parseFloat(line.substring(20,27));
		sr.psi=Float.parseFloat(line.substring(29,36));
		sr.area=Float.parseFloat(line.substring(38,43));
		sr.areaN = similarSearch.ResCharacter.sasaAreaNormalize(sr.area);
		return sr;
	}
	
	public void setResIDSD(int resIDSD)
	{
		this.resIDSD=resIDSD;
	}
	
	public boolean similarTo(StrideRes other,ResPara rPr)
	{
		if(!ResCharacter.simSecondaryStructure(this.ssType, other.ssType, rPr.ssLevel()))
			return false;
		if(!ResCharacter.simSasaArea(this.area, other.area, rPr.sasaLevel()))
			return false;
		if(!rPr.surDif())
		{
			if(!ResCharacter.simPhiPsi(this.phi, this.psi, other.phi, other.psi, rPr.ppLevel()))
				return false;
		}
		else
		{
			if(!ResCharacter.simPhiPsi(this.phi, this.psi, other.phi, other.psi, rPr.ppLevel(this.area)))
				return false;
		}
		return true;
	}
	
	public String toString()
	{
		String line=" "+this.ssType+String.format("%9.2f", phi);
		line=line+String.format("%9.2f", psi)+String.format("%7.1f", area);
		return line;
	}
	
	
	public float getAreaNormalized()
	{
		return this.areaN;
	}
	
	public static void main(String[] args)
	{
		for(float a = 113;a<114;a+=0.01)
		{
			System.out.println(a + " " +similarSearch.ResCharacter.sasaAreaNormalize(a));
		}
		
	}
}