package model;

import java.io.IOException;
import java.io.Serializable;
import java.lang.Math;
import java.util.ArrayList;

import math.Matrix;

public final class Tern implements Serializable
{
	/**
	 * 
	 */
	private static final long serialVersionUID = -2581560537312439794L;
	private float x;
	private float y;
	private float z;
	
	public Tern()
	{
		this.x=0;
		this.y=0;
		this.z=0;
	}

	
	public Tern(float x,float y,float z)
	{
		this.x=x;
		this.y=y;
		this.z=z;
	}
	
	public Tern(double x,double y,double z)
	{
		this.x=(float)x;
		this.y=(float)y;
		this.z=(float)z;
	}
	
	public static final Tern ZERO = new Tern(0,0,0);
	
	public boolean equals(Tern other)
	{
		if(this.x==other.x && this.y==other.y && this.z==other.z)
			return true;
		else
			return false;
	}
	
	public float x()
	{
		return this.x;
	}
	
	public float y()
	{
		return this.y;
	}
	
	public float z()
	{
		return this.z;
	}
	
    public float get(int i)
    {
        if(i==0) return x;
        if(i==1) return y;
        if(i==2) return z;
        throw new IllegalArgumentException("index must be 0, 1 or 2");
    }
    
	public Tern add(Tern other)
	{
		Tern newTern=new Tern();
		newTern.x=this.x+other.x;
		newTern.y=this.y+other.y;
		newTern.z=this.z+other.z;
		return newTern;
	}
	
	public Tern subtract(Tern other)
	{
		Tern newTern=new Tern();
		newTern.x=this.x-other.x;
		newTern.y=this.y-other.y;
		newTern.z=this.z-other.z;
		return newTern;
	}
	
	public Tern multiply(float k)
	{
		Tern newTern=new Tern();
		newTern.x=this.x*k;
		newTern.y=this.y*k;
		newTern.z=this.z*k;
		return newTern;
	}
	
	public Tern divide(float k)
	{
		return this.multiply(1.0f/k);
	}
	
	public float dotProduct(Tern other)
	{
		return (this.x*other.x+this.y*other.y+this.z*other.z);
	}
	
	public float length()
	{
		double x=this.x;
		double y=this.y;
		double z=this.z;
		double len=Math.sqrt(x*x+y*y+z*z);
		return (float)len;
	}
	
	public float distance(Tern other)
	{
		Tern sub=this.subtract(other);
		return sub.length();
	}
	
    public double distSquare(Tern p)
    {
        double dx = x - p.x;
        double dy = y - p.y;
        double dz = z - p.z;
        return dx*dx + dy*dy + dz*dz;
    }
    
	public boolean isZero()
	{
		return (this.x==0&&this.y==0&&this.z==0);
	}
	
    
    public Tern negate()
    {
        return new Tern(-x, -y, -z);
    }
    
	public float angle(Tern other)
	{
		if(this.isZero()||other.isZero())
			return 0;
		double cosAngle=this.dotProduct(other)/this.length()/other.length();
		cosAngle=Math.round(cosAngle*1000000)/1000000.0;
		return (float)(Math.acos(cosAngle)*180/Math.PI);
	}
	
	public Tern cross(Tern other)
	{
		float x0=this.y*other.z-this.z*other.y;                                                                                                                                                                                                                                                                                                                                                  
		float y0=this.z*other.x-this.x*other.z;
		float z0=this.x*other.y-this.y*other.x;
		return new Tern(x0,y0,z0);
	}
	
	public String toString()
	{
		String str=this.x+" "+this.y+" "+this.z;
		return str;
	}
	
	public static Tern average(ArrayList<Tern> ternList)
	{
		double xsum = 0.0;
		double ysum = 0.0;
		double zsum = 0.0;
		int num = ternList.size();
		if(num == 0)
			throw new RuntimeException("empty list");
		for(int i=0;i<ternList.size();i++)
		{
			xsum += ternList.get(i).x;
			ysum += ternList.get(i).y;
			zsum += ternList.get(i).z;
		}
		float ax = (float)(xsum/num);
		float ay = (float)(ysum/num);
		float az = (float)(zsum/num);
		
		return new Tern(ax,ay,az);
	}
	
	public static Tern transformation(Matrix mt, Tern coor)
	{
		double[][] array = mt.getArray();
		if(array.length !=3 || array[0].length != 3) 
			throw new RuntimeException("matrix is not a 3*3 matrix");
		double x = array[0][0]*coor.x + array[0][1]*coor.y + array[0][2]*coor.z;
		double y = array[1][0]*coor.x + array[1][1]*coor.y + array[1][2]*coor.z;
		double z = array[2][0]*coor.x + array[2][1]*coor.y + array[2][2]*coor.z;
		return new Tern((float)x,(float)y,(float)z);
	}
	
	public static void main(String[] args) throws IOException
	{
		Tern a = new Tern(1,0,0);
		Tern b = new Tern(0,1,0);
		Tern c = new Tern(0,0,1);
		Tern x = c.cross(a);
		System.out.println(x.toString());
	}
}