package model;

import java.util.Scanner;

public class VdwAtom {

	public final String forceFiedAtomType;
	public final double vdwSigma;
	public final double vdwWellDepth;
	public final double vdwRadius;
	private double twiceSqrtEpsilon;
	private double sqrtSigma;

//	private double beta = 0.816306927;
//    private double beta6 = 0.2958836;
    
    private double beta = 0.9;
    private double beta6 = 0.531441;
    
    private double coefficient1 = 6.0*(1.0-2.0/beta6) / (beta6*beta);
    private double coefficient2 = (13.0/beta6 - 7.0) / beta6;
	
	
	public VdwAtom(String line)
	{
		Scanner sc = new Scanner(line);
		this.forceFiedAtomType = sc.next();
		sc.next();
		this.vdwSigma = sc.nextDouble();
		this.vdwWellDepth = sc.nextDouble();
		this.vdwRadius = sc.nextDouble();
		sc.close();
		this.twiceSqrtEpsilon = 2.0*Math.sqrt(vdwWellDepth);
		this.sqrtSigma = Math.sqrt(vdwSigma);
	}
	
	private double lennardJonesEnergy(double depth, double sigma6, double distSq)
	{
		double r6 = distSq*distSq*distSq;
		double sigma6overR6 = sigma6/r6;
		double energy = depth*sigma6overR6*(sigma6overR6-1);
		return (energy<1000.0 ? energy : 1000.0);
	}
	
	private double linearEnergy(double depth, double sigma, double r)
	{
		return depth*(r*coefficient1/sigma + coefficient2);
	}
	
	public double vdwEnergy(VdwAtom other, double d)
	{
		double depth = this.twiceSqrtEpsilon * other.twiceSqrtEpsilon;
		double sigma = this.sqrtSigma * other.sqrtSigma;
		double sigma2 = sigma*sigma;
		double sigma6 = sigma2*sigma2*sigma2;
		double turnSq = beta*sigma; //turning point
	    turnSq *= turnSq;
	    double distSq = d * d;
	    
	    double attractionScale = 1.2;
	    
	    if(distSq>=turnSq)
        {
            double energy = lennardJonesEnergy(depth, sigma6, distSq);
            if(distSq<sigma*sigma) return energy;
            else return energy*attractionScale; // attractive
        }
        else return linearEnergy(depth, sigma, Math.sqrt(distSq));
	}
}
