package sequence;
import java.io.IOException;
import java.util.*;

import model.ResName;

public class HenikoffWeight {

	public static double[] getHenikoffWeight(ArrayList<String> seqs)
	{
		int seqNum  = seqs.size();
		int seqLen = seqs.get(0).length();
		double[] wt = new double[seqNum];
		int[][] count = countAA(seqs);
		for(int pos=0;pos<seqLen;pos++)
		{
			int r = nonZeroNum(count, pos);
			for(int seqID=0;seqID<seqNum;seqID++)
			{
				char type = seqs.get(seqID).charAt(pos);
				int s = appearNum(count, pos, type);
				wt[seqID] += 1.0/r/s;
			}
		}
		double max = max(wt);
		double sum = sum(wt);
		for(int i=0;i<seqNum;i++)
		{
			wt[i] = wt[i]/sum;
		}
		return wt;
	}
	
	private static int[][] countAA(ArrayList<String> seqs)
	{
		int seqLen = seqs.get(0).length();
		int[][] count = new int[seqLen][21];
		
		for(int i=0;i<seqs.size();i++)
		{
			for(int j=0;j<seqLen;j++)
			{
				char type = seqs.get(i).charAt(j);
				if(ResName.isResName(type))
				{
					int resType = ResName.sinToInt(type);
					count[j][resType] ++;
				}
				else
				{
					count[j][20] ++;
				}
			}
		}
		return count;
	}
	
	private static int nonZeroNum(int[][] count, int pos)
	{
		int n=0;
		for(int i=0;i<21;i++)
		{
			if(count[pos][i] > 0) n++;
		}
		return n;
		
	}
	
	private static int appearNum(int[][] count, int pos, char c)
	{
		if(ResName.isResName(c))
			return count[pos][ResName.sinToInt(c)];
		else
			return count[pos][20];
	}
	
	private static double max(double[] wt)
	{
		double max = 0.0;
		for(int i=0;i<wt.length;i++)
		{
			if(wt[i] > max)
			{
				max = wt[i];
			}
		}
		return max;
	}
	
	private static double sum(double[] wt)
	{
		double s = 0;
		for(int i=0;i<wt.length;i++)
		{
			s += wt[i];
		}
		return s;
	}
	
	public static void main(String[] args) throws IOException
	{
		ArrayList<String> seqs = new ArrayList<String>();
		seqs.add("GYVGS");
		seqs.add("GYVGS");
		seqs.add("GYVGS");
		seqs.add("GYVGS");
		seqs.add("GYVGS");
		seqs.add("GFDGF");
		seqs.add("GYDGF");
		seqs.add("GYQGG");
		double[] wt = getHenikoffWeight(seqs);
		for(int i=0;i<seqs.size();i++)
		{
			System.out.println(wt[i]);
		}
	}
}
