package sequence;
import java.io.*;
import math.ProbabilityArray;
import model.ResName;

public class Profile {
	private double[] prof = new double[20];
	
	public Profile(ProbabilityArray pa) throws IOException
	{
	//	AccurateBlosum62 bl = new AccurateBlosum62();
		Blosum62 bl = new Blosum62();
		for(int a=0;a<20;a++)
		{
			for(int x=0;x<20;x++)
			{
				prof[a] += pa.getArray().get(x)*bl.getScore(ResName.intToSin(a), ResName.intToSin(x));
			}
		}
	}
	
	public Profile(double[] pa) throws IOException
	{
		Blosum62 bl = new Blosum62();
		for(int a=0;a<20;a++)
		{
			for(int x=0;x<20;x++)
			{
				prof[a] += pa[x]*bl.getScore(ResName.intToSin(a), ResName.intToSin(x));
			}
		}
	}
	
	
	public Profile(ProbabilityArray pa, Blosum62 bl)
	{
		for(int a=0;a<20;a++)
		{
			for(int x=0;x<20;x++)
			{
				prof[a] += pa.getArray().get(x)*bl.getScore(ResName.intToSin(a), ResName.intToSin(x));
			}
		}
	}
	
	public Profile(ProbabilityArray pa, String bloFile) throws IOException
	{
		AccurateBlosum62 bl = new AccurateBlosum62(bloFile);
		for(int a=0;a<20;a++)
		{
			for(int x=0;x<20;x++)
			{
				prof[a] += pa.getArray().get(x)*bl.getScore(ResName.intToSin(a), ResName.intToSin(x));
			}
		}
	}

	
	public Profile(ProbabilityArray pa, boolean newProfile) throws IOException
	{
		Blosum62 bl = new Blosum62();
		for(int i=0;i<20;i++)
		{
			double[] pai = new double[20];
			for(int x=0;x<20;x++)
			{
				pai[x] += Math.exp(bl.getScore(ResName.intToSin(i), ResName.intToSin(x)));
			}
			ProbabilityArray paii = new ProbabilityArray(pai);
	//		paii.printArray(System.out);
			
			for(int x=0;x<20;x++)
			{
				prof[i] += pa.getArray().get(x)*paii.getArray().get(x);
			}
		}
		
		ProbabilityArray pa2 = new ProbabilityArray(prof);
		for(int i=0;i<20;i++)
		{
			prof[i] = Math.log(pa2.getArray().get(i));
		}
		
	}
	
	public double getScore(int type)
	{
		return this.prof[type];
	}
	
	public void dot(double x)
	{
		for(int i=0;i<this.prof.length;i++)
		{
			this.prof[i] = this.prof[i]*x;
		}
	}
	
	public Profile(ProbabilityArray pa, Blosum62 bl, boolean newProfile) throws IOException
	{
		for(int i=0;i<20;i++)
		{
			double[] pai = new double[20];
			for(int x=0;x<20;x++)
			{
				pai[x] += Math.exp(bl.getScore(ResName.intToSin(i), ResName.intToSin(x)));
			}
			ProbabilityArray paii = new ProbabilityArray(pai);
	//		paii.printArray(System.out);
			double[] pai2 = new double[20];
			for(int x=0;x<20;x++)
			{
				prof[i] += pa.getArray().get(x)*paii.getArray().get(x);
			}
		}
		
		ProbabilityArray pa2 = new ProbabilityArray(prof);
		for(int i=0;i<20;i++)
		{
			prof[i] = Math.log(pa2.getArray().get(i));
		}
		
	}
	
	public ProbabilityArray toPA()
	{
		double[] p = new double[20];
		for(int i=0;i<20;i++)
		{
			p[i] = Math.exp(prof[i]);
		}
		return new ProbabilityArray(p);
	}
	
	public static ProbabilityArray addBlosumProf(ProbabilityArray pa, Blosum62 bl) throws IOException
	{
		Profile pf = new Profile(pa,true);
		return pf.toPA();
	}
	
	public void printProfile()
	{
		for(int i=0;i<20;i++)
		{
			System.out.print("  " + ResName.intToTri(i) + "  ");
		}
		System.out.println();
		ProbabilityArray pa = this.toPA();
		for(int i=0;i<20;i++)
		{
			System.out.printf("%6.3f ",prof[i]);
		}
		System.out.println();
		for(int i=0;i<20;i++)
		{
			System.out.printf("%6.3f ",pa.getArray().get(i));
		}
		System.out.println();
	}
	
	public static void main(String[] args) throws IOException
	{
		double[] pBackground={0.0839, 0.0126, 0.0593, 0.0681, 0.0409, 
	              0.0735, 0.0235, 0.0576, 0.0567, 0.0940, 
	              0.0167, 0.0427, 0.0463, 0.0375, 0.0517, 
	              0.0588, 0.0546, 0.0713, 0.0146, 0.0358};
		Profile prof = new Profile(pBackground);
		
		double[] pa = new double[20];
		for(int i=0;i<20;i++)
			pa[i] = 0.05;
		Profile prof2 = new Profile(pa);
		
		
		for(int i=0;i<20;i++)
		{
			double s1 = prof.getScore(i);
			double s2 = prof2.getScore(i);
			System.out.printf("%s %5.2f %5.2f\n",ResName.intToTri(i),s1,s2);
			//System.out.println(prof.getScore(i));
		}
	}
}
