package sequence;


import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.Scanner;

import model.ResName;
import input.InputFile;

public class SubstitutionMatrix {
	
	public final double[][] subM = new double[20][20];
	private double mutRate;

	public SubstitutionMatrix(int idt) throws IOException
	{
		InputFile input = new InputFile();
		String file = input.get("subMatrix") + "sub" + idt + ".mtx";
		BufferedReader in = new BufferedReader(new FileReader(file));
		String line;
		for(int i=0;i<20;i++)
		{
			line = in.readLine();
			Scanner sc = new Scanner(line);
			for(int j=0;j<20;j++)
			{
				subM[i][j] = sc.nextDouble();
			}
			sc.close();
		}
		in.close();
		this.mutRate = calMutRate();
	}
	
	public SubstitutionMatrix(String file) throws IOException
	{
		BufferedReader in = new BufferedReader(new FileReader(file));
		String line;
		for(int i=0;i<20;i++)
		{
			line = in.readLine();
			Scanner sc = new Scanner(line);
			for(int j=0;j<20;j++)
			{
				subM[i][j] = sc.nextDouble();
			}
			sc.close();
		}
		in.close();
		this.mutRate = calMutRate();
	}
	
	private double calMutRate()
	{
		double[] pBackground={0.0839, 0.0126, 0.0593, 0.0681, 0.0409, 
	              0.0735, 0.0235, 0.0576, 0.0567, 0.0940, 
	              0.0167, 0.0427, 0.0463, 0.0375, 0.0517, 
	              0.0588, 0.0546, 0.0713, 0.0146, 0.0358};
		
		double wtRate = 0.0;
		for(int i=0;i<20;i++)
		{
			wtRate += pBackground[i]*subM[i][i];
		}
		return 1.0-wtRate;
	}
	
	public double getMutRate()
	{
		return this.mutRate;
	}
	
	public void printSub()
	{
		System.out.printf("  ");
		for(int i=0;i<20;i++)
			System.out.printf("  %c   ",ResName.intToSin(i));
		System.out.println();
		for(int i=0;i<20;i++)
		{
			System.out.print(ResName.intToSin(i)+" ");
			for(int j=0;j<20;j++)
			{
				double p = this.subM[j][i];
				if(p>0.001)
					System.out.printf("%5.3f ",p);
				else 
				{
			//		System.out.printf("%3.1g ",p);
					int dig = (int)Math.ceil(Math.log10(p)*(-1.0));
					double pp = p*Math.pow(10.0, dig);
					System.out.printf(" %1.0fe-%d ",pp,dig);
				}
			}
			System.out.println();
		}
	}
	
	public static void main(String[] args) 
	{
		double p = 0.0001;
		System.out.printf("%2g ",p);
		System.out.println();
		if(p>0.01)
			System.out.printf("%5.3f ",p);
		else 
		{
		//	System.out.printf("%3.1g ",p);
			int dig = (int)Math.ceil(Math.log10(p)*(-1.0));
			double pp = p*Math.pow(10.0, dig);
			System.out.printf("%1.0fe-%d ",pp,dig);
		}
	}
	
}
