package similarSearch;
import model.*;
import java.io.*;
import java.util.*;
import math.*;
import buildDataFile.SplitResidueFile;
import input.InputFile;

public class FindSimilarPairs 
{
	public static ArrayList<String> getFileList(ResPair rp, PairPara pPr) throws IOException
	{
		InputFile inputFiles = new InputFile();
		
		ArrayList<String> pairFiles= new ArrayList<String>();
		if(Math.abs(rp.seqDistance())>3)
		{
			String path = inputFiles.get("splitBySSandSASA");
	//		System.out.println(path);
			File pairFilePath = new File(path);
			String[] list = pairFilePath.list();
			if(list.length==0)
			{
				System.err.println("no file: "+path);
			}
			for(String s : list)
			{
				char ssTypeA = s.charAt(0);
				char ssTypeB = s.charAt(1);
				float sasaALow = Float.parseFloat(s.substring(2,5));
				float sasaAHigh = sasaALow + 0.1f;
				
				float sasaBLow = Float.parseFloat(s.substring(6,9));
				float sasaBHigh = sasaBLow + 0.1f;
				if(!ResCharacter.simSecondaryStructure(ssTypeA, SplitResidueFile.ssTypeTri(rp.strResA.ssType), pPr.resParaA().ssLevel()))
					continue;
				if(!ResCharacter.simSecondaryStructure(ssTypeB, SplitResidueFile.ssTypeTri(rp.strResB.ssType), pPr.resParaA().ssLevel()))
					continue;
		
				double sasaAN=ResCharacter.sasaAreaNormalize(rp.strResA.area);
				double sasaBN=ResCharacter.sasaAreaNormalize(rp.strResB.area);
				
				
				if(sasaAN > sasaALow && sasaAN < sasaAHigh && sasaBN > sasaBLow && sasaBN < sasaBHigh)
					pairFiles.add(path+s);
				else if(ResCharacter.simSasaN(sasaALow, sasaAN, pPr.resParaA().sasaLevel()) || ResCharacter.simSasaN(sasaAHigh, sasaAN, pPr.resParaA().sasaLevel()))
				{
					if(ResCharacter.simSasaN(sasaBLow, sasaBN, pPr.resParaA().sasaLevel()) || ResCharacter.simSasaN(sasaBHigh, sasaBN, pPr.resParaA().sasaLevel()))
						pairFiles.add(path+s);
				}
					
			}
		}
		else 
		{
			String path = inputFiles.get("neighbor");		
			File pairFilePath = new File(path);
			int seqDis = rp.seqDistance();
			double sasaAN = ResCharacter.sasaAreaNormalize(rp.strResA.area);
			double sasaBN = ResCharacter.sasaAreaNormalize(rp.strResB.area);
			for(String s : pairFilePath.list())
			{
				
				int fileDis = Integer.parseInt(s.substring(2,3));
				if(fileDis != seqDis)
					continue;
				double sasaALow = Double.parseDouble(s.substring(4,7));
				
				double sasaAHigh = sasaALow + 0.1;
				double sasaBLow = Double.parseDouble(s.substring(8,11));
				double sasaBHigh = sasaBLow + 0.1;
				if(sasaAN > sasaALow && sasaAN < sasaAHigh && sasaBN > sasaBLow && sasaBN < sasaBHigh)
					pairFiles.add(path+s);
				else if(ResCharacter.simSasaN(sasaALow, sasaAN, pPr.resParaA().sasaLevel()) || ResCharacter.simSasaN(sasaAHigh, sasaAN, pPr.resParaA().sasaLevel()))
				{
					if(ResCharacter.simSasaN(sasaBLow, sasaBN, pPr.resParaA().sasaLevel()) || ResCharacter.simSasaN(sasaBHigh, sasaBN, pPr.resParaA().sasaLevel()))
						pairFiles.add(path+s);
				}
			}
		}
		return pairFiles;
	}
	
	public static ProbabilityMatrix findPMofPair(ResPair rp, PairPara pPr,HashSet<String> banPDB) throws IOException
	{
		double[][] pMatrix=new double[20][20];
		ArrayList<String> files = getFileList(rp, pPr);
		PairRelativePosition prp=rp.relativePosition();
		int foundPairNum=0;
		for(String s : files)
		{
			BufferedReader in = new BufferedReader(new FileReader(s));
			String line;
			while((line=in.readLine())!=null)
			{
				StrideRes strA=new StrideRes(line.charAt(71),Float.parseFloat(line.substring(74,81).trim()),Float.parseFloat(line.substring(83,90).trim()),Float.parseFloat(line.substring(92,97).trim()));
				if(!strA.similarTo(rp.strResA, pPr.resParaA()))
					continue;
				StrideRes strB=new StrideRes(line.charAt(98),Float.parseFloat(line.substring(101,108).trim()),Float.parseFloat(line.substring(110,117).trim()),Float.parseFloat(line.substring(119,124).trim()));
				if(!strB.similarTo(rp.strResB, pPr.resParaB()))
					continue;
				if(banPDB.contains(line.substring(0,4).toLowerCase()))
				{
					continue;
				}
				double caDistance=Double.parseDouble(line.substring(28,34).trim());
				double angA=Double.parseDouble(line.substring(35,43).trim());
				double angB=Double.parseDouble(line.substring(44,52).trim());
				double dihedral=Double.parseDouble(line.substring(53,61).trim());
				
				PairRelativePosition prpL=new PairRelativePosition(caDistance,angA,angB,dihedral);
				
				double rmsd = prpL.rmsd1(prp);
				double rmsdCutoff = pPr.cutoff(rp);
				
				if(rmsd>rmsdCutoff)
					continue;
				
				foundPairNum++;
				pMatrix[ResName.triToInt(line.substring(13,16))][ResName.triToInt(line.substring(22,25))]+=1.0;
			}
			in.close();
		}
		ProbabilityMatrix pm = new ProbabilityMatrix(pMatrix,foundPairNum);
		if(rp.strResB.resID-rp.strResA.resID > 3)
		{
			ProbabilityMatrix mirrorPM = FindSimilarPairs.findPMofPair(rp.mirror(), pPr, banPDB);
			mirrorPM.transpose();
			pm=pm.merge(mirrorPM);
		}
		return pm;
	}
	
	public static ProbabilityMatrix findPMBackgroundOfPair(ResPair rp, PairPara pPr,HashSet<String> banPDB, BackgroundFasta bgFasta) throws IOException
	{
		double[][] pMatrix=new double[20][20];
		ArrayList<String> files = getFileList(rp, pPr);
		PairRelativePosition prp=rp.relativePosition();
		int foundPairNum=0;
		for(String s : files)
		{
			BufferedReader in = new BufferedReader(new FileReader(s));
			String line;
			while((line=in.readLine())!=null)
			{
				StrideRes strA=new StrideRes(line.charAt(71),Float.parseFloat(line.substring(74,81).trim()),Float.parseFloat(line.substring(83,90).trim()),Float.parseFloat(line.substring(92,97).trim()));
				if(!strA.similarTo(rp.strResA, pPr.resParaA()))
					continue;
				StrideRes strB=new StrideRes(line.charAt(98),Float.parseFloat(line.substring(101,108).trim()),Float.parseFloat(line.substring(110,117).trim()),Float.parseFloat(line.substring(119,124).trim()));
				if(!strB.similarTo(rp.strResB, pPr.resParaB()))
					continue;
				if(banPDB.contains(line.substring(0,4).toLowerCase()))
				{
					continue;
				}
				double caDistance=Double.parseDouble(line.substring(28,34).trim());
				double angA=Double.parseDouble(line.substring(35,43).trim());
				double angB=Double.parseDouble(line.substring(44,52).trim());
				double dihedral=Double.parseDouble(line.substring(53,61).trim());
				
				PairRelativePosition prpL=new PairRelativePosition(caDistance,angA,angB,dihedral);
				
				double rmsd = prpL.rmsd1(prp);
				double rmsdCutoff = pPr.cutoff(rp);
				
				if(rmsd>rmsdCutoff)
					continue;
				
				String pdbID = line.substring(0,4);
				char chainID = line.charAt(5);
				if(!bgFasta.contains(pdbID, chainID))
					continue;
				
				int resAIDSD = Integer.parseInt(line.substring(8,12).trim());
				int resBIDSD = Integer.parseInt(line.substring(17,21).trim());
				foundPairNum += bgFasta.seqNumForEachPDB();
				for(int k=0;k<bgFasta.seqNumForEachPDB();k++)
				{
					int resAType = bgFasta.getResType(pdbID, chainID, resAIDSD, k);
					int resBType = bgFasta.getResType(pdbID, chainID, resBIDSD, k);
					pMatrix[resAType][resBType]+=1.0;
				}
			}
			in.close();
		}
		ProbabilityMatrix pm = new ProbabilityMatrix(pMatrix,foundPairNum);
		if(rp.strResB.resID-rp.strResA.resID > 3)
		{
			ProbabilityMatrix mirrorPM = FindSimilarPairs.findPMBackgroundOfPair(rp.mirror(), pPr, banPDB,bgFasta);
			mirrorPM.transpose();
			pm=pm.merge(mirrorPM);
		}
		return pm;
	}
	
	
	
	public static void pdbSimPairFrom(ResPair rp, PairPara pPr, double cutFrom, double cutTo, char resA, char resB) throws IOException
	{
		ArrayList<String> files = getFileList(rp, pPr);
		PairRelativePosition prp=rp.relativePosition();
		int foundPairNum=0;
		for(String s : files)
		{
			BufferedReader in = new BufferedReader(new FileReader(s));
			String line;
			while((line=in.readLine())!=null)
			{
				StrideRes strA=new StrideRes(line.charAt(71),Float.parseFloat(line.substring(74,81).trim()),Float.parseFloat(line.substring(83,90).trim()),Float.parseFloat(line.substring(92,97).trim()));
				if(!strA.similarTo(rp.strResA, pPr.resParaA()))
					continue;
				StrideRes strB=new StrideRes(line.charAt(98),Float.parseFloat(line.substring(101,108).trim()),Float.parseFloat(line.substring(110,117).trim()),Float.parseFloat(line.substring(119,124).trim()));
				if(!strB.similarTo(rp.strResB, pPr.resParaB()))
					continue;
				double caDistance=Double.parseDouble(line.substring(28,34).trim());
				double angA=Double.parseDouble(line.substring(35,43).trim());
				double angB=Double.parseDouble(line.substring(44,52).trim());
				double dihedral=Double.parseDouble(line.substring(53,61).trim());
				
				PairRelativePosition prpL=new PairRelativePosition(caDistance,angA,angB,dihedral);
				
				double rmsd = prpL.rmsd1(prp);
				if(rmsd>cutTo || rmsd < cutFrom)
					continue;
				
				if(resA == ResName.triToSin(line.substring(13,16)) && resB == ResName.triToSin(line.substring(22,25)))
				{
					System.out.println(line.substring(0,25));
				}
				foundPairNum++;
			}
			in.close();
		}
	}
	
	public static void creatPMPoint(ProteinChain chain, StrideProtein sp, int A,int B,PrintStream out) throws IOException
	{
		ResPair rp = new ResPair(chain.getResidue(A),chain.getResidue(B));
		rp.addStrInfo(sp);
		
//		ProbabilityMatrix pm=findPMofPair(rp,new PairPara(0.5,1,1,2),0,0.5);
//		System.out.println(pm.getSampleNum()+" "+pm.entropy());
	}
	
	public static void main(String[] args) throws IOException
	{
		
		PDB pdb=new PDB("E:\\pairwiseDesign\\testPDB\\1fhlMut\\pdb1fhl.ent","1fhl");
		ProteinChain chain=pdb.getChain('A');
		Residue resA=chain.getResidue(93);
		Residue resB=chain.getResidue(143);
		ResPair rp1=new ResPair(resA,resB,"1fhl",'A');
		StrideProtein sp=new StrideProtein("E:\\pairwiseDesign\\testPDB\\1fhlMut\\1fhl.meastr");
		rp1.addStrInfo(sp);
		ArrayList<String> files = getFileList(rp1,new PairPara(40,0.5,1,3,1));

		HashSet<String> ban = new HashSet<String>();
		ban.add("1fob");
		ProbabilityMatrix pm = findPMofPair(rp1,new PairPara(40,0.5,1,3,1),ban);
		System.out.println(rp1.toString());
		System.out.println(pm.getSampleNum());
		pm.printMatrix(System.out);
	}
}
