package similarSearch;
import model.*;
import input.InputFile;

import java.util.*;
import java.io.*;

import buildDataFile.SplitResidueFile;
import math.*;

public class FindSimilarResidues 
{
	public static ArrayList<String> findFileList(StrideRes sr, ResPara rPr) throws IOException
	{
		InputFile inputFiles = new InputFile();
		
		String path = inputFiles.get("splitBySSSasa");
		ArrayList<String> relatedFileList = new ArrayList<String>(); 
		File filePath = new File(path);
		String[] allFileList = filePath.list();
		for(String fileName : allFileList)
		{
			char ss = fileName.charAt(0);
			//System.out.println(fileName);
			double sasaLow = Double.parseDouble(fileName.substring(2,6));
			double sasaHigh = sasaLow+0.1;
			
			if(!ResCharacter.simSecondaryStructure(ss, SplitResidueFile.ssTypeTri(sr.ssType), rPr.ssLevel()))
				continue;
			
			double srSasaN = ResCharacter.sasaAreaNormalize(sr.area);
			if(srSasaN > sasaLow && srSasaN < sasaHigh)
				relatedFileList.add(path+fileName);
			else if(ResCharacter.simSasaN(sasaLow, srSasaN, rPr.sasaLevel()) || ResCharacter.simSasaN(sasaHigh, srSasaN, rPr.sasaLevel()))
				relatedFileList.add(path+fileName);
		}
		return relatedFileList;
	}
	
	public static ProbabilityArray findPAofRes(StrideRes sr,ResPara rPr,HashSet<String> banList) throws IOException
	{
		double[] pa=new double[20];
		int sampleNum=0;
		char ssType=sr.ssType;
		double area=sr.area;
		double phi=sr.phi;
		double psi=sr.psi;
		
		ArrayList<String> files = findFileList(sr,rPr);
		for(String file : files)
		{
			BufferedReader in = new BufferedReader(new FileReader(file));
			String line;
			while((line=in.readLine())!=null)
			{
				if(line.length()<43)
					continue;
				if(banList.contains(line.substring(0,4)))
					continue;
				
				StrideRes srLine = StrideRes.buildFromResidueFileLine(line);
				if(!sr.similarTo(srLine, rPr))
					continue;
			
				int resTypeInt=ResName.triToInt(line.substring(12,15));
				pa[resTypeInt]+=1.0;
				sampleNum++;
			}
			in.close();
		}
		return new ProbabilityArray(pa,sampleNum);
	}
	
	public static ProbabilityArray findPAofRes(StrideRes sr,ResPara rPr,PrintStream out) throws IOException
	{
		double[] pa=new double[20];
		int sampleNum=0;
		char ssType=sr.ssType;
		double area=sr.area;
		double phi=sr.phi;
		double psi=sr.psi;
		
		ArrayList<String> files = findFileList(sr,rPr);
		for(String file : files)
		{
			BufferedReader in = new BufferedReader(new FileReader(file));
			String line;
			while((line=in.readLine())!=null)
			{
				if(line.length()<43)
					continue;
				
				StrideRes srLine = StrideRes.buildFromResidueFileLine(line);
				if(!sr.similarTo(srLine, rPr))
					continue;
			
				out.println(line);
				int resTypeInt=ResName.triToInt(line.substring(12,15));
				pa[resTypeInt]+=1.0;
				sampleNum++;
			}
		}
		out.close();
		return new ProbabilityArray(pa,sampleNum);
	}
	
	public static void main(String[] args) throws IOException
	{
		StrideProtein sp = new StrideProtein("E:\\tong\\mea.str");
		ResPara rPr = new ResPara(1,4,2);
		StrideRes sr1 = sp.getStrideChains().get(0).getResidues().get(8);
		StrideRes sr2 = sp.getStrideChains().get(0).getResidues().get(11);
		StrideRes sr3 = sp.getStrideChains().get(0).getResidues().get(25);
		StrideRes sr4 = sp.getStrideChains().get(0).getResidues().get(28);
		HashSet<String> banList = new HashSet<String>();
		ProbabilityArray paList1 =  findPAofRes(sr1,rPr,banList);
		ProbabilityArray paList2 =  findPAofRes(sr2,rPr,banList);
		ProbabilityArray paList3 =  findPAofRes(sr3,rPr,banList);
		ProbabilityArray paList4 =  findPAofRes(sr4,rPr,banList);
		ProbabilityArray pa = paList1.merge(paList2);
		pa = pa.merge(paList3);
		pa = pa.merge(paList4);
		for(int i=0;i<20;i++)
		{
			System.out.println(ResName.intToTri(i) + " " + String.format("%7.5f",pa.getArray().get(i)));
		}
	}
}
