package similarSearch;
import model.*;

import input.InputFile;

import java.util.*;
import java.io.*;

import buildDataFile.SplitResidueFile;
import math.*;

public class FindSimilarResiduesRT 
{
	public static ArrayList<String> findFileList(StrideRes sr, ResPara rPr) throws IOException
	{
		InputFile inputFiles = new InputFile();
		String path = inputFiles.get("splitBySSSasaRT");
		ArrayList<String> relatedFileList = new ArrayList<String>(); 
		File filePath = new File(path);
		String[] allFileList = filePath.list();
		for(String fileName : allFileList)
		{
			char ss = fileName.charAt(0);
			double sasaLow = Double.parseDouble(fileName.substring(2,6));
			double sasaHigh = sasaLow+0.1;
			
			if(!ResCharacter.simSecondaryStructure(ss, SplitResidueFile.ssTypeTri(sr.ssType), rPr.ssLevel()))
				continue;
			
			double srSasaN = sr.getAreaNormalized();
			if(srSasaN > sasaLow && srSasaN < sasaHigh)
				relatedFileList.add(path+fileName);
			else if(ResCharacter.simSasaN(sasaLow, srSasaN, rPr.sasaLevel()) || ResCharacter.simSasaN(sasaHigh, srSasaN, rPr.sasaLevel()))
				relatedFileList.add(path+fileName);
		}
		return relatedFileList;
	}
	
	public static ProbabilityArray findPAofRes(StrideRes sr,ResPara rPr, HashSet<String> banList) throws IOException
	{
		double[] pa=new double[20];
		for(int i=0;i<pa.length;i++)
			pa[i] = 0.0;
		int sampleNum=0;
		
		ArrayList<String> files = findFileList(sr,rPr);
		for(String file : files)
		{
			BufferedReader in = new BufferedReader(new FileReader(file));
			String line;
			while((line=in.readLine())!=null)
			{
				if(line.length()<43)
					continue;
				
				StrideRes srLine = StrideRes.buildFromResidueFileLine(line);
				if(!sr.similarTo(srLine, rPr))
					continue;
			
				int resType = ResName.triToInt(line.substring(12,15));
				
				
				if(banList.contains(line.substring(0,4)))
					continue;
				
				pa[resType]+=1.0;
				sampleNum++;
			}
			in.close();
		}
//		System.out.println(sampleNum);
		return new ProbabilityArray(pa,sampleNum);
	}
	
	public static ProbabilityArrayRT findPAofRes(StrideRes sr,ResPara rPr, Rotamers rots, HashSet<String> banList) throws IOException
	{
		double[] pa=new double[rots.rotamerNum()];
		for(int i=0;i<pa.length;i++)
			pa[i] = 0.0;
		int sampleNum=0;
		
		ArrayList<String> files = findFileList(sr,rPr);
		for(String file : files)
		{
			BufferedReader in = new BufferedReader(new FileReader(file));
			String line;
			while((line=in.readLine())!=null)
			{
				if(line.length()<43)
					continue;
				
				StrideRes srLine = StrideRes.buildFromResidueFileLine(line);
				if(!sr.similarTo(srLine, rPr))
					continue;
			
				int rotType = rots.dunbrackCodeToInt(line.substring(12,15), FindSimilarPairsRT.dunbrackCode(line.substring(44,48)));
				if(rotType<0)
					continue;
				
				if(banList.contains(line.substring(0,4)))
					continue;
				
				pa[rotType]+=1.0;
				sampleNum++;
			}
			in.close();
		}
//		System.out.println(sampleNum);
		return new ProbabilityArrayRT(pa,sampleNum);
	}
	
	
	public static void main(String[] args) throws IOException
	{
		PDB pdb=new PDB("E:\\pairwiseDesign\\testPDB\\1fhlMut\\pdb1fhl.ent","1fhl");
		ProteinChain chain=pdb.getChain('A');
		Residue resA=chain.getResidue(93);
		ResPara rpr = new ResPara(0,0,0);
		StrideProtein sp=new StrideProtein("E:\\pairwiseDesign\\testPDB\\1fhlMut\\1fhl.meastr");
		StrideRes sr = sp.getStrideResidue("A93");
		System.out.println(resA.getName());
		Rotamers rots = new Rotamers(6);
		ProbabilityArrayRT pa = findPAofRes(sr,rpr,rots,new HashSet<String>());
		System.out.println(pa.size());
		System.out.println(pa.sampleNum());
		for(int i=0;i<pa.size();i++)
		{
//			System.out.print(rots.intToDunbrackCode(i)+ " ");
			System.out.printf("%9.7f\n",pa.getArray().get(i));
		}
	}
}

