package similarSearch;
import java.math.*;
import java.util.*;

import math.*;
import model.*;

public class PairCharacter 
{
	private char ssA;
	private char ssB;
	private double sasaA;
	private double sasaB;
	private double phiA;
	private double phiB;
	private double psiA;
	private double psiB;
	private PairRelativePosition prp;
	private int seqDistance=100;
	
	public PairCharacter(ResPair rp)
	{
		StrideRes srA = rp.strResA;
		StrideRes srB = rp.strResB;
		this.ssA = srA.ssType;
		this.ssB = srB.ssType;
		this.sasaA = ResCharacter.sasaAreaNormalize(srA.area);
		this.sasaB = ResCharacter.sasaAreaNormalize(srB.area);
		this.phiA = srA.phi;
		this.phiB = srB.phi;
		this.psiA = srA.psi;
		this.psiB = srB.psi;
		this.seqDistance = Math.abs(srA.resIDSD-srB.resIDSD);
		this.prp = rp.relativePosition();
	}
	
	public char getSSA()
	{
		return this.ssA;
	}
	
	public char getSSB()
	{
		return this.ssB;
	}
	
	public int getSeqDistance()
	{
		return this.seqDistance;
	}
	
	public double rmsd(PairCharacter other)
	{
		return this.prp.rmsd1(other.prp);
	}
	
	public PairRelativePosition getPrp()
	{
		return this.prp;
	}
	
	
	public PairCharacter(char ssA,char ssB,double sasaA,double sasaB,double phiA,double phiB,double psiA,double psiB,PairRelativePosition prp)
	{
		this.ssA = ssA;
		this.ssB = ssB;
		this.sasaA = sasaA;
		this.sasaB = sasaB;
		this.phiA = phiA;
		this.phiB = phiB;
		this.psiA = psiA;
		this.psiB = psiB;
		this.prp = prp;
	}
	
	public boolean similarTo(PairCharacter other, PairPara pp)
	{
		if(!ResCharacter.simSecondaryStructure(this.ssA, other.ssA, pp.resParaA().ssLevel()))
			return false;
		if(!ResCharacter.simSecondaryStructure(this.ssB, other.ssB, pp.resParaB().ssLevel()))
			return false;
		if(!ResCharacter.simSasaN(this.sasaA, other.sasaA, pp.resParaA().sasaLevel()))
			return false;
		if(!ResCharacter.simSasaN(this.sasaB, other.sasaB, pp.resParaB().sasaLevel()))
			return false;
		if(!ResCharacter.simPhiPsi(this.phiA, this.psiA, other.phiA, other.psiA, pp.resParaA().ppLevel()))
			return false;
		if(!ResCharacter.simPhiPsi(this.phiB, this.psiB, other.phiB, other.psiB, pp.resParaB().ppLevel()))
			return false;
		if(!pairRelativePositionSimilar(this.prp,other.prp,pp.cutoff(other)))
			return false;
		
		return true;
	}
	
	public boolean similarTo(ResPair rp, PairPara pp)
	{
		PairCharacter prcOther = new PairCharacter(rp);
		return this.similarTo(prcOther, pp);
	}
	
	public static PairCharacter center(PairCharacter prcA, double weightA, PairCharacter prcB, double weightB)
	{
		char ssA = prcA.ssA;
		char ssB = prcA.ssB;
		double sasaA = (prcA.sasaA*weightA + prcB.sasaA*weightB)/(weightA + weightB);
		double sasaB = (prcA.sasaB*weightA + prcB.sasaB*weightB)/(weightA + weightB);
		double phiA = (prcA.phiA*weightA + prcB.phiA*weightB)/(weightA + weightB);
		double phiB = (prcA.phiB*weightA + prcB.phiB*weightB)/(weightA + weightB);
		double psiA = (prcA.psiA*weightA + prcB.psiA*weightB)/(weightA + weightB);
		double psiB = (prcA.psiB*weightA + prcB.psiB*weightB)/(weightA + weightB);
		
		
		PairRelativePosition prp = PairRelativePosition.center(prcA.prp, weightA, prcB.prp, weightB);
		
		return new PairCharacter(ssA,ssB,sasaA,sasaB,phiA,phiB,psiA,psiB,prp);
	}
	
	
	

	public static boolean pairRelativePositionSimilar(PairRelativePosition prpA, PairRelativePosition prpB,double cutoff)
	{
		double rmsd1=prpA.rmsd1(prpB);
		if(rmsd1<cutoff)
			return true;
		else 
			return false;
	}
	
	public static void printInfo2(ArrayList<Tern> list)
	{
		Tern A=list.get(0);
		Tern B=list.get(1);
		Tern C=list.get(2);
		Tern D=list.get(3);
		
		Tern AB=B.subtract(A);
		Tern BA=AB.negate();
		Tern AC=C.subtract(A);
		Tern CA=AC.negate();
		Tern BD=D.subtract(B);
		Tern DB=BD.negate();
		Tern CD=D.subtract(C);
		Tern DC=CD.negate();
		
		double angA=AB.angle(AC);
		double angB=BD.angle(BA);
		double angC=CA.angle(CD);
		double angD=DB.angle(DC);
		
		double cacaD=A.distance(B);
		double ggD=C.distance(D);
		
		Tern n1=AB.cross(AC.cross(AB));
		Tern n2=BA.cross(BD.cross(BA));
		double dihedral=n1.angle(n2);
		if(dihedral!=0)
		{
			double symbol=n1.cross(n2).dotProduct(BA);
			symbol=symbol/Math.abs(symbol);
			dihedral=dihedral*symbol;
		}
		
		System.out.println("angle A:"+angA);
		System.out.println("angle B:"+angB);
		System.out.println("angle C:"+angC);
		System.out.println("angle D:"+angD);
		System.out.println("ca distance:"+cacaD);
		System.out.println("ecog distance:"+ggD);
		System.out.println("dihedral:"+dihedral);
	}

}
