package similarSearch;
import model.*;

public final class PairPara 
{
	private ResPara resParaA;
	private ResPara resParaB;
	
	private double rmsdCutoff;
	int maxFqFloor = 0;
	
	public PairPara()
	{
		this.resParaA = new ResPara(1,3,2);
		this.resParaB = new ResPara(1,3,2);
		this.maxFqFloor = 60;
		this.rmsdCutoff = 0.5;
	}
	
	public PairPara(ResPara resParaA, ResPara resParaB, double cutoff,int maxFqFloor)
	{
		this.resParaA = resParaA;
		this.resParaB = resParaB;
		this.rmsdCutoff = cutoff;
		this.maxFqFloor = maxFqFloor;
	}
	
	public PairPara(int maxFqFloor,double cutoff,int ssLevel, int sasaLevel, int ppLevel)
	{
		this(new ResPara(ssLevel,sasaLevel,ppLevel),new ResPara(ssLevel,sasaLevel,ppLevel),cutoff,maxFqFloor);
	}
	
	public PairPara clone()
	{
		return new PairPara(this.maxFqFloor,this.cutoff(),this.resParaA.ssLevel(),this.resParaA.sasaLevel(),this.resParaA.ppLevel());
	}
	
	public ResPara resParaA()
	{
		return this.resParaA;
	}
	
	public ResPara resParaB()
	{
		return this.resParaB;
	}
	
	public double cutoff()
	{
		return this.rmsdCutoff;
	}
	
	public int fqFloor()
	{
		return this.maxFqFloor;
	}
	
	public double cutoff(ResPair rp)
	{
		int seqDis = rp.seqDistance();
		if(seqDis == 1)
			return this.rmsdCutoff/2.0;
		if(seqDis == 1 || seqDis == 3 || seqDis ==4)
		{
			if(rp.strResA.ssType == 'H' && rp.strResB.ssType == 'H')
				return this.rmsdCutoff/2.0;
		}
		else if(seqDis == 2)
		{
			if(rp.strResA.ssType == 'E' && rp.strResB.ssType == 'E')
				return this.rmsdCutoff/2.0;
		}
		else if(rp.strResA.ssType == 'E' && rp.strResB.ssType == 'E')
			return this.rmsdCutoff*0.75;
		
		return this.rmsdCutoff;
	}
	
	public double cutoff(RotPair rp)
	{
		int seqDis = rp.seqDistance();
		if(seqDis == 1)
			return this.rmsdCutoff/2.0;
		if(seqDis == 1 || seqDis == 3 || seqDis ==4)
		{
			if(rp.strResA.ssType == 'H' && rp.strResB.ssType == 'H')
				return this.rmsdCutoff/2.0;
		}
		else if(seqDis == 2)
		{
			if(rp.strResA.ssType == 'E' && rp.strResB.ssType == 'E')
				return this.rmsdCutoff/2.0;
		}
		else if(rp.strResA.ssType == 'E' && rp.strResB.ssType == 'E')
			return this.rmsdCutoff*0.75;
		
		return this.rmsdCutoff;
	}
	
	public double cutoff(PairCharacter prc)
	{
		int seqDis = prc.getSeqDistance();
		if(seqDis == 1)
			return this.rmsdCutoff/2.0;
		if(seqDis == 1 || seqDis == 3 || seqDis ==4)
		{
			if(prc.getSSA() == 'H' && prc.getSSB() == 'H')
				return this.rmsdCutoff/3.0;
		}
		else if(seqDis == 2)
		{
			if(prc.getSSA() == 'E' && prc.getSSB() == 'E')
				return this.rmsdCutoff/2.0;
		}
		else if(prc.getSSA() == 'E' && prc.getSSB() == 'E')
			return this.rmsdCutoff*0.75;
		
		return this.rmsdCutoff;
	}
/*	
	public PairPara demote()
	{
		if(this.resParaA.ppLevel()>0)
		{
			return new PairPara(this.maxFqFloor,this.cutoff(),this.resParaA.ssLevel(),this.resParaA.sasaLevel(),this.resParaA.ppLevel()-1);
		}
		else if(this.resParaA.sasaLevel()>2)
		{
			return new PairPara(this.maxFqFloor,this.cutoff(),this.resParaA.ssLevel(),this.resParaA.sasaLevel()-1,this.resParaA.ppLevel());
		}
		else 
		{
			return new PairPara(this.maxFqFloor,this.cutoff()*1.1,this.resParaA.ssLevel(),this.resParaA.sasaLevel(),this.resParaA.ppLevel());
		}
	}
*/
	public PairPara demote()
	{
		
		if(this.resParaA.sasaLevel()>2)
		{
			return new PairPara(this.maxFqFloor,this.cutoff(),this.resParaA.ssLevel(),this.resParaA.sasaLevel()-1,this.resParaA.ppLevel());
		}
		else if(this.resParaA.ppLevel()>0)
		{
			return new PairPara(this.maxFqFloor,this.cutoff(),this.resParaA.ssLevel(),this.resParaA.sasaLevel(),0);
		}
		else if(this.cutoff() < 0.6)
		{
			return new PairPara(this.maxFqFloor,0.6,this.resParaA.ssLevel(),this.resParaA.sasaLevel(),this.resParaA.ppLevel());
		}
		else if(this.cutoff() < 0.8)
		{
			return new PairPara(this.maxFqFloor,0.8,this.resParaA.ssLevel(),this.resParaA.sasaLevel(),this.resParaA.ppLevel());
		}
		else if(this.cutoff() < 1.1)
		{
			return new PairPara(this.maxFqFloor,1.1,this.resParaA.ssLevel(),this.resParaA.sasaLevel(),this.resParaA.ppLevel());
		}
		else
		{
			return new PairPara(this.maxFqFloor,1.5,this.resParaA.ssLevel(),this.resParaA.sasaLevel(),this.resParaA.ppLevel());
		}
	}
	
	
	public String toString()
	{
		return String.format("%4.2f", this.rmsdCutoff)+"_"+this.resParaA.toString();
	}
}
