package similarSearch;
import model.*;

public class ResCharacter 
{
	public static final int ssLevels=3;
	public static final int sasaLevels=9;
	public static final int phiPsiLevels=7;
	
	public static boolean simSecondaryStructure(char ssA,char ssB,int level)
	{
		if(level == 0)
		{
			return true;
		}
		else if(level == 1)
		{
			if(ssA==ssB)
				return true;
			
			if(ssA=='H'||ssA=='G'||ssA=='I')
				ssA='H';
			else if(ssA!='E')
				ssA='C';
			
			if(ssB=='H'||ssB=='G'||ssB=='I')
				ssB='H';
			else if(ssB!='E')
				ssB='C';
			
			return (ssA==ssB);
		}
		else if(level == 2)
		{
			return (ssA==ssB);
		}
		else
		{
			throw new RuntimeException("ss level not defined: "+level);
		}
	}
	
	public static float sasaAreaNormalizeMEA(float a)
	{
		//mea psuedo residue
		if(a<110)
		{
			return (float)(0.02073*a*(1-0.01458*a*(1-0.008224*a*(1-0.0030103*a))));
		}
		else if(a<140)
		{
			return (float)(0.2659575+0.0067*a*(1-0.002059*a));
		}
		else if(a<180)
		{
			return (float)(-0.1665238+0.0128*a-0.0000353*a*a);
		}
		else 
		{
			return 1.0f;
		}
	}
	
	public static float sasaAreaNormalizeMED(float area)
	{
		//MED psuedo residue, weight 1.0,0.6,0.3, bond length 2.0
		double x1 = area;
		double x2 = x1*x1;
		double x3 = x2*x1;
		double x4 = x3*x1;
		
		if(area < 8) return (float)(0.04223+0.0935*x1-0.01659*x2+0.00197*x3-9.457E-5*x4);
		else if(area < 40) return (float)(0.15729+0.03237*x1-0.00123*x2+2.59179E-5*x3-2.10531E-7*x4);
		else if(area < 100) return (float)(0.41824+0.00369*x1+4.71536E-5*x2-7.96524E-7*x3+4.28228E-9*x4);
		else if(area < 113.15) return (float)(1.02171 - 0.1557/(1+Math.exp((area-106.19805)/3.81997)));
		return 1.0f;	
		
	}
	
	public static float sasaAreaNormalizePSE(float area)
	{
		//PSE psuedo residue, weight 1.0,0.66,0.28, bond length 2.6, 1.5, 2.2, dihedral angle 170
		double x1 = area;
		double x2 = x1*x1;
		double x3 = x2*x1;
		double x4 = x3*x1;
		
		if(area < 3) return (float)(0.07959+0.14363*x1-0.06939*x2+0.0212*x3-0.00254*x4);
		else if(area < 24) return (float)(0.15301+0.04017*x1-0.00254*x2+9.4612E-5*x3-1.3657E-6*x4);
		else if(area < 73) return (float)(0.29182+0.0115*x1-1.25164E-4*x2+1.08372E-6*x3-2.97474E-9*x4);
		else if(area < 114.6) return (float)(-1.27204+0.06338*x1-7.05134E-4*x2+3.54395E-6*x3-6.1719E-9*x4);
		return 1.0f;	
		
	}
	
	public static float sasaAreaNormalize(float area)
	{
		//PSD psuedo residue, weight 1.0,0.56,0.4, bond length 1.7, 1.5, dihedral angle -90
		double x1 = area;
		double x2 = x1*x1;
		double x3 = x2*x1;
		double x4 = x3*x1;
		double x5 = x4*x1;
		
		if(area < 8) return (float)(0.02497+0.07342*x1-0.01202*x2+0.00146*x3-7.17891E-5*x4);
		else if(area < 43) return (float)(0.0683+0.04127*x1-0.00198*x2+6.0955E-5*x3-9.92843E-7*x4+6.60365E-9*x5);
		else if(area < 87) return (float)(0.64647-0.01494*x1+5.56039E-4*x2-6.66724E-6*x3+2.99774E-8*x4);
		else if(area < 102.3) return (float)(0.68983+0.01349*Math.exp(x1/32.64358));
		return 1.0f;	
	}
	
	public static float sasaAreaNormalizePSD2(float area)
	{
		//PSD psuedo residue, weight 1.0,0.5,0.4, bond length 2.0, dihedral angle -100
		double x1 = area;
		double x2 = x1*x1;
		double x3 = x2*x1;
		double x4 = x3*x1;
	
		
		if(area < 6.1) return (float)(0.03904+0.10277*x1-0.02275*x2+0.00347*x3-2.15905E-4*x4);
		else if(area < 43) return (float)(0.15578+0.0321*x1-0.00125*x2+2.62705E-5*x3-2.10314E-7*x4);
		else if(area < 92) return (float)(0.34041+0.00808*x1-6.13708E-5*x2+3.82624E-7*x3-2.45773E-10*x4);
		else if(area < 110.9) return (float)(22.259-0.80665*x1+0.01117*x2-6.77433E-5*x3+1.53494E-7*x4);
		return 1.0f;	
		
	}
	
	public static int findPhiPsiRegion(double phi,double psi)
	{
		double x=phi;
		double y=psi;
		if(x==360.0 || y==360.0)
			return -1;
		if(x<0)
		{
			if(y<45 && y>-90)
				return 0;
			else if(y>45 && x+y>75)
				return 2;
			else if(y>45 && x-y<-340)
				return 3;
			else if(y<-90 && x>-132.5 && x+y>-285)
				return 2;
			else if(y<-90 && x<-132.5 && x-y<20)
				return 3;
			else if((x+123.3)*(x+123.3)+(y-123.3)*(y-123.3) < 213.7)
				return 4;
			else return 1;
		}
		else if(x<62 && y<80 && y>-15)
			return 5;
		else return 6;
	}
	
	public static boolean simSasaArea(float a1,float a2,int level)
	{		
		float a1N=sasaAreaNormalize(a1);
		float a2N=sasaAreaNormalize(a2);
		double distance=Math.abs(a1N-a2N);
		
		double[] cutoff={1, 0.3, 0.2, 0.15, 0.1, 0.07, 0.05, 0.03, 0.02};
		if(level >=0 && level <cutoff.length)
			return distance<cutoff[level];
		
		else 
			throw new RuntimeException("sasa level not defined: "+level);
	}
	
	public static boolean simSasaN(double n1,double n2,int level)
	{
		double distance = Math.abs(n1-n2);
		double[] cutoff={1, 0.3, 0.2, 0.15, 0.1, 0.07, 0.05, 0.03, 0.02};
		if(level >=0 && level <cutoff.length)
			return distance<cutoff[level];
		
		else 
			throw new RuntimeException("sasa level not defined: "+level);
	}
	
	public static boolean simPhiPsi(double phiA, double psiA, double phiB, double psiB,int level)
	{
		if(level==0)
			return true;
		
		int regionA=findPhiPsiRegion(phiA,psiA);
		int regionB=findPhiPsiRegion(phiB,psiB);
		if(regionA==-1 || regionB==-1)
		{
			return (regionA==-1 && regionB==-1);
		}
		
		if(level == 1)
		{
			if(regionA>0 && regionA<5)
				regionA=1;
			else if(regionA>=5)
				regionA=5;
			
			if(regionB>0 && regionB<5)
				regionB=1;
			else if(regionB>=5)
				regionB=5;
			
			return (regionA==regionB);
		}
		
		else if(level == 2)
		{
			return (regionA==regionB);
		}
		
		Tern r1=new Tern(phiA,psiA,0);
		if(psiA<-90)
			r1=r1.add(new Tern(0,360.0,0));
		Tern r2=new Tern(phiB,psiB,0);
		if(psiB<-90)
			r2=r2.add(new Tern(0,360.0,0));
		double distance=r1.distance(r2);
		double[] cutoff={30.0, 20.0, 10.0, 5.0};
		if(level >= 3 && level <3+cutoff.length)
		{
			return distance<cutoff[level-3];
		}
		else 
			throw new RuntimeException("phiPsi level not defined: "+level);	
	}
	
	public static void main(String[] args)
	{
		
		double b = sasaAreaNormalize(24f);
		
		System.out.println(b);
	}
}
