package usr;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;

import model.Atom;
import model.PDB;
import model.ProteinChain;
import model.Residue;
import model.Tern;

public class BuildPseudoSidechain 
{
	public static void rewriteToMED(String oldPDB,String newPDB) throws IOException
	{
		PrintWriter out=new PrintWriter(newPDB);
		int atomID=1;
		PDB pdb=new PDB(oldPDB);
		{
			for(ProteinChain chain: pdb.getChains())
			{
				for(Residue res : chain.getResidue())
				{
					if(res.infoSufficient())
					{
//						Tern CaCb=res.getCaCbVectorOfGly();
						Atom C=res.getAtom("C");
						Atom CA=res.getAtom("CA");
						Atom N=res.getAtom("N");
						Atom O = res.getAtom("O");
						if(O == null)
							O = res.getAtom("OT1");
						ArrayList<Atom> atomList = res.getMEDAtoms();
						
						
						out.println(atomToPDBline(chain.getChainId(),res.getID(),atomID,C,"MED"));
						atomID++;
						out.println(atomToPDBline(chain.getChainId(),res.getID(),atomID,CA,"MED"));
						atomID++;
						out.println(atomToPDBline(chain.getChainId(),res.getID(),atomID,N,"MED"));
						atomID++;
						for(Atom at : atomList)
						{
							out.println(atomToPDBline(chain.getChainId(),res.getID(),atomID,at,"MED"));
							atomID++;
						}
						out.println(atomToPDBline(chain.getChainId(),res.getID(),atomID,O,"MED"));
						atomID++;
					}   
				}
			}
		}
		out.close();
	}
	
	public static void rewriteToPSE(String oldPDB,String newPDB) throws IOException
	{
		PrintWriter out=new PrintWriter(newPDB);
		int atomID=1;
		PDB pdb=new PDB(oldPDB);
		{
			for(ProteinChain chain: pdb.getChains())
			{
				for(Residue res : chain.getResidue())
				{
					if(res.infoSufficient())
					{
//						Tern CaCb=res.getCaCbVectorOfGly();
						Atom C=res.getAtom("C");
						Atom CA=res.getAtom("CA");
						Atom N=res.getAtom("N");
						Atom O = res.getAtom("O");
						if(O == null)
							O = res.getAtom("OT1");
						ArrayList<Atom> atomList = res.getPSEAtoms();
						
						
						out.println(atomToPDBline(chain.getChainId(),res.getID(),atomID,C,"PSE"));
						atomID++;
						out.println(atomToPDBline(chain.getChainId(),res.getID(),atomID,CA,"PSE"));
						atomID++;
						out.println(atomToPDBline(chain.getChainId(),res.getID(),atomID,N,"PSE"));
						atomID++;
						for(Atom at : atomList)
						{
							out.println(atomToPDBline(chain.getChainId(),res.getID(),atomID,at,"PSE"));
							atomID++;
						}
						out.println(atomToPDBline(chain.getChainId(),res.getID(),atomID,O,"PSE"));
						atomID++;
					}   
				}
			}
		}
		out.close();
	}
	
	public static void rewriteToPSD(String oldPDB,String newPDB) throws IOException
	{
		PrintWriter out=new PrintWriter(newPDB);
		int atomID=1;
		PDB pdb=new PDB(oldPDB);
		{
			for(ProteinChain chain: pdb.getChains())
			{
				for(Residue res : chain.getResidue())
				{
					if(res.infoSufficient())
					{
//						Tern CaCb=res.getCaCbVectorOfGly();
						Atom C=res.getAtom("C");
						Atom CA=res.getAtom("CA");
						Atom N=res.getAtom("N");
						Atom O = res.getAtom("O");
						if(O == null)
							O = res.getAtom("OT1");
						ArrayList<Atom> atomList = res.getPSDAtoms();
						
						
						out.println(atomToPDBline(chain.getChainId(),res.getID(),atomID,C,"PSD"));
						atomID++;
						out.println(atomToPDBline(chain.getChainId(),res.getID(),atomID,CA,"PSD"));
						atomID++;
						out.println(atomToPDBline(chain.getChainId(),res.getID(),atomID,N,"PSD"));
						atomID++;
						for(Atom at : atomList)
						{
							out.println(atomToPDBline(chain.getChainId(),res.getID(),atomID,at,"PSD"));
							atomID++;
						}
						out.println(atomToPDBline(chain.getChainId(),res.getID(),atomID,O,"PSD"));
						atomID++;
					}   
				}
			}
		}
		out.close();
	}
	
	public static String atomToPDBline(char chainID,int resID,int atomID,Atom atom, String resName)
	{
		String line="ATOM";
		line=line+String.format("%7d", atomID)+"  ";
		line=line+String.format("%-4s", atom.getName());
		line=line+resName + " "+chainID+String.format("%4d", resID)+"    ";
		line=line+String.format("%8.3f",atom.getCoordinate().x());
		line=line+String.format("%8.3f",atom.getCoordinate().y());
		line=line+String.format("%8.3f",atom.getCoordinate().z());
		return line;
	}
	
	public static void rewriteToEcog2(String oldPDB,String newPDB) throws IOException
	{
		PrintWriter out=new PrintWriter(newPDB);
		int atomID=1;
		PDB pdb=new PDB(oldPDB);
		{
			for(ProteinChain chain: pdb.getChains())
			{
				for(Residue res : chain.getResidue())
				{
					if(res.infoSufficient())
					{
//						Tern CaCb=res.getCaCbVectorOfGly();
						Atom C=res.getAtom("C");
						Atom CA=res.getAtom("CA");
						Atom N=res.getAtom("N");
						Atom O = res.getAtom("O");
						
						Tern ecog = res.getExpectedCOG2();
						Atom CG = new Atom("CG", ecog);
						
						
						out.println(atomToPDBline(chain.getChainId(),res.getID(),atomID,C,"COG"));
						atomID++;
						out.println(atomToPDBline(chain.getChainId(),res.getID(),atomID,CA,"COG"));
						atomID++;
						out.println(atomToPDBline(chain.getChainId(),res.getID(),atomID,N,"COG"));
						atomID++;
						if(res.getAtoms().containsKey("CB"))
						{
							Atom CB = res.getAtom("CB");
							out.println(atomToPDBline(chain.getChainId(),res.getID(),atomID,CB,"COG"));
							atomID++;
						}
						out.println(atomToPDBline(chain.getChainId(),res.getID(),atomID,CG,"COG"));
						atomID++;
						
						out.println(atomToPDBline(chain.getChainId(),res.getID(),atomID,O,"COG"));
						atomID++;
					}   
				}
			}
		}
		out.close();
	}
	
	public static void main(String[] args) throws IOException
	{
/*		String listFile = "/mnt/0a/xiongpeng/java/data/sys2Test/list.txt";
		ArrayList<String> list1 = FileToList.getList(listFile);
		for(String pdbID : list1)
		{
			String oldName = "/mnt/0a/xiongpeng/java/data/sys2Test/pdb/"+pdbID+".pdb";
			String newName = "/mnt/0a/xiongpeng/java/data/sys2Test/selectPSD/"+pdbID+".psd";
			rewriteToPSD(oldName, newName);
		}
*/		
/*
		String pdb2List = "/mnt/0a/xiongpeng/java/data/pdb2List";
		ArrayList<String> list = FileToList.getList(pdb2List);
		for(String s : list)
		{
			if(s.length() < 10) continue;
			String pdbID = s.substring(38,42);
			char third = pdbID.charAt(2);
			String newName = "/mnt/0a/xiongpeng/java/data/psdPDB2/" + third + "/" + pdbID + ".psd";
			rewriteToPSD(s, newName);
		}
*/
/*		for(int i=1;i<6;i++)
		{
			String oldFile = "/mnt/0a/xiongpeng/idealizeModel/rim" + i + "/RIM" + i + ".pdb";
			String newFile = "/mnt/0a/xiongpeng/idealizeModel/rim" + i + "/RIM" + i + ".med";
			rewriteToMED(oldFile, newFile);		
		}
*/
		rewriteToPSD(args[0], args[1]);
	}
}
