#!/usr/bin/env python

import re,os,sys,sqlite3,glob,urllib,urllib2,StringIO,gzip
import userConfig,preparation

def getpdb2acc():
	selects = 'SELECT acc, name from xdb2acc where type=\'PDB\''
	dbfile = userConfig.humanDB+'.sqlite'
	if (not os.path.exists(dbfile)) or (not os.path.isfile(dbfile)):
		print 'ERROR: '+dbfile+'does not exist.'
		sys.exit()
	conn = sqlite3.connect(dbfile)
	with conn:
		cur = conn.cursor()
		cur.execute(selects)
		dictionary = cur.fetchall()
	return dictionary

#def get_residue_mapping(humanDictionary,outDirectory):
def build_sifts_db(outDirectory):
	siftsUrl = 'ftp://ftp.ebi.ac.uk/pub/databases/msd/sifts/flatfiles/tsv/pdb_chain_uniprot.tsv.gz'
	print "Downloading "+siftsUrl+" ..."
	name = siftsUrl.split('/')[-1]
	outDirectory = preparation.check_slash(outDirectory)
	urllib.urlretrieve(siftsUrl,outDirectory+name)
	print "Decompressing "+ outDirectory+name
	cmd = 'gunzip -f '+ outDirectory+name
	preparation.run_cmd(cmd)
	print "Done."

	tCol = 9
	table = 'CREATE TABLE tempResMapping (pdbid VARCHAR(12) NOT NULL COLLATE NOCASE, chain VARCHAR(12) NOT NULL, acc VARCHAR(12) NOT NULL, resbeg INTERGER NOT NULL , resend INTERGER NOT NULL, pdbbeg INTERGER NOT NULL, pdbend INTERGER NOT NULL, spbeg INTERGER NOT NULL, spend INTERGER NOT NULL)'

	inserts = 'INSERT INTO tempResMapping VALUES ('+', '.join(('?')*tCol)+')'
	indices = ['CREATE INDEX x_respdbid ON tempResMapping (pdbid)',
			'CREATE INDEX x_resacc ON tempResMapping (acc)',
			'CREATE INDEX x_respdbidacc ON tempResMapping (pdbid, acc)']
	delect = 'DROP TABLE tempResMapping'
	
	print 'Creating SQLite table tempResMapping ...'
	
	dbfile = userConfig.humanDB+'.sqlite'
	conn = sqlite3.connect(dbfile)
	with conn:
		cur = conn.cursor()

		cur.execute('SELECT * FROM sqlite_master WHERE type = \'table\' and tbl_name = \'tempResMapping\'')
		r = cur.fetchall()	
		if len(r)>0:
			cur.execute(delect)
		cur.execute(table)
	
		print 'Loading data into tempResMapping'	
		match = re.search('^(.+)\.gz$',name)
		name = match.group(1)
		fin = open(outDirectory+name,'r')
		batchSize = 5000
		count = 0

		line = fin.readline()
		line = fin.readline()
		line = fin.readline()
		while line:
			line = re.sub('\n$','',line)
			rCells = re.split('\t',line,tCol)
			try:
				cur.execute(inserts,rCells)
				if (count % batchSize) == 0:
					conn.commit()
			except sqlite3.Error, e:
				if conn:
					conn.rollback()
				print "Error %s:" % e.args[0]
				sys.exit(1)
			count += 1
			line = fin.readline()
		fin.close()
		if count:
			conn.commit()
		print 'Indexing tempResMapping ...'
		for index in indices:
			cur.execute(index)
		print 'Optimizing tempResMapping ...'
		cur.execute('ANALYZE tempResMapping')
	conn.close()


def	get_pdb_biounit(outDirectory):
	biounitUrl = 'ftp://ftp.wwpdb.org/pub/pdb/data/biounit/PDB/all/'
	print 'Reading '+biounitUrl+' ...'
	biounitContent = preparation.readUrl(biounitUrl)
	print "Done."
	lines = biounitContent.strip().split('\n')
	biounitDic = {}
	for line in lines:
		match = re.search('\s+(\w+)\.(\w+)\.gz\\r$',line)
		if not match:
			continue
		pdbid = match.group(1)
		biounit = match.group(2)
		#print pdbid, biounit
		if biounitDic.has_key(pdbid):
			biounitDic[pdbid] += '\t'+pdbid+'.'+biounit+'.gz'
		else:
			biounitDic[pdbid] = pdbid+'.'+biounit+'.gz'
	
	tCol = 2
	table = 'CREATE TABLE tempBiounit (pdbid VARCHAR(12) NOT NULL COLLATE NOCASE, biounit VARCHAR(1000) NOT NULL)'

	inserts = 'INSERT INTO tempBiounit VALUES ('+', '.join(('?')*tCol)+')'
	indices = ['CREATE INDEX x_biopdbid ON tempBiounit (pdbid)']
	delect = 'DROP TABLE tempBiounit'
	
	print 'Creating SQLite table tempBiounit...'
	
	dbfile = userConfig.humanDB+'.sqlite'
	conn = sqlite3.connect(dbfile)
	with conn:
		cur = conn.cursor()

		cur.execute('SELECT * FROM sqlite_master WHERE type = \'table\' and tbl_name = \'tempBiounit\'')
		r = cur.fetchall()	
		if len(r)>0:
			cur.execute(delect)
		cur.execute(table)
	
		print 'Loading data into tempBiounit'	
		batchSize = 5000
		count = 0

		for pdb in biounitDic:
			rCells = [pdb, biounitDic[pdb]]
			try:
				cur.execute(inserts,rCells)
				if (count % batchSize) == 0:
					conn.commit()
			except sqlite3.Error, e:
				if conn:
					conn.rollback()
				print "Error %s:" % e.args[0]
				sys.exit(1)
			count += 1
		if count:
			conn.commit()
		print 'Indexing tempBiounit ...'
		for index in indices:
			cur.execute(index)
		print 'Optimizing tempBiounit ...'
		cur.execute('ANALYZE tempBiounit')
	conn.close()
	#ftp://ftp.wwpdb.org/pub/pdb/data/biounit/PDB/divided/03/103m.pdb1.gz
		
def	get_resolution(outDirectory):
	resUrl = 'ftp://ftp.wwpdb.org/pub/pdb/derived_data/index/resolu.idx'
	print 'Reading '+resUrl+' ...'
	resContent = preparation.readUrl(resUrl)
	print "Done."
	lines = resContent.strip().split('\n')
	resDic = {}
	for line in lines:
		match = re.search('^(\w+)\s+;\s+(\S+)\s*$',line)
		if not match:
			continue
		pdbid = match.group(1)
		resolution = match.group(2)
		#print pdbid, biounit
		resDic[pdbid] = resolution
	
	tCol = 2
	table = 'CREATE TABLE tempResolution (pdbid VARCHAR(12) NOT NULL COLLATE NOCASE, resolution INTERGER NOT NULL)'

	inserts = 'INSERT INTO tempResolution VALUES ('+', '.join(('?')*tCol)+')'
	indices = ['CREATE INDEX x_res ON tempResolution (pdbid)']
	delect = 'DROP TABLE tempResolution'
	
	print 'Creating SQLite table tempResolution...'
	
	dbfile = userConfig.humanDB+'.sqlite'
	conn = sqlite3.connect(dbfile)
	with conn:
		cur = conn.cursor()

		cur.execute('SELECT * FROM sqlite_master WHERE type = \'table\' and tbl_name = \'tempResolution\'')
		r = cur.fetchall()	
		if len(r)>0:
			cur.execute(delect)
		cur.execute(table)
	
		print 'Loading data into tempResolution'	
		batchSize = 5000
		count = 0

		for pdb in resDic:
			rCells = [pdb, resDic[pdb]]
			try:
				cur.execute(inserts,rCells)
				if (count % batchSize) == 0:
					conn.commit()
			except sqlite3.Error, e:
				if conn:
					conn.rollback()
				print "Error %s:" % e.args[0]
				sys.exit(1)
			count += 1
		if count:
			conn.commit()
		print 'Indexing tempResolution ...'
		for index in indices:
			cur.execute(index)
		print 'Optimizing tempResolution ...'
		cur.execute('ANALYZE tempResolution')
	conn.close()

def get_biounit_structure(pdbid,outDirectory):
	outDirectory = preparation.check_slash(outDirectory)
	#print outDirectory
	select = 'SELECT pdbid, biounit FROM tempBiounit WHERE pdbid = ?1'
	dbfile = userConfig.humanDB+'.sqlite' 
	if (not os.path.exists(dbfile)) or (not os.path.isfile(dbfile)):
		print 'ERROR: '+dbfile+'does not exist.'
		sys.exit()
	conn = sqlite3.connect(dbfile)
	with conn:
		cur = conn.cursor()
		#print pdbid
		cur.execute(select,[pdbid])	
		r = cur.fetchall()
		chains = ''
		biounitPdbs = glob.glob(outDirectory+pdbid.upper()+'*')
		for i in range( len(biounitPdbs) ):
			if chains =='':
				chains = biounitPdbs[i].split('.')[1]+'.'+biounitPdbs[i].split('.')[2]
			else:
				chains += ','+biounitPdbs[i].split('.')[1]+'.'+biounitPdbs[i].split('.')[2]
	
		if len(r)>0 and( len(biounitPdbs) <1):
			biounits = r[0][1].strip().split('\t')
			for biounit in biounits:
				content =''
				midleName = biounit.split('.')[0][1:3]
				biounitUrl = 'ftp://ftp.wwpdb.org/pub/pdb/data/biounit/PDB/divided/'+midleName+'/'+biounit
				#print biounitUrl
				#print outDirectory+biounit
				#urllib.urlretrieve(biounitUrl,outDirectory+biounit)
				#cmd = 'gunzip -f '+ outDirectory+biounit
				#preparation.run_cmd(cmd)

				try:
					response = urllib2.urlopen(biounitUrl)
				except Exception, e:
					print biounitUrl
					print e
					continue
				compressedFile = StringIO.StringIO()
				compressedFile.write(response.read())
				compressedFile.seek(0)
				decompressedFile = gzip.GzipFile(fileobj=compressedFile,mode='rb')
				content = decompressedFile.read()
				biounit = biounit.split('.')[0]+'.'+biounit.split('.')[1]
				postfix = biounit.split('.')[1]
				#print biounit
				with open(outDirectory+biounit,'w') as outfile:
					outfile.write(content)

				lines = content.strip().split('\n')
				index = 0 
				if chains != '':
					chains += ','
				tempChain = ''
				"""
				while index <(len(lines)):
					if re.match('^MODEL',lines[index]):
						index += 1
						while index <len(lines):
							if re.match('^ATOM',lines[index]):
								if tempChain == '':
									tempChain += lines[index][21]
								else:
									tempChain += '_'+lines[index][21]
								index += 1
								break
							else:
								index += 1
					elif re.match('^TER',lines[index]):
						index += 1
						while index <len(lines):
							if re.match('^ATOM',lines[index]):
								tempChain += lines[index][21]
								index += 1
								break
							elif re.match('^ENDMDL',lines[index]):
								index += 1
								break
							else:
								index += 1

					else:
						index += 1
				"""
				stop = 0
				while index <(len(lines)):
					if stop == 1:
						break
					if re.match('^MODEL',lines[index]):
						index += 1
						while index <len(lines):
							if re.match('^ATOM',lines[index]):
								tempChain += lines[index][21]
								index += 1
								break
							else:
								index += 1
					elif re.match('^TER',lines[index]):
						index += 1
						while index <len(lines):
							if re.match('^ATOM',lines[index]):
								tempChain += lines[index][21]
								index += 1
								break
							elif re.match('^ENDMDL',lines[index]):
								index += 1
								stop = 1
								break
							else:
								index += 1

					else:
						index += 1

				if len(tempChain)>90:
					print '******'+tempChain
					os.rename(outDirectory+biounit,outDirectory+pdbid+'.'+tempChain[0:90]+'.'+postfix)			
					chains += tempChain[0:90]+'.'+postfix				
				else:
					os.rename(outDirectory+biounit,outDirectory+pdbid+'.'+tempChain+'.'+postfix)			
					chains += tempChain+'.'+postfix				
	conn.close()
	return len(r), chains

def get_resolution(pdbid):
	select = 'SELECT resolution FROM tempResolution WHERE pdbid = ?1'
	dbfile = userConfig.humanDB+'.sqlite' 
	if (not os.path.exists(dbfile)) or (not os.path.isfile(dbfile)):
		print 'ERROR: '+dbfile+'does not exist.'
		sys.exit()
	conn = sqlite3.connect(dbfile)
	resolution = 'Null'
	with conn:
		cur = conn.cursor()
		#print pdbid
		cur.execute(select,[pdbid])	
		r = cur.fetchall()
	
		if len(r)>0:
			resolution = r[0][0]
	conn.close()
	return resolution


def pdb_structure(acc, name, pdbLib, biounitDir):
	pdbId = name.strip().split(':')[0]
	if len(name.strip().split(':'))>1:
		chain = name.strip().split(':')[1]
	else:
		chain = None

	pdbUrl = 'http://www.rcsb.org/pdb/files/'+pdbId+'.pdb' 
	pdbFile = pdbLib+'/'+pdbId+'.pdb'
	
	#if not os.path.isfile(pdbFile):
	#	preparation.download(pdbUrl,pdbFile)
	pdbContent = preparation.readUrl(pdbUrl)
	if pdbContent == '':
		#preparation.rmfile(pdbFile)	
		return None, None, None,None,None,None,None,None,None 
	
	pdbf = open(pdbFile,'w')
	pdbf.write(pdbContent)
	pdbf.close()

	lines = pdbContent.strip().split('\n')

	pdbStart = None
	pdbEnd = None
	uniprotStart = None
	structStart = None
	structEnd = None
	matchLable = 0

			
	for line in lines:
		line = line
		if chain:
			match = re.match('DBREF\s+'+pdbId+'\s+('+chain+')\s+(\-?\d+)\s+(\d+)\s+UNP\s+'+acc+'\s+\S+\s+(\d+)',line.strip())
		else:
			match = re.match('DBREF\s+'+pdbId+'\s+(\S+)\s+(\-?\d+)\s+(\d+)\s+UNP\s+'+acc+'\s+\S+\s+(\d+)',line.strip())
		if match:
			if not chain:
				chain = match.group(1)
			matchLable = 1
			tempPdbStart = match.group(2)
			tempPdbEnd = match.group(3)
			tempUniprotStart = match.group(4)
			if (not pdbStart) or (int(tempPdbStart) < int(pdbStart)):
				pdbStart = tempPdbStart
				uniprotStart = tempUniprotStart
			if (not pdbEnd) or (int(tempPdbEnd) > int(pdbEnd)):
				pdbEnd = tempPdbEnd
		elif re.match('^ATOM',line):
			tempChain = line[21]
			if (chain == tempChain) and (not structStart):
				structStart = line[22:30].strip()
		elif re.match('^TER',line):
			tempChain = line[21]
			if (chain == tempChain) and (not structEnd):
				structEnd = line[22:30].strip()
	
	biounitLable, biochains= get_biounit_structure(pdbId,biounitDir)	
	resolution = get_resolution(pdbId)
	if matchLable == 1:
		return pdbStart, pdbEnd, uniprotStart, structStart, structEnd, pdbId+':'+chain,biounitLable,biochains,resolution
	else:
		return pdbStart, pdbEnd, uniprotStart, structStart, structEnd, None, biounitLable,biochains,resolution
	
			
def createdb(dbfile,mapfile):
	tCol = 10
	table = 'CREATE TABLE resMap (acc VARCHAR(12) NOT NULL, name VARCHAR(24) NOT NULL COLLATE NOCASE, pdbstart INTERGER NOT NULL , pdbend INTERGER NOT NULL, uniprotstart INTERGER NOT NULL, structurestart INTERGER NOT NULL, structureend INTERGER NOT NULL, resolution FLOAT NOT NULL, biounit INTERGER NOT NULL,biochains VARCHAR(1000) NOT NULL)'

	inserts = 'INSERT INTO resMap VALUES ('+', '.join(('?')*tCol)+')'
	indices = ['CREATE INDEX x_accacc ON resMap (acc)',
			'CREATE INDEX x_accname ON resMap (name)',
			'CREATE INDEX x_accacna ON resMap (acc, name)']
	delect = 'DROP TABLE resMap'
	
	print 'Creating SQLite database ...'
	
	conn = sqlite3.connect(dbfile)
	with conn:
		cur = conn.cursor()

		cur.execute('SELECT * FROM sqlite_master WHERE type = \'table\' and tbl_name = \'resMap\'')
		r = cur.fetchall()	
		if len(r)>0:
			cur.execute('DROP TABLE resMap')
		cur.execute(table)
	
		print 'Loading data into resMap'	
		fin = open(mapfile,'r')
		batchSize = 5000
		count = 0

		line = fin.readline()
		while line:
			#if re.search('Null',line):
			#	line = fin.readline()
			#	continue
			line = re.sub('\n$','',line)
			rCells = re.split('\t',line,tCol)
			try:
				cur.execute(inserts,rCells)
				if (count % batchSize) == 0:
					conn.commit()
			except sqlite3.Error, e:
				if conn:
					conn.rollback()
				print "Error %s:" % e.args[0]
				sys.exit(1)
			count += 1
			line = fin.readline()
		fin.close()
		if count:
			conn.commit()
		print 'Indexing resMap ...'
		for index in indices:
			cur.execute(index)
		print 'Optimizing resMap ...'
		cur.execute('ANALYZE resMap')
	conn.close()


outFile = preparation.check_slash('/nfs/amino-home/liquan/CancerDriver/myprogram/database')
dictionary = getpdb2acc()
pdbLib = outFile+'pdb'
preparation.mkdir(pdbLib)

#build_sifts_db(outFile)
#get_pdb_biounit(outFile)
#get_resolution(outFile)
pdbLib = outFile+'pdb'
preparation.mkdir(pdbLib)
biounitDir = pdbLib+'/biounit/'
preparation.mkdir(biounitDir)
mapingFile = '/nfs/amino-home/liquan/CancerDriver/myprogram/database/human.resMap'
resMap = open(mapingFile,'w')
if len(dictionary)>0:
	for row in range(len(dictionary)):
		acc = dictionary[row][0]
		name = dictionary[row][1]
		pdbStart, pdbEnd, uniprotStart, structStart, structEnd,newname,biounitLable,biochains,resolution = pdb_structure(acc, name, pdbLib,biounitDir)
		if not newname:
			continue
		outline = acc+'\t'+newname+'\t'	
		if pdbStart:
			outline += pdbStart+'\t'
		else:
			outline += 'Null'+'\t'
		if pdbEnd:
			outline += pdbEnd+'\t'
		else:
			outline += 'Null'+'\t'
		if uniprotStart:
			outline += uniprotStart+'\t'
		else:
			outline += 'Null'+'\t'
		if structStart:
			outline += structStart+'\t'
		else:
			outline += 'Null'+'\t'
		if structEnd:
			outline += structEnd+'\t'
		else:
			outline += 'Null'+'\t'
		if resolution == 'Null':
			outline += '-2\t'
		else:
			outline += str(resolution) + '\t'
		outline += str(biounitLable) + '\t'
		if biochains == '':
			outline += 'Null'+'\n'
		else:
			outline += str(biochains) + '\n'
		resMap.write(outline)

resMap.close()
dbfile ='/nfs/amino-home/liquan/CancerDriver/myprogram/database/human.sqlite' 
createdb(dbfile,mapingFile)
